/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
#ifndef INTERCONNECT_QCOM_IPQ5424_H
#define INTERCONNECT_QCOM_IPQ5424_H

#define MASTER_ANOC_PCIE0		0
#define SLAVE_ANOC_PCIE0		1
#define MASTER_CNOC_PCIE0		2
#define SLAVE_CNOC_PCIE0		3
#define MASTER_ANOC_PCIE1		4
#define SLAVE_ANOC_PCIE1		5
#define MASTER_CNOC_PCIE1		6
#define SLAVE_CNOC_PCIE1		7
#define MASTER_ANOC_PCIE2		8
#define SLAVE_ANOC_PCIE2		9
#define MASTER_CNOC_PCIE2		10
#define SLAVE_CNOC_PCIE2		11
#define MASTER_ANOC_PCIE3		12
#define SLAVE_ANOC_PCIE3		13
#define MASTER_CNOC_PCIE3		14
#define SLAVE_CNOC_PCIE3		15
#define MASTER_CNOC_USB			16
#define SLAVE_CNOC_USB			17

#endif /* INTERCONNECT_QCOM_IPQ5424_H */
