savedcmd_arch/arm64/include/generated/asm/unistd_compat_32.h := sh ./scripts/syscallhdr.sh --emit-nr --prefix "compat32_" --abis common,32 arch/arm64/tools/syscall_32.tbl arch/arm64/include/generated/asm/unistd_compat_32.h
