#define UTS_MACHINE		"aarch64"
#define LINUX_COMPILE_BY	"builduser"
#define LINUX_COMPILE_HOST	"gitlab-runner-ampere-cax41"
#define LINUX_COMPILER		"gcc (GCC) 14.2.1 20250207, GNU ld (GNU Binutils) 2.44"
