/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
/*
 * Copyright (c) 2024, Qualcomm Innovation Center, Inc. All rights reserved.
 */

#ifndef _DT_BINDINGS_CLK_QCOM_GPUCC_QCS8300_H
#define _DT_BINDINGS_CLK_QCOM_GPUCC_QCS8300_H

#include "qcom,sa8775p-gpucc.h"

/* QCS8300 introduces below new clocks compared to SA8775P */

/* GPU_CC clocks */
#define GPU_CC_CX_ACCU_SHIFT_CLK		23
#define GPU_CC_GX_ACCU_SHIFT_CLK		24

#endif
