/* SPDX-License-Identifier: GPL-2.0 WITH Linux-syscall-note */
#ifndef _UAPI_LINUX_RPMSG_TYPES_H
#define _UAPI_LINUX_RPMSG_TYPES_H

#include <linux/types.h>

typedef __u16 __bitwise __rpmsg16;
typedef __u32 __bitwise __rpmsg32;
typedef __u64 __bitwise __rpmsg64;

#endif /* _UAPI_LINUX_RPMSG_TYPES_H */
