/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
/*
 * Copyright (c) 2023, Qualcomm Innovation Center, Inc. All rights reserved.
 */

#ifndef _DT_BINDINGS_RESET_QCOM_X1E80100_GPU_CC_H
#define _DT_BINDINGS_RESET_QCOM_X1E80100_GPU_CC_H

#define GPUCC_GPU_CC_ACD_BCR					0
#define GPUCC_GPU_CC_CB_BCR					1
#define GPUCC_GPU_CC_CX_BCR					2
#define GPUCC_GPU_CC_FAST_HUB_BCR				3
#define GPUCC_GPU_CC_FF_BCR					4
#define GPUCC_GPU_CC_GFX3D_AON_BCR				5
#define GPUCC_GPU_CC_GMU_BCR					6
#define GPUCC_GPU_CC_GX_BCR					7
#define GPUCC_GPU_CC_XO_BCR					8

#endif
