/* SPDX-License-Identifier: (GPL-2.0+ OR MIT) */
/*
 * Copyright (c) 2020 Yangtao Li <frank@allwinnertech.com>
 */

#ifndef _DT_BINDINGS_RST_SUN50I_A100_R_CCU_H_
#define _DT_BINDINGS_RST_SUN50I_A100_R_CCU_H_

#define RST_R_APB1_TIMER	0
#define RST_R_APB1_BUS_PWM	1
#define RST_R_APB1_PPU		2
#define RST_R_APB2_UART		3
#define RST_R_APB2_I2C0		4
#define RST_R_APB2_I2C1		5
#define RST_R_APB1_BUS_IR	6
#define RST_R_AHB_BUS_RTC	7

#endif /* _DT_BINDINGS_RST_SUN50I_A100_R_CCU_H_ */
