#ifndef _UAPI_ASM_UNISTD_COMPAT_32_H
#define _UAPI_ASM_UNISTD_COMPAT_32_H

#define __NR_compat32_restart_syscall 0
#define __NR_compat32_exit 1
#define __NR_compat32_fork 2
#define __NR_compat32_read 3
#define __NR_compat32_write 4
#define __NR_compat32_open 5
#define __NR_compat32_close 6
#define __NR_compat32_creat 8
#define __NR_compat32_link 9
#define __NR_compat32_unlink 10
#define __NR_compat32_execve 11
#define __NR_compat32_chdir 12
#define __NR_compat32_mknod 14
#define __NR_compat32_chmod 15
#define __NR_compat32_lchown 16
#define __NR_compat32_lseek 19
#define __NR_compat32_getpid 20
#define __NR_compat32_mount 21
#define __NR_compat32_setuid 23
#define __NR_compat32_getuid 24
#define __NR_compat32_ptrace 26
#define __NR_compat32_pause 29
#define __NR_compat32_access 33
#define __NR_compat32_nice 34
#define __NR_compat32_sync 36
#define __NR_compat32_kill 37
#define __NR_compat32_rename 38
#define __NR_compat32_mkdir 39
#define __NR_compat32_rmdir 40
#define __NR_compat32_dup 41
#define __NR_compat32_pipe 42
#define __NR_compat32_times 43
#define __NR_compat32_brk 45
#define __NR_compat32_setgid 46
#define __NR_compat32_getgid 47
#define __NR_compat32_geteuid 49
#define __NR_compat32_getegid 50
#define __NR_compat32_acct 51
#define __NR_compat32_umount2 52
#define __NR_compat32_ioctl 54
#define __NR_compat32_fcntl 55
#define __NR_compat32_setpgid 57
#define __NR_compat32_umask 60
#define __NR_compat32_chroot 61
#define __NR_compat32_ustat 62
#define __NR_compat32_dup2 63
#define __NR_compat32_getppid 64
#define __NR_compat32_getpgrp 65
#define __NR_compat32_setsid 66
#define __NR_compat32_sigaction 67
#define __NR_compat32_setreuid 70
#define __NR_compat32_setregid 71
#define __NR_compat32_sigsuspend 72
#define __NR_compat32_sigpending 73
#define __NR_compat32_sethostname 74
#define __NR_compat32_setrlimit 75
#define __NR_compat32_getrusage 77
#define __NR_compat32_gettimeofday 78
#define __NR_compat32_settimeofday 79
#define __NR_compat32_getgroups 80
#define __NR_compat32_setgroups 81
#define __NR_compat32_symlink 83
#define __NR_compat32_readlink 85
#define __NR_compat32_uselib 86
#define __NR_compat32_swapon 87
#define __NR_compat32_reboot 88
#define __NR_compat32_munmap 91
#define __NR_compat32_truncate 92
#define __NR_compat32_ftruncate 93
#define __NR_compat32_fchmod 94
#define __NR_compat32_fchown 95
#define __NR_compat32_getpriority 96
#define __NR_compat32_setpriority 97
#define __NR_compat32_statfs 99
#define __NR_compat32_fstatfs 100
#define __NR_compat32_syslog 103
#define __NR_compat32_setitimer 104
#define __NR_compat32_getitimer 105
#define __NR_compat32_stat 106
#define __NR_compat32_lstat 107
#define __NR_compat32_fstat 108
#define __NR_compat32_vhangup 111
#define __NR_compat32_wait4 114
#define __NR_compat32_swapoff 115
#define __NR_compat32_sysinfo 116
#define __NR_compat32_fsync 118
#define __NR_compat32_sigreturn 119
#define __NR_compat32_clone 120
#define __NR_compat32_setdomainname 121
#define __NR_compat32_uname 122
#define __NR_compat32_adjtimex 124
#define __NR_compat32_mprotect 125
#define __NR_compat32_sigprocmask 126
#define __NR_compat32_init_module 128
#define __NR_compat32_delete_module 129
#define __NR_compat32_quotactl 131
#define __NR_compat32_getpgid 132
#define __NR_compat32_fchdir 133
#define __NR_compat32_bdflush 134
#define __NR_compat32_sysfs 135
#define __NR_compat32_personality 136
#define __NR_compat32_setfsuid 138
#define __NR_compat32_setfsgid 139
#define __NR_compat32__llseek 140
#define __NR_compat32_getdents 141
#define __NR_compat32__newselect 142
#define __NR_compat32_flock 143
#define __NR_compat32_msync 144
#define __NR_compat32_readv 145
#define __NR_compat32_writev 146
#define __NR_compat32_getsid 147
#define __NR_compat32_fdatasync 148
#define __NR_compat32__sysctl 149
#define __NR_compat32_mlock 150
#define __NR_compat32_munlock 151
#define __NR_compat32_mlockall 152
#define __NR_compat32_munlockall 153
#define __NR_compat32_sched_setparam 154
#define __NR_compat32_sched_getparam 155
#define __NR_compat32_sched_setscheduler 156
#define __NR_compat32_sched_getscheduler 157
#define __NR_compat32_sched_yield 158
#define __NR_compat32_sched_get_priority_max 159
#define __NR_compat32_sched_get_priority_min 160
#define __NR_compat32_sched_rr_get_interval 161
#define __NR_compat32_nanosleep 162
#define __NR_compat32_mremap 163
#define __NR_compat32_setresuid 164
#define __NR_compat32_getresuid 165
#define __NR_compat32_poll 168
#define __NR_compat32_nfsservctl 169
#define __NR_compat32_setresgid 170
#define __NR_compat32_getresgid 171
#define __NR_compat32_prctl 172
#define __NR_compat32_rt_sigreturn 173
#define __NR_compat32_rt_sigaction 174
#define __NR_compat32_rt_sigprocmask 175
#define __NR_compat32_rt_sigpending 176
#define __NR_compat32_rt_sigtimedwait 177
#define __NR_compat32_rt_sigqueueinfo 178
#define __NR_compat32_rt_sigsuspend 179
#define __NR_compat32_pread64 180
#define __NR_compat32_pwrite64 181
#define __NR_compat32_chown 182
#define __NR_compat32_getcwd 183
#define __NR_compat32_capget 184
#define __NR_compat32_capset 185
#define __NR_compat32_sigaltstack 186
#define __NR_compat32_sendfile 187
#define __NR_compat32_vfork 190
#define __NR_compat32_ugetrlimit 191
#define __NR_compat32_mmap2 192
#define __NR_compat32_truncate64 193
#define __NR_compat32_ftruncate64 194
#define __NR_compat32_stat64 195
#define __NR_compat32_lstat64 196
#define __NR_compat32_fstat64 197
#define __NR_compat32_lchown32 198
#define __NR_compat32_getuid32 199
#define __NR_compat32_getgid32 200
#define __NR_compat32_geteuid32 201
#define __NR_compat32_getegid32 202
#define __NR_compat32_setreuid32 203
#define __NR_compat32_setregid32 204
#define __NR_compat32_getgroups32 205
#define __NR_compat32_setgroups32 206
#define __NR_compat32_fchown32 207
#define __NR_compat32_setresuid32 208
#define __NR_compat32_getresuid32 209
#define __NR_compat32_setresgid32 210
#define __NR_compat32_getresgid32 211
#define __NR_compat32_chown32 212
#define __NR_compat32_setuid32 213
#define __NR_compat32_setgid32 214
#define __NR_compat32_setfsuid32 215
#define __NR_compat32_setfsgid32 216
#define __NR_compat32_getdents64 217
#define __NR_compat32_pivot_root 218
#define __NR_compat32_mincore 219
#define __NR_compat32_madvise 220
#define __NR_compat32_fcntl64 221
#define __NR_compat32_gettid 224
#define __NR_compat32_readahead 225
#define __NR_compat32_setxattr 226
#define __NR_compat32_lsetxattr 227
#define __NR_compat32_fsetxattr 228
#define __NR_compat32_getxattr 229
#define __NR_compat32_lgetxattr 230
#define __NR_compat32_fgetxattr 231
#define __NR_compat32_listxattr 232
#define __NR_compat32_llistxattr 233
#define __NR_compat32_flistxattr 234
#define __NR_compat32_removexattr 235
#define __NR_compat32_lremovexattr 236
#define __NR_compat32_fremovexattr 237
#define __NR_compat32_tkill 238
#define __NR_compat32_sendfile64 239
#define __NR_compat32_futex 240
#define __NR_compat32_sched_setaffinity 241
#define __NR_compat32_sched_getaffinity 242
#define __NR_compat32_io_setup 243
#define __NR_compat32_io_destroy 244
#define __NR_compat32_io_getevents 245
#define __NR_compat32_io_submit 246
#define __NR_compat32_io_cancel 247
#define __NR_compat32_exit_group 248
#define __NR_compat32_lookup_dcookie 249
#define __NR_compat32_epoll_create 250
#define __NR_compat32_epoll_ctl 251
#define __NR_compat32_epoll_wait 252
#define __NR_compat32_remap_file_pages 253
#define __NR_compat32_set_tid_address 256
#define __NR_compat32_timer_create 257
#define __NR_compat32_timer_settime 258
#define __NR_compat32_timer_gettime 259
#define __NR_compat32_timer_getoverrun 260
#define __NR_compat32_timer_delete 261
#define __NR_compat32_clock_settime 262
#define __NR_compat32_clock_gettime 263
#define __NR_compat32_clock_getres 264
#define __NR_compat32_clock_nanosleep 265
#define __NR_compat32_statfs64 266
#define __NR_compat32_fstatfs64 267
#define __NR_compat32_tgkill 268
#define __NR_compat32_utimes 269
#define __NR_compat32_arm_fadvise64_64 270
#define __NR_compat32_pciconfig_iobase 271
#define __NR_compat32_pciconfig_read 272
#define __NR_compat32_pciconfig_write 273
#define __NR_compat32_mq_open 274
#define __NR_compat32_mq_unlink 275
#define __NR_compat32_mq_timedsend 276
#define __NR_compat32_mq_timedreceive 277
#define __NR_compat32_mq_notify 278
#define __NR_compat32_mq_getsetattr 279
#define __NR_compat32_waitid 280
#define __NR_compat32_socket 281
#define __NR_compat32_bind 282
#define __NR_compat32_connect 283
#define __NR_compat32_listen 284
#define __NR_compat32_accept 285
#define __NR_compat32_getsockname 286
#define __NR_compat32_getpeername 287
#define __NR_compat32_socketpair 288
#define __NR_compat32_send 289
#define __NR_compat32_sendto 290
#define __NR_compat32_recv 291
#define __NR_compat32_recvfrom 292
#define __NR_compat32_shutdown 293
#define __NR_compat32_setsockopt 294
#define __NR_compat32_getsockopt 295
#define __NR_compat32_sendmsg 296
#define __NR_compat32_recvmsg 297
#define __NR_compat32_semop 298
#define __NR_compat32_semget 299
#define __NR_compat32_semctl 300
#define __NR_compat32_msgsnd 301
#define __NR_compat32_msgrcv 302
#define __NR_compat32_msgget 303
#define __NR_compat32_msgctl 304
#define __NR_compat32_shmat 305
#define __NR_compat32_shmdt 306
#define __NR_compat32_shmget 307
#define __NR_compat32_shmctl 308
#define __NR_compat32_add_key 309
#define __NR_compat32_request_key 310
#define __NR_compat32_keyctl 311
#define __NR_compat32_semtimedop 312
#define __NR_compat32_vserver 313
#define __NR_compat32_ioprio_set 314
#define __NR_compat32_ioprio_get 315
#define __NR_compat32_inotify_init 316
#define __NR_compat32_inotify_add_watch 317
#define __NR_compat32_inotify_rm_watch 318
#define __NR_compat32_mbind 319
#define __NR_compat32_get_mempolicy 320
#define __NR_compat32_set_mempolicy 321
#define __NR_compat32_openat 322
#define __NR_compat32_mkdirat 323
#define __NR_compat32_mknodat 324
#define __NR_compat32_fchownat 325
#define __NR_compat32_futimesat 326
#define __NR_compat32_fstatat64 327
#define __NR_compat32_unlinkat 328
#define __NR_compat32_renameat 329
#define __NR_compat32_linkat 330
#define __NR_compat32_symlinkat 331
#define __NR_compat32_readlinkat 332
#define __NR_compat32_fchmodat 333
#define __NR_compat32_faccessat 334
#define __NR_compat32_pselect6 335
#define __NR_compat32_ppoll 336
#define __NR_compat32_unshare 337
#define __NR_compat32_set_robust_list 338
#define __NR_compat32_get_robust_list 339
#define __NR_compat32_splice 340
#define __NR_compat32_arm_sync_file_range 341
#define __NR_compat32_tee 342
#define __NR_compat32_vmsplice 343
#define __NR_compat32_move_pages 344
#define __NR_compat32_getcpu 345
#define __NR_compat32_epoll_pwait 346
#define __NR_compat32_kexec_load 347
#define __NR_compat32_utimensat 348
#define __NR_compat32_signalfd 349
#define __NR_compat32_timerfd_create 350
#define __NR_compat32_eventfd 351
#define __NR_compat32_fallocate 352
#define __NR_compat32_timerfd_settime 353
#define __NR_compat32_timerfd_gettime 354
#define __NR_compat32_signalfd4 355
#define __NR_compat32_eventfd2 356
#define __NR_compat32_epoll_create1 357
#define __NR_compat32_dup3 358
#define __NR_compat32_pipe2 359
#define __NR_compat32_inotify_init1 360
#define __NR_compat32_preadv 361
#define __NR_compat32_pwritev 362
#define __NR_compat32_rt_tgsigqueueinfo 363
#define __NR_compat32_perf_event_open 364
#define __NR_compat32_recvmmsg 365
#define __NR_compat32_accept4 366
#define __NR_compat32_fanotify_init 367
#define __NR_compat32_fanotify_mark 368
#define __NR_compat32_prlimit64 369
#define __NR_compat32_name_to_handle_at 370
#define __NR_compat32_open_by_handle_at 371
#define __NR_compat32_clock_adjtime 372
#define __NR_compat32_syncfs 373
#define __NR_compat32_sendmmsg 374
#define __NR_compat32_setns 375
#define __NR_compat32_process_vm_readv 376
#define __NR_compat32_process_vm_writev 377
#define __NR_compat32_kcmp 378
#define __NR_compat32_finit_module 379
#define __NR_compat32_sched_setattr 380
#define __NR_compat32_sched_getattr 381
#define __NR_compat32_renameat2 382
#define __NR_compat32_seccomp 383
#define __NR_compat32_getrandom 384
#define __NR_compat32_memfd_create 385
#define __NR_compat32_bpf 386
#define __NR_compat32_execveat 387
#define __NR_compat32_userfaultfd 388
#define __NR_compat32_membarrier 389
#define __NR_compat32_mlock2 390
#define __NR_compat32_copy_file_range 391
#define __NR_compat32_preadv2 392
#define __NR_compat32_pwritev2 393
#define __NR_compat32_pkey_mprotect 394
#define __NR_compat32_pkey_alloc 395
#define __NR_compat32_pkey_free 396
#define __NR_compat32_statx 397
#define __NR_compat32_rseq 398
#define __NR_compat32_io_pgetevents 399
#define __NR_compat32_migrate_pages 400
#define __NR_compat32_kexec_file_load 401
#define __NR_compat32_clock_gettime64 403
#define __NR_compat32_clock_settime64 404
#define __NR_compat32_clock_adjtime64 405
#define __NR_compat32_clock_getres_time64 406
#define __NR_compat32_clock_nanosleep_time64 407
#define __NR_compat32_timer_gettime64 408
#define __NR_compat32_timer_settime64 409
#define __NR_compat32_timerfd_gettime64 410
#define __NR_compat32_timerfd_settime64 411
#define __NR_compat32_utimensat_time64 412
#define __NR_compat32_pselect6_time64 413
#define __NR_compat32_ppoll_time64 414
#define __NR_compat32_io_pgetevents_time64 416
#define __NR_compat32_recvmmsg_time64 417
#define __NR_compat32_mq_timedsend_time64 418
#define __NR_compat32_mq_timedreceive_time64 419
#define __NR_compat32_semtimedop_time64 420
#define __NR_compat32_rt_sigtimedwait_time64 421
#define __NR_compat32_futex_time64 422
#define __NR_compat32_sched_rr_get_interval_time64 423
#define __NR_compat32_pidfd_send_signal 424
#define __NR_compat32_io_uring_setup 425
#define __NR_compat32_io_uring_enter 426
#define __NR_compat32_io_uring_register 427
#define __NR_compat32_open_tree 428
#define __NR_compat32_move_mount 429
#define __NR_compat32_fsopen 430
#define __NR_compat32_fsconfig 431
#define __NR_compat32_fsmount 432
#define __NR_compat32_fspick 433
#define __NR_compat32_pidfd_open 434
#define __NR_compat32_clone3 435
#define __NR_compat32_close_range 436
#define __NR_compat32_openat2 437
#define __NR_compat32_pidfd_getfd 438
#define __NR_compat32_faccessat2 439
#define __NR_compat32_process_madvise 440
#define __NR_compat32_epoll_pwait2 441
#define __NR_compat32_mount_setattr 442
#define __NR_compat32_quotactl_fd 443
#define __NR_compat32_landlock_create_ruleset 444
#define __NR_compat32_landlock_add_rule 445
#define __NR_compat32_landlock_restrict_self 446
#define __NR_compat32_process_mrelease 448
#define __NR_compat32_futex_waitv 449
#define __NR_compat32_set_mempolicy_home_node 450
#define __NR_compat32_cachestat 451
#define __NR_compat32_fchmodat2 452
#define __NR_compat32_map_shadow_stack 453
#define __NR_compat32_futex_wake 454
#define __NR_compat32_futex_wait 455
#define __NR_compat32_futex_requeue 456
#define __NR_compat32_statmount 457
#define __NR_compat32_listmount 458
#define __NR_compat32_lsm_get_self_attr 459
#define __NR_compat32_lsm_set_self_attr 460
#define __NR_compat32_lsm_list_modules 461
#define __NR_compat32_mseal 462
#define __NR_compat32_setxattrat 463
#define __NR_compat32_getxattrat 464
#define __NR_compat32_listxattrat 465
#define __NR_compat32_removexattrat 466
#define __NR_compat32_open_tree_attr 467

#ifdef __KERNEL__
#define __NR_compat32_syscalls 468
#endif

#endif /* _UAPI_ASM_UNISTD_COMPAT_32_H */
