/* SPDX-License-Identifier: (GPL-2.0+ OR MIT) */
/*
 * Copyright (c) 2023 Amlogic, Inc.
 * Author: hongyu chen1 <hongyu.chen1@amlogic.com>
 */
#ifndef _DT_BINDINGS_AMLOGIC_C3_POWER_H
#define _DT_BINDINGS_AMLOGIC_C3_POWER_H

#define PWRC_C3_NNA_ID				0
#define PWRC_C3_AUDIO_ID			1
#define PWRC_C3_RESV_SEC_ID			2
#define PWRC_C3_SDIOA_ID			3
#define PWRC_C3_EMMC_ID				4
#define PWRC_C3_USB_COMB_ID			5
#define PWRC_C3_SDCARD_ID			6
#define PWRC_C3_ETH_ID				7
#define PWRC_C3_RESV0_ID			8
#define PWRC_C3_GE2D_ID				9
#define PWRC_C3_CVE_ID				10
#define PWRC_C3_GDC_WRAP_ID			11
#define PWRC_C3_ISP_TOP_ID			12
#define PWRC_C3_MIPI_ISP_WRAP_ID		13
#define PWRC_C3_VCODEC_ID			14

#endif
