/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
/*
 * Copyright (C) 2023 Linaro Ltd. All rights reserved.
 */

#ifndef __DT_BINDINGS_INTERCONNECT_QCOM_MSM8996_CBF_H
#define __DT_BINDINGS_INTERCONNECT_QCOM_MSM8996_CBF_H

#define MASTER_CBF_M4M		0
#define SLAVE_CBF_M4M		1

#endif
