/* SPDX-License-Identifier: (GPL-2.0+ OR MIT) */
/*
 * Copyright (C) 2025
 */

#ifndef _DT_BINDINGS_CLK_CPU_SUN55I_A523_H_
#define _DT_BINDINGS_CLK_CPU_SUN55I_A523_H_

#define CLK_CPUL		0
#define CLK_CPUB		1
#define CLK_PLL_CPUL		2
#define CLK_PLL_CPUB		3

#define CLK_CPU_MAX_NO	(CLK_PLL_CPUB + 1)

#endif /* _DT_BINDINGS_CLK_CPU_SUN55I_A523_H_ */
