/*
 * Automatically generated file; DO NOT EDIT.
 * Linux/arm64 6.16.0-rc7-1 Kernel Configuration
 */
#define CONFIG_RTC_DRV_CROS_EC_MODULE 1
#define CONFIG_AIC79XX_RESET_DELAY_MS 5000
#define CONFIG_VIDEO_PVRUSB2_MODULE 1
#define CONFIG_NFC_MRVL_MODULE 1
#define CONFIG_SERIAL_8250_RUNTIME_UARTS 4
#define CONFIG_USB_GSPCA_PAC7311_MODULE 1
#define CONFIG_GENERIC_PHY_MIPI_DPHY 1
#define CONFIG_MAX9611_MODULE 1
#define CONFIG_NET_CLS_BPF_MODULE 1
#define CONFIG_ROCKCHIP_DW_MIPI_DSI2 1
#define CONFIG_DHT11_MODULE 1
#define CONFIG_USB_LD_MODULE 1
#define CONFIG_SPI_BCM2835AUX_MODULE 1
#define CONFIG_ARCH_USES_HIGH_VMA_FLAGS 1
#define CONFIG_USB_GSPCA_SQ930X_MODULE 1
#define CONFIG_RFKILL_MODULE 1
#define CONFIG_SPARSEMEM_VMEMMAP_ENABLE 1
#define CONFIG_DRM_PANEL_SIMPLE_MODULE 1
#define CONFIG_USB_COMMON 1
#define CONFIG_MTD_MAP_BANK_WIDTH_1 1
#define CONFIG_TABLET_SERIAL_WACOM4_MODULE 1
#define CONFIG_PCIE_IPROC 1
#define CONFIG_SCSI_ENCLOSURE_MODULE 1
#define CONFIG_BALLOON_COMPACTION 1
#define CONFIG_NFT_FIB_IPV4_MODULE 1
#define CONFIG_QRTR_MODULE 1
#define CONFIG_ARM_SMMU_V3_SVA 1
#define CONFIG_PINCTRL_RK805 1
#define CONFIG_NET_ACT_PEDIT_MODULE 1
#define CONFIG_SND_SOC_TEGRA30_AHUB_MODULE 1
#define CONFIG_MMC 1
#define CONFIG_CRYPTO_SHA256_ARM64 1
#define CONFIG_BRCM_USB_PINMAP 1
#define CONFIG_CROS_EC_MODULE 1
#define CONFIG_CM32181_MODULE 1
#define CONFIG_GPIOLIB 1
#define CONFIG_USB_DWC2 1
#define CONFIG_DVB_LGS8GXX_MODULE 1
#define CONFIG_NF_CONNTRACK_EVENTS 1
#define CONFIG_NET_EMATCH_NBYTE_MODULE 1
#define CONFIG_CRYPTO_SM3_GENERIC_MODULE 1
#define CONFIG_TOUCHSCREEN_AUO_PIXCIR_MODULE 1
#define CONFIG_DM_CACHE_SMQ_MODULE 1
#define CONFIG_IR_SANYO_DECODER_MODULE 1
#define CONFIG_NFS_V3 1
#define CONFIG_INPUT_KXTJ9_MODULE 1
#define CONFIG_VIDEO_DEV_MODULE 1
#define CONFIG_USB_GSPCA_OV534_MODULE 1
#define CONFIG_ANDROID_BINDER_DEVICES "binder,hwbinder,vndbinder"
#define CONFIG_DMARD06_MODULE 1
#define CONFIG_AD7923_MODULE 1
#define CONFIG_DRM_DW_HDMI_CEC_MODULE 1
#define CONFIG_SND_EMU10K1_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_MASQUERADE_MODULE 1
#define CONFIG_VL6180_MODULE 1
#define CONFIG_V4L2_VP9_MODULE 1
#define CONFIG_QSEMI_PHY_MODULE 1
#define CONFIG_RFKILL_INPUT 1
#define CONFIG_USB_GSPCA_TOUPTEK_MODULE 1
#define CONFIG_GENERIC_IRQ_CHIP 1
#define CONFIG_USB_ROLE_SWITCH 1
#define CONFIG_ADT7316_MODULE 1
#define CONFIG_IRQ_BYPASS_MANAGER 1
#define CONFIG_NET_HANDSHAKE 1
#define CONFIG_NF_CONNTRACK_OVS 1
#define CONFIG_SPI_AMLOGIC_SPIFC_A1_MODULE 1
#define CONFIG_TEGRA_WATCHDOG_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_RATEEST_MODULE 1
#define CONFIG_ARM_SCPI_PROTOCOL 1
#define CONFIG_HTS221_I2C_MODULE 1
#define CONFIG_LIBFDT 1
#define CONFIG_R8169_MODULE 1
#define CONFIG_IWLMVM_MODULE 1
#define CONFIG_ATH10K_CE 1
#define CONFIG_RDS_TCP_MODULE 1
#define CONFIG_WCN36XX_DEBUGFS 1
#define CONFIG_PAGE_SHIFT 12
#define CONFIG_NLS_ASCII 1
#define CONFIG_NETFILTER_XT_MATCH_TIME_MODULE 1
#define CONFIG_USB_F_UAC1_LEGACY 1
#define CONFIG_SATA_SVW_MODULE 1
#define CONFIG_UEVENT_HELPER 1
#define CONFIG_NFS_COMMON 1
#define CONFIG_MMA9551_CORE_MODULE 1
#define CONFIG_PPPOATM_MODULE 1
#define CONFIG_WL12XX_MODULE 1
#define CONFIG_CGROUP_NET_CLASSID 1
#define CONFIG_NET_DSA_TAG_EDSA_MODULE 1
#define CONFIG_LOGIG940_FF 1
#define CONFIG_DVB_ISL6423_MODULE 1
#define CONFIG_SENSORS_MAX6697_MODULE 1
#define CONFIG_PHY_CADENCE_DPHY_RX_MODULE 1
#define CONFIG_SND_SOC_ROCKCHIP_PDM_MODULE 1
#define CONFIG_IKCONFIG 1
#define CONFIG_TREE_RCU 1
#define CONFIG_LRU_GEN_WALKS_MMU 1
#define CONFIG_SYSVIPC_COMPAT 1
#define CONFIG_USB_ISP1760_HCD 1
#define CONFIG_CRYPTO_CRYPTD 1
#define CONFIG_VIDEO_SAA711X_MODULE 1
#define CONFIG_GENERIC_SCHED_CLOCK 1
#define CONFIG_CRYPTO_ECC_MODULE 1
#define CONFIG_MDIO_BCM_IPROC 1
#define CONFIG_BMC150_ACCEL_MODULE 1
#define CONFIG_CRC_CCITT 1
#define CONFIG_USB_NET_AX8817X_MODULE 1
#define CONFIG_NLS_CODEPAGE_437 1
#define CONFIG_SENSORS_GL518SM_MODULE 1
#define CONFIG_RMI4_F03_SERIO_MODULE 1
#define CONFIG_CRYPTO_GCM 1
#define CONFIG_ICE_SWITCHDEV 1
#define CONFIG_CRYPTO_WP512_MODULE 1
#define CONFIG_PINCTRL_AXP209 1
#define CONFIG_SBITMAP 1
#define CONFIG_USB_SERIAL_AIRCABLE_MODULE 1
#define CONFIG_USB_NET_CDCETHER_MODULE 1
#define CONFIG_DMA_NONCOHERENT_MMAP 1
#define CONFIG_LRU_CACHE_MODULE 1
#define CONFIG_AS_IS_GNU 1
#define CONFIG_STACKPROTECTOR_STRONG 1
#define CONFIG_RTW88_MODULE 1
#define CONFIG_IR_RC6_DECODER_MODULE 1
#define CONFIG_ENCRYPTED_KEYS 1
#define CONFIG_DVB_USB_UMT_010_MODULE 1
#define CONFIG_USB_EZUSB_FX2 1
#define CONFIG_ARM_PMUV3 1
#define CONFIG_IIO_TRIGGER 1
#define CONFIG_EXPORTFS_BLOCK_OPS 1
#define CONFIG_V4L2_FWNODE_MODULE 1
#define CONFIG_SENSORS_INA209_MODULE 1
#define CONFIG_MMC_SDHCI_OF_ARASAN 1
#define CONFIG_SLUB_TINY 1
#define CONFIG_ACPI_FAN 1
#define CONFIG_HID_STEAM_MODULE 1
#define CONFIG_RANDSTRUCT_NONE 1
#define CONFIG_HTS221_MODULE 1
#define CONFIG_RTC_I2C_AND_SPI 1
#define CONFIG_RADIO_SHARK2_MODULE 1
#define CONFIG_CMA_SIZE_SEL_MBYTES 1
#define CONFIG_ROMFS_ON_BLOCK 1
#define CONFIG_TMPFS 1
#define CONFIG_FUNCTION_ALIGNMENT_4B 1
#define CONFIG_PMBUS_MODULE 1
#define CONFIG_NETFILTER_NETLINK_MODULE 1
#define CONFIG_DM_MIRROR_MODULE 1
#define CONFIG_RTW89_8852BT_MODULE 1
#define CONFIG_TOUCHSCREEN_USB_IRTOUCH 1
#define CONFIG_CARL9170_MODULE 1
#define CONFIG_IPV6 1
#define CONFIG_GENERIC_IRQ_KEXEC_CLEAR_VM_FORWARD 1
#define CONFIG_RTC_DRV_FM3130_MODULE 1
#define CONFIG_SCSI_DMA 1
#define CONFIG_LIDAR_LITE_V2_MODULE 1
#define CONFIG_PINCTRL_SUN9I_A80_R 1
#define CONFIG_SIGNALFD 1
#define CONFIG_RTC_DRV_M48T59_MODULE 1
#define CONFIG_DVB_DDBRIDGE_MODULE 1
#define CONFIG_V4L2_JPEG_HELPER_MODULE 1
#define CONFIG_HID_ORTEK_MODULE 1
#define CONFIG_USB_PWC_INPUT_EVDEV 1
#define CONFIG_ASYNC_RAID6_RECOV_MODULE 1
#define CONFIG_ARM64_ERRATUM_1463225 1
#define CONFIG_CRYPTO_TEA_MODULE 1
#define CONFIG_CPU_FREQ_GOV_PERFORMANCE 1
#define CONFIG_F2FS_FS_LZORLE 1
#define CONFIG_USB_G_MULTI_RNDIS 1
#define CONFIG_SERIAL_AMBA_PL011_CONSOLE 1
#define CONFIG_ARM64_ERRATUM_2441009 1
#define CONFIG_NFT_TPROXY_MODULE 1
#define CONFIG_USB_MUSB_SUNXI 1
#define CONFIG_TI_ADC0832_MODULE 1
#define CONFIG_RT2X00_LIB_DEBUGFS 1
#define CONFIG_AD7746_MODULE 1
#define CONFIG_MTD_MAP_BANK_WIDTH_2 1
#define CONFIG_NET_VENDOR_ASIX 1
#define CONFIG_TRANS_TABLE 1
#define CONFIG_SND_HDA_INTEL_MODULE 1
#define CONFIG_SYSFS_SYSCALL 1
#define CONFIG_USB_SERIAL_SAFE_PADDED 1
#define CONFIG_ATH11K_DEBUGFS 1
#define CONFIG_ACPI_HED 1
#define CONFIG_USB_DWC3 1
#define CONFIG_BRIDGE_EBT_IP6_MODULE 1
#define CONFIG_BCM_FLEXRM_MBOX_MODULE 1
#define CONFIG_HAVE_UID16 1
#define CONFIG_DEVFREQ_GOV_POWERSAVE 1
#define CONFIG_ARCH_SUPPORTS_DEBUG_PAGEALLOC 1
#define CONFIG_NFS_V2 1
#define CONFIG_UHID_MODULE 1
#define CONFIG_IR_GPIO_CIR_MODULE 1
#define CONFIG_ARM64_GCS 1
#define CONFIG_HID_RAZER_MODULE 1
#define CONFIG_SECURITY_NETWORK 1
#define CONFIG_DWC_XLGMAC_MODULE 1
#define CONFIG_CRYPTO_DRBG 1
#define CONFIG_VIDEO_SUN8I_DEINTERLACE_MODULE 1
#define CONFIG_SENSORS_RM3100_I2C_MODULE 1
#define CONFIG_HOTPLUG_SMT 1
#define CONFIG_FB_DEVICE 1
#define CONFIG_NET_IPIP_MODULE 1
#define CONFIG_MTK_NET_PHYLIB_MODULE 1
#define CONFIG_AIO 1
#define CONFIG_HID_SPEEDLINK_MODULE 1
#define CONFIG_HP03_MODULE 1
#define CONFIG_RADIO_TEF6862_MODULE 1
#define CONFIG_EXTCON_USBC_CROS_EC_MODULE 1
#define CONFIG_INV_MPU6050_IIO_MODULE 1
#define CONFIG_GENERIC_IRQ_SHOW 1
#define CONFIG_SCSI_DH_RDAC_MODULE 1
#define CONFIG_BLOCK_LEGACY_AUTOLOAD 1
#define CONFIG_NET_SCH_SKBPRIO_MODULE 1
#define CONFIG_DWMAC_GENERIC 1
#define CONFIG_MPU3050_I2C_MODULE 1
#define CONFIG_VIDEO_TDA7432_MODULE 1
#define CONFIG_SPI_SUN6I 1
#define CONFIG_NETFS_SUPPORT_MODULE 1
#define CONFIG_CAN_KVASER_USB_MODULE 1
#define CONFIG_TI_ADC128S052_MODULE 1
#define CONFIG_GPIO_DWAPB 1
#define CONFIG_RPCSEC_GSS_KRB5_MODULE 1
#define CONFIG_BOOT_CONFIG_FORCE 1
#define CONFIG_INPUT_POWERMATE_MODULE 1
#define CONFIG_DP83848_PHY_MODULE 1
#define CONFIG_VIDEO_CX25821_ALSA_MODULE 1
#define CONFIG_PCI_PASID 1
#define CONFIG_WL1251_SDIO_MODULE 1
#define CONFIG_F2FS_FS 1
#define CONFIG_IP_VS_MH_TAB_INDEX 12
#define CONFIG_CRYPTO_KPP 1
#define CONFIG_GENERIC_STRNLEN_USER 1
#define CONFIG_SUN50I_H616_CCU 1
#define CONFIG_HW_RANDOM_ROCKCHIP 1
#define CONFIG_NET_FOU_MODULE 1
#define CONFIG_OF_IRQ 1
#define CONFIG_I2C_TEGRA_BPMP_MODULE 1
#define CONFIG_SQUASHFS_COMPILE_DECOMP_SINGLE 1
#define CONFIG_SND_LAYLA20_MODULE 1
#define CONFIG_BLK_SED_OPAL 1
#define CONFIG_PGTABLE_LEVELS 4
#define CONFIG_QFMT_V2 1
#define CONFIG_CRYPTO_ECB 1
#define CONFIG_B44_MODULE 1
#define CONFIG_BRCMSTB_GISB_ARB 1
#define CONFIG_BT_BNEP_MODULE 1
#define CONFIG_REGMAP 1
#define CONFIG_SLAB_OBJ_EXT 1
#define CONFIG_CRYPTO_ARCH_HAVE_LIB_POLY1305 1
#define CONFIG_BT_HCIBPA10X_MODULE 1
#define CONFIG_SENSORS_MAX1619_MODULE 1
#define CONFIG_PCI_ATS 1
#define CONFIG_SCSI_SYM53C8XX_MAX_TAGS 64
#define CONFIG_NLS_MAC_ROMANIAN_MODULE 1
#define CONFIG_USB_NET_GL620A_MODULE 1
#define CONFIG_DVB_ZL10039_MODULE 1
#define CONFIG_LED_TRIGGER_PHY 1
#define CONFIG_DRM_FBDEV_EMULATION 1
#define CONFIG_DRM_ANALOGIX_DP 1
#define CONFIG_USER_NS 1
#define CONFIG_RT73USB_MODULE 1
#define CONFIG_NFSD_V4_2_INTER_SSC 1
#define CONFIG_HWMON_VID_MODULE 1
#define CONFIG_MWL8K_MODULE 1
#define CONFIG_RTW89_MODULE 1
#define CONFIG_GENERIC_HWEIGHT 1
#define CONFIG_SECURITY_YAMA 1
#define CONFIG_VIDEO_SONY_BTF_MPX_MODULE 1
#define CONFIG_RTW88_8822BE_MODULE 1
#define CONFIG_ACPI_AC_MODULE 1
#define CONFIG_USB_SERIAL_OPTION_MODULE 1
#define CONFIG_I2C_DEMUX_PINCTRL_MODULE 1
#define CONFIG_IEEE802154_ATUSB_MODULE 1
#define CONFIG_EROFS_FS_POSIX_ACL 1
#define CONFIG_SND_AU8810_MODULE 1
#define CONFIG_ARCH_HAS_MEMBARRIER_SYNC_CORE 1
#define CONFIG_SND_SOC_RT5616_MODULE 1
#define CONFIG_USB_RTL8152_MODULE 1
#define CONFIG_AD799X_MODULE 1
#define CONFIG_W1_SLAVE_DS28E04_MODULE 1
#define CONFIG_DRM_ITE_IT6505 1
#define CONFIG_MMC_MTK 1
#define CONFIG_HAVE_C_RECORDMCOUNT 1
#define CONFIG_MAX44000_MODULE 1
#define CONFIG_DRM_PANEL_SITRONIX_ST7703_MODULE 1
#define CONFIG_NET_VENDOR_VIA 1
#define CONFIG_RESET_MESON_COMMON 1
#define CONFIG_NF_TABLES_IPV4 1
#define CONFIG_MAC80211_STA_HASH_MAX_SIZE 0
#define CONFIG_TICK_ONESHOT 1
#define CONFIG_INET_ESP_MODULE 1
#define CONFIG_USB_SERIAL_SAFE_MODULE 1
#define CONFIG_AK09911_MODULE 1
#define CONFIG_VIDEO_TW9903_MODULE 1
#define CONFIG_ARCH_WANT_HUGE_PMD_SHARE 1
#define CONFIG_RSI_USB_MODULE 1
#define CONFIG_RSI_COEX 1
#define CONFIG_DRM 1
#define CONFIG_BCMA_POSSIBLE 1
#define CONFIG_NF_REJECT_IPV6_MODULE 1
#define CONFIG_NFC_MODULE 1
#define CONFIG_NFSD_V4_SECURITY_LABEL 1
#define CONFIG_PCIE_IPROC_MSI 1
#define CONFIG_MT76_USB_MODULE 1
#define CONFIG_TCG_TPM_MODULE 1
#define CONFIG_USB_ISIGHTFW_MODULE 1
#define CONFIG_PCIE_DW_PLAT 1
#define CONFIG_COMMON_CLK_SCMI 1
#define CONFIG_NET_DSA_TAG_DSA_COMMON_MODULE 1
#define CONFIG_AD5380_MODULE 1
#define CONFIG_STMMAC_ETH 1
#define CONFIG_USB_YUREX_MODULE 1
#define CONFIG_USB_IPHETH_MODULE 1
#define CONFIG_DMA_BOUNCE_UNALIGNED_KMALLOC 1
#define CONFIG_USB_GSPCA_PAC7302_MODULE 1
#define CONFIG_IIO_TIGHTLOOP_TRIGGER_MODULE 1
#define CONFIG_BLK_DEV_MD_MODULE 1
#define CONFIG_NET_DSA_TAG_QCA_MODULE 1
#define CONFIG_CPU_LITTLE_ENDIAN 1
#define CONFIG_MEMSTICK_R592_MODULE 1
#define CONFIG_VFIO_PCI_MODULE 1
#define CONFIG_TEHUTI_MODULE 1
#define CONFIG_BATMAN_ADV_NC 1
#define CONFIG_SND_PCM_IEC958 1
#define CONFIG_TOUCHSCREEN_TSC_SERIO_MODULE 1
#define CONFIG_SENSORS_ADM1177_MODULE 1
#define CONFIG_JFS_STATISTICS 1
#define CONFIG_USB_CONFIGFS_F_UAC2 1
#define CONFIG_I2C_ALGOPCA_MODULE 1
#define CONFIG_IPW2200_PROMISCUOUS 1
#define CONFIG_ISCSI_TCP_MODULE 1
#define CONFIG_MCP4725_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_TRACE_MODULE 1
#define CONFIG_USB_SERIAL_GENERIC 1
#define CONFIG_DMIID 1
#define CONFIG_NFC_DIGITAL_MODULE 1
#define CONFIG_RCU_SCALE_TEST_MODULE 1
#define CONFIG_BRIDGE_EBT_NFLOG_MODULE 1
#define CONFIG_NF_DUP_IPV4_MODULE 1
#define CONFIG_IR_IMON_RAW_MODULE 1
#define CONFIG_SND_SOC_PCM3168A_I2C_MODULE 1
#define CONFIG_SCSI_AACRAID_MODULE 1
#define CONFIG_BRIDGE_EBT_SNAT_MODULE 1
#define CONFIG_RELAY 1
#define CONFIG_CPUSETS 1
#define CONFIG_VF610_ADC_MODULE 1
#define CONFIG_USB_KC2190 1
#define CONFIG_SND_HDA_CODEC_VIA_MODULE 1
#define CONFIG_BCM2835_VCHIQ_MODULE 1
#define CONFIG_RESET_SCMI 1
#define CONFIG_CPU_FREQ_STAT 1
#define CONFIG_LOOPBACK_TARGET_MODULE 1
#define CONFIG_SCSI_MVSAS_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_CLASSIFY_MODULE 1
#define CONFIG_LXT_PHY_MODULE 1
#define CONFIG_DRM_MESON 1
#define CONFIG_NFS_V3_ACL 1
#define CONFIG_DVB_SMIPCIE_MODULE 1
#define CONFIG_CRC_T10DIF 1
#define CONFIG_ATA_BMDMA 1
#define CONFIG_ARM_SMMU_V3 1
#define CONFIG_USB_TMC_MODULE 1
#define CONFIG_PPPOE_HASH_BITS_4 1
#define CONFIG_BCMA_HOST_PCI 1
#define CONFIG_SECURITY_LANDLOCK 1
#define CONFIG_NET_DSA_TAG_DSA_MODULE 1
#define CONFIG_X509_CERTIFICATE_PARSER 1
#define CONFIG_MAC80211_RC_DEFAULT "minstrel_ht"
#define CONFIG_MCP41010_MODULE 1
#define CONFIG_HID_CYPRESS_MODULE 1
#define CONFIG_ATH9K_HWRNG 1
#define CONFIG_SKGE_GENESIS 1
#define CONFIG_EVENTFD 1
#define CONFIG_SND_SOC_WM8903_MODULE 1
#define CONFIG_GENERIC_CPU_VULNERABILITIES 1
#define CONFIG_SENSORS_MAX31730_MODULE 1
#define CONFIG_FXAS21002C_MODULE 1
#define CONFIG_DRM_PARADE_PS8640_MODULE 1
#define CONFIG_I2C_MUX_GPIO_MODULE 1
#define CONFIG_PCIE_ROCKCHIP_DW_HOST 1
#define CONFIG_NFSD_V4 1
#define CONFIG_SND_SOC_I2C_AND_SPI 1
#define CONFIG_NET_SHAPER 1
#define CONFIG_ZLIB_DEFLATE 1
#define CONFIG_ADT7316_I2C_MODULE 1
#define CONFIG_USB_NET_DM9601_MODULE 1
#define CONFIG_SCSI_ARCMSR_MODULE 1
#define CONFIG_WLCORE_MODULE 1
#define CONFIG_IP_NF_NAT_MODULE 1
#define CONFIG_B43_PHY_HT 1
#define CONFIG_PNP 1
#define CONFIG_SATA_AHCI 1
#define CONFIG_HID_SONY_MODULE 1
#define CONFIG_CRYPTO_PCRYPT_MODULE 1
#define CONFIG_SND_HDA_CODEC_CMEDIA_MODULE 1
#define CONFIG_NF_DEFRAG_IPV6_MODULE 1
#define CONFIG_FB_TFT_S6D02A1_MODULE 1
#define CONFIG_FB_TFT_HX8340BN_MODULE 1
#define CONFIG_GCC_SUPPORTS_DYNAMIC_FTRACE_WITH_ARGS 1
#define CONFIG_SUNXI_NMI_INTC 1
#define CONFIG_ARM_SMMU 1
#define CONFIG_ZSMALLOC 1
#define CONFIG_MESON_SECURE_PM_DOMAINS 1
#define CONFIG_CRYPTO_RNG 1
#define CONFIG_HID_SENSOR_ALS_MODULE 1
#define CONFIG_TCP_CONG_DCTCP_MODULE 1
#define CONFIG_NLMON_MODULE 1
#define CONFIG_FILE_LOCKING 1
#define CONFIG_USB_U_AUDIO 1
#define CONFIG_MAG3110_MODULE 1
#define CONFIG_PATA_IT8213_MODULE 1
#define CONFIG_SND_SOC_TEGRA_ALC5632_MODULE 1
#define CONFIG_CRYPTO_HMAC 1
#define CONFIG_DMA_VIRTUAL_CHANNELS 1
#define CONFIG_SCSI_INITIO_MODULE 1
#define CONFIG_REGULATOR_FIXED_VOLTAGE 1
#define CONFIG_GPIO_VIRTIO_MODULE 1
#define CONFIG_PGTABLE_HAS_HUGE_LEAVES 1
#define CONFIG_TI_DAC082S085_MODULE 1
#define CONFIG_USB_HACKRF_MODULE 1
#define CONFIG_FS_ENCRYPTION_ALGS 1
#define CONFIG_SND_SUN4I_I2S 1
#define CONFIG_AD7766_MODULE 1
#define CONFIG_CRYPTO_USER_API_SKCIPHER 1
#define CONFIG_CRYPTO_FCRYPT_MODULE 1
#define CONFIG_RTC_INTF_DEV_UIE_EMUL 1
#define CONFIG_CROS_EC_SENSORHUB_MODULE 1
#define CONFIG_CRYPTO_AES 1
#define CONFIG_SND_HDA_PREALLOC_SIZE 4096
#define CONFIG_USB_XUSBATM_MODULE 1
#define CONFIG_PAGE_SIZE_4KB 1
#define CONFIG_CAN_EMS_PCI_MODULE 1
#define CONFIG_I2C_CHARDEV 1
#define CONFIG_BCM_NET_PHYPTP_MODULE 1
#define CONFIG_AF_UNIX_OOB 1
#define CONFIG_QCOM_NET_PHYLIB_MODULE 1
#define CONFIG_MMC_SPI 1
#define CONFIG_INPUT_JOYDEV_MODULE 1
#define CONFIG_ENCX24J600_MODULE 1
#define CONFIG_DVB_MN88472_MODULE 1
#define CONFIG_CRYPTO_DEV_CCP_DD_MODULE 1
#define CONFIG_DMI 1
#define CONFIG_HAVE_DYNAMIC_FTRACE 1
#define CONFIG_SECURITY_APPARMOR_EXPORT_BINARY 1
#define CONFIG_ARCH_SUPPORTS_CRASH_DUMP 1
#define CONFIG_DEV_SYNC_PROBE_MODULE 1
#define CONFIG_VIRTIO_CONSOLE_MODULE 1
#define CONFIG_MDIO_I2C_MODULE 1
#define CONFIG_PATA_ATP867X_MODULE 1
#define CONFIG_IPW2200_MODULE 1
#define CONFIG_ITG3200_MODULE 1
#define CONFIG_DVB_OR51211_MODULE 1
#define CONFIG_SND_HDA_ALIGNED_MMIO 1
#define CONFIG_RTC_DRV_MAX6916_MODULE 1
#define CONFIG_SENSORS_MAX127_MODULE 1
#define CONFIG_STK8312_MODULE 1
#define CONFIG_NET_IFE_SKBTCINDEX_MODULE 1
#define CONFIG_USB_SERIAL_UPD78F0730_MODULE 1
#define CONFIG_HID_ROCCAT_MODULE 1
#define CONFIG_DWMAC_TEGRA 1
#define CONFIG_TCM_PSCSI_MODULE 1
#define CONFIG_IP6_NF_TARGET_SYNPROXY_MODULE 1
#define CONFIG_RESET_MESON 1
#define CONFIG_NET_SCH_HFSC_MODULE 1
#define CONFIG_SENSORS_DRIVETEMP_MODULE 1
#define CONFIG_GENERIC_MSI_IRQ 1
#define CONFIG_FB_SYS_COPYAREA 1
#define CONFIG_OF_DYNAMIC 1
#define CONFIG_ARM_DMC620_PMU 1
#define CONFIG_POWER_SUPPLY 1
#define CONFIG_HAVE_MOD_ARCH_SPECIFIC 1
#define CONFIG_CROS_TYPEC_SWITCH_MODULE 1
#define CONFIG_WLAN_VENDOR_ATMEL 1
#define CONFIG_DEVTMPFS 1
#define CONFIG_HAVE_DEBUG_KMEMLEAK 1
#define CONFIG_NET_VENDOR_CORTINA 1
#define CONFIG_BLK_DEV_BSG 1
#define CONFIG_MEDIA_PCI_SUPPORT 1
#define CONFIG_AT803X_PHY_MODULE 1
#define CONFIG_MESON_CANVAS 1
#define CONFIG_MACVTAP_MODULE 1
#define CONFIG_IPV6_FOU_MODULE 1
#define CONFIG_VIDEO_TEGRA_MODULE 1
#define CONFIG_DRM_DISPLAY_HDMI_HELPER 1
#define CONFIG_NETFILTER_XT_MATCH_PHYSDEV_MODULE 1
#define CONFIG_USB_ANNOUNCE_NEW_DEVICES 1
#define CONFIG_USB_NET_QMI_WWAN_MODULE 1
#define CONFIG_RT2800PCI_RT53XX 1
#define CONFIG_USB_USBNET_MODULE 1
#define CONFIG_INPUT_EVDEV 1
#define CONFIG_IPV6_ROUTE_INFO 1
#define CONFIG_TI_ADS1015_MODULE 1
#define CONFIG_HUGETLBFS 1
#define CONFIG_SND_SOC_TEGRA30_I2S_MODULE 1
#define CONFIG_SND_SOC_TEGRA20_DAS_MODULE 1
#define CONFIG_KERNEL_MODE_NEON 1
#define CONFIG_BCM2712_MIP 1
#define CONFIG_CIFS_XATTR 1
#define CONFIG_MHI_BUS_MODULE 1
#define CONFIG_SENSORS_NPCM7XX_MODULE 1
#define CONFIG_PATA_SERVERWORKS_MODULE 1
#define CONFIG_DRM_MESON_DW_MIPI_DSI 1
#define CONFIG_SND_SOC_TS3A227E_MODULE 1
#define CONFIG_USB_NET_RNDIS_HOST_MODULE 1
#define CONFIG_NETWORK_FILESYSTEMS 1
#define CONFIG_PATA_SIL680_MODULE 1
#define CONFIG_XFS_QUOTA 1
#define CONFIG_IOMMU_IO_PGTABLE_ARMV7S 1
#define CONFIG_COMPACT_UNEVICTABLE_DEFAULT 1
#define CONFIG_ARCH_HAS_DEBUG_WX 1
#define CONFIG_NET_EGRESS 1
#define CONFIG_SND_USB_CAIAQ_MODULE 1
#define CONFIG_W1_MASTER_DS2490_MODULE 1
#define CONFIG_USB_XHCI_PLATFORM 1
#define CONFIG_CROSS_MEMORY_ATTACH 1
#define CONFIG_LD_ORPHAN_WARN 1
#define CONFIG_EROFS_FS_BACKED_BY_FILE 1
#define CONFIG_USB_SERIAL_CH341_MODULE 1
#define CONFIG_INET_TCP_DIAG 1
#define CONFIG_DRM_EXEC 1
#define CONFIG_AHCI_SUNXI 1
#define CONFIG_KEYBOARD_ATKBD 1
#define CONFIG_SENSORS_ASC7621_MODULE 1
#define CONFIG_MXC6255_MODULE 1
#define CONFIG_CLK_PX30 1
#define CONFIG_TMP006_MODULE 1
#define CONFIG_HID_NTRIG_MODULE 1
#define CONFIG_ARCH_WANT_LD_ORPHAN_WARN 1
#define CONFIG_EROFS_FS_MODULE 1
#define CONFIG_VIDEO_SAA7164_MODULE 1
#define CONFIG_ARM64_ERRATUM_3194386 1
#define CONFIG_RTC_DRV_STK17TA8_MODULE 1
#define CONFIG_USB_NET_CDC_SUBSET_ENABLE_MODULE 1
#define CONFIG_NF_CONNTRACK_TFTP_MODULE 1
#define CONFIG_AD7606_MODULE 1
#define CONFIG_ARCH_BCM_IPROC 1
#define CONFIG_PARAVIRT_TIME_ACCOUNTING 1
#define CONFIG_SND_GINA20_MODULE 1
#define CONFIG_BLK_DEV_INTEGRITY 1
#define CONFIG_DM_INIT 1
#define CONFIG_USB_G_HID_MODULE 1
#define CONFIG_PPPOL2TP_MODULE 1
#define CONFIG_IOMMU_SVA 1
#define CONFIG_VIDEO 1
#define CONFIG_ROCKCHIP_IOMMU 1
#define CONFIG_PCIE_IPROC_PLATFORM 1
#define CONFIG_THERMAL_GOV_BANG_BANG 1
#define CONFIG_PSTORE_ZONE_MODULE 1
#define CONFIG_SUN50I_ERRATUM_UNKNOWN1 1
#define CONFIG_RADIO_SAA7706H_MODULE 1
#define CONFIG_NET_ACT_CONNMARK_MODULE 1
#define CONFIG_GENERIC_EARLY_IOREMAP 1
#define CONFIG_SND_SOC_PCM5102A_MODULE 1
#define CONFIG_RTL8821AE_MODULE 1
#define CONFIG_PINCTRL_TEGRA234 1
#define CONFIG_BT_RFCOMM_MODULE 1
#define CONFIG_XFRM_ESP_MODULE 1
#define CONFIG_ARCH_BRCMSTB 1
#define CONFIG_USB_CDC_COMPOSITE_MODULE 1
#define CONFIG_ARCH_SUPPORTS_PAGE_TABLE_CHECK 1
#define CONFIG_RTL8188EE_MODULE 1
#define CONFIG_CXL_SUSPEND 1
#define CONFIG_DEBUG_SECTION_MISMATCH 1
#define CONFIG_PHY_ROCKCHIP_DPHY_RX0_MODULE 1
#define CONFIG_ZRAM_BACKEND_LZ4HC 1
#define CONFIG_CAN_SLCAN_MODULE 1
#define CONFIG_USB_GSPCA_NW80X_MODULE 1
#define CONFIG_SND_USB_CAIAQ_INPUT 1
#define CONFIG_RT2800USB_RT35XX 1
#define CONFIG_OF_FLATTREE 1
#define CONFIG_JOYSTICK_MAGELLAN_MODULE 1
#define CONFIG_IP_NF_MATCH_ECN_MODULE 1
#define CONFIG_DVB_B2C2_FLEXCOP_PCI_MODULE 1
#define CONFIG_SND_USB_VARIAX_MODULE 1
#define CONFIG_SERIAL_DEV_BUS 1
#define CONFIG_USB_RTL8150_MODULE 1
#define CONFIG_L2TP_MODULE 1
#define CONFIG_COMPAT_BINFMT_ELF 1
#define CONFIG_USB_SEVSEG_MODULE 1
#define CONFIG_DRM_SIMPLE_BRIDGE_MODULE 1
#define CONFIG_DRM_PANEL_SITRONIX_ST7701_MODULE 1
#define CONFIG_NETROM_MODULE 1
#define CONFIG_NETFILTER_XT_SET_MODULE 1
#define CONFIG_NF_TABLES_IPV6 1
#define CONFIG_BSD_PROCESS_ACCT 1
#define CONFIG_GAMEPORT_EMU10K1_MODULE 1
#define CONFIG_SND_SEQ_MIDI_MODULE 1
#define CONFIG_SCSI_STEX_MODULE 1
#define CONFIG_DVB_MN88473_MODULE 1
#define CONFIG_INPUT_FF_MEMLESS_MODULE 1
#define CONFIG_XZ_DEC_SPARC 1
#define CONFIG_ROCKCHIP_THERMAL_MODULE 1
#define CONFIG_B43_LEDS 1
#define CONFIG_NF_REJECT_IPV4 1
#define CONFIG_MEMSTICK_MODULE 1
#define CONFIG_MWIFIEX_PCIE_MODULE 1
#define CONFIG_W1_SLAVE_DS28E17_MODULE 1
#define CONFIG_SERIAL_8250_EXTENDED 1
#define CONFIG_CC_HAS_NO_PROFILE_FN_ATTR 1
#define CONFIG_SENSORS_EMC1403_MODULE 1
#define CONFIG_USB_U_SERIAL 1
#define CONFIG_ACPI_SPCR_TABLE 1
#define CONFIG_NET_DSA_MV88E6060_MODULE 1
#define CONFIG_NET_IPVTI_MODULE 1
#define CONFIG_KXSD9_SPI_MODULE 1
#define CONFIG_SND_SERIAL_GENERIC_MODULE 1
#define CONFIG_I2C_VIPERBOARD_MODULE 1
#define CONFIG_REGULATOR_USERSPACE_CONSUMER 1
#define CONFIG_VEXPRESS_CONFIG 1
#define CONFIG_OF_GPIO 1
#define CONFIG_MMA7660_MODULE 1
#define CONFIG_MISC_RTSX_USB_MODULE 1
#define CONFIG_TI_DAC5571_MODULE 1
#define CONFIG_DVB_MB86A20S_MODULE 1
#define CONFIG_LIBERTAS_USB_MODULE 1
#define CONFIG_RCU_CPU_STALL_TIMEOUT 60
#define CONFIG_NF_DUP_IPV6_MODULE 1
#define CONFIG_PHY_MESON_AXG_PCIE 1
#define CONFIG_IP_MROUTE_COMMON 1
#define CONFIG_USB_GADGET_TARGET_MODULE 1
#define CONFIG_MFD_NVEC_MODULE 1
#define CONFIG_USB_EHCI_HCD 1
#define CONFIG_TOUCHSCREEN_USB_EASYTOUCH 1
#define CONFIG_RTL8192DU_MODULE 1
#define CONFIG_CRYPTO_ANSI_CPRNG_MODULE 1
#define CONFIG_STRICT_KERNEL_RWX 1
#define CONFIG_ARM_SMMU_V3_PMU 1
#define CONFIG_MMC_SDHCI_UHS2 1
#define CONFIG_MAX5821_MODULE 1
#define CONFIG_FB_TFT_ILI9163_MODULE 1
#define CONFIG_USB_CONFIGFS_OBEX 1
#define CONFIG_USB_SERIAL_QUALCOMM_MODULE 1
#define CONFIG_FB_TFT_TLS8204_MODULE 1
#define CONFIG_COMMON_CLK_BD718XX_MODULE 1
#define CONFIG_RTC_DRV_TEST_MODULE 1
#define CONFIG_RTW88_SDIO_MODULE 1
#define CONFIG_LZO_COMPRESS 1
#define CONFIG_SPARSEMEM_VMEMMAP 1
#define CONFIG_MEDIA_TUNER_MT2266_MODULE 1
#define CONFIG_I2C_NFORCE2_MODULE 1
#define CONFIG_HID_SUPPORT 1
#define CONFIG_ARM64_ERRATUM_2658417 1
#define CONFIG_IP_NF_ARPFILTER_MODULE 1
#define CONFIG_CHROMEOS_OF_HW_PROBER 1
#define CONFIG_MACB 1
#define CONFIG_DEBUG_VM_IRQSOFF 1
#define CONFIG_HID_SMARTJOYPLUS_MODULE 1
#define CONFIG_I2C_CCGX_UCSI 1
#define CONFIG_ASYNC_PQ_MODULE 1
#define CONFIG_HX711_MODULE 1
#define CONFIG_VFAT_FS 1
#define CONFIG_LCD_CLASS_DEVICE_MODULE 1
#define CONFIG_FONT_8x8 1
#define CONFIG_PINCTRL_SUN55I_A523 1
#define CONFIG_EFIVAR_FS_MODULE 1
#define CONFIG_NFC_MICROREAD_I2C_MODULE 1
#define CONFIG_COMMON_CLK_MESON_EE_CLKC 1
#define CONFIG_UBIFS_FS_XATTR 1
#define CONFIG_SERIAL_8250_BCM2835AUX 1
#define CONFIG_LEDS_GPIO 1
#define CONFIG_DVB_HOPPER_MODULE 1
#define CONFIG_CRYPTO_DEFLATE 1
#define CONFIG_LTR501_MODULE 1
#define CONFIG_E100_MODULE 1
#define CONFIG_MACSEC_MODULE 1
#define CONFIG_IP_ROUTE_MULTIPATH 1
#define CONFIG_COMMON_CLK_AXG 1
#define CONFIG_WLAN_VENDOR_ADMTEK 1
#define CONFIG_TIME_NS 1
#define CONFIG_TMP007_MODULE 1
#define CONFIG_UCS2_STRING 1
#define CONFIG_DM_MULTIPATH_QL_MODULE 1
#define CONFIG_MAGIC_SYSRQ_DEFAULT_ENABLE 0x0
#define CONFIG_PACKET 1
#define CONFIG_LEDS_TRIGGER_TRANSIENT 1
#define CONFIG_VGA_ARB_MAX_GPUS 8
#define CONFIG_VF610_DAC_MODULE 1
#define CONFIG_MMC_DW_PCI 1
#define CONFIG_NO_HZ_COMMON 1
#define CONFIG_USB_SERIAL_METRO_MODULE 1
#define CONFIG_SCSI_ESAS2R_MODULE 1
#define CONFIG_MEMTEST 1
#define CONFIG_6LOWPAN_GHC_EXT_HDR_HOP_MODULE 1
#define CONFIG_IIO_HRTIMER_TRIGGER_MODULE 1
#define CONFIG_DRM_VIRTIO_GPU_MODULE 1
#define CONFIG_USB_SERIAL_MXUPORT_MODULE 1
#define CONFIG_USB_STORAGE_SDDR55_MODULE 1
#define CONFIG_CC_HAS_SANE_FUNCTION_ALIGNMENT 1
#define CONFIG_SENSORS_LM95234_MODULE 1
#define CONFIG_SND_PCM_OSS_PLUGINS 1
#define CONFIG_STACKDEPOT_MAX_FRAMES 64
#define CONFIG_ARM64_PTR_AUTH 1
#define CONFIG_CHARLCD_BL_FLASH 1
#define CONFIG_COMMON_CLK_MESON_REGMAP 1
#define CONFIG_BRIDGE_EBT_ARP_MODULE 1
#define CONFIG_COMMON_CLK_MESON_PHASE 1
#define CONFIG_SND_SERIAL_U16550_MODULE 1
#define CONFIG_PTP_1588_CLOCK_KVM_MODULE 1
#define CONFIG_FB_CFB_FILLRECT 1
#define CONFIG_BRIDGE_EBT_T_NAT_MODULE 1
#define CONFIG_NF_DEFRAG_IPV4_MODULE 1
#define CONFIG_FAIR_GROUP_SCHED 1
#define CONFIG_ZSTD_DECOMPRESS 1
#define CONFIG_TEGRA_BPMP 1
#define CONFIG_NETFILTER_ADVANCED 1
#define CONFIG_FAT_FS 1
#define CONFIG_USB_CONFIGFS_NCM 1
#define CONFIG_CDROM_PKTCDVD_MODULE 1
#define CONFIG_NVME_MULTIPATH 1
#define CONFIG_B43LEGACY_DMA_AND_PIO_MODE 1
#define CONFIG_PCI_DOMAINS_GENERIC 1
#define CONFIG_SND_MESON_GX_SOUND_CARD_MODULE 1
#define CONFIG_PSTORE_DEFAULT_KMSG_BYTES 10240
#define CONFIG_VIDEO_CX231XX_RC 1
#define CONFIG_IP6_NF_TARGET_REJECT_MODULE 1
#define CONFIG_AIC79XX_CMDS_PER_DEVICE 16
#define CONFIG_RTW89_8852BTE_MODULE 1
#define CONFIG_GENERIC_TIME_VSYSCALL 1
#define CONFIG_MMU_GATHER_RCU_TABLE_FREE 1
#define CONFIG_ARCH_SUPPORTS_PER_VMA_LOCK 1
#define CONFIG_SND_CS46XX_NEW_DSP 1
#define CONFIG_DVB_AS102_FE_MODULE 1
#define CONFIG_IR_IMON_DECODER_MODULE 1
#define CONFIG_NR_CPUS 8
#define CONFIG_V4L2_CCI_MODULE 1
#define CONFIG_ARCH_HIBERNATION_POSSIBLE 1
#define CONFIG_HID_ACCUTOUCH_MODULE 1
#define CONFIG_TOUCHSCREEN_USB_E2I 1
#define CONFIG_HAVE_FUNCTION_TRACER 1
#define CONFIG_ASYNC_MEMCPY_MODULE 1
#define CONFIG_PWM_RASPBERRYPI_POE_MODULE 1
#define CONFIG_HISILICON_ERRATUM_161600802 1
#define CONFIG_FB_TFT_S6D1121_MODULE 1
#define CONFIG_PHY_SUN9I_USB 1
#define CONFIG_GPIO_MAX7301_MODULE 1
#define CONFIG_SX9500_MODULE 1
#define CONFIG_LOG_BUF_SHIFT 18
#define CONFIG_ARM64_ERRATUM_1742098 1
#define CONFIG_LEDS_TRIGGER_DEFAULT_ON 1
#define CONFIG_IPV6_OPTIMISTIC_DAD 1
#define CONFIG_FB_EFI 1
#define CONFIG_AUDIT 1
#define CONFIG_SQUASHFS_XATTR 1
#define CONFIG_USB_F_MIDI 1
#define CONFIG_BT_HCIBTUSB_MODULE 1
#define CONFIG_DRM_DW_HDMI 1
#define CONFIG_CRYPTO_AES_ARM64_CE_CCM 1
#define CONFIG_IR_SPI_MODULE 1
#define CONFIG_TCP_CONG_SCALABLE_MODULE 1
#define CONFIG_SND_SEQ_DEVICE 1
#define CONFIG_DVB_DRX39XYJ_MODULE 1
#define CONFIG_RTW88_USB_MODULE 1
#define CONFIG_SND_SOC_CROS_EC_CODEC_MODULE 1
#define CONFIG_MOUSE_CYAPA_MODULE 1
#define CONFIG_VIDEO_TW9906_MODULE 1
#define CONFIG_GPIO_RASPBERRYPI_EXP 1
#define CONFIG_NVMEM_LAYOUT_U_BOOT_ENV_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_CONNTRACK_MODULE 1
#define CONFIG_NF_SOCKET_IPV4_MODULE 1
#define CONFIG_CAN_DEV_MODULE 1
#define CONFIG_SND_SEQ_HRTIMER_DEFAULT 1
#define CONFIG_YELLOWFIN_MODULE 1
#define CONFIG_COMPAT_NETLINK_MESSAGES 1
#define CONFIG_JOYSTICK_TWIDJOY_MODULE 1
#define CONFIG_USB_ATM_MODULE 1
#define CONFIG_REGULATOR_AXP20X 1
#define CONFIG_VSOCKETS_DIAG_MODULE 1
#define CONFIG_USB_LED_TRIG 1
#define CONFIG_NET_SCH_GRED_MODULE 1
#define CONFIG_DM_MULTIPATH_MODULE 1
#define CONFIG_AFE4403_MODULE 1
#define CONFIG_KRETPROBES 1
#define CONFIG_VIDEO_TEGRA_TPG 1
#define CONFIG_BCM_NET_PHYLIB_MODULE 1
#define CONFIG_RTC_DRV_TEGRA_MODULE 1
#define CONFIG_DVB_USB_CE6230_MODULE 1
#define CONFIG_REGULATOR_SY8824X 1
#define CONFIG_ARCH_HAS_USER_SHADOW_STACK 1
#define CONFIG_SERIAL_8250_SHARE_IRQ 1
#define CONFIG_ARM64_PSEUDO_NMI 1
#define CONFIG_LOCKD 1
#define CONFIG_USB_SERIAL_CYPRESS_M8_MODULE 1
#define CONFIG_SND_SOC_RK3288_HDMI_ANALOG_MODULE 1
#define CONFIG_6LOWPAN_GHC_UDP_MODULE 1
#define CONFIG_USB_CONFIGFS_F_UAC1 1
#define CONFIG_IP_ROUTE_VERBOSE 1
#define CONFIG_DRM_PARADE_PS8622_MODULE 1
#define CONFIG_COMPACTION 1
#define CONFIG_FB_CFB_IMAGEBLIT 1
#define CONFIG_VIDEOBUF2_VMALLOC_MODULE 1
#define CONFIG_FB_TFT_UC1701_MODULE 1
#define CONFIG_BRIDGE_EBT_IP_MODULE 1
#define CONFIG_NVMEM_U_BOOT_ENV_MODULE 1
#define CONFIG_INPUT 1
#define CONFIG_WLAN_VENDOR_RSI 1
#define CONFIG_GENERIC_IRQ_IPI 1
#define CONFIG_USB_NET_PLUSB_MODULE 1
#define CONFIG_REGULATOR_H616_MMC 1
#define CONFIG_HWSPINLOCK_SUN6I_MODULE 1
#define CONFIG_NFT_LOG_MODULE 1
#define CONFIG_PWRSEQ_EMMC 1
#define CONFIG_RADIO_ADAPTERS_MODULE 1
#define CONFIG_BGMAC_PLATFORM 1
#define CONFIG_BRIDGE_EBT_REDIRECT_MODULE 1
#define CONFIG_VXLAN_MODULE 1
#define CONFIG_SONY_FF 1
#define CONFIG_MS5611_I2C_MODULE 1
#define CONFIG_DEV_COREDUMP 1
#define CONFIG_SLAB_MERGE_DEFAULT 1
#define CONFIG_HZ_300 1
#define CONFIG_DVB_USB_MODULE 1
#define CONFIG_OF_EARLY_FLATTREE 1
#define CONFIG_FB_DMAMEM_HELPERS_DEFERRED 1
#define CONFIG_SERIAL_8250_BCM7271 1
#define CONFIG_PINCTRL_SUN50I_H616 1
#define CONFIG_USB_MIDI_GADGET_MODULE 1
#define CONFIG_DVB_B2C2_FLEXCOP_MODULE 1
#define CONFIG_GPIO_PCF857X_MODULE 1
#define CONFIG_FUNCTION_ERROR_INJECTION 1
#define CONFIG_TABLET_USB_HANWANG_MODULE 1
#define CONFIG_E1000_MODULE 1
#define CONFIG_FRAMEBUFFER_CONSOLE 1
#define CONFIG_SND_INDIGO_MODULE 1
#define CONFIG_ADT7316_SPI_MODULE 1
#define CONFIG_MTD_NAND_ECC_SW_BCH 1
#define CONFIG_JOYSTICK_XPAD_FF 1
#define CONFIG_CRYPTO_ENGINE 1
#define CONFIG_BLK_ICQ 1
#define CONFIG_MPL115_MODULE 1
#define CONFIG_CLK_RK3528 1
#define CONFIG_NF_CONNTRACK_MARK 1
#define CONFIG_RTC_DRV_MSM6242_MODULE 1
#define CONFIG_HAVE_GCC_PLUGINS 1
#define CONFIG_USB_NET_SMSC95XX_MODULE 1
#define CONFIG_SENSORS_NCT6683_MODULE 1
#define CONFIG_PATA_ARTOP_MODULE 1
#define CONFIG_MEMSTICK_TIFM_MS_MODULE 1
#define CONFIG_DVB_AF9033_MODULE 1
#define CONFIG_VIDEO_GO7007_USB_MODULE 1
#define CONFIG_TPL0102_MODULE 1
#define CONFIG_SND_SUN8I_CODEC 1
#define CONFIG_PWM_CLK_MODULE 1
#define CONFIG_CEPH_FSCACHE 1
#define CONFIG_W1_MASTER_GPIO_MODULE 1
#define CONFIG_I2C_MUX_PCA954x 1
#define CONFIG_NET_DSA_TAG_MTK_MODULE 1
#define CONFIG_USB_HSIC_USB4604_MODULE 1
#define CONFIG_DRM_V3D_MODULE 1
#define CONFIG_TOUCHSCREEN_TSC2007_MODULE 1
#define CONFIG_DM_SNAPSHOT 1
#define CONFIG_FB_TFT_TINYLCD_MODULE 1
#define CONFIG_USB_SERIAL_IUU_MODULE 1
#define CONFIG_NET_PKTGEN_MODULE 1
#define CONFIG_LLC_MODULE 1
#define CONFIG_VIDEO_AU0828_MODULE 1
#define CONFIG_MFD_SYSCON 1
#define CONFIG_VIDEO_STK1160_MODULE 1
#define CONFIG_USB_CYTHERM_MODULE 1
#define CONFIG_SENSORS_GPIO_FAN_MODULE 1
#define CONFIG_GPIOLIB_FASTPATH_LIMIT 512
#define CONFIG_MAC80211_LEDS 1
#define CONFIG_USB_NET_AQC111_MODULE 1
#define CONFIG_JFS_SECURITY 1
#define CONFIG_DRM_SIL_SII8620_MODULE 1
#define CONFIG_ARM64_ACPI_PARKING_PROTOCOL 1
#define CONFIG_NFC_MRVL_USB_MODULE 1
#define CONFIG_GENERIC_IRQ_SHOW_LEVEL 1
#define CONFIG_RT2800USB_MODULE 1
#define CONFIG_BT_BNEP_PROTO_FILTER 1
#define CONFIG_RESET_CONTROLLER 1
#define CONFIG_ARCH_SUPPORTS_HUGETLBFS 1
#define CONFIG_RTC_DRV_M41T80_WDT 1
#define CONFIG_VIDEO_MEM2MEM_DEINTERLACE_MODULE 1
#define CONFIG_RTC_DRV_MCP795_MODULE 1
#define CONFIG_PID_NS 1
#define CONFIG_DRM_GPUVM_MODULE 1
#define CONFIG_GPIO_MAX7300_MODULE 1
#define CONFIG_NVME_HWMON 1
#define CONFIG_IP6_NF_TARGET_NPT_MODULE 1
#define CONFIG_CRYPTO_SHA1_ARM64_CE 1
#define CONFIG_MB1232_MODULE 1
#define CONFIG_NET_DSA_MICROCHIP_KSZ8863_SMI_MODULE 1
#define CONFIG_HAVE_KVM_DIRTY_RING 1
#define CONFIG_AS3935_MODULE 1
#define CONFIG_CGROUP_PERF 1
#define CONFIG_L2TP_ETH_MODULE 1
#define CONFIG_IGC_LEDS 1
#define CONFIG_SOC_TEGRA_FLOWCTRL 1
#define CONFIG_VIDEO_MSP3400_MODULE 1
#define CONFIG_USB_PWC_MODULE 1
#define CONFIG_HARDIRQS_SW_RESEND 1
#define CONFIG_IIO_CONSUMERS_PER_TRIGGER 2
#define CONFIG_FRAMEBUFFER_CONSOLE_ROTATION 1
#define CONFIG_BNX2X_MODULE 1
#define CONFIG_ARCH_HAVE_NMI_SAFE_CMPXCHG 1
#define CONFIG_CRYPTO_CHACHA20_NEON 1
#define CONFIG_NETFILTER_NETLINK_GLUE_CT 1
#define CONFIG_SERIAL_8250_TEGRA_MODULE 1
#define CONFIG_SCSI_MPT2SAS_MAX_SGE 128
#define CONFIG_DRM_SUN4I 1
#define CONFIG_FREEZER 1
#define CONFIG_RFD77402_MODULE 1
#define CONFIG_RTC_DRV_HYM8563_MODULE 1
#define CONFIG_DVB_USB_V2_MODULE 1
#define CONFIG_MYRI10GE_MODULE 1
#define CONFIG_VM_EVENT_COUNTERS 1
#define CONFIG_PAGE_SIZE_LESS_THAN_64KB 1
#define CONFIG_VLAN_8021Q_MODULE 1
#define CONFIG_AQTION_MODULE 1
#define CONFIG_ASSOCIATIVE_ARRAY 1
#define CONFIG_PNFS_FILE_LAYOUT 1
#define CONFIG_RTL8723_COMMON_MODULE 1
#define CONFIG_AXP288_ADC_MODULE 1
#define CONFIG_SI7005_MODULE 1
#define CONFIG_FRAMEBUFFER_CONSOLE_LEGACY_ACCELERATION 1
#define CONFIG_JOYSTICK_SPACEBALL_MODULE 1
#define CONFIG_EFI_EARLYCON 1
#define CONFIG_MINIX_SUBPARTITION 1
#define CONFIG_EXT4_FS_SECURITY 1
#define CONFIG_SENSORS_SHT3x_MODULE 1
#define CONFIG_BRCMFMAC_USB 1
#define CONFIG_SUNRPC_SWAP 1
#define CONFIG_XFS_SUPPORT_V4 1
#define CONFIG_TOUCHSCREEN_USB_DMC_TSC10 1
#define CONFIG_BRIDGE_EBT_VLAN_MODULE 1
#define CONFIG_SENSORS_G760A_MODULE 1
#define CONFIG_SND_SOC_TEGRA210_ADMAIF_MODULE 1
#define CONFIG_SND_VIRTUOSO_MODULE 1
#define CONFIG_ATH_COMMON_MODULE 1
#define CONFIG_LED_OPENVFD_MODULE 1
#define CONFIG_ARM_GIC_MAX_NR 1
#define CONFIG_AD7816_MODULE 1
#define CONFIG_BLK_DEV 1
#define CONFIG_CRYPTO_SIG 1
#define CONFIG_RT61PCI_MODULE 1
#define CONFIG_SPI_TEGRA210_QUAD_MODULE 1
#define CONFIG_BT_HCIBTUSB_POLL_SYNC 1
#define CONFIG_THERMAL_DEBUGFS 1
#define CONFIG_USB_SERIAL_PL2303_MODULE 1
#define CONFIG_AD5449_MODULE 1
#define CONFIG_HID_SENSOR_PROX_MODULE 1
#define CONFIG_CGROUP_PIDS 1
#define CONFIG_VIDEO_SUNXI_CEDRUS_MODULE 1
#define CONFIG_VIDEO_CX88_MODULE 1
#define CONFIG_RMI4_F30 1
#define CONFIG_MMC_SDHCI_OF_AT91 1
#define CONFIG_INDIRECT_PIO 1
#define CONFIG_SENSORS_W83L786NG_MODULE 1
#define CONFIG_HISILICON_ERRATUM_162100801 1
#define CONFIG_DVB_USB_AZ6007_MODULE 1
#define CONFIG_INET6_ESP_MODULE 1
#define CONFIG_ACPI_IORT 1
#define CONFIG_CAN_GW_MODULE 1
#define CONFIG_NF_CONNTRACK_SIP_MODULE 1
#define CONFIG_NET_RX_BUSY_POLL 1
#define CONFIG_INPUT_ATI_REMOTE2_MODULE 1
#define CONFIG_EDAC_MODULE 1
#define CONFIG_MESON_IRQ_GPIO 1
#define CONFIG_USB_CHIPIDEA_HOST 1
#define CONFIG_PAHOLE_VERSION 0
#define CONFIG_HTS221_SPI_MODULE 1
#define CONFIG_SND_MPU401_MODULE 1
#define CONFIG_PKCS7_MESSAGE_PARSER 1
#define CONFIG_LRU_GEN_ENABLED 1
#define CONFIG_COMMON_CLK_AXI_CLKGEN_MODULE 1
#define CONFIG_TOUCHSCREEN_SILEAD_MODULE 1
#define CONFIG_B43LEGACY_PCICORE_AUTOSELECT 1
#define CONFIG_MEDIA_SUPPORT_MODULE 1
#define CONFIG_MOUSE_PS2_LOGIPS2PP 1
#define CONFIG_I2C_DESIGNWARE_PLATFORM 1
#define CONFIG_SENSORS_TSL2550_MODULE 1
#define CONFIG_BPQETHER_MODULE 1
#define CONFIG_NET_VENDOR_SMSC 1
#define CONFIG_SND_SOC_TEGRA_MACHINE_DRV_MODULE 1
#define CONFIG_TCG_TIS_I2C_INFINEON_MODULE 1
#define CONFIG_HAVE_KVM_IRQ_BYPASS 1
#define CONFIG_ACPI_BUTTON 1
#define CONFIG_AUXDISPLAY 1
#define CONFIG_USB_SERIAL_TI_MODULE 1
#define CONFIG_BCMA_HOST_SOC 1
#define CONFIG_RTC_INTF_SYSFS 1
#define CONFIG_MOUSE_PS2_FOCALTECH 1
#define CONFIG_PATA_HPT3X3_MODULE 1
#define CONFIG_FB_NOTIFY 1
#define CONFIG_PWM_BCM_IPROC 1
#define CONFIG_QUOTA 1
#define CONFIG_VIA_RHINE_MODULE 1
#define CONFIG_ACPI_PRMT 1
#define CONFIG_NETFILTER_XT_MATCH_TCPMSS_MODULE 1
#define CONFIG_NFT_TUNNEL_MODULE 1
#define CONFIG_ACPI_TABLE_UPGRADE 1
#define CONFIG_DVB_L64781_MODULE 1
#define CONFIG_MT7663U_MODULE 1
#define CONFIG_TASKS_RCU_GENERIC 1
#define CONFIG_CRYPTO_SHA1 1
#define CONFIG_IP_SET_HASH_IPPORT_MODULE 1
#define CONFIG_PREEMPT_NOTIFIERS 1
#define CONFIG_PINCTRL_SUNXI 1
#define CONFIG_UBIFS_FS_SECURITY 1
#define CONFIG_VIDEO_VP27SMPX_MODULE 1
#define CONFIG_SOC_TEGRA_FUSE 1
#define CONFIG_SND_VIRTIO_MODULE 1
#define CONFIG_NFT_DUP_IPV6_MODULE 1
#define CONFIG_FS_VERITY 1
#define CONFIG_RC_ATI_REMOTE_MODULE 1
#define CONFIG_PINCTRL_SUN8I_A83T 1
#define CONFIG_FB_BACKLIGHT 1
#define CONFIG_CONTEXT_TRACKING 1
#define CONFIG_USB_CHAOSKEY_MODULE 1
#define CONFIG_AC200_PHY 1
#define CONFIG_ARCH_BCM2835 1
#define CONFIG_CPU_IDLE_GOV_TEO 1
#define CONFIG_MMC_SDHCI_CADENCE 1
#define CONFIG_NET_EMATCH_U32_MODULE 1
#define CONFIG_USB_SERIAL_CYBERJACK_MODULE 1
#define CONFIG_DRM_PANTHOR_MODULE 1
#define CONFIG_VIDEO_UPD64083_MODULE 1
#define CONFIG_EDAC_DMC520_MODULE 1
#define CONFIG_AL3320A_MODULE 1
#define CONFIG_DVB_LGDT3305_MODULE 1
#define CONFIG_IP6_NF_MANGLE_MODULE 1
#define CONFIG_GPIO_MAX732X 1
#define CONFIG_CRYPTO_LZ4HC_MODULE 1
#define CONFIG_TASK_IO_ACCOUNTING 1
#define CONFIG_PSTORE_BLK_KMSG_SIZE 64
#define CONFIG_WLAN 1
#define CONFIG_INET6_AH_MODULE 1
#define CONFIG_MOUSE_PS2_SYNAPTICS_SMBUS 1
#define CONFIG_IEEE802154_6LOWPAN_MODULE 1
#define CONFIG_NET_DSA_TAG_AR9331_MODULE 1
#define CONFIG_DRM_PANEL_ORIENTATION_QUIRKS 1
#define CONFIG_ATH10K_USB_MODULE 1
#define CONFIG_FB_TFT_RA8875_MODULE 1
#define CONFIG_SOFTLOCKUP_DETECTOR 1
#define CONFIG_MTD_GEN_PROBE_MODULE 1
#define CONFIG_BT_HCIUART_BCSP 1
#define CONFIG_WLAN_VENDOR_INTERSIL 1
#define CONFIG_ISL29501_MODULE 1
#define CONFIG_MS5637_MODULE 1
#define CONFIG_IP_NF_TARGET_TTL_MODULE 1
#define CONFIG_ARCH_HAS_TICK_BROADCAST 1
#define CONFIG_FB_UDL_MODULE 1
#define CONFIG_COMMON_CLK_PWM 1
#define CONFIG_VIDEO_SYNOPSYS_HDMIRX_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_NFQUEUE_MODULE 1
#define CONFIG_KVM_GENERIC_HARDWARE_ENABLING 1
#define CONFIG_DVB_SP2_MODULE 1
#define CONFIG_SENSORS_NCT7802_MODULE 1
#define CONFIG_MMC_SDHCI_TEGRA 1
#define CONFIG_PWRSEQ_SIMPLE 1
#define CONFIG_DVB_AU8522_MODULE 1
#define CONFIG_RTL8192CE_MODULE 1
#define CONFIG_SFC_SRIOV 1
#define CONFIG_IR_FINTEK_MODULE 1
#define CONFIG_DVB_CXD2820R_MODULE 1
#define CONFIG_NF_SOCKET_IPV6_MODULE 1
#define CONFIG_PCIE_DW_HOST 1
#define CONFIG_ARCH_WANT_COMPAT_IPC_PARSE_VERSION 1
#define CONFIG_USB_MON 1
#define CONFIG_EXTCON_GPIO 1
#define CONFIG_TIPC_DIAG_MODULE 1
#define CONFIG_DAVICOM_PHY_MODULE 1
#define CONFIG_INET6_XFRM_TUNNEL_MODULE 1
#define CONFIG_ACPI_APEI_SEA 1
#define CONFIG_TOUCHSCREEN_USB_NEXIO 1
#define CONFIG_ARM64_ERRATUM_2441007 1
#define CONFIG_SND_OSSEMUL 1
#define CONFIG_DWMAC_SUN8I_MODULE 1
#define CONFIG_SND_CMIPCI_MODULE 1
#define CONFIG_CRYPTO_ECDSA_MODULE 1
#define CONFIG_COMMON_CLK_SI544_MODULE 1
#define CONFIG_DVB_DUMMY_FE_MODULE 1
#define CONFIG_ROCKCHIP_LVDS 1
#define CONFIG_KVM 1
#define CONFIG_NFT_FLOW_OFFLOAD_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_CONNLIMIT_MODULE 1
#define CONFIG_DEFAULT_MMAP_MIN_ADDR 4096
#define CONFIG_SENSORS_LM95241_MODULE 1
#define CONFIG_NLS_CODEPAGE_936_MODULE 1
#define CONFIG_ARM64_ERRATUM_827319 1
#define CONFIG_GPIO_PCA953X 1
#define CONFIG_CRYPTO_CMAC_MODULE 1
#define CONFIG_AIC7XXX_REG_PRETTY_PRINT 1
#define CONFIG_WLAN_VENDOR_BROADCOM 1
#define CONFIG_NET_SCH_MULTIQ_MODULE 1
#define CONFIG_BPF_JIT_DEFAULT_ON 1
#define CONFIG_LEDS_LT3593_MODULE 1
#define CONFIG_SERIAL_8250_EXAR 1
#define CONFIG_SPI_DW_DMA 1
#define CONFIG_NFS_V4 1
#define CONFIG_CLK_RK3368 1
#define CONFIG_DVB_TUNER_ITD1000_MODULE 1
#define CONFIG_SND_SOC_ES7134_MODULE 1
#define CONFIG_NET_NSH_MODULE 1
#define CONFIG_HID_LCPOWER_MODULE 1
#define CONFIG_F2FS_FS_SECURITY 1
#define CONFIG_SND_AC97_CODEC_MODULE 1
#define CONFIG_GENEVE_MODULE 1
#define CONFIG_DVB_NXT6000_MODULE 1
#define CONFIG_BLK_DEV_NBD_MODULE 1
#define CONFIG_HID_SENSOR_CUSTOM_SENSOR_MODULE 1
#define CONFIG_SERIAL_MESON 1
#define CONFIG_IIO_ST_GYRO_SPI_3AXIS_MODULE 1
#define CONFIG_ARM64_HW_AFDBM 1
#define CONFIG_IP_SET_HASH_NET_MODULE 1
#define CONFIG_ZPA2326_SPI_MODULE 1
#define CONFIG_KCMP 1
#define CONFIG_RTC_DRV_DS1742_MODULE 1
#define CONFIG_DEBUG_DEVRES 1
#define CONFIG_TEGRA_IOMMU_SMMU 1
#define CONFIG_SENSORS_ACPI_POWER_MODULE 1
#define CONFIG_XFRM_ALGO_MODULE 1
#define CONFIG_SD_ADC_MODULATOR_MODULE 1
#define CONFIG_NET_VENDOR_STMICRO 1
#define CONFIG_AX88796B_PHY_MODULE 1
#define CONFIG_LEDS_TRIGGER_TTY 1
#define CONFIG_BUILD_SALT ""
#define CONFIG_PINCTRL_MESON_AXG 1
#define CONFIG_ATH9K_COMMON_SPECTRAL 1
#define CONFIG_SND_BT87X_MODULE 1
#define CONFIG_SND_HDA_CODEC_CIRRUS_MODULE 1
#define CONFIG_BH1780_MODULE 1
#define CONFIG_DUMMY_CONSOLE_ROWS 25
#define CONFIG_MEDIA_CONTROLLER_DVB 1
#define CONFIG_AD5272_MODULE 1
#define CONFIG_DM_DEBUG 1
#define CONFIG_MODPROBE_PATH "/sbin/modprobe"
#define CONFIG_RTL8187_MODULE 1
#define CONFIG_POSIX_MQUEUE 1
#define CONFIG_SENSORS_POWR1220_MODULE 1
#define CONFIG_TEXTSEARCH_FSM_MODULE 1
#define CONFIG_VHOST_SCSI_MODULE 1
#define CONFIG_PSTORE_BLK_MODULE 1
#define CONFIG_NFSD_SCSILAYOUT 1
#define CONFIG_MQ_IOSCHED_DEADLINE 1
#define CONFIG_HAVE_CLK 1
#define CONFIG_USB_SERIAL_SIERRAWIRELESS_MODULE 1
#define CONFIG_MAC_PARTITION 1
#define CONFIG_SND_SOC_SIMPLE_AMPLIFIER 1
#define CONFIG_ARCH_SUPPORTS_RT 1
#define CONFIG_DEBUG_FS 1
#define CONFIG_MEDIA_CEC_SUPPORT 1
#define CONFIG_SND_SOC_RT5677_MODULE 1
#define CONFIG_CROS_KBD_LED_BACKLIGHT_MODULE 1
#define CONFIG_RTC_DRV_DS1307_MODULE 1
#define CONFIG_PPTP_MODULE 1
#define CONFIG_FUTEX 1
#define CONFIG_USB_ETH_EEM 1
#define CONFIG_TSYS01_MODULE 1
#define CONFIG_TOUCHSCREEN_WACOM_W8001_MODULE 1
#define CONFIG_IP_SET_MAX 256
#define CONFIG_ASYNC_CORE_MODULE 1
#define CONFIG_DRM_PANEL_FEIXIN_K101_IM2BA02_MODULE 1
#define CONFIG_ADXL345_I2C_MODULE 1
#define CONFIG_CRYPTO_SM4 1
#define CONFIG_CIFS_STATS2 1
#define CONFIG_ATALK_MODULE 1
#define CONFIG_NF_TABLES_INET 1
#define CONFIG_TCM_USER2_MODULE 1
#define CONFIG_VIDEO_TDA9840_MODULE 1
#define CONFIG_ARM64_ERRATUM_819472 1
#define CONFIG_JOYSTICK_XPAD_LEDS 1
#define CONFIG_RTC_DRV_DS1685 1
#define CONFIG_USB_LCD_MODULE 1
#define CONFIG_OCTEONTX2_PF_MODULE 1
#define CONFIG_CM3232_MODULE 1
#define CONFIG_ARCH_HAS_CURRENT_STACK_POINTER 1
#define CONFIG_USB_SL811_HCD_ISO 1
#define CONFIG_FB_SYSMEM_HELPERS 1
#define CONFIG_ROCKCHIP_PM_DOMAINS 1
#define CONFIG_NVMEM_MESON_EFUSE_MODULE 1
#define CONFIG_SERIAL_8250_PNP 1
#define CONFIG_USBIP_VHCI_NR_HCS 1
#define CONFIG_AD2S90_MODULE 1
#define CONFIG_USB_ETH_RNDIS 1
#define CONFIG_HAVE_FUNCTION_ERROR_INJECTION 1
#define CONFIG_CAN_CTUCANFD_PCI_MODULE 1
#define CONFIG_RADIO_SHARK_MODULE 1
#define CONFIG_TOUCHSCREEN_PIXCIR_MODULE 1
#define CONFIG_JOYSTICK_SENSEHAT_MODULE 1
#define CONFIG_SENSORS_MAX1668_MODULE 1
#define CONFIG_MEGARAID_MM_MODULE 1
#define CONFIG_HAS_DMA 1
#define CONFIG_SOFTIRQ_ON_OWN_STACK 1
#define CONFIG_OPT3001_MODULE 1
#define CONFIG_PINCTRL_SUN9I_A80 1
#define CONFIG_DRM_PANEL_KHADAS_TS050_MODULE 1
#define CONFIG_SND_MONA_MODULE 1
#define CONFIG_W1_SLAVE_DS2413_MODULE 1
#define CONFIG_MLX4_EN_MODULE 1
#define CONFIG_ARCH_WANT_BATCHED_UNMAP_TLB_FLUSH 1
#define CONFIG_SND_MESON_CODEC_GLUE_MODULE 1
#define CONFIG_CRYPTO_USER_API 1
#define CONFIG_AUDITSYSCALL 1
#define CONFIG_ANDROID_BINDERFS 1
#define CONFIG_STACKDEPOT 1
#define CONFIG_IP_SET_BITMAP_IPMAC_MODULE 1
#define CONFIG_ARM64 1
#define CONFIG_ARCH_HAS_ACPI_TABLE_UPGRADE 1
#define CONFIG_MAX11100_MODULE 1
#define CONFIG_USB_CDNS_SUPPORT_MODULE 1
#define CONFIG_TIMER_ACPI 1
#define CONFIG_TOUCHSCREEN_TOUCHIT213_MODULE 1
#define CONFIG_GENERIC_BUG 1
#define CONFIG_CRYPTO_SEED_MODULE 1
#define CONFIG_CEC_MESON_AO 1
#define CONFIG_IP_VS_MODULE 1
#define CONFIG_SND_LOLA_MODULE 1
#define CONFIG_MOUSE_ELAN_I2C_I2C 1
#define CONFIG_USB_WDM_MODULE 1
#define CONFIG_TCP_CONG_ILLINOIS_MODULE 1
#define CONFIG_THERMAL_EMULATION 1
#define CONFIG_BT_RFCOMM_TTY 1
#define CONFIG_BT_ATH3K_MODULE 1
#define CONFIG_AD3552R_MODULE 1
#define CONFIG_USBIP_CORE_MODULE 1
#define CONFIG_UNION_FIND 1
#define CONFIG_BRCMUTIL_MODULE 1
#define CONFIG_BTRFS_FS_MODULE 1
#define CONFIG_SND_HDA_CODEC_REALTEK_MODULE 1
#define CONFIG_IP_VS_IPV6 1
#define CONFIG_CPU_FREQ_GOV_COMMON 1
#define CONFIG_PPP_MPPE_MODULE 1
#define CONFIG_SCSI_QLA_FC_MODULE 1
#define CONFIG_HAVE_ALIGNED_STRUCT_PAGE 1
#define CONFIG_CB710_CORE 1
#define CONFIG_FUSE_IO_URING 1
#define CONFIG_CEPH_LIB_MODULE 1
#define CONFIG_LEGACY_DIRECT_IO 1
#define CONFIG_BATMAN_ADV_MCAST 1
#define CONFIG_LOCK_SPIN_ON_OWNER 1
#define CONFIG_HAVE_PERF_USER_STACK_DUMP 1
#define CONFIG_NVME_CORE 1
#define CONFIG_NVME_VERBOSE_ERRORS 1
#define CONFIG_NF_CT_NETLINK_TIMEOUT_MODULE 1
#define CONFIG_B43LEGACY_DMA 1
#define CONFIG_SKB_EXTENSIONS 1
#define CONFIG_HUGETLB_PAGE 1
#define CONFIG_MMC_SUNXI 1
#define CONFIG_SND_SOC_ES8328_SPI_MODULE 1
#define CONFIG_PINCTRL_TEGRA 1
#define CONFIG_XFRM_INTERFACE_MODULE 1
#define CONFIG_HAVE_ARCH_RANDOMIZE_KSTACK_OFFSET 1
#define CONFIG_SERIAL_CORE 1
#define CONFIG_DVB_USB_GP8PSK_MODULE 1
#define CONFIG_ROSE_MODULE 1
#define CONFIG_HOTPLUG_PCI_PCIE 1
#define CONFIG_NFS_USE_KERNEL_DNS 1
#define CONFIG_ARM64_ERRATUM_1286807 1
#define CONFIG_SND_SOC_SDCA_OPTIONAL 1
#define CONFIG_LEDS_TRIGGERS 1
#define CONFIG_NET_SCH_TBF_MODULE 1
#define CONFIG_SECURITY_NETWORK_XFRM 1
#define CONFIG_BRCMSMAC_MODULE 1
#define CONFIG_UBIFS_FS_LZO 1
#define CONFIG_SENSORS_LTC2945_MODULE 1
#define CONFIG_GENERIC_IRQ_STAT_SNAPSHOT 1
#define CONFIG_NET_KEY_MODULE 1
#define CONFIG_SERIAL_MCTRL_GPIO 1
#define CONFIG_NET_DSA_TAG_HELLCREEK_MODULE 1
#define CONFIG_USB_GSPCA_ZC3XX_MODULE 1
#define CONFIG_MMC_TOSHIBA_PCI 1
#define CONFIG_CRYPTO_CCM_MODULE 1
#define CONFIG_SATA_PROMISE_MODULE 1
#define CONFIG_SCSI_INIA100_MODULE 1
#define CONFIG_IP_VS_NQ_MODULE 1
#define CONFIG_SERIAL_AMBA_PL011 1
#define CONFIG_ARM64_PA_BITS_48 1
#define CONFIG_CODA_FS_MODULE 1
#define CONFIG_USB_BRCMSTB 1
#define CONFIG_ARM_SCMI_RAW_MODE_SUPPORT_COEX 1
#define CONFIG_SND_RAWMIDI 1
#define CONFIG_ARM64_LSE_ATOMICS 1
#define CONFIG_ARCH_SUPPORTS_SHADOW_CALL_STACK 1
#define CONFIG_GENERIC_GETTIMEOFDAY 1
#define CONFIG_QCOM_QDF2400_ERRATUM_0065 1
#define CONFIG_IP_SET_HASH_IPMARK_MODULE 1
#define CONFIG_IIO_CROS_EC_SENSORS_MODULE 1
#define CONFIG_FW_UPLOAD 1
#define CONFIG_TEGRA_HOST1X_MODULE 1
#define CONFIG_SND_USB 1
#define CONFIG_NET_ACT_CT_MODULE 1
#define CONFIG_VIDEO_CX18_ALSA_MODULE 1
#define CONFIG_USB_GSPCA_TV8532_MODULE 1
#define CONFIG_DVB_USB_RTL28XXU_MODULE 1
#define CONFIG_PCS_XPCS 1
#define CONFIG_ARM_SCMI_TRANSPORT_SMC_ATOMIC_ENABLE 1
#define CONFIG_CAN_C_CAN_MODULE 1
#define CONFIG_BRCMFMAC_PROTO_MSGBUF 1
#define CONFIG_GPIO_CDEV_V1 1
#define CONFIG_SYN_COOKIES 1
#define CONFIG_FB_TFT_ILI9341_MODULE 1
#define CONFIG_ARCH_HAS_FORTIFY_SOURCE 1
#define CONFIG_DRM_ETNAVIV_THERMAL 1
#define CONFIG_MT7663S_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_SCTP_MODULE 1
#define CONFIG_HID_GEMBIRD_MODULE 1
#define CONFIG_MAX77620_THERMAL_MODULE 1
#define CONFIG_NF_FLOW_TABLE_MODULE 1
#define CONFIG_CRYPTO_SHA3 1
#define CONFIG_NFC_SIM_MODULE 1
#define CONFIG_ZRAM_DEF_COMP_LZORLE 1
#define CONFIG_STACKTRACE_SUPPORT 1
#define CONFIG_WIRELESS_EXT 1
#define CONFIG_JOYSTICK_ADC_MODULE 1
#define CONFIG_TEGRA186_GPC_DMA_MODULE 1
#define CONFIG_NFT_DUP_IPV4_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_CONNMARK_MODULE 1
#define CONFIG_PCIEASPM_DEFAULT 1
#define CONFIG_USB_EHSET_TEST_FIXTURE_MODULE 1
#define CONFIG_SND 1
#define CONFIG_SENSORS_W83627EHF_MODULE 1
#define CONFIG_MEGARAID_LEGACY_MODULE 1
#define CONFIG_CRYPTO_AEAD 1
#define CONFIG_IEEE802154_MRF24J40_MODULE 1
#define CONFIG_BCMGENET_MODULE 1
#define CONFIG_ROCKCHIP_ERRATUM_3568002 1
#define CONFIG_MAX31856_MODULE 1
#define CONFIG_BLK_DEV_THROTTLING 1
#define CONFIG_NVIDIA_CARMEL_CNP_ERRATUM 1
#define CONFIG_NET_DSA_TAG_LAN9303_MODULE 1
#define CONFIG_SECTION_MISMATCH_WARN_ONLY 1
#define CONFIG_ASYNC_XOR_MODULE 1
#define CONFIG_PROC_CHILDREN 1
#define CONFIG_DRM_DISPLAY_HDCP_HELPER 1
#define CONFIG_CC_IS_GCC 1
#define CONFIG_FDMA 1
#define CONFIG_AX25_MODULE 1
#define CONFIG_CRYPTO_ARCH_HAVE_LIB_SHA256_SIMD 1
#define CONFIG_VSOCKETS_LOOPBACK_MODULE 1
#define CONFIG_MPL115_SPI_MODULE 1
#define CONFIG_PINCTRL_MESON 1
#define CONFIG_CMDLINE "console=ttyAMA0"
#define CONFIG_SENSORS_IIO_HWMON_MODULE 1
#define CONFIG_HAVE_CMPXCHG_LOCAL 1
#define CONFIG_PINCTRL_SUN50I_H616_R 1
#define CONFIG_INITRAMFS_PRESERVE_MTIME 1
#define CONFIG_ADIS16260_MODULE 1
#define CONFIG_PINCTRL_IPROC_GPIO 1
#define CONFIG_RTC_DRV_PL031 1
#define CONFIG_IPV6_ROUTER_PREF 1
#define CONFIG_CGROUP_SCHED 1
#define CONFIG_DMA_ENGINE 1
#define CONFIG_DA280_MODULE 1
#define CONFIG_SENSORS_RM3100_SPI_MODULE 1
#define CONFIG_MAX44009_MODULE 1
#define CONFIG_DVB_TDA10048_MODULE 1
#define CONFIG_SENSIRION_SGP30_MODULE 1
#define CONFIG_F2FS_FS_COMPRESSION 1
#define CONFIG_FW_LOADER_USER_HELPER 1
#define CONFIG_IP_SET_HASH_MAC_MODULE 1
#define CONFIG_NET_DSA_LOOP_MODULE 1
#define CONFIG_CRYPTO_CRC32C 1
#define CONFIG_PM_STD_PARTITION ""
#define CONFIG_MOUSE_PS2_ELANTECH_SMBUS 1
#define CONFIG_L2TP_IP_MODULE 1
#define CONFIG_DM_ZONED_MODULE 1
#define CONFIG_ARM_CCI400_COMMON 1
#define CONFIG_SND_INDIGODJ_MODULE 1
#define CONFIG_CHARGER_SBS_MODULE 1
#define CONFIG_RD_BZIP2 1
#define CONFIG_NLS_CODEPAGE_1250_MODULE 1
#define CONFIG_MEDIA_TUNER_MAX2165_MODULE 1
#define CONFIG_SENSORS_W83795_FANCTRL 1
#define CONFIG_DVB_LNBP22_MODULE 1
#define CONFIG_ECRYPT_FS_MODULE 1
#define CONFIG_PHYS_ADDR_T_64BIT 1
#define CONFIG_USB_F_NCM 1
#define CONFIG_UVC_COMMON_MODULE 1
#define CONFIG_HAVE_FUNCTION_GRAPH_TRACER 1
#define CONFIG_BPF_UNPRIV_DEFAULT_OFF 1
#define CONFIG_FB_MODE_HELPERS 1
#define CONFIG_USB_GSPCA_OV534_9_MODULE 1
#define CONFIG_SCSI_IPR_TRACE 1
#define CONFIG_CGROUP_BPF 1
#define CONFIG_MT76_LEDS 1
#define CONFIG_NET_SCH_CBS_MODULE 1
#define CONFIG_SUN8I_DE2_CCU 1
#define CONFIG_HIDRAW 1
#define CONFIG_MSPRO_BLOCK_MODULE 1
#define CONFIG_ARM64_AS_HAS_MTE 1
#define CONFIG_DVB_USB_AF9005_REMOTE_MODULE 1
#define CONFIG_SRAM 1
#define CONFIG_CRYPTO_DES_MODULE 1
#define CONFIG_IR_NEC_DECODER_MODULE 1
#define CONFIG_ARM_SDE_INTERFACE 1
#define CONFIG_LOCK_DEBUGGING_SUPPORT 1
#define CONFIG_VIDEOBUF2_DMA_CONTIG_MODULE 1
#define CONFIG_ATH10K_PCI_MODULE 1
#define CONFIG_SCSI_ISCSI_ATTRS_MODULE 1
#define CONFIG_P54_LEDS 1
#define CONFIG_BUG 1
#define CONFIG_DVB_AU8522_V4L_MODULE 1
#define CONFIG_PWM_BRCMSTB_MODULE 1
#define CONFIG_IR_HIX5HD2_MODULE 1
#define CONFIG_SND_SOC_ROCKCHIP_MODULE 1
#define CONFIG_IPVTAP_MODULE 1
#define CONFIG_M62332_MODULE 1
#define CONFIG_MEMORY 1
#define CONFIG_TTPCI_EEPROM_MODULE 1
#define CONFIG_NET_VENDOR_SYNOPSYS 1
#define CONFIG_ARCH_HAS_GCOV_PROFILE_ALL 1
#define CONFIG_DMARD09_MODULE 1
#define CONFIG_AF_KCM_MODULE 1
#define CONFIG_SND_SYNTH_EMUX_MODULE 1
#define CONFIG_SND_SOC_SUNXI_INTERNALCODEC_MODULE 1
#define CONFIG_INPUT_MMA8450_MODULE 1
#define CONFIG_NFC_SHDLC 1
#define CONFIG_ROCKCHIP_SARADC_MODULE 1
#define CONFIG_XZ_DEC_BCJ 1
#define CONFIG_USB_SERIAL_OTI6858_MODULE 1
#define CONFIG_MTD_CMDLINE_PARTS 1
#define CONFIG_QRTR_TUN_MODULE 1
#define CONFIG_ARM64_ERRATUM_2457168 1
#define CONFIG_DRM_KMS_HELPER 1
#define CONFIG_MOUSE_PS2_BYD 1
#define CONFIG_TCS3472_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_TCPOPTSTRIP_MODULE 1
#define CONFIG_SND_MESON_AXG_TDM_FORMATTER_MODULE 1
#define CONFIG_FB_SYSMEM_FOPS 1
#define CONFIG_DRM_SUN8I_TCON_TOP 1
#define CONFIG_SERIAL_AMBA_PL010 1
#define CONFIG_VIDEO_ROCKCHIP_IEP_MODULE 1
#define CONFIG_PAGE_REPORTING 1
#define CONFIG_MOUSE_PS2_TRACKPOINT 1
#define CONFIG_SCHED_HRTICK 1
#define CONFIG_TRACING_SUPPORT 1
#define CONFIG_IOMMU_MM_DATA 1
#define CONFIG_MPLS 1
#define CONFIG_MAX1118_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_HL_MODULE 1
#define CONFIG_NET_ACT_SKBMOD_MODULE 1
#define CONFIG_TEGRA_IVC 1
#define CONFIG_SCTP_DEFAULT_COOKIE_HMAC_SHA1 1
#define CONFIG_PINCTRL_SUN50I_A64_R 1
#define CONFIG_USB_EHCI_PCI 1
#define CONFIG_SPI_BCM2835_MODULE 1
#define CONFIG_ADXL345_MODULE 1
#define CONFIG_GPIO_ROCKCHIP 1
#define CONFIG_IP_SET_HASH_IP_MODULE 1
#define CONFIG_VIRTIO_MENU 1
#define CONFIG_RTC_DRV_RX8581_MODULE 1
#define CONFIG_FB_TFT_ILI9340_MODULE 1
#define CONFIG_ARM_PSCI_FW 1
#define CONFIG_NET_VENDOR_CADENCE 1
#define CONFIG_SPLIT_PTE_PTLOCKS 1
#define CONFIG_BLK_DEV_WRITE_MOUNTED 1
#define CONFIG_USB_AIRSPY_MODULE 1
#define CONFIG_MT76x02_USB_MODULE 1
#define CONFIG_USB_NET_SMSC75XX_MODULE 1
#define CONFIG_ROMFS_BACKED_BY_BLOCK 1
#define CONFIG_RTC_DRV_DS1305_MODULE 1
#define CONFIG_SENSORS_BH1770_MODULE 1
#define CONFIG_ADXL372_MODULE 1
#define CONFIG_HAVE_IOREMAP_PROT 1
#define CONFIG_CC_NO_STRINGOP_OVERFLOW 1
#define CONFIG_SECURITY_APPARMOR 1
#define CONFIG_DRM_SUN8I_MIXER 1
#define CONFIG_DRM_UDL_MODULE 1
#define CONFIG_LEDS_TRIGGER_TIMER 1
#define CONFIG_MMC_REALTEK_USB_MODULE 1
#define CONFIG_VIRTIO_FS_MODULE 1
#define CONFIG_B53_MMAP_DRIVER_MODULE 1
#define CONFIG_FB_TFT_MODULE 1
#define CONFIG_NET_SELFTESTS 1
#define CONFIG_ARCH_SUPPORTS_MEMORY_FAILURE 1
#define CONFIG_DW_DMAC_MODULE 1
#define CONFIG_SENSORS_GL520SM_MODULE 1
#define CONFIG_ROCKCHIP_VOP2 1
#define CONFIG_CC10001_ADC_MODULE 1
#define CONFIG_USB_SERIAL_QCAUX_MODULE 1
#define CONFIG_SENSORS_AD7314_MODULE 1
#define CONFIG_I2C_STUB_MODULE 1
#define CONFIG_USB_SERIAL_GARMIN_MODULE 1
#define CONFIG_SND_SOC_SUNXI_AHUB_MODULE 1
#define CONFIG_USB_CYPRESS_CY7C63_MODULE 1
#define CONFIG_AS_HAS_NON_CONST_ULEB128 1
#define CONFIG_RTL8187_LEDS 1
#define CONFIG_F2FS_UNFAIR_RWSEM 1
#define CONFIG_US5182D_MODULE 1
#define CONFIG_LLD_VERSION 0
#define CONFIG_USB_STORAGE_ONETOUCH_MODULE 1
#define CONFIG_OCFS2_FS_O2CB_MODULE 1
#define CONFIG_6LOWPAN_NHC_DEST_MODULE 1
#define CONFIG_RCU_REF_SCALE_TEST_MODULE 1
#define CONFIG_W1_MODULE 1
#define CONFIG_CRYPTO_HASH 1
#define CONFIG_SND_USB_UA101_MODULE 1
#define CONFIG_RTC_DRV_PL030 1
#define CONFIG_CRYPTO_AUTHENC_MODULE 1
#define CONFIG_DRM_CLIENT_LIB 1
#define CONFIG_ARCH_ENABLE_MEMORY_HOTPLUG 1
#define CONFIG_NET_ACT_GACT_MODULE 1
#define CONFIG_MEDIA_ATTACH 1
#define CONFIG_PINCTRL_TEGRA210 1
#define CONFIG_IIO_ST_SENSORS_SPI_MODULE 1
#define CONFIG_PATA_OF_PLATFORM_MODULE 1
#define CONFIG_IP_NF_TARGET_NETMAP_MODULE 1
#define CONFIG_USB_DWC3_HAPS 1
#define CONFIG_IP_NF_MANGLE_MODULE 1
#define CONFIG_IIO_ST_ACCEL_I2C_3AXIS_MODULE 1
#define CONFIG_NF_CONNTRACK_SANE_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_DEVGROUP_MODULE 1
#define CONFIG_IP6_NF_MATCH_EUI64_MODULE 1
#define CONFIG_SERIAL_ARC_MODULE 1
#define CONFIG_ATL1_MODULE 1
#define CONFIG_MEMREGION 1
#define CONFIG_PHY_MESON_AXG_MIPI_PCIE_ANALOG 1
#define CONFIG_LEDS_SGM3140_MODULE 1
#define CONFIG_TCP_CONG_WESTWOOD_MODULE 1
#define CONFIG_NETFILTER_EGRESS 1
#define CONFIG_IR_NUVOTON_MODULE 1
#define CONFIG_L2TP_V3 1
#define CONFIG_USB_G_ACM_MS_MODULE 1
#define CONFIG_SENSORS_MAX15301_MODULE 1
#define CONFIG_I2C_HID_OF_MODULE 1
#define CONFIG_DVB_USB_AU6610_MODULE 1
#define CONFIG_SYSTEM_TRUSTED_KEYS ""
#define CONFIG_CHARGER_SMB347_MODULE 1
#define CONFIG_IP_MROUTE_MULTIPLE_TABLES 1
#define CONFIG_NET_DSA_TAG_BRCM_COMMON_MODULE 1
#define CONFIG_NLS_CODEPAGE_1251_MODULE 1
#define CONFIG_HID_PRODIKEYS_MODULE 1
#define CONFIG_VIDEO_BCM2835_MODULE 1
#define CONFIG_GPIOLIB_IRQCHIP 1
#define CONFIG_SERIAL_MESON_CONSOLE 1
#define CONFIG_NET_SCH_SFQ_MODULE 1
#define CONFIG_GPIO_WATCHDOG_MODULE 1
#define CONFIG_ROCKCHIP_DW_DP 1
#define CONFIG_SCSI_SAS_ATTRS_MODULE 1
#define CONFIG_EFI 1
#define CONFIG_SUNRPC 1
#define CONFIG_NLS_MAC_CELTIC_MODULE 1
#define CONFIG_TI_DAC7311_MODULE 1
#define CONFIG_THERMAL_EMERGENCY_POWEROFF_DELAY_MS 0
#define CONFIG_NF_CONNTRACK_NETBIOS_NS_MODULE 1
#define CONFIG_ARCH_TEGRA_194_SOC 1
#define CONFIG_NF_NAT_OVS 1
#define CONFIG_DVB_STV0900_MODULE 1
#define CONFIG_ARM64_ERRATUM_834220 1
#define CONFIG_SUN8I_R_CCU 1
#define CONFIG_FS_STACK 1
#define CONFIG_SENSORS_LTC2947_MODULE 1
#define CONFIG_MT76x2E_MODULE 1
#define CONFIG_SOFTLOCKUP_DETECTOR_INTR_STORM 1
#define CONFIG_FB_TFT_AGM1264K_FL_MODULE 1
#define CONFIG_ARCH_SELECTS_KEXEC_FILE 1
#define CONFIG_GENERIC_CPU_DEVICES 1
#define CONFIG_LEDS_PCA963X_MODULE 1
#define CONFIG_CONNECTOR 1
#define CONFIG_ARM_SBSA_WATCHDOG_MODULE 1
#define CONFIG_SENSORS_TC654_MODULE 1
#define CONFIG_VIRTIO_PCI_LIB 1
#define CONFIG_ARCH_WANT_FRAME_POINTERS 1
#define CONFIG_DRM_VGEM_MODULE 1
#define CONFIG_SOC_TEGRA_POWERGATE_BPMP 1
#define CONFIG_TCP_MD5SIG 1
#define CONFIG_SSB_BLOCKIO 1
#define CONFIG_REGULATOR_MP5416 1
#define CONFIG_DM_UEVENT 1
#define CONFIG_UBIFS_FS_ZSTD 1
#define CONFIG_CRYPTO_LIB_CHACHA 1
#define CONFIG_GPIO_BCM_XGS_IPROC 1
#define CONFIG_BRIDGE_EBT_LOG_MODULE 1
#define CONFIG_SFP_MODULE 1
#define CONFIG_CPU_IDLE 1
#define CONFIG_CGROUP_FREEZER 1
#define CONFIG_MTD_CFI_MODULE 1
#define CONFIG_ARM64_TLB_RANGE 1
#define CONFIG_RT2X00_MODULE 1
#define CONFIG_PREEMPT_COUNT 1
#define CONFIG_KXSD9_I2C_MODULE 1
#define CONFIG_I2C_BCM2835 1
#define CONFIG_MMC_MESON_GX 1
#define CONFIG_CAN_MODULE 1
#define CONFIG_AD7150_MODULE 1
#define CONFIG_IIO_CONFIGFS_MODULE 1
#define CONFIG_IP6_NF_IPTABLES_LEGACY_MODULE 1
#define CONFIG_KUSER_HELPERS 1
#define CONFIG_HID_BATTERY_STRENGTH 1
#define CONFIG_DVB_NXT200X_MODULE 1
#define CONFIG_NET_SCH_HHF_MODULE 1
#define CONFIG_SUN50I_H6_PRCM_PPU 1
#define CONFIG_LEDS_CLASS_MULTICOLOR 1
#define CONFIG_DW_EDMA_PCIE_MODULE 1
#define CONFIG_HID_SENSOR_TEMP_MODULE 1
#define CONFIG_HID_SENSOR_HUB_MODULE 1
#define CONFIG_PINCTRL_MESON_AXG_PMX 1
#define CONFIG_IPW2200_QOS 1
#define CONFIG_HID_PICOLCD_CIR 1
#define CONFIG_PM_CLK 1
#define CONFIG_MC3230_MODULE 1
#define CONFIG_NET_XGRESS 1
#define CONFIG_SND_VX222_MODULE 1
#define CONFIG_SATA_HOST 1
#define CONFIG_SND_PCM_TIMER 1
#define CONFIG_ROCKCHIP_DW_HDMI 1
#define CONFIG_DRM_ANALOGIX_ANX6345_MODULE 1
#define CONFIG_INV_MPU6050_SPI_MODULE 1
#define CONFIG_SCSI_PROC_FS 1
#define CONFIG_DVB_STB6000_MODULE 1
#define CONFIG_CRYPTO_HW 1
#define CONFIG_FB_TFT_SSD1331_MODULE 1
#define CONFIG_LTC1660_MODULE 1
#define CONFIG_TIPC_MODULE 1
#define CONFIG_ARM_GIC_V3 1
#define CONFIG_NFT_SOCKET_MODULE 1
#define CONFIG_NETFILTER_NETLINK_QUEUE_MODULE 1
#define CONFIG_MPU3050_MODULE 1
#define CONFIG_KALLSYMS_ALL 1
#define CONFIG_USB_F_MASS_STORAGE 1
#define CONFIG_COREDUMP 1
#define CONFIG_CLK_RK3588 1
#define CONFIG_RELOCATABLE 1
#define CONFIG_PM_GENERIC_DOMAINS_OF 1
#define CONFIG_SERIAL_JSM_MODULE 1
#define CONFIG_ACPI_CONFIGFS_MODULE 1
#define CONFIG_SATA_ACARD_AHCI_MODULE 1
#define CONFIG_HAVE_KVM_MSI 1
#define CONFIG_ARM_SCMI_RAW_MODE_SUPPORT 1
#define CONFIG_CC_HAS_ZERO_CALL_USED_REGS 1
#define CONFIG_BRIDGE_EBT_PKTTYPE_MODULE 1
#define CONFIG_MESON_EE_PM_DOMAINS 1
#define CONFIG_RC_DEVICES 1
#define CONFIG_B43_BUSES_BCMA_AND_SSB 1
#define CONFIG_ARCH_KEEP_MEMBLOCK 1
#define CONFIG_CRYPTO_HASH2 1
#define CONFIG_SECCOMP 1
#define CONFIG_NET_DSA_MV88E6XXX_PTP 1
#define CONFIG_TIFM_CORE 1
#define CONFIG_CRYPTO_LIB_GF128MUL 1
#define CONFIG_SPLIT_PMD_PTLOCKS 1
#define CONFIG_HID_MICROSOFT_MODULE 1
#define CONFIG_LIBERTAS_MESH 1
#define CONFIG_USB_MUSB_DUAL_ROLE 1
#define CONFIG_USB_G_MULTI_MODULE 1
#define CONFIG_IP6_NF_IPTABLES_MODULE 1
#define CONFIG_FB_TFT_SH1106_MODULE 1
#define CONFIG_MAX1027_MODULE 1
#define CONFIG_SHMEM 1
#define CONFIG_IWLWIFI_OPMODE_MODULAR 1
#define CONFIG_IIO_BUFFER_DMAENGINE_MODULE 1
#define CONFIG_NFT_CT_MODULE 1
#define CONFIG_SUNXI_MBUS 1
#define CONFIG_RTL8723BS_MODULE 1
#define CONFIG_IO_URING_ZCRX 1
#define CONFIG_NET_ACT_IFE_MODULE 1
#define CONFIG_ARCH_HAS_NMI_SAFE_THIS_CPU_OPS 1
#define CONFIG_DVB_DIB3000MC_MODULE 1
#define CONFIG_I2C_BOARDINFO 1
#define CONFIG_SERIO_ALTERA_PS2_MODULE 1
#define CONFIG_SECCOMP_CACHE_DEBUG 1
#define CONFIG_ATL2_MODULE 1
#define CONFIG_UIO_MODULE 1
#define CONFIG_DRM_DEBUG_MODESET_LOCK 1
#define CONFIG_SSB_PCIHOST 1
#define CONFIG_NET_VENDOR_ALTEON 1
#define CONFIG_ATA_VERBOSE_ERROR 1
#define CONFIG_WEXT_CORE 1
#define CONFIG_SND_HDA_CODEC_CONEXANT_MODULE 1
#define CONFIG_PLDMFW 1
#define CONFIG_SENSORS_TPS40422_MODULE 1
#define CONFIG_THERMAL_NETLINK 1
#define CONFIG_VIRTIO_ANCHOR 1
#define CONFIG_SKGE_MODULE 1
#define CONFIG_SND_SOC_ES8328_I2C_MODULE 1
#define CONFIG_MT76_SDIO_MODULE 1
#define CONFIG_SATA_SIL_MODULE 1
#define CONFIG_SND_TIMER 1
#define CONFIG_NETLINK_DIAG_MODULE 1
#define CONFIG_NLS_MAC_ICELAND_MODULE 1
#define CONFIG_SMS_SIANO_RC 1
#define CONFIG_HAVE_POSIX_CPU_TIMERS_TASK_WORK 1
#define CONFIG_ARM_PMU_ACPI 1
#define CONFIG_NLS_CODEPAGE_949_MODULE 1
#define CONFIG_SENSORS_W83L785TS_MODULE 1
#define CONFIG_CROS_EC_LIGHTBAR_MODULE 1
#define CONFIG_BCH 1
#define CONFIG_RTLWIFI_PCI_MODULE 1
#define CONFIG_BRIDGE_MODULE 1
#define CONFIG_IRQ_FORCED_THREADING 1
#define CONFIG_VIDEO_GO7007_LOADER_MODULE 1
#define CONFIG_CRYPTO_MANAGER2 1
#define CONFIG_NET_VENDOR_MICROCHIP 1
#define CONFIG_MESSAGE_LOGLEVEL_DEFAULT 3
#define CONFIG_SND_MESON_AIU_MODULE 1
#define CONFIG_KEYBOARD_CROS_EC_MODULE 1
#define CONFIG_COMMON_CLK_FIXED_MMIO 1
#define CONFIG_RTC_DRV_DS1343_MODULE 1
#define CONFIG_IP_NF_FILTER_MODULE 1
#define CONFIG_MAX77620_WATCHDOG_MODULE 1
#define CONFIG_SQUASHFS 1
#define CONFIG_WLAN_VENDOR_ATH 1
#define CONFIG_SND_DUMMY_MODULE 1
#define CONFIG_CFG80211_WEXT 1
#define CONFIG_DVB_S921_MODULE 1
#define CONFIG_SENSORS_LM95245_MODULE 1
#define CONFIG_NLS_CODEPAGE_932_MODULE 1
#define CONFIG_WLAN_VENDOR_MARVELL 1
#define CONFIG_NET_SCH_MQPRIO_LIB_MODULE 1
#define CONFIG_SOCK_RX_QUEUE_MAPPING 1
#define CONFIG_AUXILIARY_BUS 1
#define CONFIG_ARCH_HIBERNATION_HEADER 1
#define CONFIG_DECOMPRESS_LZ4 1
#define CONFIG_SENSORS_VT1211_MODULE 1
#define CONFIG_DVB_CX24117_MODULE 1
#define CONFIG_USB_GSPCA_ETOMS_MODULE 1
#define CONFIG_ARCH_WANT_DEFAULT_BPF_JIT 1
#define CONFIG_GPIO_SIM_MODULE 1
#define CONFIG_NLS_ISO8859_1_MODULE 1
#define CONFIG_ARCH_HAS_KCOV 1
#define CONFIG_HID_TOPSEED_MODULE 1
#define CONFIG_NET_EMATCH_META_MODULE 1
#define CONFIG_MPLS_IPTUNNEL_MODULE 1
#define CONFIG_PM_SLEEP_DEBUG 1
#define CONFIG_BCM_PMB 1
#define CONFIG_BT_HCIUART_RTL 1
#define CONFIG_GENERIC_IRQ_INJECTION 1
#define CONFIG_USB_TRANCEVIBRATOR_MODULE 1
#define CONFIG_WLAN_VENDOR_ZYDAS 1
#define CONFIG_AD3552R_LIB_MODULE 1
#define CONFIG_LIBERTAS_SDIO_MODULE 1
#define CONFIG_SENSORS_SHT21_MODULE 1
#define CONFIG_USB_GSPCA_MR97310A_MODULE 1
#define CONFIG_DTC 1
#define CONFIG_PATA_TIMINGS 1
#define CONFIG_NETFILTER_XT_MARK_MODULE 1
#define CONFIG_COMPAT 1
#define CONFIG_USB_SERIAL_DIGI_ACCELEPORT_MODULE 1
#define CONFIG_ARCH_ENABLE_HUGEPAGE_MIGRATION 1
#define CONFIG_COMMON_CLK_GXBB 1
#define CONFIG_PSTORE 1
#define CONFIG_IIO_ST_LSM6DSX_SPI_MODULE 1
#define CONFIG_NET_CLS_ROUTE4_MODULE 1
#define CONFIG_IIO_TRIGGERED_EVENT_MODULE 1
#define CONFIG_DVB_USB_AF9035_MODULE 1
#define CONFIG_RESET_SIMPLE 1
#define CONFIG_NET_SCH_CHOKE_MODULE 1
#define CONFIG_WILC1000_MODULE 1
#define CONFIG_EXCLUSIVE_SYSTEM_RAM 1
#define CONFIG_FW_LOADER_COMPRESS 1
#define CONFIG_CRYPTO_LIB_POLY1305_RSIZE 9
#define CONFIG_INIT_ENV_ARG_LIMIT 32
#define CONFIG_CACHEFILES_MODULE 1
#define CONFIG_PCI_HOST_GENERIC 1
#define CONFIG_SUN8I_H3_CCU 1
#define CONFIG_STANDALONE 1
#define CONFIG_USB_SERIAL_CONSOLE 1
#define CONFIG_USB_CONFIGFS_EEM 1
#define CONFIG_XFRM_MIGRATE 1
#define CONFIG_SND_DYNAMIC_MINORS 1
#define CONFIG_IP_PIMSM_V1 1
#define CONFIG_CC_HAS_AUTO_VAR_INIT_ZERO 1
#define CONFIG_SCSI_VIRTIO_MODULE 1
#define CONFIG_ARCH_HAVE_TRACE_MMIO_ACCESS 1
#define CONFIG_DM_FLAKEY_MODULE 1
#define CONFIG_RTC_DRV_MAX77686_MODULE 1
#define CONFIG_ZD1211RW_MODULE 1
#define CONFIG_VIDEO_GO7007_USB_S2250_BOARD_MODULE 1
#define CONFIG_USB_GSPCA_XIRLINK_CIT_MODULE 1
#define CONFIG_DVB_DRXK_MODULE 1
#define CONFIG_I2C_PCA_PLATFORM_MODULE 1
#define CONFIG_CAVIUM_ERRATUM_22375 1
#define CONFIG_EROFS_FS_SECURITY 1
#define CONFIG_I2C_HID 1
#define CONFIG_PREEMPT_BUILD 1
#define CONFIG_AIC_FW_PATH "/lib/firmware/aic8800_sdio"
#define CONFIG_BT_HCIUART_MRVL 1
#define CONFIG_DRM_I2C_ADV7511_MODULE 1
#define CONFIG_LOGIWHEELS_FF 1
#define CONFIG_MANAGER_SBS_MODULE 1
#define CONFIG_JOYSTICK_STINGER_MODULE 1
#define CONFIG_842_COMPRESS_MODULE 1
#define CONFIG_SERIAL_ARC_NR_PORTS 1
#define CONFIG_DVB_USB_ANYSEE_MODULE 1
#define CONFIG_DEFAULT_TCP_CONG "cubic"
#define CONFIG_VL53L0X_I2C_MODULE 1
#define CONFIG_I2C_TINY_USB_MODULE 1
#define CONFIG_IR_TTUSBIR_MODULE 1
#define CONFIG_SUNXI_WATCHDOG_MODULE 1
#define CONFIG_SPI_MESON_SPICC_MODULE 1
#define CONFIG_DEVFREQ_EVENT_ROCKCHIP_DFI 1
#define CONFIG_DWMAC_ROCKCHIP 1
#define CONFIG_MTD_CFI_UTIL_MODULE 1
#define CONFIG_NETFILTER_NETLINK_ACCT_MODULE 1
#define CONFIG_WLAN_VENDOR_SILABS 1
#define CONFIG_IPW2100_MONITOR 1
#define CONFIG_VORTEX_MODULE 1
#define CONFIG_CRAMFS_BLOCKDEV 1
#define CONFIG_ARCH_SUPPORTS_LTO_CLANG 1
#define CONFIG_SND_SOC_AC97_BUS 1
#define CONFIG_SMC91X_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_L2TP_MODULE 1
#define CONFIG_VIDEO_PVRUSB2_SYSFS 1
#define CONFIG_IP6_NF_SECURITY_MODULE 1
#define CONFIG_USB_STORAGE_REALTEK_MODULE 1
#define CONFIG_ARM_CCN 1
#define CONFIG_RTW88_8821CS_MODULE 1
#define CONFIG_WIL6210_MODULE 1
#define CONFIG_CRYPTO_CRC32 1
#define CONFIG_SPI_MEM 1
#define CONFIG_MMC_BLOCK 1
#define CONFIG_RTC_DRV_RX8025_MODULE 1
#define CONFIG_GCC_NO_STRINGOP_OVERFLOW 1
#define CONFIG_CONTIG_ALLOC 1
#define CONFIG_CRYPTO_LIB_SM3 1
#define CONFIG_SMBFS_MODULE 1
#define CONFIG_DRM_PANFROST_MODULE 1
#define CONFIG_SND_SOC_TEGRA20_SPDIF_MODULE 1
#define CONFIG_USB_DWC3_PCI 1
#define CONFIG_USB_SI4713_MODULE 1
#define CONFIG_AX25_DAMA_SLAVE 1
#define CONFIG_CAN_PEAK_PCIEC 1
#define CONFIG_CPU_FREQ_GOV_ATTR_SET 1
#define CONFIG_RWSEM_SPIN_ON_OWNER 1
#define CONFIG_ARCH_HAS_STRICT_MODULE_RWX 1
#define CONFIG_SPI_PL022 1
#define CONFIG_DVB_DIB3000MB_MODULE 1
#define CONFIG_COMMON_CLK_A1_PERIPHERALS 1
#define CONFIG_DVB_TS2020_MODULE 1
#define CONFIG_DRM_DISPLAY_DP_AUX_BUS 1
#define CONFIG_LRU_GEN 1
#define CONFIG_SERIAL_TEGRA_TCU_MODULE 1
#define CONFIG_WLAN_VENDOR_INTEL 1
#define CONFIG_VIDEO_CX88_MPEG_MODULE 1
#define CONFIG_CRYPTO_ESSIV_MODULE 1
#define CONFIG_USB_SERIAL_SPCP8X5_MODULE 1
#define CONFIG_ATH5K_DEBUG 1
#define CONFIG_GPIO_GENERIC_PLATFORM 1
#define CONFIG_ARCH_HAS_KERNEL_FPU_SUPPORT 1
#define CONFIG_POWER_RESET_BRCMSTB 1
#define CONFIG_AFE4404_MODULE 1
#define CONFIG_HID_CMEDIA_MODULE 1
#define CONFIG_DVB_LNBP21_MODULE 1
#define CONFIG_RPCSEC_GSS_KRB5_ENCTYPES_AES_SHA1 1
#define CONFIG_HOTPLUG_CORE_SYNC 1
#define CONFIG_HID_ICADE_MODULE 1
#define CONFIG_DVB_AU8522_DTV_MODULE 1
#define CONFIG_NET_VENDOR_ATHEROS 1
#define CONFIG_DVB_SP887X_MODULE 1
#define CONFIG_LEDS_TRIGGER_CAMERA 1
#define CONFIG_DVB_TDA10021_MODULE 1
#define CONFIG_VIRTIO_MMIO_MODULE 1
#define CONFIG_ELFCORE 1
#define CONFIG_CARL9170_WPC 1
#define CONFIG_BLK_PM 1
#define CONFIG_SND_VMASTER 1
#define CONFIG_SERIAL_8250_RSA 1
#define CONFIG_I2C_TEGRA_MODULE 1
#define CONFIG_SENSORS_STTS751_MODULE 1
#define CONFIG_CIFS_DEBUG 1
#define CONFIG_ARM_SMMU_MMU_500_CPRE_ERRATA 1
#define CONFIG_USBIP_VHCI_HCD_MODULE 1
#define CONFIG_DVB_CX24116_MODULE 1
#define CONFIG_BRIDGE_EBT_DNAT_MODULE 1
#define CONFIG_MEDIA_TUNER_MXL5005S_MODULE 1
#define CONFIG_NET_DSA_MV88E6XXX_LEDS 1
#define CONFIG_HID_MULTITOUCH 1
#define CONFIG_SND_HDA_CODEC_SI3054_MODULE 1
#define CONFIG_TOUCHSCREEN_ELO_MODULE 1
#define CONFIG_BMG160_SPI_MODULE 1
#define CONFIG_SENSORS_UCD9000_MODULE 1
#define CONFIG_SND_SOC_PCM3168A_MODULE 1
#define CONFIG_SND_VERBOSE_PROCFS 1
#define CONFIG_GADGET_UAC1 1
#define CONFIG_NET_SCH_INGRESS_MODULE 1
#define CONFIG_BRIDGE_EBT_AMONG_MODULE 1
#define CONFIG_MEDIA_TUNER_TDA827X_MODULE 1
#define CONFIG_SENSORS_HMC5843_MODULE 1
#define CONFIG_SND_SOC_TEGRA20_AC97_MODULE 1
#define CONFIG_GENERIC_CPU_AUTOPROBE 1
#define CONFIG_WLAN_VENDOR_ST 1
#define CONFIG_SOLARIS_X86_PARTITION 1
#define CONFIG_NFC_MICROREAD_MODULE 1
#define CONFIG_INPUT_MOUSEDEV 1
#define CONFIG_NET_SCH_CAKE_MODULE 1
#define CONFIG_NETDEVICES 1
#define CONFIG_DRM_CLIENT_DEFAULT "fbdev"
#define CONFIG_LAN743X_MODULE 1
#define CONFIG_IIO_BUFFER_DMA_MODULE 1
#define CONFIG_NLS_CODEPAGE_874_MODULE 1
#define CONFIG_VIDEO_TEA6415C_MODULE 1
#define CONFIG_RTL8180_MODULE 1
#define CONFIG_USB_GSPCA_BENQ_MODULE 1
#define CONFIG_BMP280_MODULE 1
#define CONFIG_INET_XFRM_TUNNEL_MODULE 1
#define CONFIG_INPUT_MISC 1
#define CONFIG_UBIFS_FS_ZLIB 1
#define CONFIG_NF_NAT_H323_MODULE 1
#define CONFIG_IP_PIMSM_V2 1
#define CONFIG_KGDB_HONOUR_BLOCKLIST 1
#define CONFIG_ETHTOOL_NETLINK 1
#define CONFIG_AD7606_IFACE_SPI_MODULE 1
#define CONFIG_GPIO_MAX730X_MODULE 1
#define CONFIG_VIRTIO_PCI_LIB_LEGACY 1
#define CONFIG_COMMON_CLK 1
#define CONFIG_HAVE_ARM_SMCCC 1
#define CONFIG_USBPCWATCHDOG 1
#define CONFIG_KXSD9_MODULE 1
#define CONFIG_SND_RME96_MODULE 1
#define CONFIG_VIDEO_CX2341X_MODULE 1
#define CONFIG_RTC_DRV_DS1302_MODULE 1
#define CONFIG_ACPI_APEI_EINJ 1
#define CONFIG_RMI4_F3A 1
#define CONFIG_RT2X00_LIB_LEDS 1
#define CONFIG_MD_CLUSTER_MODULE 1
#define CONFIG_LEDS_SYSCON 1
#define CONFIG_RTC_DRV_DS1374_WDT 1
#define CONFIG_HID_PICOLCD_FB 1
#define CONFIG_SCSI_SCAN_ASYNC 1
#define CONFIG_CRYPTO_BLOWFISH_MODULE 1
#define CONFIG_RTW89_PCI_MODULE 1
#define CONFIG_USB_PULSE8_CEC_MODULE 1
#define CONFIG_PHYLINK 1
#define CONFIG_SENSORS_ISL29028_MODULE 1
#define CONFIG_SATA_PMP 1
#define CONFIG_PCIE_ECRC 1
#define CONFIG_DMA_SHARED_BUFFER 1
#define CONFIG_MODULES 1
#define CONFIG_SCHED_HW_PRESSURE 1
#define CONFIG_RT2800PCI_RT35XX 1
#define CONFIG_IPV6_SEG6_HMAC 1
#define CONFIG_SND_SEQ_VIRMIDI_MODULE 1
#define CONFIG_SND_SIMPLE_CARD_UTILS 1
#define CONFIG_NF_NAT_MODULE 1
#define CONFIG_CACHESTAT_SYSCALL 1
#define CONFIG_CMA_SIZE_MBYTES 256
#define CONFIG_CROS_EC_SPI_MODULE 1
#define CONFIG_SND_HDA_INPUT_BEEP 1
#define CONFIG_USB_CONFIGFS_ACM 1
#define CONFIG_I2C_BRCMSTB 1
#define CONFIG_IIO_ST_ACCEL_SPI_3AXIS_MODULE 1
#define CONFIG_NFS_DISABLE_UDP_SUPPORT 1
#define CONFIG_MAX_SKB_FRAGS 17
#define CONFIG_RTC_DRV_RK808_MODULE 1
#define CONFIG_SENSORS_PC87427_MODULE 1
#define CONFIG_BOOT_PRINTK_DELAY 1
#define CONFIG_AD5360_MODULE 1
#define CONFIG_CRYPTO_ACOMP2 1
#define CONFIG_SND_CA0106_MODULE 1
#define CONFIG_VIDEO_AU0828_V4L2 1
#define CONFIG_DM_WRITECACHE_MODULE 1
#define CONFIG_SENSORS_W83773G_MODULE 1
#define CONFIG_V4L2_CCI_I2C_MODULE 1
#define CONFIG_NF_NAT_TFTP_MODULE 1
#define CONFIG_RTW88_8723D_MODULE 1
#define CONFIG_PPS_CLIENT_GPIO_MODULE 1
#define CONFIG_INET_TABLE_PERTURB_ORDER 16
#define CONFIG_SND_SOC_ROCKCHIP_SPDIF_MODULE 1
#define CONFIG_LEDS_TRIGGER_MTD 1
#define CONFIG_VIRTIO_INPUT_MODULE 1
#define CONFIG_IIO_SW_TRIGGER_MODULE 1
#define CONFIG_HID_LOGITECH_MODULE 1
#define CONFIG_MESON_SM 1
#define CONFIG_SND_NM256_MODULE 1
#define CONFIG_MAX30102_MODULE 1
#define CONFIG_RT2X00_LIB_USB_MODULE 1
#define CONFIG_SENSORS_HMC5843_SPI_MODULE 1
#define CONFIG_IIO_BUFFER 1
#define CONFIG_DS1803_MODULE 1
#define CONFIG_DVB_B2C2_FLEXCOP_USB_MODULE 1
#define CONFIG_VIDEO_HANTRO_SUNXI 1
#define CONFIG_ISCSI_BOOT_SYSFS 1
#define CONFIG_TCM_IBLOCK_MODULE 1
#define CONFIG_TSL4531_MODULE 1
#define CONFIG_DRM_PANEL_FEIYANG_FY07024DI26A30D_MODULE 1
#define CONFIG_IIO_SYSFS_TRIGGER_MODULE 1
#define CONFIG_HID_SENSOR_MAGNETOMETER_3D_MODULE 1
#define CONFIG_NFC_PN544_I2C_MODULE 1
#define CONFIG_NET_VENDOR_I825XX 1
#define CONFIG_DMA_ENGINE_RAID 1
#define CONFIG_SENSORS_NCT7904_MODULE 1
#define CONFIG_NF_CONNTRACK_FTP_MODULE 1
#define CONFIG_MFD_MP2629_MODULE 1
#define CONFIG_REGULATOR_PCA9450 1
#define CONFIG_BCM2835_VCHIQ_MMAL_MODULE 1
#define CONFIG_ZPA2326_MODULE 1
#define CONFIG_MMU_GATHER_TABLE_FREE 1
#define CONFIG_PERSISTENT_KEYRINGS 1
#define CONFIG_PMS7003_MODULE 1
#define CONFIG_USB_GSPCA_TOPRO_MODULE 1
#define CONFIG_SENSORS_PWM_FAN_MODULE 1
#define CONFIG_BITREVERSE 1
#define CONFIG_ARM64_ERRATUM_2067961 1
#define CONFIG_PSI 1
#define CONFIG_MAX517_MODULE 1
#define CONFIG_HAVE_ARCH_KASAN_VMALLOC 1
#define CONFIG_KEYBOARD_ADC_MODULE 1
#define CONFIG_IPV6_NDISC_NODETYPE 1
#define CONFIG_USB_ISP1761_UDC 1
#define CONFIG_HID_MONTEREY_MODULE 1
#define CONFIG_NET_VENDOR_PACKET_ENGINES 1
#define CONFIG_GFS2_FS_LOCKING_DLM 1
#define CONFIG_IIO_BACKEND_MODULE 1
#define CONFIG_USB_NET_NET1080_MODULE 1
#define CONFIG_KHADAS_MCU_FAN_THERMAL_MODULE 1
#define CONFIG_AHCI_TEGRA 1
#define CONFIG_BLK_DEV_SD 1
#define CONFIG_IIO_ST_GYRO_I2C_3AXIS_MODULE 1
#define CONFIG_MT76x0U_MODULE 1
#define CONFIG_USB_XHCI_PCI 1
#define CONFIG_RT2800USB_RT3573 1
#define CONFIG_SENSORS_VT8231_MODULE 1
#define CONFIG_BCM_SR_THERMAL 1
#define CONFIG_HAVE_FTRACE_GRAPH_FUNC 1
#define CONFIG_ATH9K_BTCOEX_SUPPORT 1
#define CONFIG_SSB_DRIVER_PCICORE_POSSIBLE 1
#define CONFIG_CDROM_PKTCDVD_BUFFERS 8
#define CONFIG_IIO_ST_MAGN_SPI_3AXIS_MODULE 1
#define CONFIG_NFC_PN544_MODULE 1
#define CONFIG_SCHED_INFO 1
#define CONFIG_FUSION_CTL_MODULE 1
#define CONFIG_PHY_ROCKCHIP_PCIE 1
#define CONFIG_IP_VS_LC_MODULE 1
#define CONFIG_NLS_ISO8859_3_MODULE 1
#define CONFIG_CC_OPTIMIZE_FOR_PERFORMANCE 1
#define CONFIG_USB_CHIPIDEA 1
#define CONFIG_W1_SLAVE_DS2780_MODULE 1
#define CONFIG_ROCKCHIP_RK3066_HDMI 1
#define CONFIG_UDF_FS_MODULE 1
#define CONFIG_CRYPTO_SHA3_ARM64 1
#define CONFIG_HAVE_KPROBES 1
#define CONFIG_NET_SCH_ETF_MODULE 1
#define CONFIG_IIO_INV_SENSORS_TIMESTAMP_MODULE 1
#define CONFIG_IGB_HWMON 1
#define CONFIG_ACPI_HOTPLUG_CPU 1
#define CONFIG_GPIO_ALTERA_MODULE 1
#define CONFIG_USB_SERIAL_KEYSPAN_PDA_MODULE 1
#define CONFIG_RASPBERRYPI_POWER 1
#define CONFIG_VIDEO_CX231XX_DVB_MODULE 1
#define CONFIG_RC_MAP_MODULE 1
#define CONFIG_RTLWIFI_USB_MODULE 1
#define CONFIG_RTL8192D_COMMON_MODULE 1
#define CONFIG_CM3605_MODULE 1
#define CONFIG_BCM7120_L2_IRQ 1
#define CONFIG_NLS_UCS2_UTILS_MODULE 1
#define CONFIG_SND_HDA_CODEC_CA0110_MODULE 1
#define CONFIG_MEDIA_PLATFORM_DRIVERS 1
#define CONFIG_RADIO_TEA5764_MODULE 1
#define CONFIG_MEDIA_TEST_SUPPORT 1
#define CONFIG_ARM64_ERRATUM_2077057 1
#define CONFIG_ACPI_MCFG 1
#define CONFIG_MFD_AXP20X 1
#define CONFIG_HAVE_KVM_CPU_RELAX_INTERCEPT 1
#define CONFIG_SND_VX_LIB_MODULE 1
#define CONFIG_HID_JABRA_MODULE 1
#define CONFIG_ASN1_ENCODER_MODULE 1
#define CONFIG_DVB_MB86A16_MODULE 1
#define CONFIG_PINCTRL_MCP23S08_SPI_MODULE 1
#define CONFIG_LWTUNNEL 1
#define CONFIG_SND_ICE1724_MODULE 1
#define CONFIG_SND_RIPTIDE_MODULE 1
#define CONFIG_CMA_ALIGNMENT 8
#define CONFIG_COMMON_CLK_MESON_CPU_DYNDIV 1
#define CONFIG_SND_HDA_SCODEC_COMPONENT_MODULE 1
#define CONFIG_MT7601U_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_BPF_MODULE 1
#define CONFIG_SUN6I_R_INTC 1
#define CONFIG_IWLEGACY_MODULE 1
#define CONFIG_USB_SERIAL_IPAQ_MODULE 1
#define CONFIG_RTW88_8723CS_MODULE 1
#define CONFIG_LAN966X_SWITCH_MODULE 1
#define CONFIG_ACPI_BGRT 1
#define CONFIG_ENC28J60_MODULE 1
#define CONFIG_DMA_DIRECT_REMAP 1
#define CONFIG_SSB_SDIOHOST 1
#define CONFIG_VIDEO_OV9734_MODULE 1
#define CONFIG_8139TOO_MODULE 1
#define CONFIG_TOUCHSCREEN_FUJITSU_MODULE 1
#define CONFIG_ARCH_ROCKCHIP 1
#define CONFIG_VIDEO_CX88_VP3054_MODULE 1
#define CONFIG_SYSVIPC_SYSCTL 1
#define CONFIG_MESON_GXL_PHY 1
#define CONFIG_USB_SERIAL_OMNINET_MODULE 1
#define CONFIG_I2C_MUX 1
#define CONFIG_DRM_PANEL 1
#define CONFIG_MDIO_BUS_MUX_MULTIPLEXER 1
#define CONFIG_AIC7XXX_CMDS_PER_DEVICE 16
#define CONFIG_FANOTIFY 1
#define CONFIG_PHY_ROCKCHIP_INNO_HDMI 1
#define CONFIG_SND_SUN8I_CODEC_ANALOG 1
#define CONFIG_DMADEVICES 1
#define CONFIG_SND_SOC_SIMPLE_MUX_MODULE 1
#define CONFIG_TCP_CONG_YEAH_MODULE 1
#define CONFIG_FS_MBCACHE 1
#define CONFIG_ARM64_AMU_EXTN 1
#define CONFIG_RTW88_8821CU_MODULE 1
#define CONFIG_ZPA2326_I2C_MODULE 1
#define CONFIG_BCM87XX_PHY_MODULE 1
#define CONFIG_SQUASHFS_FRAGMENT_CACHE_SIZE 3
#define CONFIG_TIMER_OF 1
#define CONFIG_IIO_ST_PRESS_MODULE 1
#define CONFIG_AXP20X_POWER_MODULE 1
#define CONFIG_KSM 1
#define CONFIG_USB_UAS 1
#define CONFIG_CAN_RX_OFFLOAD 1
#define CONFIG_SND_HWDEP_MODULE 1
#define CONFIG_AHCI_DWC 1
#define CONFIG_RC_XBOX_DVD_MODULE 1
#define CONFIG_CRYPTO_LIB_SHA256 1
#define CONFIG_IIO_ST_LSM6DSX_MODULE 1
#define CONFIG_LZ4HC_COMPRESS 1
#define CONFIG_SND_SOC_RT5677_SPI_MODULE 1
#define CONFIG_SERIAL_8250_DMA 1
#define CONFIG_BT_HCIBTUSB_BCM 1
#define CONFIG_NET_IP_TUNNEL_MODULE 1
#define CONFIG_AD5592R_MODULE 1
#define CONFIG_FXAS21002C_I2C_MODULE 1
#define CONFIG_I2C_DESIGNWARE_PCI 1
#define CONFIG_SCSI_BNX2X_FCOE_MODULE 1
#define CONFIG_HAVE_PCI 1
#define CONFIG_DVB_TDA665x_MODULE 1
#define CONFIG_MEDIA_TUNER_QM1D1C0042_MODULE 1
#define CONFIG_USB_DWC3_MESON_G12A 1
#define CONFIG_DM_CLONE_MODULE 1
#define CONFIG_AD7124_MODULE 1
#define CONFIG_NEED_SG_DMA_FLAGS 1
#define CONFIG_IP_NF_TARGET_SYNPROXY_MODULE 1
#define CONFIG_USB_KEENE_MODULE 1
#define CONFIG_RC_DECODERS 1
#define CONFIG_MT76x2_COMMON_MODULE 1
#define CONFIG_BME680_SPI_MODULE 1
#define CONFIG_W1_SLAVE_SMEM_MODULE 1
#define CONFIG_CAVIUM_TX2_ERRATUM_219 1
#define CONFIG_RTW89_8852B_COMMON_MODULE 1
#define CONFIG_ISL29020_MODULE 1
#define CONFIG_VIDEO_ADV_DEBUG 1
#define CONFIG_PHY_MESON_G12A_USB2 1
#define CONFIG_6PACK_MODULE 1
#define CONFIG_BRIDGE_EBT_802_3_MODULE 1
#define CONFIG_ACPI_CONTAINER 1
#define CONFIG_SFC_FALCON_MTD 1
#define CONFIG_CEC_GPIO 1
#define CONFIG_DVB_USB_M920X_MODULE 1
#define CONFIG_DVB_TDA10023_MODULE 1
#define CONFIG_CHR_DEV_SG 1
#define CONFIG_ARCH_HAS_HW_PTE_YOUNG 1
#define CONFIG_PINCTRL_SUN50I_A64 1
#define CONFIG_MMU 1
#define CONFIG_SSB_DRIVER_PCICORE 1
#define CONFIG_MEDIA_TUNER_MT2131_MODULE 1
#define CONFIG_IEEE802154_SOCKET_MODULE 1
#define CONFIG_SND_SOC_TEGRA_RT5677_MODULE 1
#define CONFIG_IP6_NF_MATCH_MH_MODULE 1
#define CONFIG_SECURITY 1
#define CONFIG_DRM_MIPI_DSI 1
#define CONFIG_FS_ENCRYPTION 1
#define CONFIG_SENSORS_ARM_SCMI_MODULE 1
#define CONFIG_IR_XMP_DECODER_MODULE 1
#define CONFIG_MEDIA_COMMON_OPTIONS 1
#define CONFIG_CRYPTO_RNG2 1
#define CONFIG_NLS_ISO8859_2_MODULE 1
#define CONFIG_HID_TIVO_MODULE 1
#define CONFIG_COMMON_CLK_AXG_AUDIO 1
#define CONFIG_W1_SLAVE_DS2781_MODULE 1
#define CONFIG_VIDEO_OV7640_MODULE 1
#define CONFIG_IPV6_GRE_MODULE 1
#define CONFIG_KEYBOARD_GPIO_MODULE 1
#define CONFIG_MUX_ADGS1408_MODULE 1
#define CONFIG_WILC1000_SDIO_MODULE 1
#define CONFIG_MMC_DW_PLTFM 1
#define CONFIG_NET_FOU_IP_TUNNELS 1
#define CONFIG_MMC_MESON_MX_SDIO 1
#define CONFIG_DRM_LOAD_EDID_FIRMWARE 1
#define CONFIG_MDIO_BUS_MUX_GPIO_MODULE 1
#define CONFIG_INPUT_PWM_VIBRA_MODULE 1
#define CONFIG_MINIX_FS_MODULE 1
#define CONFIG_SND_USB_AUDIO_MODULE 1
#define CONFIG_RC_CORE_MODULE 1
#define CONFIG_BCM2835_MBOX 1
#define CONFIG_CRYPTO_DEV_ALLWINNER 1
#define CONFIG_BMP280_SPI_MODULE 1
#define CONFIG_HAMACHI_MODULE 1
#define CONFIG_CRYPTO_DRBG_CTR 1
#define CONFIG_NET_SCH_QFQ_MODULE 1
#define CONFIG_MANTIS_CORE_MODULE 1
#define CONFIG_SPI_DESIGNWARE 1
#define CONFIG_TOUCHSCREEN_USB_ZYTRONIC 1
#define CONFIG_NFT_OSF_MODULE 1
#define CONFIG_BME680_I2C_MODULE 1
#define CONFIG_MOUSE_SYNAPTICS_USB_MODULE 1
#define CONFIG_MFD_CROS_EC_DEV_MODULE 1
#define CONFIG_RESET_ATTACK_MITIGATION 1
#define CONFIG_DRM_TEGRA_STAGING 1
#define CONFIG_VCAP 1
#define CONFIG_CFG80211_DEFAULT_PS 1
#define CONFIG_PHY_SUN6I_MIPI_DPHY 1
#define CONFIG_BT_LE_L2CAP_ECRED 1
#define CONFIG_MT76_CONNAC_LIB_MODULE 1
#define CONFIG_CAN_KVASER_PCI_MODULE 1
#define CONFIG_SND_SEQUENCER_MODULE 1
#define CONFIG_OPENVSWITCH_GENEVE_MODULE 1
#define CONFIG_WLAN_VENDOR_MEDIATEK 1
#define CONFIG_CAN_CALC_BITTIMING 1
#define CONFIG_IPC_NS 1
#define CONFIG_RTC_DRV_DS1390_MODULE 1
#define CONFIG_ARCH_HAS_DEBUG_VIRTUAL 1
#define CONFIG_HID_PID 1
#define CONFIG_IOMMU_SUPPORT 1
#define CONFIG_MUX_GPIO_MODULE 1
#define CONFIG_ARCH_HAS_RESET_CONTROLLER 1
#define CONFIG_IPMI_WATCHDOG_MODULE 1
#define CONFIG_NET_ACT_NAT_MODULE 1
#define CONFIG_BROADCOM_PHY_MODULE 1
#define CONFIG_USB_CHIPIDEA_MSM 1
#define CONFIG_DPLL 1
#define CONFIG_SND_SOC_MAX98357A_MODULE 1
#define CONFIG_HID_MACALLY_MODULE 1
#define CONFIG_JOYSTICK_ADI_MODULE 1
#define CONFIG_MEMBARRIER 1
#define CONFIG_ROCKCHIP_INNO_HDMI 1
#define CONFIG_SUN8I_THERMAL_MODULE 1
#define CONFIG_SENSORS_DME1737_MODULE 1
#define CONFIG_ENERGY_MODEL 1
#define CONFIG_B43_BCMA 1
#define CONFIG_PCI_IOV 1
#define CONFIG_USB_VL600_MODULE 1
#define CONFIG_CPU_IDLE_GOV_MENU 1
#define CONFIG_NETFILTER_XT_MATCH_HELPER_MODULE 1
#define CONFIG_OF_ADDRESS 1
#define CONFIG_CRC32_ARCH 1
#define CONFIG_CRYPTO_ARCH_HAVE_LIB_CHACHA 1
#define CONFIG_I2C_SI4713_MODULE 1
#define CONFIG_I40E_DCB 1
#define CONFIG_MFD_SUN6I_PRCM 1
#define CONFIG_FB_TFT_ILI9325_MODULE 1
#define CONFIG_USB_SERIAL_MCT_U232_MODULE 1
#define CONFIG_VFIO_PLATFORM_CALXEDAXGMAC_RESET_MODULE 1
#define CONFIG_ARM_CCI 1
#define CONFIG_VIDEO_OV2640_MODULE 1
#define CONFIG_VSOCKETS_MODULE 1
#define CONFIG_CHECK_SIGNATURE 1
#define CONFIG_MEDIA_DIGITAL_TV_SUPPORT 1
#define CONFIG_USB_CONFIGFS_RNDIS 1
#define CONFIG_MXC4005_MODULE 1
#define CONFIG_NET_CLS_CGROUP 1
#define CONFIG_CRYPTO_DRBG_MENU 1
#define CONFIG_CLS_U32_MARK 1
#define CONFIG_SCSI_SNIC_MODULE 1
#define CONFIG_DVB_LNBH25_MODULE 1
#define CONFIG_BATTERY_CW2015_MODULE 1
#define CONFIG_DRM_HISI_KIRIN_MODULE 1
#define CONFIG_W1_SLAVE_THERM_MODULE 1
#define CONFIG_MAX30100_MODULE 1
#define CONFIG_DRM_TI_TFP410_MODULE 1
#define CONFIG_SND_SOC_SUNXI_SUN50IW9_CODEC_MODULE 1
#define CONFIG_VHOST_IOTLB_MODULE 1
#define CONFIG_IPMI_DMI_DECODE 1
#define CONFIG_SUN50I_A64_CCU 1
#define CONFIG_CW1200_WLAN_SPI_MODULE 1
#define CONFIG_XZ_DEC 1
#define CONFIG_ARM64_CONT_PTE_SHIFT 4
#define CONFIG_SENSORS_ADS7828_MODULE 1
#define CONFIG_IIO_MS_SENSORS_I2C_MODULE 1
#define CONFIG_USB_DSBR_MODULE 1
#define CONFIG_CC_HAS_KASAN_SW_TAGS 1
#define CONFIG_SND_SOC_ROCKCHIP_I2S_MODULE 1
#define CONFIG_CIFS_ALLOW_INSECURE_LEGACY 1
#define CONFIG_ATH11K_MODULE 1
#define CONFIG_THERMAL_HWMON 1
#define CONFIG_LEDS_TRIGGER_CPU 1
#define CONFIG_ROCKCHIP_DW_HDMI_QP 1
#define CONFIG_PCI_QUIRKS 1
#define CONFIG_DRM_DW_HDMI_GP_AUDIO 1
#define CONFIG_NET_DSA_TAG_BRCM_MODULE 1
#define CONFIG_NET_SCH_FQ_CODEL 1
#define CONFIG_IPV6_ILA_MODULE 1
#define CONFIG_TEGRA_TIMER 1
#define CONFIG_PATA_MARVELL_MODULE 1
#define CONFIG_GENERIC_PINCONF 1
#define CONFIG_MOUSE_PS2_MODULE 1
#define CONFIG_ARCH_HAS_CRC32 1
#define CONFIG_MEDIA_TUNER_SI2157_MODULE 1
#define CONFIG_DW_DMAC_PCI_MODULE 1
#define CONFIG_CROS_EC_VBC_MODULE 1
#define CONFIG_MD 1
#define CONFIG_I2C_VIRTIO_MODULE 1
#define CONFIG_PCPU_DEV_REFCNT 1
#define CONFIG_I40E_MODULE 1
#define CONFIG_IR_ITE_CIR_MODULE 1
#define CONFIG_RTC_DRV_DS1347_MODULE 1
#define CONFIG_SENSORS_RASPBERRYPI_HWMON_MODULE 1
#define CONFIG_MODULE_UNLOAD 1
#define CONFIG_NET_VENDOR_INTEL 1
#define CONFIG_THERMAL_DEFAULT_GOV_STEP_WISE 1
#define CONFIG_DVB_TDA8261_MODULE 1
#define CONFIG_PINCTRL_SUN8I_H3_R 1
#define CONFIG_ATH9K_RFKILL 1
#define CONFIG_CONSOLE_LOGLEVEL_DEFAULT 3
#define CONFIG_RT2800USB_RT55XX 1
#define CONFIG_DVB_DIB7000M_MODULE 1
#define CONFIG_USB_GSPCA_T613_MODULE 1
#define CONFIG_VITESSE_PHY_MODULE 1
#define CONFIG_TCP_CONG_BIC_MODULE 1
#define CONFIG_ARM64_BTI 1
#define CONFIG_IGBVF_MODULE 1
#define CONFIG_PAGE_COUNTER 1
#define CONFIG_CONSOLE_TRANSLATIONS 1
#define CONFIG_IP_VS_RR_MODULE 1
#define CONFIG_CPU_FREQ_GOV_POWERSAVE 1
#define CONFIG_CRYPTO_TWOFISH_COMMON_MODULE 1
#define CONFIG_USB_ACM_MODULE 1
#define CONFIG_DVB_CX22700_MODULE 1
#define CONFIG_WATCHDOG_OPEN_TIMEOUT 0
#define CONFIG_BUILDTIME_TABLE_SORT 1
#define CONFIG_NLS_ISO8859_5_MODULE 1
#define CONFIG_DVB_USB_VP702X_MODULE 1
#define CONFIG_E1000E_MODULE 1
#define CONFIG_HAVE_CONTEXT_TRACKING_USER 1
#define CONFIG_IEEE802154_DRIVERS_MODULE 1
#define CONFIG_BINFMT_SCRIPT 1
#define CONFIG_CPU_IDLE_GOV_LADDER 1
#define CONFIG_SENSORS_CORSAIR_PSU_MODULE 1
#define CONFIG_MD_RAID1_MODULE 1
#define CONFIG_VIRTIO_NET_MODULE 1
#define CONFIG_USB_GADGET_DEBUG_FILES 1
#define CONFIG_PINMUX 1
#define CONFIG_CRYPTO_SIG2 1
#define CONFIG_MAC80211_MODULE 1
#define CONFIG_DVB_STV0910_MODULE 1
#define CONFIG_THRUSTMASTER_FF 1
#define CONFIG_SND_MIXER_OSS_MODULE 1
#define CONFIG_ARCH_HAVE_ELF_PROT 1
#define CONFIG_RTW88_8822CU_MODULE 1
#define CONFIG_MEDIA_TUNER_TDA18271_MODULE 1
#define CONFIG_MULTIUSER 1
#define CONFIG_CRYPTO_842_MODULE 1
#define CONFIG_USB_AN2720 1
#define CONFIG_CRYPTO_USER_API_ENABLE_OBSOLETE 1
#define CONFIG_FONT_8x16 1
#define CONFIG_LIBERTAS_MODULE 1
#define CONFIG_NLS_MAC_ROMAN_MODULE 1
#define CONFIG_RTW88_8822C_MODULE 1
#define CONFIG_PPP_MODULE 1
#define CONFIG_ZOPT2201_MODULE 1
#define CONFIG_PNP_DEBUG_MESSAGES 1
#define CONFIG_HAVE_KVM_IRQ_ROUTING 1
#define CONFIG_NETFILTER_SKIP_EGRESS 1
#define CONFIG_TOOLS_SUPPORT_RELR 1
#define CONFIG_SPMI 1
#define CONFIG_BRIDGE_IGMP_SNOOPING 1
#define CONFIG_USB_GSPCA_JL2005BCD_MODULE 1
#define CONFIG_VCNL4035_MODULE 1
#define CONFIG_CMA 1
#define CONFIG_SND_USB_TONEPORT_MODULE 1
#define CONFIG_IP6_NF_FILTER_MODULE 1
#define CONFIG_HID_SAITEK_MODULE 1
#define CONFIG_CRYPTO_AES_ARM64_CE_BLK 1
#define CONFIG_IP_NF_ARPTABLES_MODULE 1
#define CONFIG_ARCH_FORCE_MAX_ORDER 10
#define CONFIG_NETFILTER_XT_MATCH_MARK_MODULE 1
#define CONFIG_CHARGER_GPIO_MODULE 1
#define CONFIG_LAN966X_DCB 1
#define CONFIG_ARM_SCMI_QUIRKS 1
#define CONFIG_SCSI_DH 1
#define CONFIG_HID_ACRUX_MODULE 1
#define CONFIG_PINCTRL_MESON_A1 1
#define CONFIG_NETFILTER_XT_MATCH_LENGTH_MODULE 1
#define CONFIG_RTC_DRV_M41T80_MODULE 1
#define CONFIG_TMPFS_XATTR 1
#define CONFIG_TOUCHSCREEN_USB_GUNZE 1
#define CONFIG_BRCMFMAC_PCIE 1
#define CONFIG_TCG_INFINEON_MODULE 1
#define CONFIG_DVB_BCM3510_MODULE 1
#define CONFIG_ARM64_ERRATUM_824069 1
#define CONFIG_BLOCK 1
#define CONFIG_CYPRESS_FIRMWARE_MODULE 1
#define CONFIG_PCI_LABEL 1
#define CONFIG_SND_LX6464ES_MODULE 1
#define CONFIG_AIC_USB_WLAN_SUPPORT 1
#define CONFIG_PTP_1588_CLOCK_OPTIONAL 1
#define CONFIG_CRYPTO_JITTERENTROPY_MEMORY_BLOCKS 64
#define CONFIG_DM_BUFIO 1
#define CONFIG_DEBUG_MISC 1
#define CONFIG_NET_VENDOR_3COM 1
#define CONFIG_ARM64_PTR_AUTH_KERNEL 1
#define CONFIG_CPU_FREQ 1
#define CONFIG_KEYBOARD_SUN4I_LRADC_MODULE 1
#define CONFIG_USB_G_SERIAL_MODULE 1
#define CONFIG_FAT_DEFAULT_IOCHARSET "ascii"
#define CONFIG_SENSORS_DS620_MODULE 1
#define CONFIG_NFSD_PNFS 1
#define CONFIG_TTY 1
#define CONFIG_CRYPTO_SM4_ARM64_CE_BLK 1
#define CONFIG_DVB_USB_ZD1301_MODULE 1
#define CONFIG_DECOMPRESS_LZMA 1
#define CONFIG_DVB_MANTIS_MODULE 1
#define CONFIG_IR_IGUANA_MODULE 1
#define CONFIG_CEC_NXP_TDA9950_MODULE 1
#define CONFIG_IPW2200_MONITOR 1
#define CONFIG_ARM_RASPBERRYPI_CPUFREQ_MODULE 1
#define CONFIG_842_DECOMPRESS_MODULE 1
#define CONFIG_I40EVF_MODULE 1
#define CONFIG_CFS_BANDWIDTH 1
#define CONFIG_LEDS_LM3530_MODULE 1
#define CONFIG_MOUSE_PS2_CYPRESS 1
#define CONFIG_B43_BCMA_PIO 1
#define CONFIG_ACPI_TAD_MODULE 1
#define CONFIG_SYMBOLIC_ERRNAME 1
#define CONFIG_SENSORS_LINEAGE_MODULE 1
#define CONFIG_VIDEO_CX88_ENABLE_VP3054 1
#define CONFIG_CAN_PEAK_USB_MODULE 1
#define CONFIG_QUEUED_SPINLOCKS 1
#define CONFIG_SENSORS_LTC2978_MODULE 1
#define CONFIG_ARCH_SUPPORTS_INT128 1
#define CONFIG_TCP_CONG_BBR_MODULE 1
#define CONFIG_CRYPTO_MD4 1
#define CONFIG_SERIAL_8250_CONSOLE 1
#define CONFIG_LOCK_MM_AND_FIND_VMA 1
#define CONFIG_SOC_BUS 1
#define CONFIG_SND_SOC_RK3399_GRU_SOUND_MODULE 1
#define CONFIG_NETWORK_PHY_TIMESTAMPING 1
#define CONFIG_PROC_KCORE 1
#define CONFIG_ARCH_USE_CMPXCHG_LOCKREF 1
#define CONFIG_ARCH_HAS_KEEPINITRD 1
#define CONFIG_CLK_SP810 1
#define CONFIG_FW_LOADER_DEBUG 1
#define CONFIG_ARCH_HAS_SUBPAGE_FAULTS 1
#define CONFIG_RTW88_LEDS 1
#define CONFIG_VIDEO_CX25840_MODULE 1
#define CONFIG_REGULATOR_GPIO 1
#define CONFIG_XFS_POSIX_ACL 1
#define CONFIG_BLK_MQ_STACKING 1
#define CONFIG_SWP_EMULATION 1
#define CONFIG_SFC_SIENA_MCDI_MON 1
#define CONFIG_DRM_TOSHIBA_TC358767_MODULE 1
#define CONFIG_HAVE_TRUSTED_KEYS 1
#define CONFIG_CARL9170_LEDS 1
#define CONFIG_NF_DUP_NETDEV_MODULE 1
#define CONFIG_NET_ACT_CSUM_MODULE 1
#define CONFIG_AD5593R_MODULE 1
#define CONFIG_SND_MESON_AXG_TDM_INTERFACE_MODULE 1
#define CONFIG_MEDIA_TUNER_TDA9887_MODULE 1
#define CONFIG_CROS_EC_CHARDEV_MODULE 1
#define CONFIG_SCSI_UFSHCD 1
#define CONFIG_POWERCAP 1
#define CONFIG_I2C_MUX_GPMUX_MODULE 1
#define CONFIG_IOMMUFD_MODULE 1
#define CONFIG_INET6_IPCOMP_MODULE 1
#define CONFIG_CRYPTO_DEV_SP_CCP 1
#define CONFIG_FW_LOADER_SYSFS 1
#define CONFIG_NFT_COMPAT_MODULE 1
#define CONFIG_TEGRA20_APB_DMA 1
#define CONFIG_NET_DSA_TAG_XRS700X_MODULE 1
#define CONFIG_ARM64_TAGGED_ADDR_ABI 1
#define CONFIG_DRM_VC4_HDMI_CEC 1
#define CONFIG_CRYPTO_ALGAPI2 1
#define CONFIG_SCSI_PMCRAID_MODULE 1
#define CONFIG_DMA_CMA 1
#define CONFIG_BCM7XXX_PHY_MODULE 1
#define CONFIG_CPU_MITIGATIONS 1
#define CONFIG_MEDIA_TUNER_R820T_MODULE 1
#define CONFIG_NLS_ISO8859_4_MODULE 1
#define CONFIG_RAID6_PQ_MODULE 1
#define CONFIG_HID_WACOM_MODULE 1
#define CONFIG_MD_RAID0_MODULE 1
#define CONFIG_CLZ_TAB 1
#define CONFIG_PROC_MEM_ALWAYS_FORCE 1
#define CONFIG_SND_SOC_ROCKCHIP_SAI_MODULE 1
#define CONFIG_SND_CS4281_MODULE 1
#define CONFIG_CHROME_PLATFORMS 1
#define CONFIG_ARCH_USES_PG_ARCH_2 1
#define CONFIG_PINCONF 1
#define CONFIG_PM_DEVFREQ_EVENT 1
#define CONFIG_SND_HDA_PATCH_LOADER 1
#define CONFIG_FUNCTION_ALIGNMENT 4
#define CONFIG_F2FS_STAT_FS 1
#define CONFIG_ARM_SCMI_TRANSPORT_MAILBOX 1
#define CONFIG_ATH9K_AHB 1
#define CONFIG_NETFILTER_XT_MATCH_PKTTYPE_MODULE 1
#define CONFIG_CRYPTO_XTS 1
#define CONFIG_NETFILTER_XT_MATCH_MAC_MODULE 1
#define CONFIG_IIO_SW_DEVICE_MODULE 1
#define CONFIG_DVB_TC90522_MODULE 1
#define CONFIG_LEDS_TRIGGER_HEARTBEAT 1
#define CONFIG_INET_SCTP_DIAG_MODULE 1
#define CONFIG_SI1133_MODULE 1
#define CONFIG_RTW88_8822B_MODULE 1
#define CONFIG_PPS 1
#define CONFIG_NFC_PORT100_MODULE 1
#define CONFIG_INTERCONNECT 1
#define CONFIG_MAC80211_HAS_RC 1
#define CONFIG_ARM_ARCH_TIMER 1
#define CONFIG_NLS_MAC_GREEK_MODULE 1
#define CONFIG_ARCH_USE_GNU_PROPERTY 1
#define CONFIG_SND_SOC_WM8753_MODULE 1
#define CONFIG_VIDEO_EM28XX_MODULE 1
#define CONFIG_GLOB 1
#define CONFIG_CAN_BCM_MODULE 1
#define CONFIG_PINCTRL_MAX77620 1
#define CONFIG_SCSI_SYM53C8XX_DMA_ADDRESSING_MODE 1
#define CONFIG_SND_SOC_RK3328_MODULE 1
#define CONFIG_SENSORS_TMP421_MODULE 1
#define CONFIG_USB_GSPCA_MARS_MODULE 1
#define CONFIG_IP_ROUTE_CLASSID 1
#define CONFIG_ST_UVIS25_I2C_MODULE 1
#define CONFIG_DEBUG_VM_PGTABLE 1
#define CONFIG_FM10K_MODULE 1
#define CONFIG_DEVMEM 1
#define CONFIG_REGULATOR_RK808 1
#define CONFIG_SENSORS_ADS7871_MODULE 1
#define CONFIG_USBIP_HOST_MODULE 1
#define CONFIG_BACKLIGHT_GPIO_MODULE 1
#define CONFIG_USB_GADGET_VBUS_DRAW 500
#define CONFIG_NETFILTER_XT_CONNMARK_MODULE 1
#define CONFIG_NFS_V4_SECURITY_LABEL 1
#define CONFIG_ISCSI_IBFT 1
#define CONFIG_KEXEC_CORE 1
#define CONFIG_MFD_BD9571MWV 1
#define CONFIG_MEDIA_TUNER_TDA18218_MODULE 1
#define CONFIG_DVB_DRXD_MODULE 1
#define CONFIG_LSM "landlock,lockdown,yama,loadpin,safesetid,integrity,selinux,smack,tomoyo,apparmor,bpf"
#define CONFIG_ARCH_HAS_PKEYS 1
#define CONFIG_USB_FUNCTIONFS_RNDIS 1
#define CONFIG_BMC150_ACCEL_I2C_MODULE 1
#define CONFIG_TEGRA_HOST1X_FIREWALL 1
#define CONFIG_GPIO_MAX77620 1
#define CONFIG_HID_RETRODE_MODULE 1
#define CONFIG_I2C_MESON 1
#define CONFIG_USB_NET_MCS7830_MODULE 1
#define CONFIG_FB_CFB_COPYAREA 1
#define CONFIG_CRYPTO_ALGAPI 1
#define CONFIG_KVM_XFER_TO_GUEST_WORK 1
#define CONFIG_DWMAC_SUNXI 1
#define CONFIG_HTU21_MODULE 1
#define CONFIG_SND_PCXHR_MODULE 1
#define CONFIG_DWMAC_MESON 1
#define CONFIG_CLONE_BACKWARDS 1
#define CONFIG_SND_SOC_TLV320AIC23_MODULE 1
#define CONFIG_SERIAL_EARLYCON 1
#define CONFIG_RTC_DRV_M41T94_MODULE 1
#define CONFIG_NF_NAT_IRC_MODULE 1
#define CONFIG_GET_FREE_REGION 1
#define CONFIG_USB_F_HID 1
#define CONFIG_AD5770R_MODULE 1
#define CONFIG_DEVPORT 1
#define CONFIG_CRYPTO_HKDF 1
#define CONFIG_USB 1
#define CONFIG_NETFILTER_XT_TARGET_CT_MODULE 1
#define CONFIG_ARM64_ERRATUM_1542419 1
#define CONFIG_N_HDLC_MODULE 1
#define CONFIG_SCTP_COOKIE_HMAC_SHA1 1
#define CONFIG_ATH9K_STATION_STATISTICS 1
#define CONFIG_ELF_CORE 1
#define CONFIG_FW_LOADER_USER_HELPER_FALLBACK 1
#define CONFIG_PCI_HOST_COMMON 1
#define CONFIG_TASK_DELAY_ACCT 1
#define CONFIG_F2FS_FS_XATTR 1
#define CONFIG_DVB_ATBM8830_MODULE 1
#define CONFIG_DM_DELAY_MODULE 1
#define CONFIG_SCSI_QLOGIC_1280_MODULE 1
#define CONFIG_DVB_PLL_MODULE 1
#define CONFIG_THERMAL 1
#define CONFIG_SPI_AX88796C_COMPRESSION 1
#define CONFIG_NFC_ST21NFCA_MODULE 1
#define CONFIG_CRYPTO_AES_ARM64_CE 1
#define CONFIG_SI7020_MODULE 1
#define CONFIG_NETFILTER_BPF_LINK 1
#define CONFIG_SND_BCM2835_MODULE 1
#define CONFIG_DVB_LG2160_MODULE 1
#define CONFIG_BEFS_FS_MODULE 1
#define CONFIG_REGULATOR_SY8106A 1
#define CONFIG_NET_VENDOR_DLINK 1
#define CONFIG_IP_SET_HASH_IPPORTNET_MODULE 1
#define CONFIG_PPPOE_HASH_BITS 4
#define CONFIG_CAN_EMS_USB_MODULE 1
#define CONFIG_NET_TEAM_MODE_ROUNDROBIN_MODULE 1
#define CONFIG_TICK_CPU_ACCOUNTING 1
#define CONFIG_CAN_VXCAN_MODULE 1
#define CONFIG_DVB_USB_NOVA_T_USB2_MODULE 1
#define CONFIG_SENSORS_LTC4245_MODULE 1
#define CONFIG_CRYPTO_MD5 1
#define CONFIG_SND_DRIVERS 1
#define CONFIG_REGULATOR_FAN53555 1
#define CONFIG_SND_AC97_POWER_SAVE_DEFAULT 0
#define CONFIG_INPUT_CMA3000_MODULE 1
#define CONFIG_SMSC9420_MODULE 1
#define CONFIG_ATH6KL_MODULE 1
#define CONFIG_INPUT_KEYSPAN_REMOTE_MODULE 1
#define CONFIG_ATH9K_HTC_MODULE 1
#define CONFIG_DM_AUDIT 1
#define CONFIG_NETFILTER_XT_MATCH_POLICY_MODULE 1
#define CONFIG_XZ_DEC_X86 1
#define CONFIG_USB_ISP1301_MODULE 1
#define CONFIG_SND_HDA_HWDEP 1
#define CONFIG_VT_CONSOLE_SLEEP 1
#define CONFIG_NLS_KOI8_U_MODULE 1
#define CONFIG_PINCTRL_SUN50I_H6 1
#define CONFIG_SCSI_SPI_ATTRS_MODULE 1
#define CONFIG_RUSTC_VERSION 0
#define CONFIG_CRYPTO_SERPENT_MODULE 1
#define CONFIG_MTD_BLKDEVS 1
#define CONFIG_MCP4018_MODULE 1
#define CONFIG_DEBUG_KERNEL 1
#define CONFIG_PHY_ROCKCHIP_SAMSUNG_DCPHY_MODULE 1
#define CONFIG_ASYNC_TX_DMA 1
#define CONFIG_INIT_STACK_NONE 1
#define CONFIG_AS_HAS_MOPS 1
#define CONFIG_VIDEO_SUN50I_DEINTERLACE_MODULE 1
#define CONFIG_SCSI_QLA_ISCSI_MODULE 1
#define CONFIG_CAN_M_CAN_MODULE 1
#define CONFIG_MOUSE_ELAN_I2C_SMBUS 1
#define CONFIG_CEC_CORE 1
#define CONFIG_RTC_DRV_EFI_MODULE 1
#define CONFIG_WLAN_VENDOR_TI 1
#define CONFIG_EEPROM_93CX6 1
#define CONFIG_VIDEOBUF2_CORE_MODULE 1
#define CONFIG_ATM_CLIP_MODULE 1
#define CONFIG_USB_SERIAL_DEBUG_MODULE 1
#define CONFIG_HID_VIEWSONIC_MODULE 1
#define CONFIG_MISC_FILESYSTEMS 1
#define CONFIG_QUOTA_TREE 1
#define CONFIG_NFT_FIB_MODULE 1
#define CONFIG_NET_DSA_REALTEK_MODULE 1
#define CONFIG_CRYPTO_GHASH 1
#define CONFIG_MDIO_BITBANG_MODULE 1
#define CONFIG_NET_ACT_TUNNEL_KEY_MODULE 1
#define CONFIG_CGROUP_RDMA 1
#define CONFIG_CAN_PLX_PCI_MODULE 1
#define CONFIG_INOTIFY_USER 1
#define CONFIG_DVB_CX22702_MODULE 1
#define CONFIG_USB_GSPCA_VICAM_MODULE 1
#define CONFIG_NLS_ISO8859_7_MODULE 1
#define CONFIG_SENSORS_LTC2947_SPI_MODULE 1
#define CONFIG_PHYLIB 1
#define CONFIG_B43_HWRNG 1
#define CONFIG_DVB_GP8PSK_FE_MODULE 1
#define CONFIG_SPI_MUX_MODULE 1
#define CONFIG_ARCH_HAS_GIGANTIC_PAGE 1
#define CONFIG_NET_EMATCH_IPT_MODULE 1
#define CONFIG_USB_MICROTEK_MODULE 1
#define CONFIG_HDMI 1
#define CONFIG_ARCH_USES_PG_ARCH_3 1
#define CONFIG_NET_UDP_TUNNEL_MODULE 1
#define CONFIG_TOUCHSCREEN_USB_COMPOSITE_MODULE 1
#define CONFIG_DVB_USB_DIB3000MC_MODULE 1
#define CONFIG_RTW88_8822CS_MODULE 1
#define CONFIG_NET_DSA_TAG_RTL8_4_MODULE 1
#define CONFIG_USB_DWC3_OF_SIMPLE 1
#define CONFIG_INPUT_TABLET 1
#define CONFIG_V4L_PLATFORM_DRIVERS 1
#define CONFIG_MAXIM_THERMOCOUPLE_MODULE 1
#define CONFIG_PPPOE_MODULE 1
#define CONFIG_LIBERTAS_THINFIRM_MODULE 1
#define CONFIG_DP83867_PHY_MODULE 1
#define CONFIG_CAN_GS_USB_MODULE 1
#define CONFIG_HAVE_DYNAMIC_FTRACE_WITH_ARGS 1
#define CONFIG_TSL2583_MODULE 1
#define CONFIG_ARCH_HAS_SYSCALL_WRAPPER 1
#define CONFIG_REED_SOLOMON_MODULE 1
#define CONFIG_HAVE_ARCH_MMAP_RND_COMPAT_BITS 1
#define CONFIG_FAT_DEFAULT_CODEPAGE 437
#define CONFIG_DMA_COHERENT_POOL 1
#define CONFIG_RMI4_2D_SENSOR 1
#define CONFIG_JSA1212_MODULE 1
#define CONFIG_SENSORS_LM25066_MODULE 1
#define CONFIG_AD9834_MODULE 1
#define CONFIG_SENSORS_PC87360_MODULE 1
#define CONFIG_DRM_PANEL_EDP_MODULE 1
#define CONFIG_REGULATOR_ARM_SCMI 1
#define CONFIG_NLATTR 1
#define CONFIG_DVB_SI21XX_MODULE 1
#define CONFIG_ACPI_PCI_SLOT 1
#define CONFIG_PDC_ADMA_MODULE 1
#define CONFIG_RT2800PCI_RT3290 1
#define CONFIG_HAVE_GUP_FAST 1
#define CONFIG_MLX4_EN_DCB 1
#define CONFIG_ARM64_MTE 1
#define CONFIG_NFT_REJECT_INET_MODULE 1
#define CONFIG_SND_OXYGEN_LIB_MODULE 1
#define CONFIG_CORDIC_MODULE 1
#define CONFIG_ADXL372_SPI_MODULE 1
#define CONFIG_ARM_SCMI_HAVE_TRANSPORT 1
#define CONFIG_RTC_DRV_M48T35_MODULE 1
#define CONFIG_SCHED_MC 1
#define CONFIG_RTC_DRV_PCF2127_MODULE 1
#define CONFIG_TCP_CONG_ADVANCED 1
#define CONFIG_CEPH_FS_SECURITY_LABEL 1
#define CONFIG_AD5761_MODULE 1
#define CONFIG_DRM_DW_DP 1
#define CONFIG_ARCH_HAS_CC_PLATFORM 1
#define CONFIG_USB_KAWETH_MODULE 1
#define CONFIG_USB_NET_CDC_MBIM_MODULE 1
#define CONFIG_SENSORS_LM90_MODULE 1
#define CONFIG_MMU_NOTIFIER 1
#define CONFIG_FUSION 1
#define CONFIG_CRYPTO_CAMELLIA_MODULE 1
#define CONFIG_HID_GFRM_MODULE 1
#define CONFIG_RTC_DRV_M41T93_MODULE 1
#define CONFIG_MEDIA_USB_SUPPORT 1
#define CONFIG_MOUSE_ELAN_I2C_MODULE 1
#define CONFIG_RTLWIFI_MODULE 1
#define CONFIG_HAVE_EBPF_JIT 1
#define CONFIG_USB_SERIAL_EMPEG_MODULE 1
#define CONFIG_NET_ACT_MIRRED_MODULE 1
#define CONFIG_NET_VENDOR_WANGXUN 1
#define CONFIG_PM_OPP 1
#define CONFIG_RFS_ACCEL 1
#define CONFIG_SCSI_MPT3SAS_MODULE 1
#define CONFIG_YAM_MODULE 1
#define CONFIG_BRCMSTB_DPFE 1
#define CONFIG_KALLSYMS 1
#define CONFIG_INPUT_AXP20X_PEK_MODULE 1
#define CONFIG_FUSION_MAX_SGE 128
#define CONFIG_LEDS_TRIGGER_ACTIVITY 1
#define CONFIG_HID_PLANTRONICS_MODULE 1
#define CONFIG_SMS_SIANO_MDTV_MODULE 1
#define CONFIG_INPUT_E3X0_BUTTON_MODULE 1
#define CONFIG_MDIO_BUS 1
#define CONFIG_ARM64_WORKAROUND_SPECULATIVE_UNPRIV_LOAD 1
#define CONFIG_NVMEM_LAYOUTS 1
#define CONFIG_SCSI_MOD 1
#define CONFIG_ZSMALLOC_CHAIN_SIZE 8
#define CONFIG_CUSE_MODULE 1
#define CONFIG_IIO 1
#define CONFIG_ATH9K_HW_MODULE 1
#define CONFIG_BCMA_DRIVER_GPIO 1
#define CONFIG_NET_DSA_AR9331_MODULE 1
#define CONFIG_DRM_SUN6I_DSI 1
#define CONFIG_I2C_HELPER_AUTO 1
#define CONFIG_TOUCHSCREEN_TOUCHWIN_MODULE 1
#define CONFIG_HZ 300
#define CONFIG_ACPI 1
#define CONFIG_HAMRADIO 1
#define CONFIG_ARCH_USE_QUEUED_SPINLOCKS 1
#define CONFIG_HW_RANDOM_CCTRNG_MODULE 1
#define CONFIG_IWLWIFI_LEDS 1
#define CONFIG_CHECKPOINT_RESTORE 1
#define CONFIG_BLK_DEV_DM 1
#define CONFIG_ICE_HWMON 1
#define CONFIG_NF_CONNTRACK_LABELS 1
#define CONFIG_NETFILTER_NETLINK_OSF_MODULE 1
#define CONFIG_KVM_COMMON 1
#define CONFIG_W1_SLAVE_DS2406_MODULE 1
#define CONFIG_SENSORS_ISL29018_MODULE 1
#define CONFIG_RTW88_PCI_MODULE 1
#define CONFIG_VIRTUALIZATION 1
#define CONFIG_EDAC_LEGACY_SYSFS 1
#define CONFIG_USB_SERIAL_NAVMAN_MODULE 1
#define CONFIG_USB_CONFIGFS_F_PRINTER 1
#define CONFIG_TCS3414_MODULE 1
#define CONFIG_DVB_USB_AZ6027_MODULE 1
#define CONFIG_PINCTRL_SUN50I_H5 1
#define CONFIG_NET_CLS_FLOWER_MODULE 1
#define CONFIG_PLATFORM_MHU 1
#define CONFIG_DVB_MT312_MODULE 1
#define CONFIG_ARM_TEGRA186_CPUFREQ 1
#define CONFIG_TASKSTATS 1
#define CONFIG_TI_TLC4541_MODULE 1
#define CONFIG_TARGET_CORE_MODULE 1
#define CONFIG_USB_CONFIGFS_F_FS 1
#define CONFIG_NFT_FIB_NETDEV_MODULE 1
#define CONFIG_IP6_NF_TARGET_MASQUERADE_MODULE 1
#define CONFIG_CRYPTO_ANUBIS_MODULE 1
#define CONFIG_SND_MIA_MODULE 1
#define CONFIG_VIDEO_IMX258_MODULE 1
#define CONFIG_MDIO_REGMAP_MODULE 1
#define CONFIG_CXL_REGION 1
#define CONFIG_DRM_DW_MIPI_DSI2 1
#define CONFIG_ZONEFS_FS_MODULE 1
#define CONFIG_MT76_CORE_MODULE 1
#define CONFIG_CFG80211_CERTIFICATION_ONUS 1
#define CONFIG_TOUCHSCREEN_MTOUCH_MODULE 1
#define CONFIG_ATH9K_DEBUGFS 1
#define CONFIG_CXL_PORT_MODULE 1
#define CONFIG_UIO_SERCOS3_MODULE 1
#define CONFIG_OPENVSWITCH_MODULE 1
#define CONFIG_COMMON_CLK_CS2000_CP 1
#define CONFIG_SFC_MCDI_LOGGING 1
#define CONFIG_IPV6_SIT_MODULE 1
#define CONFIG_RT2800_LIB_MMIO_MODULE 1
#define CONFIG_CRYPTO_KHAZAD_MODULE 1
#define CONFIG_AD9523_MODULE 1
#define CONFIG_DMABUF_HEAPS 1
#define CONFIG_SND_SOC_RT5645_MODULE 1
#define CONFIG_STE10XP_MODULE 1
#define CONFIG_FS_POSIX_ACL 1
#define CONFIG_DVB_CX24110_MODULE 1
#define CONFIG_BLK_DEV_RAM_COUNT 16
#define CONFIG_INTERVAL_TREE_SPAN_ITER 1
#define CONFIG_MESON_WATCHDOG_MODULE 1
#define CONFIG_NLS_ISO8859_6_MODULE 1
#define CONFIG_USB_STORAGE_USBAT_MODULE 1
#define CONFIG_SENSORS_ADC128D818_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_DSCP_MODULE 1
#define CONFIG_USB_GSPCA_SQ905C_MODULE 1
#define CONFIG_MT7921U_MODULE 1
#define CONFIG_INET_RAW_DIAG 1
#define CONFIG_CRYPTO_DEV_ROCKCHIP_MODULE 1
#define CONFIG_DVB_TDA10086_MODULE 1
#define CONFIG_POWER_RESET_VEXPRESS 1
#define CONFIG_DW_WATCHDOG_MODULE 1
#define CONFIG_VALIDATE_FS_PARSER 1
#define CONFIG_IR_IMON_MODULE 1
#define CONFIG_SND_SOC_TEGRA_WM8903_MODULE 1
#define CONFIG_SLIP_COMPRESSED 1
#define CONFIG_PREEMPTION 1
#define CONFIG_BINARY_PRINTF 1
#define CONFIG_PL330_DMA 1
#define CONFIG_TIGON3_HWMON 1
#define CONFIG_HAVE_ARCH_KCSAN 1
#define CONFIG_HAVE_PERF_EVENTS_NMI 1
#define CONFIG_HAVE_ARCH_KASAN_SW_TAGS 1
#define CONFIG_MFD_CORE 1
#define CONFIG_TRACE_IRQFLAGS_SUPPORT 1
#define CONFIG_MTD_SPI_NAND_MODULE 1
#define CONFIG_IR_MESON_MODULE 1
#define CONFIG_IPVLAN_L3S 1
#define CONFIG_CRYPTO_XXHASH_MODULE 1
#define CONFIG_IR_SHARP_DECODER_MODULE 1
#define CONFIG_TIMERFD 1
#define CONFIG_RTC_HCTOSYS_DEVICE "rtc0"
#define CONFIG_SND_USB_POD_MODULE 1
#define CONFIG_UFS_FS_MODULE 1
#define CONFIG_PSTORE_BLK_BLKDEV ""
#define CONFIG_USB_STORAGE_FREECOM_MODULE 1
#define CONFIG_PINCTRL_BCM4908 1
#define CONFIG_SENSORS_W83627HF_MODULE 1
#define CONFIG_GENERIC_VDSO_TIME_NS 1
#define CONFIG_SND_JACK_INPUT_DEV 1
#define CONFIG_IRQ_MSI_IOMMU 1
#define CONFIG_COMMON_CLK_MESON_PLL 1
#define CONFIG_DRM_CLIENT_SETUP 1
#define CONFIG_NET_VENDOR_MYRI 1
#define CONFIG_NET_VENDOR_META 1
#define CONFIG_IKCONFIG_PROC 1
#define CONFIG_NET_TEAM_MODE_LOADBALANCE_MODULE 1
#define CONFIG_USB_SERIAL_ARK3116_MODULE 1
#define CONFIG_SECURITY_PATH 1
#define CONFIG_LEDS_PCA955X_MODULE 1
#define CONFIG_HID_SUNPLUS_MODULE 1
#define CONFIG_PHY_TEGRA_XUSB_MODULE 1
#define CONFIG_NET_IFE_SKBMARK_MODULE 1
#define CONFIG_USB_OTG 1
#define CONFIG_MCP3422_MODULE 1
#define CONFIG_NF_NAT_SNMP_BASIC_MODULE 1
#define CONFIG_ZPOOL 1
#define CONFIG_ADIS16201_MODULE 1
#define CONFIG_COMMON_CLK_C3_PERIPHERALS 1
#define CONFIG_SND_EMU10K1_SEQ_MODULE 1
#define CONFIG_VMCORE_INFO 1
#define CONFIG_SND_KORG1212_MODULE 1
#define CONFIG_TI_ADS7950_MODULE 1
#define CONFIG_DLM_DEBUG 1
#define CONFIG_GPIO_XGENE 1
#define CONFIG_ARM64_HAFT 1
#define CONFIG_ARM_MHU 1
#define CONFIG_USB_NET_CH9200_MODULE 1
#define CONFIG_USB_SI470X_MODULE 1
#define CONFIG_NFT_FWD_NETDEV_MODULE 1
#define CONFIG_AD5766_MODULE 1
#define CONFIG_FB_CORE 1
#define CONFIG_VIDEO_HANTRO_HEVC_RFC 1
#define CONFIG_PINCTRL_MESON_S4 1
#define CONFIG_DRM_GEM_DMA_HELPER 1
#define CONFIG_SENSORS_LM93_MODULE 1
#define CONFIG_IRQ_DOMAIN 1
#define CONFIG_HID_ELO_MODULE 1
#define CONFIG_ARM_TIMER_SP804 1
#define CONFIG_LDISC_AUTOLOAD 1
#define CONFIG_DYNAMIC_DEBUG_CORE 1
#define CONFIG_XDP_SOCKETS_DIAG_MODULE 1
#define CONFIG_PCI_PRI 1
#define CONFIG_PINCTRL_AMLOGIC_T7 1
#define CONFIG_CRYPTO_AKCIPHER2 1
#define CONFIG_WATCHDOG_HANDLE_BOOT_ENABLED 1
#define CONFIG_VIDEO_SUNXI 1
#define CONFIG_NF_NAT_REDIRECT 1
#define CONFIG_SWIOTLB 1
#define CONFIG_SENSORS_SMSC47B397_MODULE 1
#define CONFIG_SCHED_AUTOGROUP 1
#define CONFIG_IGC_MODULE 1
#define CONFIG_PINCTRL_SUN50I_A100 1
#define CONFIG_IP_VS_SH_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_CPU_MODULE 1
#define CONFIG_HAVE_ARCH_HUGE_VMALLOC 1
#define CONFIG_DEVFREQ_GOV_USERSPACE 1
#define CONFIG_ARCH_SUNXI 1
#define CONFIG_NFT_CONNLIMIT_MODULE 1
#define CONFIG_SCREEN_INFO 1
#define CONFIG_NLS_MAC_CYRILLIC_MODULE 1
#define CONFIG_BT_BCM_MODULE 1
#define CONFIG_CRYPTO_LIB_BLAKE2S_GENERIC 1
#define CONFIG_HARDENED_USERCOPY 1
#define CONFIG_FB_VIRTUAL 1
#define CONFIG_TOUCHSCREEN_EETI_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_MARK_MODULE 1
#define CONFIG_HAS_IOMEM 1
#define CONFIG_HAVE_CMPXCHG_DOUBLE 1
#define CONFIG_PCIEASPM 1
#define CONFIG_CMA_AREAS 7
#define CONFIG_MVMDIO_MODULE 1
#define CONFIG_SECURITY_APPARMOR_PARANOID_LOAD 1
#define CONFIG_NETFILTER_XTABLES 1
#define CONFIG_MT7915E_MODULE 1
#define CONFIG_CM36651_MODULE 1
#define CONFIG_MEDIA_TUNER_MXL5007T_MODULE 1
#define CONFIG_BT_MRVL_SDIO_MODULE 1
#define CONFIG_UIO_AEC_MODULE 1
#define CONFIG_IR_GPIO_TX_MODULE 1
#define CONFIG_LIBIPW_MODULE 1
#define CONFIG_CLKSRC_MMIO 1
#define CONFIG_B43LEGACY_MODULE 1
#define CONFIG_W1_SLAVE_DS2405_MODULE 1
#define CONFIG_KEY_NOTIFICATIONS 1
#define CONFIG_MDIO_BUS_MUX_MMIOREG 1
#define CONFIG_IPV6_MULTIPLE_TABLES 1
#define CONFIG_BRIDGE_EBT_T_FILTER_MODULE 1
#define CONFIG_USB_ADUTUX_MODULE 1
#define CONFIG_ARM_ARCH_TIMER_OOL_WORKAROUND 1
#define CONFIG_BT_HCIBTSDIO_MODULE 1
#define CONFIG_DVB_USB_A800_MODULE 1
#define CONFIG_IR_STREAMZAP_MODULE 1
#define CONFIG_NF_CONNTRACK_PROCFS 1
#define CONFIG_NF_CONNTRACK_H323_MODULE 1
#define CONFIG_MEDIA_TUNER_FC0011_MODULE 1
#define CONFIG_VHOST_NET_MODULE 1
#define CONFIG_SND_SOC_RT5514_SPI_MODULE 1
#define CONFIG_MFD_RK8XX_SPI 1
#define CONFIG_B53_SPI_DRIVER_MODULE 1
#define CONFIG_IP_VS_PROTO_UDP 1
#define CONFIG_USB_CHIPIDEA_UDC 1
#define CONFIG_XZ_DEC_POWERPC 1
#define CONFIG_BPF 1
#define CONFIG_USB_CONFIGFS_F_MIDI 1
#define CONFIG_XZ_DEC_ARM64 1
#define CONFIG_SND_SUN8I_ADDA_PR_REGMAP 1
#define CONFIG_LOCALVERSION "-MANJARO-ARM"
#define CONFIG_IIO_CROS_EC_BARO_MODULE 1
#define CONFIG_USB_CONFIGFS_F_HID 1
#define CONFIG_HID_GT683R_MODULE 1
#define CONFIG_MOUSE_PS2_SYNAPTICS 1
#define CONFIG_BT_INTEL_MODULE 1
#define CONFIG_IOSCHED_BFQ 1
#define CONFIG_LIBETH_MODULE 1
#define CONFIG_ARM64_SVE 1
#define CONFIG_MICROCHIP_PHY_MODULE 1
#define CONFIG_PINCTRL_AMD 1
#define CONFIG_CPUFREQ_DT 1
#define CONFIG_SUN50I_IOMMU 1
#define CONFIG_DVB_RTL2830_MODULE 1
#define CONFIG_MDIO_BUS_MUX_MESON_GXL_MODULE 1
#define CONFIG_POWER_RESET_SYSCON 1
#define CONFIG_USB_STORAGE_DATAFAB_MODULE 1
#define CONFIG_SND_AUDIO_GRAPH_CARD2_MODULE 1
#define CONFIG_DVB_USB_AF9015_MODULE 1
#define CONFIG_PATA_HPT366_MODULE 1
#define CONFIG_HID_TWINHAN_MODULE 1
#define CONFIG_NET_DSA_SJA1105_MODULE 1
#define CONFIG_BCM2711_THERMAL_MODULE 1
#define CONFIG_REGMAP_MMIO 1
#define CONFIG_IOMMU_IO_PGTABLE 1
#define CONFIG_KEXEC 1
#define CONFIG_SND_ALOOP_MODULE 1
#define CONFIG_SND_HRTIMER_MODULE 1
#define CONFIG_NF_NAT_SIP_MODULE 1
#define CONFIG_SFC_SIENA_MCDI_LOGGING 1
#define CONFIG_B53_MDIO_DRIVER_MODULE 1
#define CONFIG_SND_SOC_TEGRA_AUDIO_GRAPH_CARD_MODULE 1
#define CONFIG_RT2X00_LIB_MMIO_MODULE 1
#define CONFIG_NLS_ISO8859_9_MODULE 1
#define CONFIG_IP_NF_TARGET_ECN_MODULE 1
#define CONFIG_CRYPTO_ADIANTUM_MODULE 1
#define CONFIG_DRM_VIRTIO_GPU_KMS 1
#define CONFIG_ARM_SCMI_PROTOCOL 1
#define CONFIG_TEGRA_BPMP_THERMAL_MODULE 1
#define CONFIG_CRYPTO_BLOWFISH_COMMON_MODULE 1
#define CONFIG_CROS_EC_TYPEC_MODULE 1
#define CONFIG_DVB_TTUSB_BUDGET_MODULE 1
#define CONFIG_SLUB 1
#define CONFIG_NET_SCH_PRIO_MODULE 1
#define CONFIG_IFB_MODULE 1
#define CONFIG_GPIO_74X164_MODULE 1
#define CONFIG_RESET_BRCMSTB 1
#define CONFIG_BCM7038_L1_IRQ 1
#define CONFIG_STACKPROTECTOR 1
#define CONFIG_EXT4_FS_POSIX_ACL 1
#define CONFIG_T5403_MODULE 1
#define CONFIG_MOUSE_PS2_ALPS 1
#define CONFIG_IR_MCEUSB_MODULE 1
#define CONFIG_USB_SERIAL_CP210X_MODULE 1
#define CONFIG_SLIP_MODULE 1
#define CONFIG_DVB_USB_DW2102_MODULE 1
#define CONFIG_IP_NF_IPTABLES 1
#define CONFIG_USB_IOWARRIOR_MODULE 1
#define CONFIG_ST_UVIS25_SPI_MODULE 1
#define CONFIG_HID_KENSINGTON_MODULE 1
#define CONFIG_USB_GSPCA_STV0680_MODULE 1
#define CONFIG_B43_PCICORE_AUTOSELECT 1
#define CONFIG_HAVE_REGS_AND_STACK_ACCESS_API 1
#define CONFIG_NVEC_POWER_MODULE 1
#define CONFIG_GENERIC_ADC_THERMAL_MODULE 1
#define CONFIG_USB_GSPCA_SN9C20X_MODULE 1
#define CONFIG_ROCKCHIP_CDN_DP 1
#define CONFIG_DVB_ZL10353_MODULE 1
#define CONFIG_TABLET_USB_ACECAD_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_DSCP_MODULE 1
#define CONFIG_HAVE_SAMPLE_FTRACE_DIRECT 1
#define CONFIG_SND_USB_PODHD_MODULE 1
#define CONFIG_DVB_TDA8083_MODULE 1
#define CONFIG_B53_SERDES_MODULE 1
#define CONFIG_USB_DWC2_PCI 1
#define CONFIG_SUN4I_EMAC_MODULE 1
#define CONFIG_DRM_HDLCD_MODULE 1
#define CONFIG_KGDB_TESTS 1
#define CONFIG_AD9832_MODULE 1
#define CONFIG_FIRMWARE_MEMMAP 1
#define CONFIG_ARCH_DMA_ADDR_T_64BIT 1
#define CONFIG_DVB_STV090x_MODULE 1
#define CONFIG_AIC7XXX_RESET_DELAY_MS 5000
#define CONFIG_VIRTIO_VSOCKETS_COMMON_MODULE 1
#define CONFIG_NFT_COMPAT_ARP_MODULE 1
#define CONFIG_HID_A4TECH_MODULE 1
#define CONFIG_ACPI_CPPC_CPUFREQ_MODULE 1
#define CONFIG_USB_F_FS 1
#define CONFIG_SENSORS_ADT7X10_MODULE 1
#define CONFIG_LEDS_CLASS_FLASH_MODULE 1
#define CONFIG_VIDEO_CX88_BLACKBIRD_MODULE 1
#define CONFIG_EFI_ESRT 1
#define CONFIG_MEDIA_TUNER_TEA5767_MODULE 1
#define CONFIG_WATCH_QUEUE 1
#define CONFIG_ARM_SCMI_TRANSPORT_SMC 1
#define CONFIG_CRYPTO_LIB_ARC4_MODULE 1
#define CONFIG_SENSORS_LM92_MODULE 1
#define CONFIG_TEXTSEARCH_BM_MODULE 1
#define CONFIG_RTC_DRV_DS1553_MODULE 1
#define CONFIG_RTC_DRV_HID_SENSOR_TIME_MODULE 1
#define CONFIG_SND_PROC_FS 1
#define CONFIG_USB_GSPCA_SUNPLUS_MODULE 1
#define CONFIG_HID_SENSOR_INCLINOMETER_3D_MODULE 1
#define CONFIG_BATMAN_ADV_DAT 1
#define CONFIG_UTS_NS 1
#define CONFIG_HAVE_IRQ_TIME_ACCOUNTING 1
#define CONFIG_CRYPTO_CTR 1
#define CONFIG_DVB_M88DS3103_MODULE 1
#define CONFIG_SERIAL_8250_PCILIB 1
#define CONFIG_FB_TFT_ILI9320_MODULE 1
#define CONFIG_WFX_MODULE 1
#define CONFIG_BCM_NS_THERMAL 1
#define CONFIG_DVB_AF9013_MODULE 1
#define CONFIG_CAN_VCAN_MODULE 1
#define CONFIG_POSIX_CPU_TIMERS_TASK_WORK 1
#define CONFIG_SND_INTEL_NHLT 1
#define CONFIG_HID_ALPS_MODULE 1
#define CONFIG_IGB_MODULE 1
#define CONFIG_SCSI_AIC94XX_MODULE 1
#define CONFIG_MWIFIEX_MODULE 1
#define CONFIG_TOUCHSCREEN_USB_ITM 1
#define CONFIG_VHOST_MENU 1
#define CONFIG_GPIO_MAX3191X_MODULE 1
#define CONFIG_LOG_CPU_MAX_BUF_SHIFT 12
#define CONFIG_LEDS_USER_MODULE 1
#define CONFIG_AD5686_SPI_MODULE 1
#define CONFIG_BCM4908_ENET 1
#define CONFIG_PCIE_BRCMSTB 1
#define CONFIG_USB_EHCI_TT_NEWSCHED 1
#define CONFIG_SENSORS_NCT6775_CORE_MODULE 1
#define CONFIG_BMG160_I2C_MODULE 1
#define CONFIG_RMI4_F03 1
#define CONFIG_CRYPTO_CBC 1
#define CONFIG_SND_PCM 1
#define CONFIG_ROCKCHIP_GRF 1
#define CONFIG_SND_SEQ_MIDI_EVENT_MODULE 1
#define CONFIG_GPIO_SYSCON 1
#define CONFIG_6LOWPAN_NHC_MOBILITY_MODULE 1
#define CONFIG_PATA_PLATFORM_MODULE 1
#define CONFIG_NFS_ACL_SUPPORT 1
#define CONFIG_CGROUP_WRITEBACK 1
#define CONFIG_I2C_MUX_PCA9541_MODULE 1
#define CONFIG_NET_DEVLINK 1
#define CONFIG_IIO_ST_LSM6DSX_I2C_MODULE 1
#define CONFIG_OF_RESOLVE 1
#define CONFIG_USB_CATC_MODULE 1
#define CONFIG_ARCH_TEGRA_132_SOC 1
#define CONFIG_RMI4_F12 1
#define CONFIG_HOTPLUG_CORE_SYNC_DEAD 1
#define CONFIG_SPI_ROCKCHIP_SFC 1
#define CONFIG_EFI_GENERIC_STUB 1
#define CONFIG_AD8366_MODULE 1
#define CONFIG_BMG160_MODULE 1
#define CONFIG_MAC80211_RC_DEFAULT_MINSTREL 1
#define CONFIG_NF_CONNTRACK_SNMP_MODULE 1
#define CONFIG_NLS_KOI8_R_MODULE 1
#define CONFIG_SND_SOC_DA7219_MODULE 1
#define CONFIG_TEXTSEARCH_KMP_MODULE 1
#define CONFIG_DVB_EC100_MODULE 1
#define CONFIG_MTD_OF_PARTS_BCM4908 1
#define CONFIG_EXPERT 1
#define CONFIG_REBOOT_MODE 1
#define CONFIG_P54_USB_MODULE 1
#define CONFIG_IIO_ST_SENSORS_I2C_MODULE 1
#define CONFIG_DM_MULTIPATH_ST_MODULE 1
#define CONFIG_NETFILTER_INGRESS 1
#define CONFIG_W1_SLAVE_DS2433_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_CONNMARK_MODULE 1
#define CONFIG_WL1251_SPI_MODULE 1
#define CONFIG_RT2800PCI_RT33XX 1
#define CONFIG_PCIE_TEGRA194_HOST_MODULE 1
#define CONFIG_ARM_PSCI_CPUIDLE_DOMAIN 1
#define CONFIG_IIO_BUFFER_HW_CONSUMER_MODULE 1
#define CONFIG_CRYPTO_SHA512_ARM64 1
#define CONFIG_ARM_SCMI_POWERCAP_MODULE 1
#define CONFIG_FB_TFT_HX8353D_MODULE 1
#define CONFIG_DVB_DS3000_MODULE 1
#define CONFIG_NF_FLOW_TABLE_INET_MODULE 1
#define CONFIG_AM2315_MODULE 1
#define CONFIG_USB_NET_HUAWEI_CDC_NCM_MODULE 1
#define CONFIG_USB_STORAGE_JUMPSHOT_MODULE 1
#define CONFIG_BCMA_DRIVER_PCI 1
#define CONFIG_CAN_ROCKCHIP_CANFD_MODULE 1
#define CONFIG_BT_LEDS 1
#define CONFIG_PHY_TEGRA194_P2U_MODULE 1
#define CONFIG_USB_F_UVC_MODULE 1
#define CONFIG_NLS_ISO8859_8_MODULE 1
#define CONFIG_TOUCHSCREEN_ATMEL_MXT_MODULE 1
#define CONFIG_USB_CHIPIDEA_TEGRA 1
#define CONFIG_UID16 1
#define CONFIG_SND_SOC_ALC5623_MODULE 1
#define CONFIG_TIPC_MEDIA_UDP 1
#define CONFIG_SWAP 1
#define CONFIG_CB710_DEBUG_ASSUMPTIONS 1
#define CONFIG_EPIC100_MODULE 1
#define CONFIG_MAGIC_SYSRQ_SERIAL 1
#define CONFIG_NET_VENDOR_MOTORCOMM 1
#define CONFIG_HAVE_STACKPROTECTOR 1
#define CONFIG_BNX2_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_LOG_MODULE 1
#define CONFIG_CRYPTO_LIB_CURVE25519_INTERNAL_MODULE 1
#define CONFIG_PINCTRL_NS2_MUX 1
#define CONFIG_MT7615E_MODULE 1
#define CONFIG_ACPI_MDIO 1
#define CONFIG_PCIE_ROCKCHIP 1
#define CONFIG_SENSORS_THMC50_MODULE 1
#define CONFIG_SND_SOC_ALC5632_MODULE 1
#define CONFIG_MFD_MAX14577_MODULE 1
#define CONFIG_SND_SEQ_MIDI_EMUL_MODULE 1
#define CONFIG_HAVE_ARCH_JUMP_LABEL 1
#define CONFIG_COMMON_CLK_C3_PLL 1
#define CONFIG_USB_CONFIGFS_ECM 1
#define CONFIG_GPIO_VIPERBOARD_MODULE 1
#define CONFIG_CRYPTO_PCBC_MODULE 1
#define CONFIG_BCM_VIDEOCORE 1
#define CONFIG_PLATFORM_SI4713_MODULE 1
#define CONFIG_NETDEVSIM_MODULE 1
#define CONFIG_BCMA_MODULE 1
#define CONFIG_HID_HOLTEK_MODULE 1
#define CONFIG_NETCONSOLE_DYNAMIC 1
#define CONFIG_NF_CONNTRACK_BROADCAST_MODULE 1
#define CONFIG_MMC_SDHCI_IPROC 1
#define CONFIG_SND_MESON_AXG_TDMIN_MODULE 1
#define CONFIG_ARCH_HAS_NON_OVERLAPPING_ADDRESS_SPACE 1
#define CONFIG_RT2800USB_RT33XX 1
#define CONFIG_ISL29125_MODULE 1
#define CONFIG_MMC_SDHCI_PLTFM 1
#define CONFIG_MT76x0_COMMON_MODULE 1
#define CONFIG_WLCORE_SPI_MODULE 1
#define CONFIG_CXL_MEM_MODULE 1
#define CONFIG_SND_OPL3_LIB_MODULE 1
#define CONFIG_PINCTRL_MCP23S08_I2C_MODULE 1
#define CONFIG_HID_ZYDACRON_MODULE 1
#define CONFIG_PREEMPT_RCU 1
#define CONFIG_USB_NET_ZAURUS_MODULE 1
#define CONFIG_VIDEO_SAA6588_MODULE 1
#define CONFIG_ADIS16203_MODULE 1
#define CONFIG_CC_HAS_BRANCH_PROT_PAC_RET_BTI 1
#define CONFIG_HAVE_SYSCALL_TRACEPOINTS 1
#define CONFIG_IOMMU_IOVA 1
#define CONFIG_XOR_BLOCKS_MODULE 1
#define CONFIG_DNOTIFY 1
#define CONFIG_LEDS_TRIGGER_BACKLIGHT 1
#define CONFIG_BPF_STREAM_PARSER 1
#define CONFIG_MEDIA_SUBDRV_AUTOSELECT 1
#define CONFIG_RDS_MODULE 1
#define CONFIG_NET_DSA_MICROCHIP_KSZ9477_I2C_MODULE 1
#define CONFIG_MACVLAN_MODULE 1
#define CONFIG_TEGRA_MC 1
#define CONFIG_SENSORS_SHT15_MODULE 1
#define CONFIG_ARCH_SUPPORTS_KEXEC 1
#define CONFIG_IR_MESON_TX_MODULE 1
#define CONFIG_IP_NF_RAW_MODULE 1
#define CONFIG_HAVE_HW_BREAKPOINT 1
#define CONFIG_BRIDGE_NF_EBTABLES_LEGACY_MODULE 1
#define CONFIG_AD5764_MODULE 1
#define CONFIG_GPIO_BRCMSTB 1
#define CONFIG_MOUSE_VSXXXAA_MODULE 1
#define CONFIG_VIDEO_TEA6420_MODULE 1
#define CONFIG_ARCH_BCMBCA 1
#define CONFIG_SCSI_MPT3SAS_MAX_SGE 128
#define CONFIG_VIDEO_DT3155_MODULE 1
#define CONFIG_CRYPTO_CTS 1
#define CONFIG_DVB_USB_EC168_MODULE 1
#define CONFIG_RSI_91X_MODULE 1
#define CONFIG_USB_GSPCA_SPCA1528_MODULE 1
#define CONFIG_TOUCHSCREEN_ILI210X_MODULE 1
#define CONFIG_HAVE_NMI 1
#define CONFIG_DVB_RTL2832_SDR_MODULE 1
#define CONFIG_ARM_SP805_WATCHDOG_MODULE 1
#define CONFIG_CC_HAS_INT128 1
#define CONFIG_SENSORS_F75375S_MODULE 1
#define CONFIG_SND_INDIGOIOX_MODULE 1
#define CONFIG_SND_MESON_G12A_TOACODEC_MODULE 1
#define CONFIG_SCSI_NETLINK 1
#define CONFIG_CRYPTO_USER_API_AEAD 1
#define CONFIG_NETCONSOLE_MODULE 1
#define CONFIG_PINCTRL_SUN50I_H6_R 1
#define CONFIG_HAVE_VIRT_CPU_ACCOUNTING_GEN 1
#define CONFIG_SND_HDSPM_MODULE 1
#define CONFIG_GENERIC_CLOCKEVENTS_BROADCAST 1
#define CONFIG_CRYPTO_DEV_SAFEXCEL_MODULE 1
#define CONFIG_DVB_TTUSB_DEC_MODULE 1
#define CONFIG_RTC_NVMEM 1
#define CONFIG_SCSI_HPTIOP_MODULE 1
#define CONFIG_JFS_POSIX_ACL 1
#define CONFIG_VIDEO_AU0828_RC 1
#define CONFIG_P54_SPI_MODULE 1
#define CONFIG_I2C_RK3X 1
#define CONFIG_COMMON_CLK_XGENE 1
#define CONFIG_MEDIA_ALTERA_CI_MODULE 1
#define CONFIG_USB_SERIAL_QT2_MODULE 1
#define CONFIG_SPI_GPIO 1
#define CONFIG_HID_LED_MODULE 1
#define CONFIG_FB_SYS_FILLRECT 1
#define CONFIG_I2C_DIOLAN_U2C_MODULE 1
#define CONFIG_XARRAY_MULTI 1
#define CONFIG_USB_SERIAL_IPW_MODULE 1
#define CONFIG_USBIP_VUDC_MODULE 1
#define CONFIG_RMI4_F11 1
#define CONFIG_ARCH_SUPPORTS_KEXEC_FILE 1
#define CONFIG_DCB 1
#define CONFIG_ZSWAP 1
#define CONFIG_SENSORS_TMP103_MODULE 1
#define CONFIG_PHY_MESON_G12A_MIPI_DPHY_ANALOG 1
#define CONFIG_MT7921_COMMON_MODULE 1
#define CONFIG_PCIE_ROCKCHIP_HOST 1
#define CONFIG_RTLWIFI_DEBUG 1
#define CONFIG_ZSWAP_COMPRESSOR_DEFAULT_LZO 1
#define CONFIG_TI_ADS8344_MODULE 1
#define CONFIG_MEDIA_TUNER_FC0013_MODULE 1
#define CONFIG_SENSORS_ADM1025_MODULE 1
#define CONFIG_CRYPTO_TWOFISH_MODULE 1
#define CONFIG_HAVE_ARCH_PREL32_RELOCATIONS 1
#define CONFIG_PACKING 1
#define CONFIG_IPV6_MIP6 1
#define CONFIG_SND_FM801_TEA575X_BOOL 1
#define CONFIG_W1_SLAVE_DS2430_MODULE 1
#define CONFIG_BT_HCIUART_QCA 1
#define CONFIG_MTD_SWAP 1
#define CONFIG_CRYPTO_RNG_DEFAULT 1
#define CONFIG_STREAM_PARSER 1
#define CONFIG_KVM_GENERIC_MMU_NOTIFIER 1
#define CONFIG_UEFI_CPER 1
#define CONFIG_IP_VS_SED_MODULE 1
#define CONFIG_LIBFCOE_MODULE 1
#define CONFIG_XZ_DEC_ARM 1
#define CONFIG_DVB_RTL2832_MODULE 1
#define CONFIG_NET_DSA_MT7530_MODULE 1
#define CONFIG_NET_DSA_MODULE 1
#define CONFIG_HPFS_FS_MODULE 1
#define CONFIG_SENSORS_INA2XX_MODULE 1
#define CONFIG_MTD_UBI_BEB_LIMIT 20
#define CONFIG_HOTPLUG_PCI 1
#define CONFIG_USB_VIDEO_CLASS_MODULE 1
#define CONFIG_F2FS_FS_LZO 1
#define CONFIG_SENSORS_AD7418_MODULE 1
#define CONFIG_NFC_NCI_MODULE 1
#define CONFIG_SND_SOC_RT5640_MODULE 1
#define CONFIG_AXP20X_ADC_MODULE 1
#define CONFIG_IEEE802154_MODULE 1
#define CONFIG_RTC_DRV_DS1374_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_IDLETIMER_MODULE 1
#define CONFIG_ZSMALLOC_STAT 1
#define CONFIG_CIFS_POSIX 1
#define CONFIG_ARM_CCI_PMU 1
#define CONFIG_HID_COUGAR_MODULE 1
#define CONFIG_ROCKCHIP_IODOMAIN 1
#define CONFIG_MMC_HSQ 1
#define CONFIG_SND_SOC_MAX98090_MODULE 1
#define CONFIG_PWM 1
#define CONFIG_AD7780_MODULE 1
#define CONFIG_CONFIGFS_FS 1
#define CONFIG_PINCTRL_BCM2835 1
#define CONFIG_DEFAULT_INIT ""
#define CONFIG_RT2500PCI_MODULE 1
#define CONFIG_SND_SPI 1
#define CONFIG_KXCJK1013_MODULE 1
#define CONFIG_DMABUF_HEAPS_CMA 1
#define CONFIG_COMMON_CLK_VC3 1
#define CONFIG_ATA_ACPI 1
#define CONFIG_RTW88_8723DU_MODULE 1
#define CONFIG_NET_CLS 1
#define CONFIG_NET_VENDOR_AGERE 1
#define CONFIG_RAID6_PQ_BENCHMARK 1
#define CONFIG_W1_CON 1
#define CONFIG_HID_ACRUX_FF 1
#define CONFIG_OF_RESERVED_MEM 1
#define CONFIG_NFT_REJECT_IPV4_MODULE 1
#define CONFIG_IP6_NF_MATCH_OPTS_MODULE 1
#define CONFIG_MV_XOR_V2 1
#define CONFIG_SERIAL_AMBA_PL010_CONSOLE 1
#define CONFIG_TOUCHSCREEN_USB_EGALAX 1
#define CONFIG_SATA_MV_MODULE 1
#define CONFIG_SCHEDSTATS 1
#define CONFIG_ALTERA_STAPL_MODULE 1
#define CONFIG_VIA_VELOCITY_MODULE 1
#define CONFIG_REALTEK_PHY 1
#define CONFIG_CRYPTO_KPP2 1
#define CONFIG_NET_ACT_SKBEDIT_MODULE 1
#define CONFIG_DRM_DISPLAY_DP_AUX_CEC 1
#define CONFIG_ARCH_SUPPORTS_KEXEC_SIG 1
#define CONFIG_MOUSE_GPIO_MODULE 1
#define CONFIG_VHOST_VSOCK_MODULE 1
#define CONFIG_BLK_DEV_LOOP_MIN_COUNT 0
#define CONFIG_DRM_DISPLAY_CONNECTOR 1
#define CONFIG_B43LEGACY_HWRNG 1
#define CONFIG_NFT_QUEUE_MODULE 1
#define CONFIG_CRYPTO_DRBG_HASH 1
#define CONFIG_MCP4922_MODULE 1
#define CONFIG_HAVE_ARM_SMCCC_DISCOVERY 1
#define CONFIG_DCACHE_WORD_ACCESS 1
#define CONFIG_ARCH_SUPPORTS_CFI_CLANG 1
#define CONFIG_RANDOMIZE_KSTACK_OFFSET_DEFAULT 1
#define CONFIG_SND_SUN4I_SPDIF 1
#define CONFIG_CDROM 1
#define CONFIG_RCU_TORTURE_TEST_MODULE 1
#define CONFIG_P54_COMMON_MODULE 1
#define CONFIG_SUN6I_RTC_CCU 1
#define CONFIG_USB_EHCI_BRCMSTB 1
#define CONFIG_SFC_SIENA_MTD 1
#define CONFIG_MEDIA_TUNER_TEA5761_MODULE 1
#define CONFIG_SENSORS_SCH56XX_COMMON_MODULE 1
#define CONFIG_USB_GSPCA_VC032X_MODULE 1
#define CONFIG_MMC_DW 1
#define CONFIG_AS_HAS_ARMV8_5 1
#define CONFIG_RUSTC_LLVM_VERSION 0
#define CONFIG_EXTCON 1
#define CONFIG_BAYCOM_SER_FDX_MODULE 1
#define CONFIG_IIO_INTERRUPT_TRIGGER_MODULE 1
#define CONFIG_INPUT_RK805_PWRKEY 1
#define CONFIG_GPIO_TEGRA186 1
#define CONFIG_MT7663_USB_SDIO_COMMON_MODULE 1
#define CONFIG_ARCH_HAS_PTDUMP 1
#define CONFIG_DRM_I2C_NXP_TDA998X_MODULE 1
#define CONFIG_USB_GSPCA_STK014_MODULE 1
#define CONFIG_DMA_ACPI 1
#define CONFIG_INPUT_KEYBOARD 1
#define CONFIG_GENERIC_IDLE_POLL_SETUP 1
#define CONFIG_TRUSTED_KEYS_MODULE 1
#define CONFIG_ATH9K_COMMON_MODULE 1
#define CONFIG_MQ_IOSCHED_KYBER 1
#define CONFIG_SND_SEQ_DUMMY_MODULE 1
#define CONFIG_XZ_DEC_RISCV 1
#define CONFIG_ARM64_ERRATUM_832075 1
#define CONFIG_SND_MPU401_UART_MODULE 1
#define CONFIG_DRAGONRISE_FF 1
#define CONFIG_REGULATOR_VCTRL 1
#define CONFIG_THREAD_INFO_IN_TASK 1
#define CONFIG_SYSFS 1
#define CONFIG_RD_ZSTD 1
#define CONFIG_IRQ_TIME_ACCOUNTING 1
#define CONFIG_USB_MR800_MODULE 1
#define CONFIG_HAVE_ARCH_THREAD_STRUCT_WHITELIST 1
#define CONFIG_MMC_BCM2835 1
#define CONFIG_BRCMFMAC_PROTO_BCDC 1
#define CONFIG_MFD_SM501_GPIO 1
#define CONFIG_NET_CLS_FLOW_MODULE 1
#define CONFIG_TOUCHSCREEN_ST1232_MODULE 1
#define CONFIG_DM_THIN_PROVISIONING_MODULE 1
#define CONFIG_ASYNC_TX_ENABLE_CHANNEL_SWITCH 1
#define CONFIG_TCP_CONG_HYBLA_MODULE 1
#define CONFIG_SND_UMP_MODULE 1
#define CONFIG_VIDEO_SUN6I_CSI_MODULE 1
#define CONFIG_FB_TFT_UPD161704_MODULE 1
#define CONFIG_MEDIA_TUNER_XC5000_MODULE 1
#define CONFIG_HAVE_RSEQ 1
#define CONFIG_ATLAS_PH_SENSOR_MODULE 1
#define CONFIG_ACPI_CCA_REQUIRED 1
#define CONFIG_HID_AUREAL_MODULE 1
#define CONFIG_DVB_CORE_MODULE 1
#define CONFIG_IP_NF_TARGET_REJECT_MODULE 1
#define CONFIG_REGMAP_IRQ 1
#define CONFIG_NET_EMATCH_TEXT_MODULE 1
#define CONFIG_HAVE_KCSAN_COMPILER 1
#define CONFIG_EFI_VARS_PSTORE_MODULE 1
#define CONFIG_SND_ENS1370_MODULE 1
#define CONFIG_SFC_MTD 1
#define CONFIG_IPVLAN_MODULE 1
#define CONFIG_SERIO_RAW_MODULE 1
#define CONFIG_STRICT_DEVMEM 1
#define CONFIG_GCC10_NO_ARRAY_BOUNDS 1
#define CONFIG_ROCKCHIP_RGB 1
#define CONFIG_RTC_DRV_DS1672_MODULE 1
#define CONFIG_SENSORS_TMP102_MODULE 1
#define CONFIG_RTW88_8821C_MODULE 1
#define CONFIG_SND_INDIGOIO_MODULE 1
#define CONFIG_MEDIA_TUNER_FC0012_MODULE 1
#define CONFIG_SENSORS_ADM1026_MODULE 1
#define CONFIG_EEPROM_AT25_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_STATE_MODULE 1
#define CONFIG_DRM_DRAW 1
#define CONFIG_SND_SOC_ES8328_MODULE 1
#define CONFIG_SENSORS_NCT6775_I2C_MODULE 1
#define CONFIG_PINCTRL_TEGRA_XUSB 1
#define CONFIG_MMC35240_MODULE 1
#define CONFIG_SPI_DYNAMIC 1
#define CONFIG_PPP_DEFLATE_MODULE 1
#define CONFIG_W1_SLAVE_DS2431_MODULE 1
#define CONFIG_CPU_FREQ_GOV_ONDEMAND 1
#define CONFIG_SENSORS_MAX8688_MODULE 1
#define CONFIG_MMC_DW_ROCKCHIP 1
#define CONFIG_VT_HW_CONSOLE_BINDING 1
#define CONFIG_I2C_ALGOBIT_MODULE 1
#define CONFIG_PHY_BCM_SR_PCIE 1
#define CONFIG_I2C_HID_ACPI_MODULE 1
#define CONFIG_CRYPTO_CAST_COMMON_MODULE 1
#define CONFIG_NF_NAT_MASQUERADE 1
#define CONFIG_GENERIC_CLOCKEVENTS 1
#define CONFIG_EXT2_FS 1
#define CONFIG_NF_CT_NETLINK_HELPER_MODULE 1
#define CONFIG_CMA_DEBUGFS 1
#define CONFIG_MOUSE_SYNAPTICS_I2C_MODULE 1
#define CONFIG_NLS_CODEPAGE_852_MODULE 1
#define CONFIG_USB_SERIAL_WHITEHEAT_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_ECN_MODULE 1
#define CONFIG_BMC150_MAGN_I2C_MODULE 1
#define CONFIG_BT_HCIUART_MODULE 1
#define CONFIG_IPMI_DEVICE_INTERFACE_MODULE 1
#define CONFIG_COMMON_CLK_RK808 1
#define CONFIG_DVB_USB_AF9005_MODULE 1
#define CONFIG_VIDEO_USBTV_MODULE 1
#define CONFIG_SND_HDSP_MODULE 1
#define CONFIG_RTC_CLASS 1
#define CONFIG_INPUT_PWM_BEEPER_MODULE 1
#define CONFIG_ARM64_4K_PAGES 1
#define CONFIG_CRYPTO_LZO 1
#define CONFIG_ROCKCHIP_ANALOGIX_DP 1
#define CONFIG_NLS_CODEPAGE_869_MODULE 1
#define CONFIG_CPUFREQ_DT_PLATDEV 1
#define CONFIG_FWNODE_MDIO 1
#define CONFIG_BATTERY_AXP20X_MODULE 1
#define CONFIG_PERF_EVENTS 1
#define CONFIG_IP6_NF_MATCH_AH_MODULE 1
#define CONFIG_HAVE_HARDLOCKUP_DETECTOR_PERF 1
#define CONFIG_P54_SPI_DEFAULT_EEPROM 1
#define CONFIG_SENSORS_ADT7470_MODULE 1
#define CONFIG_NET_ACT_VLAN_MODULE 1
#define CONFIG_BGMAC 1
#define CONFIG_NETFILTER_XT_TARGET_LED_MODULE 1
#define CONFIG_SERIAL_8250_NR_UARTS 32
#define CONFIG_NO_HZ_IDLE 1
#define CONFIG_GP2AP020A00F_MODULE 1
#define CONFIG_ADJD_S311_MODULE 1
#define CONFIG_CRC8_MODULE 1
#define CONFIG_PCIEPORTBUS 1
#define CONFIG_USB_STORAGE 1
#define CONFIG_DVB_USB_LME2510_MODULE 1
#define CONFIG_SENSORS_TPS53679_MODULE 1
#define CONFIG_PATA_IT821X_MODULE 1
#define CONFIG_IOMMU_API 1
#define CONFIG_ZISOFS 1
#define CONFIG_MAX5481_MODULE 1
#define CONFIG_CRYPTO_RSA 1
#define CONFIG_HID_LOGITECH_HIDPP_MODULE 1
#define CONFIG_RVU_ESWITCH_MODULE 1
#define CONFIG_SERIAL_8250_MANY_PORTS 1
#define CONFIG_IR_JVC_DECODER_MODULE 1
#define CONFIG_SCSI_AIC79XX_MODULE 1
#define CONFIG_NET_IPGRE_BROADCAST 1
#define CONFIG_SENSORS_SHTC1_MODULE 1
#define CONFIG_LOCKD_V4 1
#define CONFIG_DM_DUST_MODULE 1
#define CONFIG_CIFS_DFS_UPCALL 1
#define CONFIG_TYPEC_TCPCI 1
#define CONFIG_TI_ADC12138_MODULE 1
#define CONFIG_RELR 1
#define CONFIG_AQUANTIA_PHY_MODULE 1
#define CONFIG_WATCHDOG_CORE 1
#define CONFIG_IIO_ST_ACCEL_3AXIS_MODULE 1
#define CONFIG_AD5421_MODULE 1
#define CONFIG_HAVE_ARCH_KASAN_HW_TAGS 1
#define CONFIG_LTC2485_MODULE 1
#define CONFIG_SND_HDA_CODEC_HDMI_MODULE 1
#define CONFIG_NVMEM_REBOOT_MODE 1
#define CONFIG_SCSI_DH_EMC_MODULE 1
#define CONFIG_ARCH_HAS_SYNC_DMA_FOR_CPU 1
#define CONFIG_REGULATOR_VIRTUAL_CONSUMER 1
#define CONFIG_CROS_USBPD_NOTIFY_MODULE 1
#define CONFIG_BQL 1
#define CONFIG_IO_URING 1
#define CONFIG_HID_SENSOR_ACCEL_3D_MODULE 1
#define CONFIG_IIO_ADIS_LIB_BUFFER 1
#define CONFIG_ASYMMETRIC_KEY_TYPE 1
#define CONFIG_RTC_DRV_PCF8523_MODULE 1
#define CONFIG_SENSORS_MAX20751_MODULE 1
#define CONFIG_INPUT_MOUSE 1
#define CONFIG_PHY_SUN4I_USB 1
#define CONFIG_RTC_DRV_X1205_MODULE 1
#define CONFIG_MMC_USHC_MODULE 1
#define CONFIG_SATA_SX4_MODULE 1
#define CONFIG_KVM_GENERIC_DIRTYLOG_READ_PROTECT 1
#define CONFIG_DECOMPRESS_XZ 1
#define CONFIG_SND_HDA_CODEC_SIGMATEL_MODULE 1
#define CONFIG_LEDS_PWM 1
#define CONFIG_VIDEO_SAA7134_MODULE 1
#define CONFIG_USB_LEGOTOWER_MODULE 1
#define CONFIG_RTC_DRV_MAX6900_MODULE 1
#define CONFIG_DRM_VC4 1
#define CONFIG_MTD_BLOCK_MODULE 1
#define CONFIG_SND_RME9652_MODULE 1
#define CONFIG_NET_DSA_BCM_SF2_MODULE 1
#define CONFIG_TIPC_CRYPTO 1
#define CONFIG_ARM_SCPI_POWER_DOMAIN 1
#define CONFIG_NLS_CODEPAGE_737_MODULE 1
#define CONFIG_SENSORS_SMSC47M192_MODULE 1
#define CONFIG_MODULES_USE_ELF_RELA 1
#define CONFIG_NFS_V4_2 1
#define CONFIG_DW_AXI_DMAC 1
#define CONFIG_CPU_FREQ_DEFAULT_GOV_SCHEDUTIL 1
#define CONFIG_RTC_DRV_RV3029C2_MODULE 1
#define CONFIG_RT2400PCI_MODULE 1
#define CONFIG_CLOSURES 1
#define CONFIG_MFD_AXP20X_I2C 1
#define CONFIG_ARCH_TEGRA 1
#define CONFIG_BOOT_CONFIG 1
#define CONFIG_MUX_ADG792A_MODULE 1
#define CONFIG_HID_KYE_MODULE 1
#define CONFIG_USB_XHCI_HCD 1
#define CONFIG_ETHERNET 1
#define CONFIG_GPIO_MAX732X_IRQ 1
#define CONFIG_NFSD_V3_ACL 1
#define CONFIG_NETFILTER_CONNCOUNT_MODULE 1
#define CONFIG_MTD_CFI_I2 1
#define CONFIG_VIDEO_MESON_VDEC_MODULE 1
#define CONFIG_AD7266_MODULE 1
#define CONFIG_SND_ENS1371_MODULE 1
#define CONFIG_CEC_TEGRA_MODULE 1
#define CONFIG_ARM64_VA_BITS 48
#define CONFIG_TREE_SRCU 1
#define CONFIG_NETFS_STATS 1
#define CONFIG_RADIO_SI470X_MODULE 1
#define CONFIG_STK8BA50_MODULE 1
#define CONFIG_NTFS_FS_MODULE 1
#define CONFIG_IWLWIFI_MODULE 1
#define CONFIG_BT_MTK_MODULE 1
#define CONFIG_NF_CONNTRACK_AMANDA_MODULE 1
#define CONFIG_IRQ_FASTEOI_HIERARCHY_HANDLERS 1
#define CONFIG_EEPROM_AT24_MODULE 1
#define CONFIG_BT_HIDP_MODULE 1
#define CONFIG_SCSI_UFSHCD_PLATFORM 1
#define CONFIG_ACPI_PROCESSOR 1
#define CONFIG_POWER_RESET_SYSCON_POWEROFF 1
#define CONFIG_ARCH_HAS_STRICT_KERNEL_RWX 1
#define CONFIG_SUNRPC_BACKCHANNEL 1
#define CONFIG_I2C_SLAVE 1
#define CONFIG_RTL8192SE_MODULE 1
#define CONFIG_ARM64_CONT_PMD_SHIFT 4
#define CONFIG_WL1251_MODULE 1
#define CONFIG_ARMV8_DEPRECATED 1
#define CONFIG_NF_CONNTRACK_TIMEOUT 1
#define CONFIG_SUN50I_H6_CCU 1
#define CONFIG_HW_RANDOM_BCM2835 1
#define CONFIG_NET_ACT_POLICE_MODULE 1
#define CONFIG_HID_UCLOGIC_MODULE 1
#define CONFIG_BLOCK_HOLDER_DEPRECATED 1
#define CONFIG_HID_WALTOP_MODULE 1
#define CONFIG_PREEMPT 1
#define CONFIG_USB_STORAGE_ISD200_MODULE 1
#define CONFIG_DRM_MIPI_DBI_MODULE 1
#define CONFIG_USB_EHCI_TEGRA 1
#define CONFIG_VIDEO_SUN8I_ROTATE_MODULE 1
#define CONFIG_REALTEK_AUTOPM 1
#define CONFIG_MCP3911_MODULE 1
#define CONFIG_VIDEO_IR_I2C_MODULE 1
#define CONFIG_CC_HAVE_STACKPROTECTOR_SYSREG 1
#define CONFIG_LEDS_TRIGGER_GPIO 1
#define CONFIG_LV0104CS_MODULE 1
#define CONFIG_NFT_REJECT_MODULE 1
#define CONFIG_SND_RME32_MODULE 1
#define CONFIG_VIDEO_IVTV_ALSA_MODULE 1
#define CONFIG_8139CP_MODULE 1
#define CONFIG_NET_NS 1
#define CONFIG_NET_TEAM_MODE_BROADCAST_MODULE 1
#define CONFIG_CRC16 1
#define CONFIG_SOCK_CGROUP_DATA 1
#define CONFIG_SCHED_CLUSTER 1
#define CONFIG_TABLET_USB_PEGASUS_MODULE 1
#define CONFIG_ISL29003_MODULE 1
#define CONFIG_AD7606_IFACE_PARALLEL_MODULE 1
#define CONFIG_USB_AUTOSUSPEND_DELAY 2
#define CONFIG_CRYPTO_SEQIV 1
#define CONFIG_DP83640_PHY_MODULE 1
#define CONFIG_ARCH_MMAP_RND_COMPAT_BITS_MAX 16
#define CONFIG_NLS_CODEPAGE_866_MODULE 1
#define CONFIG_BRCMSMAC_LEDS 1
#define CONFIG_IPMI_HANDLER_MODULE 1
#define CONFIG_MUTEX_SPIN_ON_OWNER 1
#define CONFIG_ACPI_APEI_GHES 1
#define CONFIG_FSNOTIFY 1
#define CONFIG_ARM_TEGRA_DEVFREQ_MODULE 1
#define CONFIG_NET_FLOW_LIMIT 1
#define CONFIG_NET_DSA_TAG_KSZ_MODULE 1
#define CONFIG_OF_OVERLAY 1
#define CONFIG_NFT_REJECT_IPV6_MODULE 1
#define CONFIG_ARCH_BCM 1
#define CONFIG_CIFS_MODULE 1
#define CONFIG_DW_EDMA_MODULE 1
#define CONFIG_RTC_DRV_MESON_VRTC_MODULE 1
#define CONFIG_OCFS2_FS_USERSPACE_CLUSTER_MODULE 1
#define CONFIG_DRM_ANALOGIX_ANX78XX_MODULE 1
#define CONFIG_NET_DSA_MICROCHIP_KSZ_COMMON_MODULE 1
#define CONFIG_TEGRA_HSP_MBOX 1
#define CONFIG_ALX_MODULE 1
#define CONFIG_INPUT_TOUCHSCREEN 1
#define CONFIG_BFQ_GROUP_IOSCHED 1
#define CONFIG_PCI_TEGRA 1
#define CONFIG_CICADA_PHY_MODULE 1
#define CONFIG_VIRTIO 1
#define CONFIG_DWMAC_SUN55I_MODULE 1
#define CONFIG_IP_SET_HASH_NETIFACE_MODULE 1
#define CONFIG_I2C_SI470X_MODULE 1
#define CONFIG_SND_INTEL_DSP_CONFIG_MODULE 1
#define CONFIG_CRYPTO_DEV_CCREE_MODULE 1
#define CONFIG_VIDEO_UPD64031A_MODULE 1
#define CONFIG_NET_DSA_TAG_BRCM_LEGACY_MODULE 1
#define CONFIG_SND_SOC_ROCKCHIP_MAX98090_MODULE 1
#define CONFIG_CLS_U32_PERF 1
#define CONFIG_ACPI_GTDT 1
#define CONFIG_MMC_BLOCK_MINORS 32
#define CONFIG_CRYPTO_USER_API_HASH 1
#define CONFIG_PARTITION_ADVANCED 1
#define CONFIG_NFT_FIB_INET_MODULE 1
#define CONFIG_DVB_MT352_MODULE 1
#define CONFIG_PWM_SUN4I 1
#define CONFIG_ALTERA_TSE_MODULE 1
#define CONFIG_SENSORS_LTC4151_MODULE 1
#define CONFIG_ZRAM_DEF_COMP "lzo-rle"
#define CONFIG_FB_TFT_ST7735R_MODULE 1
#define CONFIG_SUN6I_MSGBOX 1
#define CONFIG_ATH5K_PCI 1
#define CONFIG_TCG_ATMEL_MODULE 1
#define CONFIG_DEBUG_LIST 1
#define CONFIG_MDIO_BUS_MUX_BCM_IPROC 1
#define CONFIG_NFSD_MODULE 1
#define CONFIG_PM_GENERIC_DOMAINS_SLEEP 1
#define CONFIG_HID_CHICONY_MODULE 1
#define CONFIG_USB_MA901_MODULE 1
#define CONFIG_CRYPTO_SHA512_ARM64_CE 1
#define CONFIG_SENSORS_MAX197_MODULE 1
#define CONFIG_PCI_ECAM 1
#define CONFIG_SYSTEM_TRUSTED_KEYRING 1
#define CONFIG_CRYPTO_ZSTD 1
#define CONFIG_ARCH_MMAP_RND_BITS_MAX 33
#define CONFIG_BMP280_I2C_MODULE 1
#define CONFIG_VIDEO_TUNER_MODULE 1
#define CONFIG_LTC2497_MODULE 1
#define CONFIG_NET_EMATCH_IPSET_MODULE 1
#define CONFIG_ACPI_NHLT 1
#define CONFIG_IP_VS_MH_MODULE 1
#define CONFIG_MEDIA_TUNER_XC2028_MODULE 1
#define CONFIG_BATMAN_ADV_BLA 1
#define CONFIG_AD7192_MODULE 1
#define CONFIG_CGROUP_DEVICE 1
#define CONFIG_USB_NET_CDC_SUBSET_MODULE 1
#define CONFIG_SENSORS_NTC_THERMISTOR_MODULE 1
#define CONFIG_VIDEO_MESON_GE2D_MODULE 1
#define CONFIG_SCSI_FC_ATTRS_MODULE 1
#define CONFIG_FB_TFT_SEPS525_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_QUOTA_MODULE 1
#define CONFIG_INPUT_CM109_MODULE 1
#define CONFIG_VIDEO_SAA7134_DVB_MODULE 1
#define CONFIG_DVB_USB_OPERA1_MODULE 1
#define CONFIG_ARCH_TEGRA_210_SOC 1
#define CONFIG_RTC_DRV_PCF8583_MODULE 1
#define CONFIG_PCIEAER_INJECT_MODULE 1
#define CONFIG_MEDIA_TUNER_MODULE 1
#define CONFIG_SYNC_FILE 1
#define CONFIG_BPF_SYSCALL 1
#define CONFIG_CRYPTO_SM3_ARM64_CE 1
#define CONFIG_CAN_SOFTING_MODULE 1
#define CONFIG_DEVFREQ_GOV_PASSIVE 1
#define CONFIG_ARM_SCMI_POWER_DOMAIN 1
#define CONFIG_VIDEO_GO7007_MODULE 1
#define CONFIG_SND_PCI 1
#define CONFIG_VZ89X_MODULE 1
#define CONFIG_USB_NET_INT51X1_MODULE 1
#define CONFIG_USB_EHCI_HCD_PLATFORM 1
#define CONFIG_HID_PENMOUNT_MODULE 1
#define CONFIG_IP_SET_LIST_SET_MODULE 1
#define CONFIG_LIBFC_MODULE 1
#define CONFIG_RTC_DRV_RS5C348_MODULE 1
#define CONFIG_ICPLUS_PHY 1
#define CONFIG_BRIDGE_EBT_ARPREPLY_MODULE 1
#define CONFIG_NFS_V4_2_SSC_HELPER 1
#define CONFIG_IIO_KFIFO_BUF 1
#define CONFIG_MTD_CFI_I1 1
#define CONFIG_LEDS_LP3944_MODULE 1
#define CONFIG_VIRTIO_PCI 1
#define CONFIG_RD_LZMA 1
#define CONFIG_ADIS16480_MODULE 1
#define CONFIG_DA311_MODULE 1
#define CONFIG_W1_SLAVE_DS2408_MODULE 1
#define CONFIG_VIRTIO_PCI_LEGACY 1
#define CONFIG_NTFS3_LZX_XPRESS 1
#define CONFIG_RTC_DRV_DS1685_FAMILY_MODULE 1
#define CONFIG_RUSTC_SUPPORTS_ARM64 1
#define CONFIG_MEDIA_TUNER_MXL301RF_MODULE 1
#define CONFIG_ARCH_SUPPORTS_UPROBES 1
#define CONFIG_MEMORY_ISOLATION 1
#define CONFIG_IWLEGACY_DEBUGFS 1
#define CONFIG_IP_MROUTE 1
#define CONFIG_ARCH_HAS_DEBUG_VM_PGTABLE 1
#define CONFIG_VIDEO_CS3308_MODULE 1
#define CONFIG_FB_SYS_IMAGEBLIT 1
#define CONFIG_NET_DSA_TAG_NONE_MODULE 1
#define CONFIG_MMA7455_MODULE 1
#define CONFIG_ARCH_SUPPORTS_ACPI 1
#define CONFIG_RTL8XXXU_MODULE 1
#define CONFIG_ROCKCHIP_TIMER 1
#define CONFIG_USB_NET_KALMIA_MODULE 1
#define CONFIG_USB_SERIAL_BELKIN_MODULE 1
#define CONFIG_DRM_ANALOGIX_ANX7625_MODULE 1
#define CONFIG_CRYPTO_JITTERENTROPY_OSR 1
#define CONFIG_HAVE_KRETPROBES 1
#define CONFIG_TOUCHSCREEN_USB_GOTOP 1
#define CONFIG_SPI_SPIDEV_MODULE 1
#define CONFIG_USB_GSPCA_PAC207_MODULE 1
#define CONFIG_SI1145_MODULE 1
#define CONFIG_NLS_CODEPAGE_850_MODULE 1
#define CONFIG_IIO_RESCALE_MODULE 1
#define CONFIG_AD7949_MODULE 1
#define CONFIG_IR_SUNXI_MODULE 1
#define CONFIG_OPENVSWITCH_GRE_MODULE 1
#define CONFIG_NLS_MAC_INUIT_MODULE 1
#define CONFIG_NF_CONNTRACK_BRIDGE_MODULE 1
#define CONFIG_SND_MESON_AXG_SPDIFIN_MODULE 1
#define CONFIG_HID_SENSOR_DEVICE_ROTATION_MODULE 1
#define CONFIG_KEYS 1
#define CONFIG_VIDEO_TVAUDIO_MODULE 1
#define CONFIG_QNX4FS_FS_MODULE 1
#define CONFIG_INPUT_YEALINK_MODULE 1
#define CONFIG_MT7921S_MODULE 1
#define CONFIG_RTC_DRV_ISL1208_MODULE 1
#define CONFIG_DVB_OR51132_MODULE 1
#define CONFIG_INA2XX_ADC_MODULE 1
#define CONFIG_I2C_HID_OF_GOODIX_MODULE 1
#define CONFIG_MMC_DW_K3 1
#define CONFIG_MFD_VIPERBOARD_MODULE 1
#define CONFIG_IIO_ST_MAGN_I2C_3AXIS_MODULE 1
#define CONFIG_HID_GREENASIA_MODULE 1
#define CONFIG_CW1200_WLAN_SDIO_MODULE 1
#define CONFIG_TOUCHSCREEN_MMS114_MODULE 1
#define CONFIG_HID_SENSOR_GYRO_3D_MODULE 1
#define CONFIG_IPMI_POWEROFF_MODULE 1
#define CONFIG_RTC_DRV_ISL12022_MODULE 1
#define CONFIG_ARM64_CNP 1
#define CONFIG_NET_SWITCHDEV 1
#define CONFIG_ARM_PSCI_CPUIDLE 1
#define CONFIG_ADXL372_I2C_MODULE 1
#define CONFIG_SCSI_SYM53C8XX_2_MODULE 1
#define CONFIG_PNFS_BLOCK 1
#define CONFIG_V4L2_ASYNC_MODULE 1
#define CONFIG_AUDIT_GENERIC 1
#define CONFIG_FB_TFT_SSD1306_MODULE 1
#define CONFIG_PSAMPLE_MODULE 1
#define CONFIG_RTW88_8822BS_MODULE 1
#define CONFIG_I2C_SLAVE_EEPROM_MODULE 1
#define CONFIG_ACPI_DOCK 1
#define CONFIG_FB_TFT_ILI9481_MODULE 1
#define CONFIG_IO_WQ 1
#define CONFIG_COMMON_CLK_ROCKCHIP 1
#define CONFIG_VHOST_MODULE 1
#define CONFIG_USB_MASS_STORAGE_MODULE 1
#define CONFIG_RADIO_SI4713_MODULE 1
#define CONFIG_VIDEO_HANTRO_MODULE 1
#define CONFIG_HAVE_ARCH_HUGE_VMAP 1
#define CONFIG_VDSO_GETRANDOM 1
#define CONFIG_RTLBTCOEXIST_MODULE 1
#define CONFIG_NET_DSA_MV88E6XXX_MODULE 1
#define CONFIG_VIA_RHINE_MMIO 1
#define CONFIG_CLK_BCM2835 1
#define CONFIG_ADM8211_MODULE 1
#define CONFIG_AS_VERSION 24400
#define CONFIG_CRYPTO_DEV_CCP 1
#define CONFIG_RTW89_8852BE_MODULE 1
#define CONFIG_BLK_CGROUP_PUNT_BIO 1
#define CONFIG_REGULATOR_VEXPRESS 1
#define CONFIG_INET_UDP_DIAG 1
#define CONFIG_USB_STORAGE_ALAUDA_MODULE 1
#define CONFIG_NET_SCH_HTB_MODULE 1
#define CONFIG_IP_VS_FO_MODULE 1
#define CONFIG_USB_NET_DRIVERS 1
#define CONFIG_ARCH_USE_QUEUED_RWLOCKS 1
#define CONFIG_SCSI_SAS_LIBSAS_MODULE 1
#define CONFIG_ROCKCHIP_MBOX 1
#define CONFIG_LTC2471_MODULE 1
#define CONFIG_DVB_TUA6100_MODULE 1
#define CONFIG_NFT_HASH_MODULE 1
#define CONFIG_IEEE802154_FAKELB_MODULE 1
#define CONFIG_DRM_I2C_ADV7511_AUDIO 1
#define CONFIG_ARCH_HAS_RELR 1
#define CONFIG_USB_MAX3421_HCD_MODULE 1
#define CONFIG_BRCMFMAC_SDIO 1
#define CONFIG_EXTCON_USB_GPIO 1
#define CONFIG_LEDS_TRIGGER_PATTERN 1
#define CONFIG_CLK_BCM_SR 1
#define CONFIG_RTC_DRV_ABX80X_MODULE 1
#define CONFIG_DM_CRYPT_MODULE 1
#define CONFIG_IPV6_FOU_TUNNEL_MODULE 1
#define CONFIG_IRQ_SIM 1
#define CONFIG_HID_THRUSTMASTER_MODULE 1
#define CONFIG_NFS_FS 1
#define CONFIG_SOUND_OSS_CORE_PRECLAIM 1
#define CONFIG_DVB_S5H1420_MODULE 1
#define CONFIG_IIO_CROS_EC_ACCEL_LEGACY_MODULE 1
#define CONFIG_RTC_DRV_BQ32K_MODULE 1
#define CONFIG_PINCTRL_AMLOGIC_C3 1
#define CONFIG_MD_BITMAP_FILE 1
#define CONFIG_FUTEX_PI 1
#define CONFIG_HAS_IOPORT_MAP 1
#define CONFIG_XFRM_STATISTICS 1
#define CONFIG_PHY_ROCKCHIP_EMMC 1
#define CONFIG_ZRAM_BACKEND_LZO 1
#define CONFIG_RTC_DRV_MAX6902_MODULE 1
#define CONFIG_IP_VS_PROTO_AH 1
#define CONFIG_RTC_INTF_DEV 1
#define CONFIG_NETFILTER_SYNPROXY_MODULE 1
#define CONFIG_DVB_M88RS2000_MODULE 1
#define CONFIG_IP6_NF_MATCH_IPV6HEADER_MODULE 1
#define CONFIG_CRYPTO_DRBG_HMAC 1
#define CONFIG_ARCH_BINFMT_ELF_EXTRA_PHDRS 1
#define CONFIG_BRIDGE_EBT_STP_MODULE 1
#define CONFIG_ZSWAP_ZPOOL_DEFAULT_ZSMALLOC 1
#define CONFIG_USB_SERIAL_KEYSPAN_MODULE 1
#define CONFIG_DVB_SI2165_MODULE 1
#define CONFIG_MFD_WL1273_CORE_MODULE 1
#define CONFIG_RTL8723BE_MODULE 1
#define CONFIG_IP6_NF_MATCH_FRAG_MODULE 1
#define CONFIG_SENSORS_ADCXX_MODULE 1
#define CONFIG_REALTEK_PHY_HWMON 1
#define CONFIG_RTC_DRV_PCF85063_MODULE 1
#define CONFIG_TSYS02D_MODULE 1
#define CONFIG_SENSORS_EMC6W201_MODULE 1
#define CONFIG_ARM64_ERRATUM_1508412 1
#define CONFIG_USB_G_NCM_MODULE 1
#define CONFIG_HIBERNATION_DEF_COMP "lzo"
#define CONFIG_SENSORS_AMC6821_MODULE 1
#define CONFIG_MTD_NAND_ECC_SW_HAMMING 1
#define CONFIG_SPI_OFFLOAD 1
#define CONFIG_MT7615_COMMON_MODULE 1
#define CONFIG_DRM_DISPLAY_HELPER 1
#define CONFIG_PWM_MESON_MODULE 1
#define CONFIG_BRIDGE_EBT_MARK_T_MODULE 1
#define CONFIG_SQUASHFS_FILE_CACHE 1
#define CONFIG_MFD_AXP20X_RSB 1
#define CONFIG_CPU_IDLE_MULTIPLE_DRIVERS 1
#define CONFIG_CROS_EC_SYSFS_MODULE 1
#define CONFIG_SUN50I_A100_CCU 1
#define CONFIG_NF_CONNTRACK_PPTP_MODULE 1
#define CONFIG_ARCH_SUPPORTS_LTO_CLANG_THIN 1
#define CONFIG_ARCH_MMAP_RND_BITS 18
#define CONFIG_TYPEC_FUSB302 1
#define CONFIG_SENSORS_W83792D_MODULE 1
#define CONFIG_SENSORS_TMP464_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_IPCOMP_MODULE 1
#define CONFIG_SFC_SIENA_SRIOV 1
#define CONFIG_PCIE_DW 1
#define CONFIG_XFRM_SUB_POLICY 1
#define CONFIG_ARCH_DEFAULT_CRASH_DUMP 1
#define CONFIG_RT2800PCI_MODULE 1
#define CONFIG_IP6_NF_MATCH_RPFILTER_MODULE 1
#define CONFIG_DRM_BOCHS_MODULE 1
#define CONFIG_HID_SENSOR_PRESS_MODULE 1
#define CONFIG_TI_DAC7612_MODULE 1
#define CONFIG_MEDIA_SDR_SUPPORT 1
#define CONFIG_MTD_CFI_INTELEXT_MODULE 1
#define CONFIG_NET_SCH_SFB_MODULE 1
#define CONFIG_GENERIC_PINCTRL_GROUPS 1
#define CONFIG_HID_SENSOR_HUMIDITY_MODULE 1
#define CONFIG_BMC150_MAGN_SPI_MODULE 1
#define CONFIG_MDIO_MODULE 1
#define CONFIG_AD7298_MODULE 1
#define CONFIG_NF_CT_PROTO_UDPLITE 1
#define CONFIG_AD5504_MODULE 1
#define CONFIG_XFS_FS_MODULE 1
#define CONFIG_SRF04_MODULE 1
#define CONFIG_DMA_BCM2835 1
#define CONFIG_DVB_USB_DVBSKY_MODULE 1
#define CONFIG_RESET_BRCMSTB_RESCAL 1
#define CONFIG_SENSORS_AD7414_MODULE 1
#define CONFIG_CAN_PEAK_PCI_MODULE 1
#define CONFIG_BMI160_MODULE 1
#define CONFIG_NLS_CODEPAGE_857_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_TEE_MODULE 1
#define CONFIG_CRYPTO_SM4_ARM64_NEON_BLK 1
#define CONFIG_QCOM_FALKOR_ERRATUM_E1041 1
#define CONFIG_PROFILING 1
#define CONFIG_ARCH_HAS_ZONE_DMA_SET 1
#define CONFIG_VIDEO_SAA7134_GO7007_MODULE 1
#define CONFIG_BRCMSTB_L2_IRQ 1
#define CONFIG_ARM64_ERRATUM_1024718 1
#define CONFIG_CROS_EC_TYPEC_ALTMODES 1
#define CONFIG_HW_RANDOM_TIMERIOMEM_MODULE 1
#define CONFIG_RT2X00_LIB_FIRMWARE 1
#define CONFIG_BT_HCIBCM203X_MODULE 1
#define CONFIG_IAQCORE_MODULE 1
#define CONFIG_BCM2835_POWER 1
#define CONFIG_NFS_V4_2_READ_PLUS 1
#define CONFIG_DRM_MESON_DW_HDMI 1
#define CONFIG_SUN50I_DE2_BUS 1
#define CONFIG_HID_SAMSUNG_MODULE 1
#define CONFIG_SND_SOC_SPDIF 1
#define CONFIG_MEDIA_TUNER_IT913X_MODULE 1
#define CONFIG_SND_SIMPLE_CARD 1
#define CONFIG_VFIO_DEVICE_CDEV 1
#define CONFIG_NLS_CODEPAGE_864_MODULE 1
#define CONFIG_CAN_8DEV_USB_MODULE 1
#define CONFIG_USB_CXACRU_MODULE 1
#define CONFIG_KVM_VFIO 1
#define CONFIG_F2FS_IOSTAT 1
#define CONFIG_6LOWPAN_NHC_ROUTING_MODULE 1
#define CONFIG_USB_SERIAL_FTDI_SIO_MODULE 1
#define CONFIG_ADIS16130_MODULE 1
#define CONFIG_SCSI_CONSTANTS 1
#define CONFIG_HAVE_ARCH_TRANSPARENT_HUGEPAGE 1
#define CONFIG_NET_SCH_FIFO 1
#define CONFIG_BRIDGE_VLAN_FILTERING 1
#define CONFIG_TRACE_IRQFLAGS_NMI_SUPPORT 1
#define CONFIG_BT_HCIUART_ATH3K 1
#define CONFIG_TYPEC_TCPM 1
#define CONFIG_PCI_XGENE 1
#define CONFIG_ARM64_WORKAROUND_REPEAT_TLBI 1
#define CONFIG_IIO_ST_MAGN_3AXIS_MODULE 1
#define CONFIG_ARCH_TEGRA_234_SOC 1
#define CONFIG_ARCH_BINFMT_ELF_STATE 1
#define CONFIG_NLS_DEFAULT "utf8"
#define CONFIG_SND_MESON_CARD_UTILS_MODULE 1
#define CONFIG_FB_TFT_SSD1305_MODULE 1
#define CONFIG_MEDIA_TUNER_XC4000_MODULE 1
#define CONFIG_FB_TFT_ILI9486_MODULE 1
#define CONFIG_CRYPTO_ARC4_MODULE 1
#define CONFIG_TOUCHSCREEN_USB_ELO 1
#define CONFIG_RTW88_8723X_MODULE 1
#define CONFIG_SENSORS_ADT7462_MODULE 1
#define CONFIG_MOUSE_PS2_SENTELIC 1
#define CONFIG_MESON_GX_SOCINFO 1
#define CONFIG_CC_HAS_MIN_FUNCTION_ALIGNMENT 1
#define CONFIG_JOYSTICK_XPAD_MODULE 1
#define CONFIG_BT_MRVL_MODULE 1
#define CONFIG_SENSORS_TC74_MODULE 1
#define CONFIG_DMA_NEED_SYNC 1
#define CONFIG_NLS_ISO8859_13_MODULE 1
#define CONFIG_MMC_SDHCI_PCI 1
#define CONFIG_TOUCHSCREEN_EGALAX_MODULE 1
#define CONFIG_RESET_SUNXI 1
#define CONFIG_NFS_V4_1_MIGRATION 1
#define CONFIG_REGULATOR 1
#define CONFIG_TYPEC_MUX_PI3USB30532 1
#define CONFIG_SENSORS_INA3221_MODULE 1
#define CONFIG_ACPI_I2C_OPREGION 1
#define CONFIG_PINCTRL_SUN8I_A83T_R 1
#define CONFIG_6LOWPAN_GHC_EXT_HDR_DEST_MODULE 1
#define CONFIG_INPUT_SPARSEKMAP_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_CONNLABEL_MODULE 1
#define CONFIG_TORTURE_TEST_MODULE 1
#define CONFIG_DVB_LGDT3306A_MODULE 1
#define CONFIG_CLK_RK3562 1
#define CONFIG_GRO_CELLS 1
#define CONFIG_TOUCHSCREEN_GOODIX_MODULE 1
#define CONFIG_VIDEO_CX25821_MODULE 1
#define CONFIG_CRYPTO_JITTERENTROPY_MEMORY_BLOCKSIZE 32
#define CONFIG_PACKET_DIAG_MODULE 1
#define CONFIG_GENERIC_STRNCPY_FROM_USER 1
#define CONFIG_HID_RMI_MODULE 1
#define CONFIG_TEXTSEARCH 1
#define CONFIG_HAVE_MOVE_PMD 1
#define CONFIG_IWLWIFI_DEBUGFS 1
#define CONFIG_USB_SERIAL_WISHBONE_MODULE 1
#define CONFIG_KEYBOARD_NVEC_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_RECENT_MODULE 1
#define CONFIG_PROC_EVENTS 1
#define CONFIG_FAILOVER_MODULE 1
#define CONFIG_CRYPTO_ARCH_HAVE_LIB_SHA256 1
#define CONFIG_PAGE_MAPCOUNT 1
#define CONFIG_LINEAR_RANGES 1
#define CONFIG_NFS_V4_1 1
#define CONFIG_SATA_VIA_MODULE 1
#define CONFIG_WEXT_PROC 1
#define CONFIG_NF_TPROXY_IPV4_MODULE 1
#define CONFIG_NF_CONNTRACK_IRC_MODULE 1
#define CONFIG_DM_LOG_USERSPACE_MODULE 1
#define CONFIG_DRM_PANEL_ILITEK_ILI9881C_MODULE 1
#define CONFIG_BRIDGE_EBT_BROUTE_MODULE 1
#define CONFIG_AD5696_I2C_MODULE 1
#define CONFIG_VIRTIO_DMA_SHARED_BUFFER_MODULE 1
#define CONFIG_SND_HDA_CORE_MODULE 1
#define CONFIG_NET_VENDOR_ALLWINNER 1
#define CONFIG_HID 1
#define CONFIG_CRYPTO_LZ4_MODULE 1
#define CONFIG_ARM64_RAS_EXTN 1
#define CONFIG_ACPI_REDUCED_HARDWARE_ONLY 1
#define CONFIG_NET_TEAM_MODE_RANDOM_MODULE 1
#define CONFIG_SATA_VITESSE_MODULE 1
#define CONFIG_USB_GADGETFS_MODULE 1
#define CONFIG_SND_SOC_GENERIC_DMAENGINE_PCM 1
#define CONFIG_NFT_MASQ_MODULE 1
#define CONFIG_NET_IFE_SKBPRIO_MODULE 1
#define CONFIG_AD7887_MODULE 1
#define CONFIG_HID_CP2112_MODULE 1
#define CONFIG_NET_SCH_TEQL_MODULE 1
#define CONFIG_CRYPTO_LIB_UTILS 1
#define CONFIG_VEML6070_MODULE 1
#define CONFIG_SG_POOL 1
#define CONFIG_ARM_ARCH_TIMER_EVTSTREAM 1
#define CONFIG_CLK_RASPBERRYPI 1
#define CONFIG_CRYPTO_DH 1
#define CONFIG_MTD_BLOCK2MTD_MODULE 1
#define CONFIG_CC_HAS_WORKING_NOSANITIZE_ADDRESS 1
#define CONFIG_NETFILTER_XT_TARGET_HMARK_MODULE 1
#define CONFIG_BLK_CGROUP_RWSTAT 1
#define CONFIG_USB_CDNS3_MODULE 1
#define CONFIG_SMS_SDIO_DRV_MODULE 1
#define CONFIG_GENERIC_LIB_DEVMEM_IS_ALLOWED 1
#define CONFIG_USB_G_WEBCAM_MODULE 1
#define CONFIG_CROS_EC_PROTO_MODULE 1
#define CONFIG_CORE_DUMP_DEFAULT_ELF_HEADERS 1
#define CONFIG_NET_EMATCH 1
#define CONFIG_SND_HDA_POWER_SAVE_DEFAULT 0
#define CONFIG_B44_PCI_AUTOSELECT 1
#define CONFIG_TYPEC 1
#define CONFIG_VIDEO_HANTRO_ROCKCHIP 1
#define CONFIG_IP6_NF_MATCH_HL_MODULE 1
#define CONFIG_ARCH_HAS_DMA_PREP_COHERENT 1
#define CONFIG_SENSORS_SMSC47M1_MODULE 1
#define CONFIG_SENSORS_NCT6775_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_OWNER_MODULE 1
#define CONFIG_SND_SOC_TEGRA210_AHUB_MODULE 1
#define CONFIG_BINFMT_MISC 1
#define CONFIG_MT792x_LIB_MODULE 1
#define CONFIG_DVB_STV6110x_MODULE 1
#define CONFIG_USB_PCI 1
#define CONFIG_SND_SOC_MSM8916_WCD_DIGITAL_MODULE 1
#define CONFIG_USB_GSPCA_FINEPIX_MODULE 1
#define CONFIG_MRP_MODULE 1
#define CONFIG_SCHED_MM_CID 1
#define CONFIG_ACPI_VIOT 1
#define CONFIG_NFSD_BLOCKLAYOUT 1
#define CONFIG_PWM_DWC_CORE_MODULE 1
#define CONFIG_MCP4131_MODULE 1
#define CONFIG_ADVISE_SYSCALLS 1
#define CONFIG_MT76x2U_MODULE 1
#define CONFIG_THERMAL_GOV_STEP_WISE 1
#define CONFIG_NET 1
#define CONFIG_MOUSE_APPLETOUCH_MODULE 1
#define CONFIG_ARCH_ENABLE_SPLIT_PMD_PTLOCK 1
#define CONFIG_WLAN_VENDOR_MICROCHIP 1
#define CONFIG_CPU_PM 1
#define CONFIG_MPL115_I2C_MODULE 1
#define CONFIG_SCSI_MVSAS_TASKLET 1
#define CONFIG_NET_DSA_TAG_GSWIP_MODULE 1
#define CONFIG_XFRM_IPCOMP_MODULE 1
#define CONFIG_DVB_USB_PCTV452E_MODULE 1
#define CONFIG_GPIO_ACPI 1
#define CONFIG_DEFAULT_CUBIC 1
#define CONFIG_XFRM_USER_MODULE 1
#define CONFIG_NLS_CODEPAGE_865_MODULE 1
#define CONFIG_USB_DEFAULT_AUTHORIZATION_MODE 1
#define CONFIG_NET_REDIRECT 1
#define CONFIG_ACPI_THERMAL 1
#define CONFIG_VFIO_PLATFORM_AMDXGBE_RESET_MODULE 1
#define CONFIG_USB_LAN78XX_MODULE 1
#define CONFIG_AD2S1200_MODULE 1
#define CONFIG_SMS_USB_DRV_MODULE 1
#define CONFIG_GACT_PROB 1
#define CONFIG_ATL1E_MODULE 1
#define CONFIG_VIDEO_CAMERA_SENSOR 1
#define CONFIG_SCHED_SMT 1
#define CONFIG_LIRC 1
#define CONFIG_UNINLINE_SPIN_UNLOCK 1
#define CONFIG_INITRAMFS_SOURCE ""
#define CONFIG_EXFAT_FS_MODULE 1
#define CONFIG_B43_PHY_N 1
#define CONFIG_ARM64_EPAN 1
#define CONFIG_UNIX98_PTYS 1
#define CONFIG_RANDOMIZE_KSTACK_OFFSET 1
#define CONFIG_ARM_SMMU_DISABLE_BYPASS_BY_DEFAULT 1
#define CONFIG_USB_APPLEDISPLAY_MODULE 1
#define CONFIG_PPP_SYNC_TTY_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_CONNBYTES_MODULE 1
#define CONFIG_ARCH_HAS_NONLEAF_PMD_YOUNG 1
#define CONFIG_RTW88_8822BU_MODULE 1
#define CONFIG_DMA_DECLARE_COHERENT 1
#define CONFIG_DRM_CIRRUS_QEMU_MODULE 1
#define CONFIG_SENSORS_MAX1111_MODULE 1
#define CONFIG_PPP_BSDCOMP_MODULE 1
#define CONFIG_USB_GADGET 1
#define CONFIG_BRIDGE_EBT_LIMIT_MODULE 1
#define CONFIG_HID_WIIMOTE_MODULE 1
#define CONFIG_CLK_BCM_63XX 1
#define CONFIG_FB_SIMPLE 1
#define CONFIG_CRYPTO_SM4_GENERIC_MODULE 1
#define CONFIG_FUJITSU_ERRATUM_010001 1
#define CONFIG_CRYPTO_GHASH_ARM64_CE 1
#define CONFIG_MEMORY_BALLOON 1
#define CONFIG_VFIO_MODULE 1
#define CONFIG_NFC_HCI_MODULE 1
#define CONFIG_NLS_ISO8859_14_MODULE 1
#define CONFIG_EXECMEM 1
#define CONFIG_DVB_ZD1301_DEMOD_MODULE 1
#define CONFIG_NET_VENDOR_REALTEK 1
#define CONFIG_ADIS16209_MODULE 1
#define CONFIG_IPV6_SEG6_LWTUNNEL 1
#define CONFIG_MEDIA_TUNER_TDA18212_MODULE 1
#define CONFIG_CHARGER_CROS_PCHG_MODULE 1
#define CONFIG_NET_DSA_TAG_RTL4_A_MODULE 1
#define CONFIG_PCIE_ROCKCHIP_DW 1
#define CONFIG_SENSORS_LTC2947_I2C_MODULE 1
#define CONFIG_OVERLAY_FS_MODULE 1
#define CONFIG_SND_SUN50I_CODEC_ANALOG 1
#define CONFIG_BT_MODULE 1
#define CONFIG_ZRAM_BACKEND_DEFLATE 1
#define CONFIG_TOUCHSCREEN_USB_IDEALTEK 1
#define CONFIG_BMA180_MODULE 1
#define CONFIG_VT 1
#define CONFIG_USB_GSPCA_JEILINJ_MODULE 1
#define CONFIG_NET_CLS_ACT 1
#define CONFIG_ARM_TEGRA194_CPUFREQ 1
#define CONFIG_DVB_USB_MXL111SF_MODULE 1
#define CONFIG_ET131X_MODULE 1
#define CONFIG_COMPAT_OLD_SIGACTION 1
#define CONFIG_EEPROM_MAX6875_MODULE 1
#define CONFIG_IP_SET_HASH_IPPORTIP_MODULE 1
#define CONFIG_SMP 1
#define CONFIG_ARCH_MMAP_RND_COMPAT_BITS 11
#define CONFIG_SUN55I_PCK600 1
#define CONFIG_VLAN_8021Q_MVRP 1
#define CONFIG_SCSI_BNX2_ISCSI_MODULE 1
#define CONFIG_R8169_LEDS 1
#define CONFIG_BT_HCIBTUSB_RTL 1
#define CONFIG_W1_SLAVE_DS250X_MODULE 1
#define CONFIG_DETECT_HUNG_TASK_BLOCKER 1
#define CONFIG_CFG80211_MODULE 1
#define CONFIG_NFT_QUOTA_MODULE 1
#define CONFIG_ARCH_SUPPORTS_MSEAL_SYSTEM_MAPPINGS 1
#define CONFIG_CEPH_FS_POSIX_ACL 1
#define CONFIG_FB_TFT_UC1611_MODULE 1
#define CONFIG_HID_REDRAGON_MODULE 1
#define CONFIG_BT_RTL_MODULE 1
#define CONFIG_SECURITYFS 1
#define CONFIG_VIDEO_CX88_DVB_MODULE 1
#define CONFIG_VIDEO_SOLO6X10_MODULE 1
#define CONFIG_QUEUED_RWLOCKS 1
#define CONFIG_BT_HCIVHCI_MODULE 1
#define CONFIG_KGDB_SERIAL_CONSOLE 1
#define CONFIG_NETFILTER_XT_MATCH_DCCP_MODULE 1
#define CONFIG_MAC80211_RC_MINSTREL 1
#define CONFIG_VIDEO_CX231XX_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_HASHLIMIT_MODULE 1
#define CONFIG_HAVE_ARCH_AUDITSYSCALL 1
#define CONFIG_REGULATOR_PFUZE100 1
#define CONFIG_FIREWIRE_SBP2_MODULE 1
#define CONFIG_ROCKCHIP_PHY 1
#define CONFIG_SND_SOC_STA350_MODULE 1
#define CONFIG_BLK_DEV_LOOP_MODULE 1
#define CONFIG_IPV6_SUBTREES 1
#define CONFIG_SENSORS_ADT7310_MODULE 1
#define CONFIG_SETEND_EMULATION 1
#define CONFIG_HID_EMS_FF_MODULE 1
#define CONFIG_HID_SENSOR_IIO_TRIGGER_MODULE 1
#define CONFIG_BTRFS_FS_POSIX_ACL 1
#define CONFIG_GPIO_PL061 1
#define CONFIG_HID_UDRAW_PS3_MODULE 1
#define CONFIG_HID_APPLE_MODULE 1
#define CONFIG_SND_SOC_TEGRA_SGTL5000_MODULE 1
#define CONFIG_DVB_USB_DIGITV_MODULE 1
#define CONFIG_ACPI_GENERIC_GSI 1
#define CONFIG_AC97_BUS 1
#define CONFIG_STK3310_MODULE 1
#define CONFIG_VIDEO_SAA7134_ALSA_MODULE 1
#define CONFIG_USB_BELKIN 1
#define CONFIG_RCU_STALL_COMMON 1
#define CONFIG_USB_UEAGLEATM_MODULE 1
#define CONFIG_BLK_DEV_ZONED 1
#define CONFIG_RT2X00_LIB_MODULE 1
#define CONFIG_HAVE_ASM_MODVERSIONS 1
#define CONFIG_DVB_STB6100_MODULE 1
#define CONFIG_HID_SENSOR_IIO_COMMON_MODULE 1
#define CONFIG_TCP_CONG_CDG_MODULE 1
#define CONFIG_APDS9960_MODULE 1
#define CONFIG_NFS_DEBUG 1
#define CONFIG_AUDIT_ARCH_COMPAT_GENERIC 1
#define CONFIG_NET_FC 1
#define CONFIG_BT_HCIRSI_MODULE 1
#define CONFIG_SCSI_LOGGING 1
#define CONFIG_HAVE_PERF_EVENTS 1
#define CONFIG_TI_ADC084S021_MODULE 1
#define CONFIG_RFKILL_LEDS 1
#define CONFIG_AD7476_MODULE 1
#define CONFIG_DRM_FBDEV_OVERALLOC 100
#define CONFIG_ARM_SCMI_POWER_CONTROL_MODULE 1
#define CONFIG_SECRETMEM 1
#define CONFIG_DM_VERITY_MODULE 1
#define CONFIG_CONSOLE_POLL 1
#define CONFIG_NFT_NUMGEN_MODULE 1
#define CONFIG_ARM_GIC_V3_ITS 1
#define CONFIG_ATM_MODULE 1
#define CONFIG_NET_SCH_MQPRIO_MODULE 1
#define CONFIG_SENSORS_APDS990X_MODULE 1
#define CONFIG_AIC7XXX_DEBUG_MASK 0
#define CONFIG_W1_SLAVE_DS2423_MODULE 1
#define CONFIG_FB_TFT_BD663474_MODULE 1
#define CONFIG_ROCKCHIP_VOP 1
#define CONFIG_NETFILTER_XT_MATCH_U32_MODULE 1
#define CONFIG_DVB_DM1105_MODULE 1
#define CONFIG_SCSI_SYM53C8XX_MMIO 1
#define CONFIG_IPW2100_MODULE 1
#define CONFIG_NLS_CODEPAGE_855_MODULE 1
#define CONFIG_USB_DWC2_DUAL_ROLE 1
#define CONFIG_RTC_DRV_DS1511_MODULE 1
#define CONFIG_FB_TFT_PCD8544_MODULE 1
#define CONFIG_BCMASP 1
#define CONFIG_IP6_NF_TARGET_HL_MODULE 1
#define CONFIG_HUGETLB_PMD_PAGE_TABLE_SHARING 1
#define CONFIG_NET_KEY_MIGRATE 1
#define CONFIG_SND_INDIGODJX_MODULE 1
#define CONFIG_USB_STORAGE_KARMA_MODULE 1
#define CONFIG_HID_MAYFLASH_MODULE 1
#define CONFIG_B43LEGACY_PCI_AUTOSELECT 1
#define CONFIG_ARM64_ERRATUM_826319 1
#define CONFIG_MEMSTICK_JMICRON_38X_MODULE 1
#define CONFIG_MEDIA_TUNER_QT1010_MODULE 1
#define CONFIG_BONDING_MODULE 1
#define CONFIG_LTO_NONE 1
#define CONFIG_NLS_CODEPAGE_862_MODULE 1
#define CONFIG_HAVE_DMA_CONTIGUOUS 1
#define CONFIG_COMMON_CLK_VC5 1
#define CONFIG_USB_TEGRA_PHY 1
#define CONFIG_DVB_ISL6405_MODULE 1
#define CONFIG_ADIS16136_MODULE 1
#define CONFIG_CM3323_MODULE 1
#define CONFIG_MTD_PHYSMAP_MODULE 1
#define CONFIG_RTW88_8723DS_MODULE 1
#define CONFIG_SENSORS_ADT7475_MODULE 1
#define CONFIG_ARCH_PROC_KCORE_TEXT 1
#define CONFIG_MTD_CFI_STAA_MODULE 1
#define CONFIG_FB_TFT_SSD1289_MODULE 1
#define CONFIG_CAN_NETLINK 1
#define CONFIG_KEYBOARD_MATRIX_MODULE 1
#define CONFIG_BACKLIGHT_CLASS_DEVICE 1
#define CONFIG_CFG80211_DEBUGFS 1
#define CONFIG_KEYBOARD_TEGRA_MODULE 1
#define CONFIG_NF_TABLES_MODULE 1
#define CONFIG_CRYPTO_MANAGER 1
#define CONFIG_USB_F_EEM 1
#define CONFIG_HIBERNATION_COMP_LZO 1
#define CONFIG_CRYPTO_LIB_SHA256_GENERIC 1
#define CONFIG_SND_SOC_SUNXI_AHUB_DAM_MODULE 1
#define CONFIG_SENSORS_PCF8591_MODULE 1
#define CONFIG_SUSPEND_FREEZER 1
#define CONFIG_NET_VENDOR_MARVELL 1
#define CONFIG_NVMEM_ROCKCHIP_EFUSE_MODULE 1
#define CONFIG_RTW89_CORE_MODULE 1
#define CONFIG_SCSI_DC395x_MODULE 1
#define CONFIG_DVB_NET 1
#define CONFIG_SSB_MODULE 1
#define CONFIG_ADXRS450_MODULE 1
#define CONFIG_CRYPTO_LIB_POLY1305_GENERIC 1
#define CONFIG_DECOMPRESS_LZO 1
#define CONFIG_QRTR_MHI_MODULE 1
#define CONFIG_ARM_TEGRA124_CPUFREQ 1
#define CONFIG_OVERLAY_FS_REDIRECT_ALWAYS_FOLLOW 1
#define CONFIG_NLS_ISO8859_15_MODULE 1
#define CONFIG_HAVE_ARCH_STACKLEAK 1
#define CONFIG_MEDIA_CAMERA_SUPPORT 1
#define CONFIG_RFKILL_GPIO_MODULE 1
#define CONFIG_NET_DSA_SMSC_LAN9303_I2C_MODULE 1
#define CONFIG_PINCTRL_ROCKCHIP 1
#define CONFIG_ARM64_USE_LSE_ATOMICS 1
#define CONFIG_NET_SCH_PLUG_MODULE 1
#define CONFIG_NET_DSA_TAG_SJA1105_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_CLUSTER_MODULE 1
#define CONFIG_VIDEO_SAA6752HS_MODULE 1
#define CONFIG_PPP_MULTILINK 1
#define CONFIG_QCOM_QMI_HELPERS_MODULE 1
#define CONFIG_LZ4_DECOMPRESS 1
#define CONFIG_USB_SERIAL_IR_MODULE 1
#define CONFIG_DRM_SUN8I_DW_HDMI 1
#define CONFIG_MFD_RK8XX_I2C 1
#define CONFIG_SND_HDA_CODEC_CA0132_MODULE 1
#define CONFIG_IP_VS_PROTO_ESP 1
#define CONFIG_FB_TFT_HX8347D_MODULE 1
#define CONFIG_SCSI_CHELSIO_FCOE_MODULE 1
#define CONFIG_QUOTA_NETLINK_INTERFACE 1
#define CONFIG_USB_EMI62_MODULE 1
#define CONFIG_NFT_DUP_NETDEV_MODULE 1
#define CONFIG_SCSI_SAS_HOST_SMP 1
#define CONFIG_IIO_ST_SENSORS_CORE_MODULE 1
#define CONFIG_IP6_NF_NAT_MODULE 1
#define CONFIG_RD_LZO 1
#define CONFIG_NET_VENDOR_AQUANTIA 1
#define CONFIG_HID_LENOVO_MODULE 1
#define CONFIG_CEPH_FS_MODULE 1
#define CONFIG_DRM_DISPLAY_HDMI_STATE_HELPER 1
#define CONFIG_OVPN_MODULE 1
#define CONFIG_RASPBERRYPI_FIRMWARE 1
#define CONFIG_ATA_SFF 1
#define CONFIG_INPUT_MOUSEDEV_SCREEN_X 1024
#define CONFIG_NF_CT_PROTO_DCCP 1
#define CONFIG_USB_RAREMONO_MODULE 1
#define CONFIG_PINCTRL 1
#define CONFIG_NET_DSA_VITESSE_VSC73XX_SPI_MODULE 1
#define CONFIG_SUN55I_A523_R_CCU 1
#define CONFIG_DRM_DW_MIPI_DSI 1
#define CONFIG_STP_MODULE 1
#define CONFIG_PHY_MESON_G12A_USB3_PCIE 1
#define CONFIG_JOYSTICK_SPACEORB_MODULE 1
#define CONFIG_TCM_QLA2XXX_MODULE 1
#define CONFIG_MCP320X_MODULE 1
#define CONFIG_JME_MODULE 1
#define CONFIG_USB_SERIAL_VISOR_MODULE 1
#define CONFIG_ARM64_ERRATUM_845719 1
#define CONFIG_MTD_SPI_NOR_USE_4K_SECTORS 1
#define CONFIG_NET_SCH_RED_MODULE 1
#define CONFIG_COMMON_CLK_SCPI 1
#define CONFIG_DVB_VES1X93_MODULE 1
#define CONFIG_VIDEO_MT9V011_MODULE 1
#define CONFIG_VIDEO_TVEEPROM_MODULE 1
#define CONFIG_TI_ADS8688_MODULE 1
#define CONFIG_SUN4I_GPADC_MODULE 1
#define CONFIG_NF_TPROXY_IPV6_MODULE 1
#define CONFIG_ARM64_SME 1
#define CONFIG_STAGING 1
#define CONFIG_SCSI_DH_HP_SW_MODULE 1
#define CONFIG_MISC_RTSX_MODULE 1
#define CONFIG_MEDIA_CONTROLLER 1
#define CONFIG_RTC_DRV_EM3027_MODULE 1
#define CONFIG_DRM_DW_HDMI_I2S_AUDIO 1
#define CONFIG_USB_SUPPORT 1
#define CONFIG_WLCORE_SDIO_MODULE 1
#define CONFIG_DVB_MXL5XX_MODULE 1
#define CONFIG_IP_VS_LBLC_MODULE 1
#define CONFIG_ACPI_PROCESSOR_IDLE 1
#define CONFIG_GCC_PLUGINS 1
#define CONFIG_CRYPTO_JITTERENTROPY 1
#define CONFIG_PREVENT_FIRMWARE_BUILD 1
#define CONFIG_SENSORS_ATXP1_MODULE 1
#define CONFIG_HW_RANDOM_VIRTIO_MODULE 1
#define CONFIG_MEGARAID_SAS_MODULE 1
#define CONFIG_PINCTRL_AMLOGIC_A4 1
#define CONFIG_IP_VS_TAB_BITS 12
#define CONFIG_USB_CONFIGFS_SERIAL 1
#define CONFIG_PSTORE_COMPRESS 1
#define CONFIG_USB_ULPI 1
#define CONFIG_DM_ZERO 1
#define CONFIG_HW_PERF_EVENTS 1
#define CONFIG_NVMEM_BCM_OCOTP 1
#define CONFIG_I2C_GPIO_MODULE 1
#define CONFIG_IP_SET_HASH_NETNET_MODULE 1
#define CONFIG_RESET_MESON_AUDIO_ARB 1
#define CONFIG_FW_LOADER_COMPRESS_ZSTD 1
#define CONFIG_AIC8800_WLAN_SUPPORT_MODULE 1
#define CONFIG_IR_REDRAT3_MODULE 1
#define CONFIG_IEEE802154_HWSIM_MODULE 1
#define CONFIG_NFT_LIMIT_MODULE 1
#define CONFIG_IPV6_TUNNEL_MODULE 1
#define CONFIG_XZ_DEC_ARMTHUMB 1
#define CONFIG_BT_HCIUART_SERDEV 1
#define CONFIG_ZONE_DMA32 1
#define CONFIG_B43_DEBUG 1
#define CONFIG_ARM64_ERRATUM_1165522 1
#define CONFIG_CC_HAS_KASAN_GENERIC 1
#define CONFIG_NET_FAILOVER_MODULE 1
#define CONFIG_KEXEC_FILE 1
#define CONFIG_B43_PIO 1
#define CONFIG_SND_SOC_RK817_MODULE 1
#define CONFIG_SENSORS_ADM1031_MODULE 1
#define CONFIG_PRINTK_TIME 1
#define CONFIG_PHY_ROCKCHIP_USBDP_MODULE 1
#define CONFIG_TSL2772_MODULE 1
#define CONFIG_TIFM_7XX1_MODULE 1
#define CONFIG_ARM_ALLWINNER_SUN50I_CPUFREQ_NVMEM 1
#define CONFIG_HOLTEK_FF 1
#define CONFIG_I2C_CROS_EC_TUNNEL_MODULE 1
#define CONFIG_GREENASIA_FF 1
#define CONFIG_FSCACHE_STATS 1
#define CONFIG_APPLE_MFI_FASTCHARGE_MODULE 1
#define CONFIG_ACPI_CPPC_LIB 1
#define CONFIG_DMI_SYSFS 1
#define CONFIG_ZLIB_INFLATE 1
#define CONFIG_ARCH_WANTS_THP_SWAP 1
#define CONFIG_MMA7455_SPI_MODULE 1
#define CONFIG_HID_BETOP_FF_MODULE 1
#define CONFIG_HAVE_KVM_VCPU_RUN_PID_CHANGE 1
#define CONFIG_IPW2200_RADIOTAP 1
#define CONFIG_W1_MASTER_DS2482_MODULE 1
#define CONFIG_MTD_UBI_MODULE 1
#define CONFIG_NLS_CODEPAGE_863_MODULE 1
#define CONFIG_RTC_DRV_RP5C01_MODULE 1
#define CONFIG_BLK_DEV_RAM_MODULE 1
#define CONFIG_VGA_ARB 1
#define CONFIG_CRYPTO_LIB_CURVE25519_MODULE 1
#define CONFIG_LOGIRUMBLEPAD2_FF 1
#define CONFIG_GENERIC_IOREMAP 1
#define CONFIG_ATL1C_MODULE 1
#define CONFIG_APDS9802ALS_MODULE 1
#define CONFIG_USB_EPSON2888 1
#define CONFIG_ARCH_USE_MEMTEST 1
#define CONFIG_HID_GENERIC 1
#define CONFIG_ACPI_PFRUT_MODULE 1
#define CONFIG_USB_PHY 1
#define CONFIG_TABLET_USB_AIPTEK_MODULE 1
#define CONFIG_PMIC_OPREGION 1
#define CONFIG_CAN_RAW_MODULE 1
#define CONFIG_ARM_CCI5xx_PMU 1
#define CONFIG_TI_ADC108S102_MODULE 1
#define CONFIG_CRC_OPTIMIZATIONS 1
#define CONFIG_RODATA_FULL_DEFAULT_ENABLED 1
#define CONFIG_NET_DSA_XRS700X_MODULE 1
#define CONFIG_MAX5487_MODULE 1
#define CONFIG_DS4424_MODULE 1
#define CONFIG_APERTURE_HELPERS 1
#define CONFIG_UNICODE_MODULE 1
#define CONFIG_GAMEPORT_MODULE 1
#define CONFIG_DM_LOG_WRITES_MODULE 1
#define CONFIG_B44_PCI 1
#define CONFIG_USB_SIERRA_NET_MODULE 1
#define CONFIG_SPI_BITBANG 1
#define CONFIG_RTC_DRV_AMLOGIC_A4 1
#define CONFIG_SND_MESON_AXG_TODDR_MODULE 1
#define CONFIG_SWPHY 1
#define CONFIG_CGROUP_CPUACCT 1
#define CONFIG_SENSORS_ADM1275_MODULE 1
#define CONFIG_SND_I2S_HI6210_I2S_MODULE 1
#define CONFIG_VIDEO_SAA717X_MODULE 1
#define CONFIG_CLK_RK3576 1
#define CONFIG_MFD_AC200 1
#define CONFIG_MEMSTICK_REALTEK_USB_MODULE 1
#define CONFIG_USB_CHIPIDEA_IMX 1
#define CONFIG_VCHIQ_CDEV 1
#define CONFIG_SND_SOC_ROCKCHIP_RT5645_MODULE 1
#define CONFIG_PA12203001_MODULE 1
#define CONFIG_USB_STORAGE_CYPRESS_ATACB_MODULE 1
#define CONFIG_DRM_SCHED 1
#define CONFIG_DRM_PANEL_BRIDGE 1
#define CONFIG_DVB_USB_GL861_MODULE 1
#define CONFIG_IP_VS_WLC_MODULE 1
#define CONFIG_NET_POLL_CONTROLLER 1
#define CONFIG_DW_DMAC_CORE_MODULE 1
#define CONFIG_FIREWIRE_OHCI_MODULE 1
#define CONFIG_TOUCHSCREEN_EDT_FT5X06_MODULE 1
#define CONFIG_PHY_ROCKCHIP_NANENG_COMBO_PHY 1
#define CONFIG_RTW89_8852B_MODULE 1
#define CONFIG_BT_HCIUART_3WIRE 1
#define CONFIG_DIMLIB 1
#define CONFIG_OCTEONTX2_MBOX_MODULE 1
#define CONFIG_MMC_SDHCI_ACPI 1
#define CONFIG_6LOWPAN_GHC_ICMPV6_MODULE 1
#define CONFIG_HAVE_KVM_READONLY_MEM 1
#define CONFIG_SERIAL_OF_PLATFORM 1
#define CONFIG_GPIO_MB86S7X 1
#define CONFIG_INPUT_MOUSEDEV_SCREEN_Y 768
#define CONFIG_CP15_BARRIER_EMULATION 1
#define CONFIG_MEMCG 1
#define CONFIG_DRM_DISPLAY_DP_AUX_CHARDEV 1
#define CONFIG_PWM_ROCKCHIP 1
#define CONFIG_COMMON_CLK_SI5341_MODULE 1
#define CONFIG_BH1750_MODULE 1
#define CONFIG_CRYPTO_LIB_CHACHA20POLY1305_MODULE 1
#define CONFIG_ZSTD_COMPRESS 1
#define CONFIG_MAILBOX 1
#define CONFIG_SPARSE_IRQ 1
#define CONFIG_SENSORS_TSL2563_MODULE 1
#define CONFIG_NEED_SG_DMA_LENGTH 1
#define CONFIG_VHOST_TASK 1
#define CONFIG_IPV6_SEG6_BPF 1
#define CONFIG_KMX61_MODULE 1
#define CONFIG_DVB_TUNER_DIB0070_MODULE 1
#define CONFIG_SUSPEND 1
#define CONFIG_IOMMU_DEFAULT_DMA_STRICT 1
#define CONFIG_USB_SERIAL_F8153X_MODULE 1
#define CONFIG_SND_USB_AUDIO_MIDI_V2 1
#define CONFIG_KGDB 1
#define CONFIG_NL80211_TESTMODE 1
#define CONFIG_GRACE_PERIOD 1
#define CONFIG_SND_SOC_TEGRA186_DSPK_MODULE 1
#define CONFIG_ATH6KL_DEBUG 1
#define CONFIG_PINCTRL_SINGLE 1
#define CONFIG_RTL8192DE_MODULE 1
#define CONFIG_CRC_T10DIF_ARCH 1
#define CONFIG_HAVE_FUNCTION_ARG_ACCESS_API 1
#define CONFIG_MMC_ARMMMCI 1
#define CONFIG_DRM_DW_HDMI_QP_CEC 1
#define CONFIG_SND_SOC_ES8316_MODULE 1
#define CONFIG_ENVELOPE_DETECTOR_MODULE 1
#define CONFIG_RT2800_LIB_MODULE 1
#define CONFIG_MDIO_SUN4I_MODULE 1
#define CONFIG_CC_IMPLICIT_FALLTHROUGH "-Wimplicit-fallthrough=5"
#define CONFIG_FB_DMAMEM_HELPERS 1
#define CONFIG_IP_MULTIPLE_TABLES 1
#define CONFIG_USB_F_RNDIS 1
#define CONFIG_RT2X00_LIB_CRYPTO 1
#define CONFIG_PPP_FILTER 1
#define CONFIG_PTP_1588_CLOCK_VMCLOCK_MODULE 1
#define CONFIG_GPIO_PCA953X_IRQ 1
#define CONFIG_NET_DSA_TAG_OCELOT_8021Q_MODULE 1
#define CONFIG_USB_F_PRINTER 1
#define CONFIG_WLAN_VENDOR_RALINK 1
#define CONFIG_IR_MCE_KBD_DECODER_MODULE 1
#define CONFIG_RATIONAL 1
#define CONFIG_NF_CONNTRACK_MODULE 1
#define CONFIG_GENERIC_VDSO_DATA_STORE 1
#define CONFIG_EXTCON_ADC_JACK_MODULE 1
#define CONFIG_STACKPROTECTOR_PER_TASK 1
#define CONFIG_W1_SLAVE_DS2438_MODULE 1
#define CONFIG_CRC64 1
#define CONFIG_TRUSTED_KEYS_TPM 1
#define CONFIG_MS5611_SPI_MODULE 1
#define CONFIG_MS5611_MODULE 1
#define CONFIG_KEY_DH_OPERATIONS 1
#define CONFIG_HID_GYRATION_MODULE 1
#define CONFIG_NTSYNC_MODULE 1
#define CONFIG_USB_U_ETHER 1
#define CONFIG_VIDEO_TW2804_MODULE 1
#define CONFIG_ZONE_DMA 1
#define CONFIG_CRYPTO_AES_ARM64_NEON_BLK 1
#define CONFIG_SND_USB_LINE6_MODULE 1
#define CONFIG_DVB_USB_TTUSB2_MODULE 1
#define CONFIG_WIL6210_ISR_COR 1
#define CONFIG_SND_MAX_CARDS 32
#define CONFIG_IIO_MUX_MODULE 1
#define CONFIG_RD_GZIP 1
#define CONFIG_DRM_ROCKCHIP 1
#define CONFIG_FB_DEFERRED_IO 1
#define CONFIG_NET_SCH_NETEM_MODULE 1
#define CONFIG_IP_NF_MATCH_RPFILTER_MODULE 1
#define CONFIG_MSDOS_PARTITION 1
#define CONFIG_B43_SDIO 1
#define CONFIG_TIMER_PROBE 1
#define CONFIG_PCC 1
#define CONFIG_DVB_USB_DTV5100_MODULE 1
#define CONFIG_POWER_SUPPLY_HWMON 1
#define CONFIG_CMDLINE_FROM_BOOTLOADER 1
#define CONFIG_PHYLIB_LEDS 1
#define CONFIG_NETFILTER_NETLINK_LOG_MODULE 1
#define CONFIG_SUNXI_SRAM 1
#define CONFIG_CPU_FREQ_GOV_CONSERVATIVE 1
#define CONFIG_ZSTD_COMMON 1
#define CONFIG_DEFAULT_SECURITY_DAC 1
#define CONFIG_PHY_ROCKCHIP_DP 1
#define CONFIG_IIO_CROS_EC_LIGHT_PROX_MODULE 1
#define CONFIG_TOUCHSCREEN_USB_3M 1
#define CONFIG_WATCHDOG 1
#define CONFIG_BRIDGE_NETFILTER_MODULE 1
#define CONFIG_NLS_CODEPAGE_860_MODULE 1
#define CONFIG_LOGITECH_FF 1
#define CONFIG_IR_IGORPLUGUSB_MODULE 1
#define CONFIG_I2C_HID_CORE_MODULE 1
#define CONFIG_HW_RANDOM_ARM_SMCCC_TRNG 1
#define CONFIG_MWIFIEX_SDIO_MODULE 1
#define CONFIG_XFRM 1
#define CONFIG_IPV6_PIMSM_V2 1
#define CONFIG_FIX_EARLYCON_MEM 1
#define CONFIG_TCG_TIS_I2C_ATMEL_MODULE 1
#define CONFIG_NVMEM_SUNXI_SID 1
#define CONFIG_DVB_A8293_MODULE 1
#define CONFIG_MLX90614_MODULE 1
#define CONFIG_TOUCHSCREEN_DYNAPRO_MODULE 1
#define CONFIG_IRQCHIP 1
#define CONFIG_ARCH_HAS_FAST_MULTIPLIER 1
#define CONFIG_IOMMUFD_DRIVER_CORE 1
#define CONFIG_CGROUP_HUGETLB 1
#define CONFIG_TOUCHSCREEN_RASPBERRYPI_FW_MODULE 1
#define CONFIG_FW_LOADER 1
#define CONFIG_USB_GSPCA_KINECT_MODULE 1
#define CONFIG_RTC_SYSTOHC 1
#define CONFIG_JFS_FS_MODULE 1
#define CONFIG_SENSORS_W83791D_MODULE 1
#define CONFIG_REGULATOR_SY8827N 1
#define CONFIG_INET_IPCOMP_MODULE 1
#define CONFIG_ROMFS_FS_MODULE 1
#define CONFIG_IP6_NF_MATCH_RT_MODULE 1
#define CONFIG_MMC_USDHI6ROL0 1
#define CONFIG_SCSI_WD719X_MODULE 1
#define CONFIG_MEDIA_TUNER_MT20XX_MODULE 1
#define CONFIG_PHY_ROCKCHIP_SAMSUNG_HDPTX 1
#define CONFIG_SND_SOC_WM9712_MODULE 1
#define CONFIG_L2TP_DEBUGFS_MODULE 1
#define CONFIG_PATA_TOSHIBA_MODULE 1
#define CONFIG_ARCH_HAS_CACHE_LINE_SIZE 1
#define CONFIG_DRM_BRIDGE_CONNECTOR 1
#define CONFIG_DM_RAID_MODULE 1
#define CONFIG_EXPORTFS 1
#define CONFIG_RPR0521_MODULE 1
#define CONFIG_VIDEO_HDPVR_MODULE 1
#define CONFIG_MMC_DW_EXYNOS 1
#define CONFIG_MAC80211_HWSIM_MODULE 1
#define CONFIG_TCP_CONG_VEGAS_MODULE 1
#define CONFIG_PANIC_TIMEOUT 0
#define CONFIG_VFIO_PLATFORM_BASE_MODULE 1
#define CONFIG_EXT3_FS 1
#define CONFIG_PTP_1588_CLOCK 1
#define CONFIG_MDIO_BCM_UNIMAC 1
#define CONFIG_DRM_SII902X_MODULE 1
#define CONFIG_FW_LOADER_COMPRESS_XZ 1
#define CONFIG_SUN4I_TIMER 1
#define CONFIG_NET_CORE 1
#define CONFIG_DVB_LGDT330X_MODULE 1
#define CONFIG_AT76C50X_USB_MODULE 1
#define CONFIG_SOFT_WATCHDOG_MODULE 1
#define CONFIG_LD_IS_BFD 1
#define CONFIG_SND_SUN4I_CODEC 1
#define CONFIG_MMU_LAZY_TLB_REFCOUNT 1
#define CONFIG_NAMESPACES 1
#define CONFIG_SND_PCM_ELD 1
#define CONFIG_DECOMPRESS_GZIP 1
#define CONFIG_CROS_EC_I2C_MODULE 1
#define CONFIG_HWMON 1
#define CONFIG_TYPEC_TPS6598X 1
#define CONFIG_BRCMFMAC_MODULE 1
#define CONFIG_ACPI_BATTERY_MODULE 1
#define CONFIG_SND_SOC_MESON_T9015_MODULE 1
#define CONFIG_PHY_MESON_AXG_MIPI_DPHY 1
#define CONFIG_IP_NF_ARP_MANGLE_MODULE 1
#define CONFIG_RTW88_8821CE_MODULE 1
#define CONFIG_USB_SISUSBVGA_MODULE 1
#define CONFIG_CPU_RMAP 1
#define CONFIG_IP6_NF_RAW_MODULE 1
#define CONFIG_VFIO_PCI_CORE_MODULE 1
#define CONFIG_ARCH_SUPPORTS_NUMA_BALANCING 1
#define CONFIG_BUFFER_HEAD 1
#define CONFIG_DVB_NGENE_MODULE 1
#define CONFIG_MWIFIEX_USB_MODULE 1
#define CONFIG_CC_HAS_AUTO_VAR_INIT_PATTERN 1
#define CONFIG_VIDEO_CX23885_MODULE 1
#define CONFIG_FIRMWARE_EDID 1
#define CONFIG_FORTIFY_SOURCE 1
#define CONFIG_HWSPINLOCK 1
#define CONFIG_I2C_DESIGNWARE_CORE 1
#define CONFIG_ENCLOSURE_SERVICES_MODULE 1
#define CONFIG_NAU7802_MODULE 1
#define CONFIG_DMA_OF 1
#define CONFIG_SND_SOC_RL6231_MODULE 1
#define CONFIG_DVB_SI2168_MODULE 1
#define CONFIG_FB_SYSMEM_HELPERS_DEFERRED 1
#define CONFIG_MFD_SIMPLE_MFD_I2C_MODULE 1
#define CONFIG_6LOWPAN_MODULE 1
#define CONFIG_ARM64_ERRATUM_858921 1
#define CONFIG_YT6801_ETHERNET_MODULE 1
#define CONFIG_ARM64_ERRATUM_1418040 1
#define CONFIG_DUMMY_MODULE 1
#define CONFIG_SYSCON_REBOOT_MODE 1
#define CONFIG_VIDEO_EM28XX_V4L2_MODULE 1
#define CONFIG_DM_BIO_PRISON_MODULE 1
#define CONFIG_MMC_SDHCI_IO_ACCESSORS 1
#define CONFIG_CRAMFS_MTD 1
#define CONFIG_BMC150_ACCEL_SPI_MODULE 1
#define CONFIG_NETFILTER_FAMILY_ARP 1
#define CONFIG_HIBERNATE_CALLBACKS 1
#define CONFIG_DVB_USB_CXUSB_MODULE 1
#define CONFIG_IR_RCMM_DECODER_MODULE 1
#define CONFIG_TASKS_TRACE_RCU 1
#define CONFIG_6LOWPAN_NHC_MODULE 1
#define CONFIG_MOUSE_PS2_ELANTECH 1
#define CONFIG_HARDENED_USERCOPY_DEFAULT_ON 1
#define CONFIG_VIPERBOARD_ADC_MODULE 1
#define CONFIG_SERIO_AMBAKMI 1
#define CONFIG_NET_VENDOR_MELLANOX 1
#define CONFIG_IR_SERIAL_MODULE 1
#define CONFIG_OCFS2_FS_MODULE 1
#define CONFIG_SKY2_MODULE 1
#define CONFIG_ARCH_HAS_CRC_T10DIF 1
#define CONFIG_USB_GSPCA_SPCA501_MODULE 1
#define CONFIG_BT_LE 1
#define CONFIG_IXGBE_MODULE 1
#define CONFIG_SENSORS_NZXT_SMART2_MODULE 1
#define CONFIG_MT76x02_LIB_MODULE 1
#define CONFIG_USB_CONFIGFS_F_UVC 1
#define CONFIG_DYNAMIC_DEBUG 1
#define CONFIG_ZRAM_MODULE 1
#define CONFIG_SERIAL_DEV_CTRL_TTYPORT 1
#define CONFIG_PCI_DOMAINS 1
#define CONFIG_FUSION_SAS_MODULE 1
#define CONFIG_CPU_FREQ_GOV_SCHEDUTIL 1
#define CONFIG_PCI_XGENE_MSI 1
#define CONFIG_BMC150_MAGN_MODULE 1
#define CONFIG_IR_PWM_TX_MODULE 1
#define CONFIG_EFI_RUNTIME_WRAPPERS 1
#define CONFIG_USB_MDC800_MODULE 1
#define CONFIG_NF_CT_NETLINK_MODULE 1
#define CONFIG_IIO_TRIGGERED_BUFFER 1
#define CONFIG_IOMMU_IO_PGTABLE_LPAE 1
#define CONFIG_NETFILTER_XT_NAT_MODULE 1
#define CONFIG_CAVIUM_ERRATUM_27456 1
#define CONFIG_ATH10K_LEDS 1
#define CONFIG_6LOWPAN_NHC_FRAGMENT_MODULE 1
#define CONFIG_PROC_SYSCTL 1
#define CONFIG_HDC100X_MODULE 1
#define CONFIG_MIGRATION 1
#define CONFIG_DVB_PLUTO2_MODULE 1
#define CONFIG_HAVE_EFFICIENT_UNALIGNED_ACCESS 1
#define CONFIG_ATH9K_MODULE 1
#define CONFIG_CRYPTO_AKCIPHER 1
#define CONFIG_HID_APPLEIR_MODULE 1
#define CONFIG_MTD_SPI_NOR_SWP_DISABLE_ON_VOLATILE 1
#define CONFIG_NEED_KVM_DIRTY_RING_WITH_BITMAP 1
#define CONFIG_NET_SCHED 1
#define CONFIG_NET_CLS_FW_MODULE 1
#define CONFIG_HAVE_HARDLOCKUP_DETECTOR_BUDDY 1
#define CONFIG_NLS_CODEPAGE_861_MODULE 1
#define CONFIG_CRYPTO_CHACHA20 1
#define CONFIG_MPLS_ROUTING_MODULE 1
#define CONFIG_PAGE_POOL 1
#define CONFIG_CRYPTO_USER_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_ESP_MODULE 1
#define CONFIG_RT2500USB_MODULE 1
#define CONFIG_HID_XINMO_MODULE 1
#define CONFIG_NTFS3_FS_MODULE 1
#define CONFIG_NETPOLL 1
#define CONFIG_SCSI_DMX3191D_MODULE 1
#define CONFIG_SPI_MESON_SPIFC_MODULE 1
#define CONFIG_SERIAL_TEGRA_MODULE 1
#define CONFIG_DRM_TEGRA_MODULE 1
#define CONFIG_VIDEOBUF2_MEMOPS_MODULE 1
#define CONFIG_IIO_ST_PRESS_SPI_MODULE 1
#define CONFIG_NET_MPLS_GSO_MODULE 1
#define CONFIG_MII 1
#define CONFIG_BCMA_DRIVER_GMAC_CMN 1
#define CONFIG_OF_MDIO 1
#define CONFIG_NVMEM_SYSFS 1
#define CONFIG_SND_SOC_TEGRA_TRIMSLICE_MODULE 1
#define CONFIG_SENSORS_ADM1266_MODULE 1
#define CONFIG_DVB_TDA18271C2DD_MODULE 1
#define CONFIG_USB_STORAGE_ENE_UB6250_MODULE 1
#define CONFIG_B53_SRAB_DRIVER_MODULE 1
#define CONFIG_SCSI_COMMON 1
#define CONFIG_SPARSEMEM_EXTREME 1
#define CONFIG_PHY_ROCKCHIP_TYPEC 1
#define CONFIG_CEC_MESON_G12A_AO 1
#define CONFIG_USB_M5602_MODULE 1
#define CONFIG_RSEQ 1
#define CONFIG_SND_PCM_OSS_MODULE 1
#define CONFIG_IOMMU_DMA 1
#define CONFIG_NET_SCH_DRR_MODULE 1
#define CONFIG_DRM_DW_HDMI_QP 1
#define CONFIG_TOUCHSCREEN_ILITEK_MODULE 1
#define CONFIG_SYSCTL_EXCEPTION_TRACE 1
#define CONFIG_NEW_LEDS 1
#define CONFIG_ARM_PMU 1
#define CONFIG_USB_G_PRINTER_MODULE 1
#define CONFIG_SPI_AX88796C_MODULE 1
#define CONFIG_RAS 1
#define CONFIG_PINCTRL_MESON_GXBB 1
#define CONFIG_NF_TABLES_NETDEV 1
#define CONFIG_SSB_SPROM 1
#define CONFIG_SND_USB_6FIRE_MODULE 1
#define CONFIG_PCIE_TEGRA194_MODULE 1
#define CONFIG_BT_6LOWPAN_MODULE 1
#define CONFIG_DVB_STV0367_MODULE 1
#define CONFIG_PWM_DWC_MODULE 1
#define CONFIG_SCSI_IPR_MODULE 1
#define CONFIG_KVM_MMIO 1
#define CONFIG_MOUSE_SERIAL_MODULE 1
#define CONFIG_PADATA 1
#define CONFIG_VIDEO_FB_IVTV_MODULE 1
#define CONFIG_KEYBOARD_GPIO_POLLED_MODULE 1
#define CONFIG_ARCH_HAS_PTE_SPECIAL 1
#define CONFIG_SND_DARLA20_MODULE 1
#define CONFIG_ARM64_ERRATUM_2054223 1
#define CONFIG_ARM64_PA_BITS 48
#define CONFIG_MEDIA_TUNER_MT2060_MODULE 1
#define CONFIG_CC_HAS_ASM_INLINE 1
#define CONFIG_SATA_AHCI_PLATFORM 1
#define CONFIG_VFIO_AMBA_MODULE 1
#define CONFIG_SENSORS_W83781D_MODULE 1
#define CONFIG_FB 1
#define CONFIG_IP_VS_FTP_MODULE 1
#define CONFIG_PATA_NS87410_MODULE 1
#define CONFIG_HID_STEELSERIES_MODULE 1
#define CONFIG_SQUASHFS_DECOMP_SINGLE 1
#define CONFIG_USB_LEDS_TRIGGER_USBPORT_MODULE 1
#define CONFIG_SENSORS_TMP513_MODULE 1
#define CONFIG_VIDEO_CS5345_MODULE 1
#define CONFIG_F2FS_FS_LZ4 1
#define CONFIG_HAS_SECURITY_AUDIT 1
#define CONFIG_VIRT_WIFI_MODULE 1
#define CONFIG_QCOM_FALKOR_ERRATUM_1003 1
#define CONFIG_VIDEO_CX88_ALSA_MODULE 1
#define CONFIG_HAVE_ACPI_APEI 1
#define CONFIG_COMMON_CLK_MESON_SCLK_DIV 1
#define CONFIG_TOUCHSCREEN_PENMOUNT_MODULE 1
#define CONFIG_NLS_CODEPAGE_950_MODULE 1
#define CONFIG_IR_RC5_DECODER_MODULE 1
#define CONFIG_SENSORS_LM78_MODULE 1
#define CONFIG_BT_HCIUART_INTEL 1
#define CONFIG_MLX4_DEBUG 1
#define CONFIG_F2FS_FS_POSIX_ACL 1
#define CONFIG_RTC_DRV_SUN6I 1
#define CONFIG_CC_HAS_ASM_GOTO_TIED_OUTPUT 1
#define CONFIG_ISCSI_TARGET_MODULE 1
#define CONFIG_TABLET_USB_KBTAB_MODULE 1
#define CONFIG_BLK_DEV_INITRD 1
#define CONFIG_CIFS_FSCACHE 1
#define CONFIG_MEDIA_TUNER_TDA8290_MODULE 1
#define CONFIG_BE2ISCSI_MODULE 1
#define CONFIG_DWC_PCIE_PMU_MODULE 1
#define CONFIG_USB_F_SERIAL 1
#define CONFIG_AFFS_FS_MODULE 1
#define CONFIG_NEED_TASKS_RCU 1
#define CONFIG_ADXL345_SPI_MODULE 1
#define CONFIG_USB_SERIAL_OPTICON_MODULE 1
#define CONFIG_USB_GSPCA_SPCA500_MODULE 1
#define CONFIG_B43_PHY_LP 1
#define CONFIG_INET 1
#define CONFIG_DL2K_MODULE 1
#define CONFIG_VIDEO_CX18_MODULE 1
#define CONFIG_PATA_HPT3X2N_MODULE 1
#define CONFIG_HID_PETALYNX_MODULE 1
#define CONFIG_DEVFREQ_GOV_SIMPLE_ONDEMAND 1
#define CONFIG_BLK_DEV_SR 1
#define CONFIG_PTP_1588_CLOCK_DTE_MODULE 1
#define CONFIG_AD5686_MODULE 1
#define CONFIG_ATH11K_PCI_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_REDIRECT_MODULE 1
#define CONFIG_POWER_RESET 1
#define CONFIG_ATA 1
#define CONFIG_BLK_DEV_RAM_SIZE 16384
#define CONFIG_MMA7455_I2C_MODULE 1
#define CONFIG_POWER_RESET_REGULATOR 1
#define CONFIG_USB_EMI26_MODULE 1
#define CONFIG_SENSORS_ZL6100_MODULE 1
#define CONFIG_ATH6KL_USB_MODULE 1
#define CONFIG_RPS 1
#define CONFIG_HW_RANDOM_MESON_MODULE 1
#define CONFIG_HID_PICOLCD_LCD 1
#define CONFIG_SND_MESON_G12A_TOHDMITX_MODULE 1
#define CONFIG_TOUCHSCREEN_ELAN_MODULE 1
#define CONFIG_CRYPTO_CAST6_MODULE 1
#define CONFIG_GENERIC_ARCH_TOPOLOGY 1
#define CONFIG_ARM64_VA_BITS_48 1
#define CONFIG_LEDS_TRIGGER_NETDEV 1
#define CONFIG_RESET_TEGRA_BPMP 1
#define CONFIG_CGROUP_NET_PRIO 1
#define CONFIG_MPILIB 1
#define CONFIG_MT7603E_MODULE 1
#define CONFIG_MEDIA_TUNER_MC44S803_MODULE 1
#define CONFIG_GARP_MODULE 1
#define CONFIG_FIRMWARE_TABLE 1
#define CONFIG_SND_DMAENGINE_PCM 1
#define CONFIG_NET_ACT_SIMP_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_CHECKSUM_MODULE 1
#define CONFIG_IP_NF_SECURITY_MODULE 1
#define CONFIG_EPOLL 1
#define CONFIG_USB_FUNCTIONFS_MODULE 1
#define CONFIG_ACPI_PLATFORM_PROFILE_MODULE 1
#define CONFIG_ARM_GIC_V2M 1
#define CONFIG_USB_FUNCTIONFS_ETH 1
#define CONFIG_RTC_DRV_DS3232_MODULE 1
#define CONFIG_COMMON_CLK_MESON_MPLL 1
#define CONFIG_CGROUPS 1
#define CONFIG_UDMABUF 1
#define CONFIG_CNIC_MODULE 1
#define CONFIG_NFSD_FLEXFILELAYOUT 1
#define CONFIG_CLK_RK3328 1
#define CONFIG_SND_SUN50IW9_CODEC_MODULE 1
#define CONFIG_PATA_JMICRON_MODULE 1
#define CONFIG_USB_F_SUBSET 1
#define CONFIG_RTC_DRV_PCF85363_MODULE 1
#define CONFIG_ACPI_CPPC_CPUFREQ_FIE 1
#define CONFIG_AD7791_MODULE 1
#define CONFIG_SND_SOC_SGTL5000_MODULE 1
#define CONFIG_NET_CLS_MATCHALL_MODULE 1
#define CONFIG_RTC_DRV_BRCMSTB 1
#define CONFIG_PKCS8_PRIVATE_KEY_PARSER 1
#define CONFIG_AD2S1210_MODULE 1
#define CONFIG_SERIAL_8250 1
#define CONFIG_ABP060MG_MODULE 1
#define CONFIG_NET_DSA_MT7530_MMIO_MODULE 1
#define CONFIG_HAVE_ARCH_KASAN 1
#define CONFIG_USB_SERIAL 1
#define CONFIG_CRYPTO_CHACHA20POLY1305_MODULE 1
#define CONFIG_MFD_MAX77620 1
#define CONFIG_CRYPTO_HASH_INFO 1
#define CONFIG_NETFILTER_XT_MATCH_IPVS_MODULE 1
#define CONFIG_CRC32 1
#define CONFIG_EXT4_FS 1
#define CONFIG_AK8975_MODULE 1
#define CONFIG_SENSORS_INA238_MODULE 1
#define CONFIG_TOUCHSCREEN_TOUCHRIGHT_MODULE 1
#define CONFIG_HID_EZKEY_MODULE 1
#define CONFIG_FANOTIFY_ACCESS_PERMISSIONS 1
#define CONFIG_IR_SERIAL_TRANSMITTER 1
#define CONFIG_JOYSTICK_ZHENHUA_MODULE 1
#define CONFIG_PHY_ROCKCHIP_INNO_USB2 1
#define CONFIG_AD7768_1_MODULE 1
#define CONFIG_TCP_CONG_HSTCP_MODULE 1
#define CONFIG_RTL8192CU_MODULE 1
#define CONFIG_SND_HDA_INPUT_BEEP_MODE 0
#define CONFIG_GPIO_CDEV 1
#define CONFIG_VIDEO_PVRUSB2_DVB 1
#define CONFIG_USB_SERIAL_XSENS_MT_MODULE 1
#define CONFIG_BATTERY_SBS_MODULE 1
#define CONFIG_ATH9K_PCOEM 1
#define CONFIG_SOUND_OSS_CORE 1
#define CONFIG_LIBIE_MODULE 1
#define CONFIG_QNX6FS_FS_MODULE 1
#define CONFIG_BSD_PROCESS_ACCT_V3 1
#define CONFIG_PHY_SUN50I_USB3 1
#define CONFIG_FUSE_PASSTHROUGH 1
#define CONFIG_CAN_C_CAN_PCI_MODULE 1
#define CONFIG_RTL8192C_COMMON_MODULE 1
#define CONFIG_NET_DSA_TAG_VSC73XX_8021Q_MODULE 1
#define CONFIG_USB_F_OBEX 1
#define CONFIG_DT_IDLE_STATES 1
#define CONFIG_USBIP_VHCI_HC_PORTS 8
#define CONFIG_PATA_NETCELL_MODULE 1
#define CONFIG_ARCH_HAS_SET_DIRECT_MAP 1
#define CONFIG_USB_SERIAL_SIMPLE_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_REALM_MODULE 1
#define CONFIG_USB_CHIPIDEA_PCI 1
#define CONFIG_CIFS_UPCALL 1
#define CONFIG_SOCIONEXT_SYNQUACER_PREITS 1
#define CONFIG_USB_SL811_HCD_MODULE 1
#define CONFIG_USB_TEST_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_RATEEST_MODULE 1
#define CONFIG_SND_MIXART_MODULE 1
#define CONFIG_SATA_MOBILE_LPM_POLICY 0
#define CONFIG_NET_CRC32C 1
#define CONFIG_DMARD10_MODULE 1
#define CONFIG_VCNL4000_MODULE 1
#define CONFIG_SND_HDA_GENERIC_MODULE 1
#define CONFIG_WL18XX_MODULE 1
#define CONFIG_WILC1000_SPI_MODULE 1
#define CONFIG_IIO_ADC_HELPER_MODULE 1
#define CONFIG_SND_SOC_TEGRA_MODULE 1
#define CONFIG_MFD_ROHM_BD718XX 1
#define CONFIG_DECOMPRESS_BZIP2 1
#define CONFIG_HAVE_ARCH_VMAP_STACK 1
#define CONFIG_EFI_STUB 1
#define CONFIG_BT_HCIUART_H4 1
#define CONFIG_DMABUF_HEAPS_SYSTEM 1
#define CONFIG_ARM64_WORKAROUND_TSB_FLUSH_FAILURE 1
#define CONFIG_CAN_SJA1000_MODULE 1
#define CONFIG_SSB_DRIVER_GPIO 1
#define CONFIG_RTC_LIB 1
#define CONFIG_HAVE_MOVE_PUD 1
#define CONFIG_SENSORS_LM77_MODULE 1
#define CONFIG_SND_MESON_AXG_SOUND_CARD_MODULE 1
#define CONFIG_RADIO_WL1273_MODULE 1
#define CONFIG_PWRSEQ_SD8787_MODULE 1
#define CONFIG_INPUT_UINPUT_MODULE 1
#define CONFIG_INPUT_GPIO_ROTARY_ENCODER_MODULE 1
#define CONFIG_SCSI_DEBUG_MODULE 1
#define CONFIG_SENSORS_TMP108_MODULE 1
#define CONFIG_PER_VMA_LOCK 1
#define CONFIG_NFT_REDIR_MODULE 1
#define CONFIG_CRYPTO_POLY1305_NEON_MODULE 1
#define CONFIG_USB_GSPCA_SONIXB_MODULE 1
#define CONFIG_DQL 1
#define CONFIG_I2C_MV64XXX 1
#define CONFIG_RSI_DEBUGFS 1
#define CONFIG_DRM_I2C_ADV7511_CEC 1
#define CONFIG_USB_CONFIGFS_F_UAC1_LEGACY 1
#define CONFIG_IR_TOY_MODULE 1
#define CONFIG_SECCOMP_FILTER 1
#define CONFIG_MEGARAID_NEWGEN 1
#define CONFIG_TASKS_RCU 1
#define CONFIG_GCC_VERSION 140201
#define CONFIG_6LOWPAN_NHC_IPV6_MODULE 1
#define CONFIG_HAVE_ARCH_JUMP_LABEL_RELATIVE 1
#define CONFIG_WEXT_PRIV 1
#define CONFIG_POWER_RESET_GPIO 1
#define CONFIG_GPIO_SYSFS 1
#define CONFIG_I2C_MUX_PINCTRL_MODULE 1
#define CONFIG_SERIAL_MAX310X_MODULE 1
#define CONFIG_SYSCTL 1
#define CONFIG_THERMAL_GOV_USER_SPACE 1
#define CONFIG_ARM64_WORKAROUND_CLEAN_CACHE 1
#define CONFIG_PANIC_ON_OOPS_VALUE 0
#define CONFIG_NVMEM 1
#define CONFIG_CACHEFILES_DEBUG 1
#define CONFIG_CRYPTO_LIB_SHA1 1
#define CONFIG_VIDEO_EM28XX_ALSA_MODULE 1
#define CONFIG_VLAN_8021Q_GVRP 1
#define CONFIG_IP_NF_IPTABLES_LEGACY_MODULE 1
#define CONFIG_USB_ISP1760_MODULE 1
#define CONFIG_HID_DRAGONRISE_MODULE 1
#define CONFIG_OCTEONTX2_AF_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_STRING_MODULE 1
#define CONFIG_SND_AU8830_MODULE 1
#define CONFIG_GPIO_TEGRA 1
#define CONFIG_BCM2835_WDT_MODULE 1
#define CONFIG_SSB_POSSIBLE 1
#define CONFIG_AIC79XX_REG_PRETTY_PRINT 1
#define CONFIG_TIGON3_MODULE 1
#define CONFIG_SENSORS_MAX31785_MODULE 1
#define CONFIG_CEC_CROS_EC_MODULE 1
#define CONFIG_NLS_UTF8_MODULE 1
#define CONFIG_SERIAL_8250_FSL 1
#define CONFIG_WLAN_VENDOR_REALTEK 1
#define CONFIG_B43_PCI_AUTOSELECT 1
#define CONFIG_USB_GSPCA_STK1135_MODULE 1
#define CONFIG_GPIO_GENERIC 1
#define CONFIG_BACKLIGHT_LP855X_MODULE 1
#define CONFIG_PM_SLEEP_SMP 1
#define CONFIG_POWER_RESET_GPIO_RESTART 1
#define CONFIG_FIXED_PHY 1
#define CONFIG_CRYPTO_NULL 1
#define CONFIG_POSIX_MQUEUE_SYSCTL 1
#define CONFIG_HOTPLUG_PCI_ACPI 1
#define CONFIG_MULTIPLEXER 1
#define CONFIG_HIBERNATION 1
#define CONFIG_DRM_MALI_DISPLAY_MODULE 1
#define CONFIG_IP_VS_DH_MODULE 1
#define CONFIG_COMMON_CLK_MESON_CLKC_UTILS 1
#define CONFIG_AD_SIGMA_DELTA_MODULE 1
#define CONFIG_MESON_CLK_MEASURE 1
#define CONFIG_IOMMU_IO_PGTABLE_DART 1
#define CONFIG_NETFILTER_XT_MATCH_OSF_MODULE 1
#define CONFIG_IPMI_SI_MODULE 1
#define CONFIG_IWL4965_MODULE 1
#define CONFIG_RTC_DRV_DS3232_HWMON 1
#define CONFIG_SERIAL_CORE_CONSOLE 1
#define CONFIG_HP206C_MODULE 1
#define CONFIG_AK8974_MODULE 1
#define CONFIG_XXHASH 1
#define CONFIG_NET_L3_MASTER_DEV 1
#define CONFIG_FB_TFT_HX8357D_MODULE 1
#define CONFIG_TASK_XACCT 1
#define CONFIG_PHY_ROCKCHIP_INNO_USB3_MODULE 1
#define CONFIG_NET_SOCK_MSG 1
#define CONFIG_SENSORS_LTC2992_MODULE 1
#define CONFIG_SENSORS_ADM9240_MODULE 1
#define CONFIG_CRYPTO_LIB_DES_MODULE 1
#define CONFIG_SND_SEQUENCER_OSS_MODULE 1
#define CONFIG_EFI_PARAMS_FROM_FDT 1
#define CONFIG_MESON_SARADC_MODULE 1
#define CONFIG_CRYPTO_ECDH_MODULE 1
#define CONFIG_RTC_DRV_DS2404_MODULE 1
#define CONFIG_ACPI_APMT 1
#define CONFIG_HW_RANDOM_BCM74110 1
#define CONFIG_PHY_MESON_GXL_USB2 1
#define CONFIG_DVB_DYNAMIC_MINORS 1
#define CONFIG_HID_PICOLCD_MODULE 1
#define CONFIG_AIC8800_BTLPM_SUPPORT_MODULE 1
#define CONFIG_IRQ_DOMAIN_HIERARCHY 1
#define CONFIG_SND_JACK 1
#define CONFIG_COMMON_CLK_IPROC 1
#define CONFIG_IWLDVM_MODULE 1
#define CONFIG_NET_ACT_BPF_MODULE 1
#define CONFIG_UNMAP_KERNEL_AT_EL0 1
#define CONFIG_CAN_CTUCANFD_MODULE 1
#define CONFIG_DVB_USB_DIB0700_MODULE 1
#define CONFIG_SENSORS_TMP401_MODULE 1
#define CONFIG_CRYPTO_LIB_POLY1305_MODULE 1
#define CONFIG_I2C_SCMI 1
#define CONFIG_MTD_NAND_CORE 1
#define CONFIG_NET_DSA_XRS700X_I2C_MODULE 1
#define CONFIG_GENERIC_IRQ_PROBE 1
#define CONFIG_TCP_CONG_VENO_MODULE 1
#define CONFIG_USB_GSPCA_MODULE 1
#define CONFIG_SQUASHFS_LZO 1
#define CONFIG_MOUSE_PS2_SMBUS 1
#define CONFIG_FRAMEBUFFER_CONSOLE_DETECT_PRIMARY 1
#define CONFIG_NET_SCH_TAPRIO_MODULE 1
#define CONFIG_SENSORS_W83795_MODULE 1
#define CONFIG_FRAME_WARN 1024
#define CONFIG_SERIAL_MAX3100_MODULE 1
#define CONFIG_NLS_MAC_CENTEURO_MODULE 1
#define CONFIG_LD_VERSION 24400
#define CONFIG_NET_CLS_U32_MODULE 1
#define CONFIG_VT_CONSOLE 1
#define CONFIG_INET_AH_MODULE 1
#define CONFIG_I2C 1
#define CONFIG_RESET_RASPBERRYPI 1
#define CONFIG_NET_SCH_ETS_MODULE 1
#define CONFIG_VFIO_PLATFORM_MODULE 1
#define CONFIG_SND_SOC_TEGRA210_I2S_MODULE 1
#define CONFIG_SND_SOC 1
#define CONFIG_USB_MUSB_HDRC 1
#define CONFIG_SENSORS_EMC2103_MODULE 1
#define CONFIG_SCSI_MVUMI_MODULE 1
#define CONFIG_PCIE_PME 1
#define CONFIG_MD_RAID10_MODULE 1
#define CONFIG_LEDS_CLASS 1
#define CONFIG_MEDIA_RADIO_SUPPORT 1
#define CONFIG_LSI_ET1011C_PHY_MODULE 1
#define CONFIG_NET_VENDOR_ADI 1
#define CONFIG_LEDS_TRIGGER_PANIC 1
#define CONFIG_PM_SLEEP 1
#define CONFIG_NET_SCH_CODEL_MODULE 1
#define CONFIG_SENSORS_ADM1029_MODULE 1
#define CONFIG_DRM_BRIDGE 1
#define CONFIG_PINCTRL_SUN55I_A523_R 1
#define CONFIG_REED_SOLOMON_DEC8 1
#define CONFIG_MKISS_MODULE 1
#define CONFIG_VIDEO_SAA7134_RC 1
#define CONFIG_USB_GSPCA_SPCA506_MODULE 1
#define CONFIG_MFD_SUN4I_GPADC_MODULE 1
#define CONFIG_SENSORS_ADT7410_MODULE 1
#define CONFIG_IWLWIFI_DEBUG 1
#define CONFIG_HID_MAGICMOUSE_MODULE 1
#define CONFIG_NET_SCH_FQ_MODULE 1
#define CONFIG_THERMAL_OF 1
#define CONFIG_VIDEO_BT848_MODULE 1
#define CONFIG_DRM_LIMA_MODULE 1
#define CONFIG_RT2800USB_RT53XX 1
#define CONFIG_DETECT_HUNG_TASK 1
#define CONFIG_VIDEO_M52790_MODULE 1
#define CONFIG_USB_G_MULTI_CDC 1
#define CONFIG_VIRTIO_IOMMU_MODULE 1
#define CONFIG_CC_HAS_BRANCH_PROT_PAC_RET 1
#define CONFIG_HIBERNATION_SNAPSHOT_DEV 1
#define CONFIG_ARM_CMN 1
#define CONFIG_HID_ZEROPLUS_MODULE 1
#define CONFIG_SND_HDA_DSP_LOADER 1
#define CONFIG_NET_PTP_CLASSIFY 1
#define CONFIG_NET_DSA_LANTIQ_GSWIP_MODULE 1
#define CONFIG_VMAP_STACK 1
#define CONFIG_SPI_SUN4I 1
#define CONFIG_ICE_MODULE 1
#define CONFIG_PM 1
#define CONFIG_AD5064_MODULE 1
#define CONFIG_DST_CACHE 1
#define CONFIG_CRYPTO_DEV_VIRTIO_MODULE 1
#define CONFIG_CRYPTO_DEV_CCP_CRYPTO_MODULE 1
#define CONFIG_CRYPTO_SHA512 1
#define CONFIG_LIST_HARDENED 1
#define CONFIG_IIO_ST_PRESS_I2C_MODULE 1
#define CONFIG_SENSORS_F71882FG_MODULE 1
#define CONFIG_IRQ_POLL 1
#define CONFIG_VIDEO_FIXED_MINOR_RANGES 1
#define CONFIG_CHARGER_MAX14577_MODULE 1
#define CONFIG_ARCH_SPARSEMEM_ENABLE 1
#define CONFIG_SCSI_AIC7XXX_MODULE 1
#define CONFIG_ACPI_THERMAL_LIB 1
#define CONFIG_USB_EHCI_ROOT_HUB_TT 1
#define CONFIG_USB_NET_AX88179_178A_MODULE 1
#define CONFIG_NLS_CODEPAGE_775_MODULE 1
#define CONFIG_CMA_SYSFS 1
#define CONFIG_MARVELL_PHY_MODULE 1
#define CONFIG_HID_PRIMAX_MODULE 1
#define CONFIG_NLS 1
#define CONFIG_USB_S2255_MODULE 1
#define CONFIG_IP_SET_HASH_NETPORTNET_MODULE 1
#define CONFIG_OID_REGISTRY 1
#define CONFIG_MEDIATEK_GE_PHY_MODULE 1
#define CONFIG_USER_STACKTRACE_SUPPORT 1
#define CONFIG_SND_CTXFI_MODULE 1
#define CONFIG_THERMAL_MMIO_MODULE 1
#define CONFIG_B43_PHY_G 1
#define CONFIG_INPUT_CMA3000_I2C_MODULE 1
#define CONFIG_TOUCHSCREEN_WACOM_I2C_MODULE 1
#define CONFIG_AD7793_MODULE 1
#define CONFIG_ACPI_ALS_MODULE 1
#define CONFIG_GENERIC_IRQ_MIGRATION 1
#define CONFIG_SQUASHFS_ZLIB 1
#define CONFIG_MLX4_CORE_MODULE 1
#define CONFIG_USB_AUDIO_MODULE 1
#define CONFIG_SUNXI_CCU 1
#define CONFIG_ARCH_DEFAULT_KEXEC_IMAGE_VERIFY_SIG 1
#define CONFIG_SENSORS_LM87_MODULE 1
#define CONFIG_BACKLIGHT_PWM_MODULE 1
#define CONFIG_APDS9300_MODULE 1
#define CONFIG_USB_VIDEO_CLASS_INPUT_EVDEV 1
#define CONFIG_SOC_TEGRA_PMC 1
#define CONFIG_DVB_STV0297_MODULE 1
#define CONFIG_NET_DSA_TAG_BRCM_PREPEND_MODULE 1
#define CONFIG_SENSORS_MAX6621_MODULE 1
#define CONFIG_SENSORS_LTC4215_MODULE 1
#define CONFIG_AUDIT_COMPAT_GENERIC 1
#define CONFIG_NETFILTER_FAMILY_BRIDGE 1
#define CONFIG_CC_NO_ARRAY_BOUNDS 1
#define CONFIG_6LOWPAN_GHC_EXT_HDR_ROUTE_MODULE 1
#define CONFIG_MLX90632_MODULE 1
#define CONFIG_FIB_RULES 1
#define CONFIG_NFT_XFRM_MODULE 1
#define CONFIG_CONTEXT_TRACKING_IDLE 1
#define CONFIG_BLK_DEBUG_FS 1
#define CONFIG_SND_SUN50I_DMIC_MODULE 1
#define CONFIG_JBD2 1
#define CONFIG_FONT_SUPPORT 1
#define CONFIG_SENSORS_LTC4222_MODULE 1
#define CONFIG_ARCH_WANTS_EXECMEM_LATE 1
#define CONFIG_ARM_SCMI_HAVE_SHMEM 1
#define CONFIG_GENERIC_IRQ_EFFECTIVE_AFF_MASK 1
#define CONFIG_PINCTRL_TEGRA124 1
#define CONFIG_WIRELESS 1
#define CONFIG_THERMAL_GOV_FAIR_SHARE 1
#define CONFIG_BLK_DEV_BSGLIB 1
#define CONFIG_COMMON_CLK_MAX77686 1
#define CONFIG_SUN50I_A100_R_CCU 1
#define CONFIG_USB_NET_SR9800_MODULE 1
#define CONFIG_NET_SCH_FQ_PIE_MODULE 1
#define CONFIG_SATA_SIL24_MODULE 1
#define CONFIG_MAX1363_MODULE 1
#define CONFIG_SCSI_DH_ALUA_MODULE 1
#define CONFIG_SENSORS_MCP3021_MODULE 1
#define CONFIG_SND_CS46XX_MODULE 1
#define CONFIG_NETFILTER 1
#define CONFIG_LIBERTAS_THINFIRM_USB_MODULE 1
#define CONFIG_MEDIA_TUNER_MT2063_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_HL_MODULE 1
#define CONFIG_SND_MESON_AXG_TDMOUT_MODULE 1
#define CONFIG_DVB_STV6111_MODULE 1
#define CONFIG_REGMAP_SPI 1
#define CONFIG_NET_ACT_GATE_MODULE 1
#define CONFIG_ADIS16080_MODULE 1
#define CONFIG_CXL_PCI_MODULE 1
#define CONFIG_SND_VIRMIDI_MODULE 1
#define CONFIG_MMA9553_MODULE 1
#define CONFIG_USB_GSPCA_SN9C2028_MODULE 1
#define CONFIG_NET_IPGRE_MODULE 1
#define CONFIG_LEDS_BCM63138 1
#define CONFIG_SND_USB_AUDIO_USE_MEDIA_CONTROLLER 1
#define CONFIG_BSD_DISKLABEL 1
#define CONFIG_USB_STORAGE_SDDR09_MODULE 1
#define CONFIG_DRM_PANEL_RASPBERRYPI_TOUCHSCREEN_MODULE 1
#define CONFIG_AD5446_MODULE 1
#define CONFIG_BLK_WBT_MQ 1
#define CONFIG_CHROMEOS_ACPI_MODULE 1
#define CONFIG_DVB_CXD2841ER_MODULE 1
#define CONFIG_DRM_DISPLAY_DP_HELPER 1
#define CONFIG_EFI_VARS_PSTORE_DEFAULT_DISABLE 1
#define CONFIG_VIDEO_TVP5150_MODULE 1
#define CONFIG_HFSPLUS_FS_MODULE 1
#define CONFIG_SENSORS_ARM_SCPI_MODULE 1
#define CONFIG_CRYPTO_SKCIPHER 1
#define CONFIG_USB_CONFIGFS_F_TCM 1
#define CONFIG_NET_DSA_VITESSE_VSC73XX_MODULE 1
#define CONFIG_SENSORS_LM75_MODULE 1
#define CONFIG_USB_SERIAL_MOS7840_MODULE 1
#define CONFIG_ATH11K_SPECTRAL 1
#define CONFIG_IR_SONY_DECODER_MODULE 1
#define CONFIG_SERIO_ARC_PS2_MODULE 1
#define CONFIG_HID_ITE_MODULE 1
#define CONFIG_HAVE_FUNCTION_GRAPH_FREGS 1
#define CONFIG_SND_SOC_TLV320AIC23_I2C_MODULE 1
#define CONFIG_PCP_BATCH_SCALE_MAX 5
#define CONFIG_NETFILTER_XT_MATCH_NFACCT_MODULE 1
#define CONFIG_PATA_RDC_MODULE 1
#define CONFIG_PM_GENERIC_DOMAINS 1
#define CONFIG_CRYPTO_LRW_MODULE 1
#define CONFIG_DEFAULT_HUNG_TASK_TIMEOUT 120
#define CONFIG_SERIO_LIBPS2 1
#define CONFIG_MMA8452_MODULE 1
#define CONFIG_USB_GSPCA_SPCA505_MODULE 1
#define CONFIG_TAP_MODULE 1
#define CONFIG_CAN_CC770_PLATFORM_MODULE 1
#define CONFIG_HAVE_PREEMPT_DYNAMIC 1
#define CONFIG_ARM64_PLATFORM_DEVICES 1
#define CONFIG_USB_DEFAULT_PERSIST 1
#define CONFIG_SENSORS_ADT7411_MODULE 1
#define CONFIG_TCP_CONG_CUBIC 1
#define CONFIG_CLK_BCM2711_DVP 1
#define CONFIG_NVMEM_MESON_MX_EFUSE_MODULE 1
#define CONFIG_HID_THINGM_MODULE 1
#define CONFIG_ATM_BR2684_MODULE 1
#define CONFIG_SCTP_COOKIE_HMAC_MD5 1
#define CONFIG_XDP_SOCKETS 1
#define CONFIG_CRYPTO_ECHAINIV_MODULE 1
#define CONFIG_PINCTRL_TEGRA194 1
#define CONFIG_MEGARAID_MAILBOX_MODULE 1
#define CONFIG_ARCH_HAS_UBSAN 1
#define CONFIG_AD7291_MODULE 1
#define CONFIG_AIC_SDIO_WLAN_SUPPORT 1
#define CONFIG_HID_PICOLCD_LEDS 1
#define CONFIG_DVB_DIB7000P_MODULE 1
#define CONFIG_CW1200_MODULE 1
#define CONFIG_IP_SET_BITMAP_IP_MODULE 1
#define CONFIG_VMAP_PFN 1
#define CONFIG_ARCH_WANT_DEFAULT_TOPDOWN_MMAP_LAYOUT 1
#define CONFIG_HAVE_SOFTIRQ_ON_OWN_STACK 1
#define CONFIG_MTD_CFI_AMDSTD_MODULE 1
#define CONFIG_ACPI_PPTT 1
#define CONFIG_EFI_ARMSTUB_DTB_LOADER 1
#define CONFIG_BPF_JIT 1
#define CONFIG_MOUSE_PS2_TOUCHKIT 1
#define CONFIG_ARCH_HAS_MEM_ENCRYPT 1
#define CONFIG_SUNRPC_DEBUG 1
#define CONFIG_HID_MALTRON_MODULE 1
#define CONFIG_CRYPTO_CAST5_MODULE 1
#define CONFIG_PM_TEST_SUSPEND 1
#define CONFIG_NET_TEAM_MODE_ACTIVEBACKUP_MODULE 1
#define CONFIG_MT7921E_MODULE 1
#define CONFIG_DVB_AS102_MODULE 1
#define CONFIG_HID_PANTHERLORD_MODULE 1
#define CONFIG_HAVE_CLK_PREPARE 1
#define CONFIG_USB_CONFIGFS_MASS_STORAGE 1
#define CONFIG_IR_ENE_MODULE 1
#define CONFIG_FB_SM712 1
#define CONFIG_SERIAL_NONSTANDARD 1
#define CONFIG_SPI_ROCKCHIP 1
#define CONFIG_AS_HAS_CFI_NEGATE_RA_STATE 1
#define CONFIG_HAVE_RUST 1
#define CONFIG_BLK_DEV_BSG_COMMON 1
#define CONFIG_CFG80211_CRDA_SUPPORT 1
#define CONFIG_SND_SOC_RT5514_MODULE 1
#define CONFIG_MFD_RK8XX 1
#define CONFIG_IP_ADVANCED_ROUTER 1
#define CONFIG_JOYSTICK_IFORCE_MODULE 1
#define CONFIG_SENSORS_MAX20730_MODULE 1
#define CONFIG_GENERIC_CSUM 1
#define CONFIG_IP_VS_OVF_MODULE 1
#define CONFIG_CRYPTO_GENIV 1
#define CONFIG_ARM_TEGRA20_CPUFREQ 1
#define CONFIG_VIDEO_EM28XX_RC_MODULE 1
#define CONFIG_BAYCOM_SER_HDX_MODULE 1
#define CONFIG_ARM_SMC_WATCHDOG_MODULE 1
#define CONFIG_RTL8192EE_MODULE 1
#define CONFIG_USB_DWC3_ULPI 1
#define CONFIG_TI_ADC161S626_MODULE 1
#define CONFIG_SND_SOC_HDMI_CODEC 1
#define CONFIG_DNS_RESOLVER 1
#define CONFIG_COMMON_CLK_S4_PLL 1
#define CONFIG_SUNRPC_GSS_MODULE 1
#define CONFIG_VIDEOMODE_HELPERS 1
#define CONFIG_SQUASHFS_LZ4 1
#define CONFIG_MITIGATE_SPECTRE_BRANCH_HISTORY 1
#define CONFIG_EDAC_SUPPORT 1
#define CONFIG_USB_PRINTER_MODULE 1
#define CONFIG_FW_CFG_SYSFS_MODULE 1
#define CONFIG_HAVE_ARCH_COMPILER_H 1
#define CONFIG_ARM64_ERRATUM_2645198 1
#define CONFIG_B44_PCICORE_AUTOSELECT 1
#define CONFIG_NETFILTER_XT_TARGET_NETMAP_MODULE 1
#define CONFIG_ACPI_IPMI_MODULE 1
#define CONFIG_SPI_MASTER 1
#define CONFIG_TEGRA_HOST1X_CONTEXT_BUS 1
#define CONFIG_SND_INTEL_SOUNDWIRE_ACPI_MODULE 1
#define CONFIG_F2FS_FS_LZ4HC 1
#define CONFIG_USB_GSPCA_SE401_MODULE 1
#define CONFIG_STRIP_ASM_SYMS 1
#define CONFIG_SENSORS_SIS5595_MODULE 1
#define CONFIG_EFI_PARTITION 1
#define CONFIG_SENSORS_MAX6620_MODULE 1
#define CONFIG_MTD_SPI_NOR 1
#define CONFIG_SND_AC97_POWER_SAVE 1
#define CONFIG_PCIE_BUS_DEFAULT 1
#define CONFIG_ARM_SCMI_NEED_DEBUGFS 1
#define CONFIG_BUILTIN_RETURN_ADDRESS_STRIPS_PAC 1
#define CONFIG_SENSORS_LTC2990_MODULE 1
#define CONFIG_MARVELL_10G_PHY_MODULE 1
#define CONFIG_ST_UVIS25_MODULE 1
#define CONFIG_MACB_PCI 1
#define CONFIG_SCSI_UFS_ROCKCHIP 1
#define CONFIG_VIDEO_SAA7127_MODULE 1
#define CONFIG_MT792x_USB_MODULE 1
#define CONFIG_IP_NF_MATCH_TTL_MODULE 1
#define CONFIG_HW_RANDOM_IPROC_RNG200 1
#define CONFIG_FHANDLE 1
#define CONFIG_SND_USB_HIFACE_MODULE 1
#define CONFIG_ARM64_ERRATUM_843419 1
#define CONFIG_XFS_SUPPORT_ASCII_CI 1
#define CONFIG_NETFILTER_XT_MATCH_CGROUP_MODULE 1
#define CONFIG_USB_SERIAL_SSU100_MODULE 1
#define CONFIG_SENSORS_G762_MODULE 1
#define CONFIG_ARCH_MHP_MEMMAP_ON_MEMORY_ENABLE 1
#define CONFIG_USB_SERIAL_F81232_MODULE 1
#define CONFIG_USB_GSPCA_SPCA561_MODULE 1
#define CONFIG_SND_DARLA24_MODULE 1
#define CONFIG_SGL_ALLOC 1
#define CONFIG_TYPEC_HD3SS3220_MODULE 1
#define CONFIG_GENERIC_PCI_IOMAP 1
#define CONFIG_CRYPTO_USER_API_RNG 1
#define CONFIG_DVB_STV6110_MODULE 1
#define CONFIG_NF_CT_PROTO_GRE 1
#define CONFIG_SUN20I_GPADC_MODULE 1
#define CONFIG_BT_HCIBFUSB_MODULE 1
#define CONFIG_SOUND 1
#define CONFIG_SENSORS_W83793_MODULE 1
#define CONFIG_ARM64_ERRATUM_2966298 1
#define CONFIG_NET_DSA_TAG_TRAILER_MODULE 1
#define CONFIG_CRAMFS 1
#define CONFIG_USB_DWC3_DUAL_ROLE 1
#define CONFIG_SND_AD1889_MODULE 1
#define CONFIG_NLS_MAC_TURKISH_MODULE 1
#define CONFIG_GENERIC_ALLOCATOR 1
#define CONFIG_DVB_TDA1004X_MODULE 1
#define CONFIG_SND_MESON_AXG_PDM_MODULE 1
#define CONFIG_HAVE_ARCH_KGDB 1
#define CONFIG_CAN_C_CAN_PLATFORM_MODULE 1
#define CONFIG_MTD_NAND_ECC 1
#define CONFIG_NET_INGRESS 1
#define CONFIG_CRYPTO_AEAD2 1
#define CONFIG_DM_PERSISTENT_DATA_MODULE 1
#define CONFIG_PHY_BCM_SR_USB 1
#define CONFIG_SND_SOC_TEGRA_RT5640_MODULE 1
#define CONFIG_GENERIC_BUG_RELATIVE_POINTERS 1
#define CONFIG_SND_YMFPCI_MODULE 1
#define CONFIG_ARCH_STACKWALK 1
#define CONFIG_SPI 1
#define CONFIG_P54_PCI_MODULE 1
#define CONFIG_LCD_PLATFORM_MODULE 1
#define CONFIG_USB_HIDDEV 1
#define CONFIG_CEC_CH7322_MODULE 1
#define CONFIG_SENSORS_IT87_MODULE 1
#define CONFIG_TCM_FILEIO_MODULE 1
#define CONFIG_CRYPTO_AES_ARM64_BS 1
#define CONFIG_DEBUG_INFO_NONE 1
#define CONFIG_HID_U2FZERO_MODULE 1
#define CONFIG_TOUCHSCREEN_ZFORCE_MODULE 1
#define CONFIG_HFS_FS_MODULE 1
#define CONFIG_HAVE_ARCH_MMAP_RND_BITS 1
#define CONFIG_SND_FM801_MODULE 1
#define CONFIG_HAVE_PERF_REGS 1
#define CONFIG_SND_MESON_AXG_FRDDR_MODULE 1
#define CONFIG_MOTORCOMM_PHY 1
#define CONFIG_SND_SOC_TEGRA186_ASRC_MODULE 1
#define CONFIG_MTD_PHYSMAP_OF 1
#define CONFIG_DVB_STB0899_MODULE 1
#define CONFIG_UEFI_CPER_ARM 1
#define CONFIG_PAGE_SIZE_LESS_THAN_256KB 1
#define CONFIG_IPV6_SIT_6RD 1
#define CONFIG_HAS_IOPORT 1
#define CONFIG_STRICT_MODULE_RWX 1
#define CONFIG_MCP4531_MODULE 1
#define CONFIG_IXGBEVF_MODULE 1
#define CONFIG_CXL_BUS_MODULE 1
#define CONFIG_MAC802154_MODULE 1
#define CONFIG_USB_NET_SR9700_MODULE 1
#define CONFIG_PCS_MTK_LYNXI_MODULE 1
#define CONFIG_LIBERTAS_SPI_MODULE 1
#define CONFIG_PCI 1
#define CONFIG_SENSORS_PMBUS_MODULE 1
#define CONFIG_USB_OHCI_HCD_PCI 1
#define CONFIG_SND_OPL3_LIB_SEQ_MODULE 1
#define CONFIG_USB_GSPCA_CPIA1_MODULE 1
#define CONFIG_CHARGER_RK817 1
#define CONFIG_CLANG_VERSION 0
#define CONFIG_CEC_NOTIFIER 1
#define CONFIG_BRIDGE_EBT_MARK_MODULE 1
#define CONFIG_B43_MODULE 1
#define CONFIG_DM_CACHE_MODULE 1
#define CONFIG_NETFILTER_XTABLES_COMPAT 1
#define CONFIG_B43_SSB 1
#define CONFIG_SBP_TARGET_MODULE 1
#define CONFIG_SECURITY_APPARMOR_INTROSPECT_POLICY 1
#define CONFIG_ARCH_HAS_ELF_RANDOMIZE 1
#define CONFIG_NFS_V4_1_IMPLEMENTATION_ID_DOMAIN "kernel.org"
#define CONFIG_SCSI 1
#define CONFIG_DVB_ZL10036_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_IPRANGE_MODULE 1
#define CONFIG_NF_CONNTRACK_TIMESTAMP 1
#define CONFIG_SND_SOC_TEGRA_MAX98090_MODULE 1
#define CONFIG_SENSORS_SHT4x_MODULE 1
#define CONFIG_INPUT_LEDS 1
#define CONFIG_USB_RTL8153_ECM_MODULE 1
#define CONFIG_IIO_CROS_EC_SENSORS_CORE_MODULE 1
#define CONFIG_USB_CHIPIDEA_GENERIC 1
#define CONFIG_MAGIC_SYSRQ 1
#define CONFIG_GENERIC_CALIBRATE_DELAY 1
#define CONFIG_CRYPTO_KDF800108_CTR 1
#define CONFIG_NET_VENDOR_SOLARFLARE 1
#define CONFIG_MAGIC_SYSRQ_SERIAL_SEQUENCE ""
#define CONFIG_BT_BNEP_MC_FILTER 1
#define CONFIG_SENSORS_MAX34440_MODULE 1
#define CONFIG_CRC7 1
#define CONFIG_MEDIA_ANALOG_TV_SUPPORT 1
#define CONFIG_HAVE_KVM_IRQCHIP 1
#define CONFIG_LD_ORPHAN_WARN_LEVEL "warn"
#define CONFIG_USB_OHCI_HCD_PLATFORM 1
#define CONFIG_PCI_MESON 1
#define CONFIG_RTW88_8703B_MODULE 1
#define CONFIG_DVB_TDA10071_MODULE 1
#define CONFIG_I2C_MUX_REG_MODULE 1
#define CONFIG_MEDIA_TUNER_FC2580_MODULE 1
#define CONFIG_TOUCHSCREEN_USB_JASTEC 1
#define CONFIG_SFC_SIENA_MODULE 1
#define CONFIG_CC_HAS_ASM_GOTO_OUTPUT 1
#define CONFIG_64BIT 1
#define CONFIG_CRYPTO_DEV_BCM_SPU_MODULE 1
#define CONFIG_USB_CONFIGFS_ECM_SUBSET 1
#define CONFIG_DVB_IX2505V_MODULE 1
#define CONFIG_USB_OHCI_LITTLE_ENDIAN 1
#define CONFIG_ISO9660_FS_MODULE 1
#define CONFIG_UNIX_DIAG_MODULE 1
#define CONFIG_NFS_SWAP 1
#define CONFIG_SND_HDA_GENERIC_LEDS 1
#define CONFIG_DVB_VES1820_MODULE 1
#define CONFIG_SENSORS_LM85_MODULE 1
#define CONFIG_CRYPTO_LIB_CHACHA_GENERIC 1
#define CONFIG_AD5758_MODULE 1
#define CONFIG_COMMON_CLK_MESON_VCLK 1
#define CONFIG_DMA_OPS_HELPERS 1
#define CONFIG_VIDEOBUF2_DVB_MODULE 1
#define CONFIG_DVB_STV0299_MODULE 1
#define CONFIG_IP_SCTP_MODULE 1
#define CONFIG_IP_VS_PROTO_SCTP 1
#define CONFIG_MESON_GX_PM 1
#define CONFIG_USB_GADGET_STORAGE_NUM_BUFFERS 2
#define CONFIG_RADIO_MAXIRADIO_MODULE 1
#define CONFIG_PM_DEBUG 1
#define CONFIG_VIDEOBUF2_DMA_SG_MODULE 1
#define CONFIG_NET_DSA_SMSC_LAN9303_MDIO_MODULE 1
#define CONFIG_MMC_SDHCI_XENON 1
#define CONFIG_USB_F_UAC1 1
#define CONFIG_TEGRA_SOCTHERM_MODULE 1
#define CONFIG_BACKLIGHT_LED_MODULE 1
#define CONFIG_BLK_CGROUP 1
#define CONFIG_BNX2X_SRIOV 1
#define CONFIG_PINCTRL_SUN8I_H3 1
#define CONFIG_HAVE_ARCH_KFENCE 1
#define CONFIG_HID_CORSAIR_MODULE 1
#define CONFIG_IPV6_VTI_MODULE 1
#define CONFIG_RTL8XXXU_UNTESTED 1
#define CONFIG_COMMON_CLK_MESON_DUALDIV 1
#define CONFIG_CLK_RK3568 1
#define CONFIG_HID_NTI_MODULE 1
#define CONFIG_DECOMPRESS_ZSTD 1
#define CONFIG_ARM_MHU_V3_MODULE 1
#define CONFIG_NF_LOG_ARP_MODULE 1
#define CONFIG_MTD_MAP_BANK_WIDTH_4 1
#define CONFIG_NF_NAT_AMANDA_MODULE 1
#define CONFIG_OLD_SIGSUSPEND3 1
#define CONFIG_NET_TEAM_MODULE 1
#define CONFIG_MDIO_BUS_MUX 1
#define CONFIG_SSB_PCIHOST_POSSIBLE 1
#define CONFIG_SND_HDA_CODEC_ANALOG_MODULE 1
#define CONFIG_XPS 1
#define CONFIG_BRCMSTB_MEMC 1
#define CONFIG_AMLOGIC_THERMAL_MODULE 1
#define CONFIG_6LOWPAN_GHC_EXT_HDR_FRAG_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_STATISTIC_MODULE 1
#define CONFIG_ZRAM_BACKEND_LZ4 1
#define CONFIG_DRM_PL111_MODULE 1
#define CONFIG_COMPAT_32BIT_TIME 1
#define CONFIG_NVMEM_ROCKCHIP_OTP_MODULE 1
#define CONFIG_MMA9551_MODULE 1
#define CONFIG_POWER_RESET_RESTART 1
#define CONFIG_SENSORS_SCH5627_MODULE 1
#define CONFIG_HOTPLUG_CPU 1
#define CONFIG_PAGE_POOL_STATS 1
#define CONFIG_WANT_DEV_COREDUMP 1
#define CONFIG_NET_IPGRE_DEMUX_MODULE 1
#define CONFIG_USB_LIBCOMPOSITE 1
#define CONFIG_DVB_PT1_MODULE 1
#define CONFIG_F2FS_FS_ZSTD 1
#define CONFIG_RTC_DRV_CADENCE_MODULE 1
#define CONFIG_SENSORS_SCH5636_MODULE 1
#define CONFIG_BLK_DEV_NVME 1
#define CONFIG_ARM_GIC_PM 1
#define CONFIG_BT_HCIUART_AML 1
#define CONFIG_NET_DSA_VITESSE_VSC73XX_PLATFORM_MODULE 1
#define CONFIG_SENSORS_LM73_MODULE 1
#define CONFIG_DVB_USB_VP7045_MODULE 1
#define CONFIG_ARCH_HAS_SET_MEMORY 1
#define CONFIG_DEBUG_VFS 1
#define CONFIG_DEBUG_VM 1
#define CONFIG_DVB_USB_DTT200U_MODULE 1
#define CONFIG_SERIO_SUN4I_PS2_MODULE 1
#define CONFIG_PSTORE_BLK_MAX_REASON 2
#define CONFIG_HID_CHERRY_MODULE 1
#define CONFIG_MAC80211_MESH 1
#define CONFIG_BLK_WBT 1
#define CONFIG_MTD_UBI_WL_THRESHOLD 4096
#define CONFIG_USB_SERIAL_WWAN_MODULE 1
#define CONFIG_IP_SET_HASH_NETPORT_MODULE 1
#define CONFIG_NET_ACT_SAMPLE_MODULE 1
#define CONFIG_OF_CONFIGFS 1
#define CONFIG_PINCTRL_MESON8_PMX 1
#define CONFIG_USB_ARMLINUX 1
#define CONFIG_ZEROPLUS_FF 1
#define CONFIG_HID_KEYTOUCH_MODULE 1
#define CONFIG_REGULATOR_PWM 1
#define CONFIG_REGULATOR_MP886X 1
#define CONFIG_SMSC_PHY 1
#define CONFIG_SND_SOC_DMIC_MODULE 1
#define CONFIG_AD7293_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_ADDRTYPE_MODULE 1
#define CONFIG_SENSORS_LTC4260_MODULE 1
#define CONFIG_CC_CAN_LINK 1
#define CONFIG_BT_MTKSDIO_MODULE 1
#define CONFIG_USB_FUNCTIONFS_GENERIC 1
#define CONFIG_B53_MODULE 1
#define CONFIG_JOYSTICK_IFORCE_USB_MODULE 1
#define CONFIG_PCI_HISI 1
#define CONFIG_NF_LOG_IPV6_MODULE 1
#define CONFIG_CC_HAS_AUTO_VAR_INIT_ZERO_BARE 1
#define CONFIG_MMC_VUB300_MODULE 1
#define CONFIG_IPMI_PLAT_DATA 1
#define CONFIG_KERNFS 1
#define CONFIG_6LOWPAN_NHC_UDP_MODULE 1
#define CONFIG_COMMON_CLK_S4_PERIPHERALS 1
#define CONFIG_SERIO_SERPORT 1
#define CONFIG_TOUCHSCREEN_GUNZE_MODULE 1
#define CONFIG_V4L2_H264_MODULE 1
#define CONFIG_RD_XZ 1
#define CONFIG_HAVE_PAGE_SIZE_4KB 1
#define CONFIG_VIDEO_TW68_MODULE 1
#define CONFIG_VIDEO_V4L2_I2C 1
#define CONFIG_IIO_ADIS_LIB_MODULE 1
#define CONFIG_MDIO_BUS_MUX_MESON_G12A_MODULE 1
#define CONFIG_MSDOS_FS 1
#define CONFIG_CRYPTO_AES_ARM64 1
#define CONFIG_USB_ULPI_BUS 1
#define CONFIG_DM_EBS_MODULE 1
#define CONFIG_ARCH_PKEY_BITS 3
#define CONFIG_JOYSTICK_WARRIOR_MODULE 1
#define CONFIG_PCIEAER 1
#define CONFIG_IP_VS_SH_TAB_BITS 8
#define CONFIG_SND_MTPAV_MODULE 1
#define CONFIG_DVB_S5H1411_MODULE 1
#define CONFIG_ATH6KL_SDIO_MODULE 1
#define CONFIG_COMMON_CLK_A1_PLL 1
#define CONFIG_HAVE_SCHED_AVG_IRQ 1
#define CONFIG_TI_ADS124S08_MODULE 1
#define CONFIG_CRC_ITU_T 1
#define CONFIG_TOUCHSCREEN_INEXIO_MODULE 1
#define CONFIG_SCSI_SRP_ATTRS_MODULE 1
#define CONFIG_SND_ECHO3G_MODULE 1
#define CONFIG_DT_IDLE_GENPD 1
#define CONFIG_SSB_SDIOHOST_POSSIBLE 1
#define CONFIG_DEVFREQ_GOV_PERFORMANCE 1
#define CONFIG_SENSORS_UCD9200_MODULE 1
#define CONFIG_SENSORS_RM3100_MODULE 1
#define CONFIG_IP_FIB_TRIE_STATS 1
#define CONFIG_PROC_PAGE_MONITOR 1
#define CONFIG_HAVE_ARCH_SECCOMP_FILTER 1
#define CONFIG_NET_DSA_TAG_OCELOT_MODULE 1
#define CONFIG_DEBUG_BUGVERBOSE 1
#define CONFIG_ATH9K_COMMON_DEBUG 1
#define CONFIG_USB_PEGASUS_MODULE 1
#define CONFIG_UIO_PCI_GENERIC_MODULE 1
#define CONFIG_OF_IOMMU 1
#define CONFIG_FXAS21002C_SPI_MODULE 1
#define CONFIG_ARCH_SUSPEND_POSSIBLE 1
#define CONFIG_USB_IDMOUSE_MODULE 1
#define CONFIG_KPROBES 1
#define CONFIG_RT_MUTEXES 1
#define CONFIG_USB_F_UAC2 1
#define CONFIG_PCI_DOE 1
#define CONFIG_CEC_PIN 1
#define CONFIG_ARM64_PAN 1
#define CONFIG_ANDROID_BINDER_IPC 1
#define CONFIG_PARTITION_PERCPU 1
#define CONFIG_NET_IFE_MODULE 1
#define CONFIG_CHARGER_AXP20X_MODULE 1
#define CONFIG_RD_LZ4 1
#define CONFIG_USB_SERIAL_EDGEPORT_MODULE 1
#define CONFIG_PLFXLC_MODULE 1
#define CONFIG_IPMI_SSIF_MODULE 1
#define CONFIG_IEEE802154_AT86RF230_MODULE 1
#define CONFIG_ARM_MHU_V2_MODULE 1
#define CONFIG_NET_DSA_XRS700X_MDIO_MODULE 1
#define CONFIG_SENSORS_LIS3_I2C_MODULE 1
#define CONFIG_DVB_MXL692_MODULE 1
#define CONFIG_VIDEO_CX231XX_ALSA_MODULE 1
#define CONFIG_DEFAULT_HOSTNAME "(none)"
#define CONFIG_VFIO_PCI_INTX 1
#define CONFIG_GENERIC_NET_UTILS 1
#define CONFIG_SND_AUDIO_GRAPH_CARD_MODULE 1
#define CONFIG_REGULATOR_TPS65132 1
#define CONFIG_SND_SOC_ROCKCHIP_I2S_TDM_MODULE 1
#define CONFIG_USB_CONN_GPIO 1
#define CONFIG_LMP91000_MODULE 1
#define CONFIG_DM_ERA_MODULE 1
#define CONFIG_IP_VS_WRR_MODULE 1
#define CONFIG_SYSFB 1
#define CONFIG_LEDS_TRIGGER_ONESHOT 1
#define CONFIG_FRAME_POINTER 1
#define CONFIG_BLK_DEV_DRBD_MODULE 1
#define CONFIG_INPUT_JOYSTICK 1
#define CONFIG_BLK_DEV_DM_BUILTIN 1
#define CONFIG_SMARTJOYPLUS_FF 1
#define CONFIG_IP5XXX_POWER_MODULE 1
#define CONFIG_SCSI_LOWLEVEL 1
#define CONFIG_SENSORS_LM63_MODULE 1
#define CONFIG_SOC_TEGRA_CBB 1
#define CONFIG_INTERVAL_TREE 1
#define CONFIG_RTC_DRV_PCF8563_MODULE 1
#define CONFIG_QCOM_FALKOR_ERRATUM_1009 1
#define CONFIG_MESON_GXBB_WATCHDOG_MODULE 1
#define CONFIG_W1_SLAVE_DS2805_MODULE 1
#define CONFIG_SPARSEMEM 1
#define CONFIG_BCACHE_MODULE 1
#define CONFIG_HVC_DRIVER 1
#define CONFIG_NLS_MAC_GAELIC_MODULE 1
#define CONFIG_REGULATOR_MAX8973 1
#define CONFIG_SND_HDA_TEGRA_MODULE 1
#define CONFIG_NET_VENDOR_BROADCOM 1
#define CONFIG_RTC_SYSTOHC_DEVICE "rtc0"
#define CONFIG_ARM64_WORKAROUND_SPECULATIVE_AT 1
#define CONFIG_PWM_BCM2835_MODULE 1
#define CONFIG_RTC_INTF_PROC 1
#define CONFIG_USB_CHIPIDEA_NPCM 1
#define CONFIG_RC_LOOPBACK_MODULE 1
#define CONFIG_AUTOFS_FS 1
#define CONFIG_AD5791_MODULE 1
#define CONFIG_DVB_CX24123_MODULE 1
#define CONFIG_ARM_SUN8I_A33_MBUS_DEVFREQ 1
#define CONFIG_LD_CAN_USE_KEEP_IN_OVERLAY 1
#define CONFIG_I2C_SIMTEC_MODULE 1
#define CONFIG_IPV6_MROUTE_MULTIPLE_TABLES 1
#define CONFIG_ARCH_HAS_PTE_DEVMAP 1
#define CONFIG_BT_BREDR 1
#define CONFIG_TYPEC_DP_ALTMODE 1
#define CONFIG_ARCH_WANTS_NO_INSTR 1
#define CONFIG_NET_DSA_QCA8K_MODULE 1
#define CONFIG_USB_HID 1
#define CONFIG_SMSC911X 1
#define CONFIG_SFC_MCDI_MON 1
#define CONFIG_USB_HSO_MODULE 1
#define CONFIG_NFT_BRIDGE_REJECT_MODULE 1
#define CONFIG_SND_HDA_RECONFIG 1
#define CONFIG_PHY_ROCKCHIP_USB 1
#define CONFIG_IIO_BUFFER_CB 1
#define CONFIG_CRYPTO_SKCIPHER2 1
#define CONFIG_MTD 1
#define CONFIG_SENSORS_LTC4261_MODULE 1
#define CONFIG_PCI_MSI 1
#define CONFIG_SRF08_MODULE 1
#define CONFIG_CLK_RK3308 1
#define CONFIG_RMI4_CORE_MODULE 1
#define CONFIG_PATA_PDC2027X_MODULE 1
#define CONFIG_HAVE_GENERIC_VDSO 1
#define CONFIG_ARM_SCPI_CPUFREQ 1
#define CONFIG_SND_HDA_MODULE 1
#define CONFIG_TOUCHSCREEN_USB_PANJIT 1
#define CONFIG_SOC_BRCMSTB 1
#define CONFIG_WLAN_VENDOR_PURELIFI 1
#define CONFIG_NF_TABLES_ARP 1
#define CONFIG_TMPFS_POSIX_ACL 1
#define CONFIG_PCIE_DW_PLAT_HOST 1
#define CONFIG_MODULES_TREE_LOOKUP 1
#define CONFIG_FW_LOADER_PAGED_BUF 1
#define CONFIG_AIC79XX_DEBUG_MASK 0
#define CONFIG_IRQ_MSI_LIB 1
#define CONFIG_NILFS2_FS_MODULE 1
#define CONFIG_IP_VS_PE_SIP_MODULE 1
#define CONFIG_USB_SERIAL_KOBIL_SCT_MODULE 1
#define CONFIG_PHY_NS2_PCIE 1
#define CONFIG_BATMAN_ADV_MODULE 1
#define CONFIG_SATA_INIC162X_MODULE 1
#define CONFIG_NVMEM_RMEM_MODULE 1
#define CONFIG_USB_NET_CDC_EEM_MODULE 1
#define CONFIG_BMI160_I2C_MODULE 1
#define CONFIG_MEDIA_TUNER_E4000_MODULE 1
#define CONFIG_RTC_DRV_DS1286_MODULE 1
#define CONFIG_RTL8723AE_MODULE 1
#define CONFIG_REGULATOR_MP8859 1
#define CONFIG_DRM_CLIENT_DEFAULT_FBDEV 1
#define CONFIG_SND_AU8820_MODULE 1
#define CONFIG_FB_SSD1307_MODULE 1
#define CONFIG_VIDEO_ROCKCHIP_VDEC_MODULE 1
#define CONFIG_CPU_FREQ_GOV_USERSPACE 1
#define CONFIG_TOUCHSCREEN_USB_ETT_TC45USB 1
#define CONFIG_SUNXI_RSB 1
#define CONFIG_SCSI_SYM53C8XX_DEFAULT_TAGS 16
#define CONFIG_PHY_ROCKCHIP_SNPS_PCIE3 1
#define CONFIG_BLK_DEV_NULL_BLK_MODULE 1
#define CONFIG_IP_VS_PROTO_AH_ESP 1
#define CONFIG_HAVE_ARCH_BITREVERSE 1
#define CONFIG_JOLIET 1
#define CONFIG_DEVFREQ_THERMAL 1
#define CONFIG_V4L_MEM2MEM_DRIVERS 1
#define CONFIG_JOYSTICK_IFORCE_232_MODULE 1
#define CONFIG_COMMON_CLK_G12A 1
#define CONFIG_EROFS_FS_ZIP 1
#define CONFIG_DVB_S5H1409_MODULE 1
#define CONFIG_SENSORS_LM83_MODULE 1
#define CONFIG_BCM2835_THERMAL_MODULE 1
#define CONFIG_B43LEGACY_PIO 1
#define CONFIG_ARCH_HAS_PREEMPT_LAZY 1
#define CONFIG_SERIAL_8250_DW 1
#define CONFIG_SFC_FALCON_MODULE 1
#define CONFIG_ACENIC_MODULE 1
#define CONFIG_SENSORS_MAX16601_MODULE 1
#define CONFIG_TEGRA_CLK_DFLL 1
#define CONFIG_MD_RAID456_MODULE 1
#define CONFIG_PATA_PDC_OLD_MODULE 1
#define CONFIG_TEGRA_AHB 1
#define CONFIG_W1_SLAVE_DS2433_CRC 1
#define CONFIG_HW_RANDOM_TPM 1
#define CONFIG_ILLEGAL_POINTER_VALUE 0xdead000000000000
#define CONFIG_HI8435_MODULE 1
#define CONFIG_IWLEGACY_DEBUG 1
#define CONFIG_ARCH_HAS_SETUP_DMA_OPS 1
#define CONFIG_INPUT_MATRIXKMAP 1
#define CONFIG_IP_NF_MATCH_AH_MODULE 1
#define CONFIG_GROUP_SCHED_WEIGHT 1
#define CONFIG_MEDIA_TUNER_SIMPLE_MODULE 1
#define CONFIG_ARM_SCMI_CPUFREQ 1
#define CONFIG_ADIS16400_MODULE 1
#define CONFIG_NFT_NAT_MODULE 1
#define CONFIG_ZSWAP_COMPRESSOR_DEFAULT "lzo"
#define CONFIG_VIDEO_CS53L32A_MODULE 1
#define CONFIG_MAX31827_MODULE 1
#define CONFIG_NF_CONNTRACK_ZONES 1
#define CONFIG_ROCKCHIP_ERRATUM_3588001 1
#define CONFIG_DRM_CLIENT 1
#define CONFIG_SCSI_AM53C974_MODULE 1
#define CONFIG_USB_UHCI_HCD_MODULE 1
#define CONFIG_SENSORS_VIA686A_MODULE 1
#define CONFIG_MT76x0E_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_MULTIPORT_MODULE 1
#define CONFIG_CRYPTO_NHPOLY1305 1
#define CONFIG_GENERIC_PHY 1
#define CONFIG_FCOE_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_LIMIT_MODULE 1
#define CONFIG_IPV6_MROUTE 1
#define CONFIG_USB_RAINSHADOW_CEC_MODULE 1
#define CONFIG_INET6_TUNNEL_MODULE 1
#define CONFIG_REGULATOR_MAX77620 1
#define CONFIG_GUEST_PERF_EVENTS 1
#define CONFIG_TEGRA210_ADMA 1
#define CONFIG_MPL3115_MODULE 1
#define CONFIG_DVB_BT8XX_MODULE 1
#define CONFIG_IWL3945_MODULE 1
#define CONFIG_DUMMY_CONSOLE 1
#define CONFIG_FB_TILEBLITTING 1
#define CONFIG_RTL_CARDS_MODULE 1
#define CONFIG_PANTHERLORD_FF 1
#define CONFIG_DVB_TUNER_DIB0090_MODULE 1
#define CONFIG_USB_F_TCM_MODULE 1
#define CONFIG_HAVE_ARCH_TRACEHOOK 1
#define CONFIG_IXGBE_HWMON 1
#define CONFIG_CRYPTO_DEV_AMLOGIC_GXL 1
#define CONFIG_DVB_PT3_MODULE 1
#define CONFIG_ARM_AMBA 1
#define CONFIG_NET_DSA_SMSC_LAN9303_MODULE 1
#define CONFIG_USB_SERIAL_XR_MODULE 1
#define CONFIG_IP_SET_MODULE 1
#define CONFIG_SND_SOC_SUNXI_AAUDIO_MODULE 1
#define CONFIG_IP_SET_BITMAP_PORT_MODULE 1
#define CONFIG_ACPI_APEI 1
#define CONFIG_TCP_CONG_NV_MODULE 1
#define CONFIG_AD8801_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_TCPMSS_MODULE 1
#define CONFIG_SLHC_MODULE 1
#define CONFIG_VIDEO_WM8775_MODULE 1
#define CONFIG_PPS_CLIENT_LDISC_MODULE 1
#define CONFIG_USB_GSPCA_SQ905_MODULE 1
#define CONFIG_SND_BCM2835_SOC_I2S_MODULE 1
#define CONFIG_ARM64_ERRATUM_3117295 1
#define CONFIG_CCS811_MODULE 1
#define CONFIG_TOUCHSCREEN_USB_GENERAL_TOUCH 1
#define CONFIG_CLK_RK3399 1
#define CONFIG_CPU_FREQ_THERMAL 1
#define CONFIG_DVB_CX24120_MODULE 1
#define CONFIG_PINCTRL_MCP23S08_MODULE 1
#define CONFIG_ATH10K_SDIO_MODULE 1
#define CONFIG_EFI_CAPSULE_LOADER_MODULE 1
#define CONFIG_DVB_USB_DIBUSB_MB_MODULE 1
#define CONFIG_DRM_NXP_PTN3460_MODULE 1
#define CONFIG_BAREUDP_MODULE 1
#define CONFIG_PATA_HPT37X_MODULE 1
#define CONFIG_LEDS_BLINKM_MODULE 1
#define CONFIG_SFC_MODULE 1
#define CONFIG_RADIO_TEA575X_MODULE 1
#define CONFIG_CPU_THERMAL 1
#define CONFIG_MMC_RICOH_MMC 1
#define CONFIG_CRYPTO_SHA256 1
#define CONFIG_ATH10K_MODULE 1
#define CONFIG_ARCH_SUPPORTS_KEXEC_IMAGE_VERIFY_SIG 1
#define CONFIG_SND_CTL_LED_MODULE 1
#define CONFIG_ARM_CCI400_PMU 1
#define CONFIG_CLK_TEGRA_BPMP 1
#define CONFIG_HAVE_BUILDTIME_MCOUNT_SORT 1
#define CONFIG_ATM_LANE_MODULE 1
#define CONFIG_SND_SOC_TEGRA_WM9712_MODULE 1
#define CONFIG_USB_GSPCA_KONICA_MODULE 1
#define CONFIG_NET_EMATCH_CMP_MODULE 1
#define CONFIG_QUOTACTL 1
#define CONFIG_BRIDGE_NF_EBTABLES_MODULE 1
#define CONFIG_CAVIUM_ERRATUM_23154 1
#define CONFIG_EXFAT_DEFAULT_IOCHARSET "utf8"
#define CONFIG_VIDEOBUF2_V4L2_MODULE 1
#define CONFIG_PHY_MESON8B_USB2 1
#define CONFIG_COMMON_CLK_MESON_VID_PLL_DIV 1
#define CONFIG_WIL6210_DEBUGFS 1
#define CONFIG_BT_HCIUART_BCM 1
#define CONFIG_UNIX 1
#define CONFIG_CLK_BCM_NS2 1
#define CONFIG_MEDIA_TUNER_M88RS6000T_MODULE 1
#define CONFIG_IRQ_WORK 1
#define CONFIG_PHY_BRCM_USB 1
#define CONFIG_ARCH_ENABLE_MEMORY_HOTREMOVE 1
#define CONFIG_NETFILTER_XT_MATCH_COMMENT_MODULE 1
#define CONFIG_PINCTRL_MESON_G12A 1
#define CONFIG_EROFS_FS_XATTR 1
#define CONFIG_CRYPTO_LIB_CURVE25519_GENERIC_MODULE 1
#define CONFIG_NLS_MAC_CROATIAN_MODULE 1
#define CONFIG_SCSI_HPSA_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_TPROXY_MODULE 1
#define CONFIG_DUMMY_CONSOLE_COLUMNS 80
#define CONFIG_NF_LOG_IPV4_MODULE 1
#define CONFIG_TLS_MODULE 1
#define CONFIG_NET_DEVMEM 1
#define CONFIG_ATH9K_PCI 1
#define CONFIG_ARM64_ERRATUM_1530923 1
#define CONFIG_B43LEGACY_LEDS 1
#define CONFIG_MFD_KHADAS_MCU_MODULE 1
#define CONFIG_TCP_CONG_HTCP_MODULE 1
#define CONFIG_CAN_SJA1000_PLATFORM_MODULE 1
#define CONFIG_SND_LAYLA24_MODULE 1
#define CONFIG_HID_PICOLCD_BACKLIGHT 1
#define CONFIG_STACKTRACE 1
#define CONFIG_AD7280_MODULE 1
#define CONFIG_WCN36XX_MODULE 1
#define CONFIG_COMMON_CLK_MESON_AO_CLKC 1
#define CONFIG_DVB_USB_CINERGY_T2_MODULE 1
#define CONFIG_BCMA_SFLASH 1
#define CONFIG_TYPHOON_MODULE 1
#define CONFIG_8139TOO_8129 1
#define CONFIG_6LOWPAN_NHC_HOP_MODULE 1
#define CONFIG_SND_SOC_SUNXI_MACH_MODULE 1
#define CONFIG_ARM64_ERRATUM_2051678 1
#define CONFIG_NET_IEEE8021Q_HELPERS 1
#define CONFIG_TUN_MODULE 1
#define CONFIG_SCSI_IPR_DUMP 1
#define CONFIG_SCA3000_MODULE 1
#define CONFIG_CAN_CC770_MODULE 1
#define CONFIG_NET_CLS_BASIC_MODULE 1
#define CONFIG_TCP_CONG_LP_MODULE 1
#define CONFIG_ADF4350_MODULE 1
#define CONFIG_DVB_USB_TECHNISAT_USB2_MODULE 1
#define CONFIG_SATA_QSTOR_MODULE 1
#define CONFIG_ARCH_USE_SYM_ANNOTATIONS 1
#define CONFIG_RTC_DRV_RV3029_HWMON 1
#define CONFIG_DRM_CLIENT_LOG 1
#define CONFIG_MAC80211_DEBUGFS 1
#define CONFIG_SUN50I_H6_R_CCU 1
#define CONFIG_B43LEGACY_DEBUG 1
#define CONFIG_IXGBE_DCB 1
#define CONFIG_MOUSE_BCM5974_MODULE 1
#define CONFIG_ATH5K_MODULE 1
#define CONFIG_SENSORS_MAX6639_MODULE 1
#define CONFIG_SENSORS_MAX31790_MODULE 1
#define CONFIG_CRYPTO_XCBC_MODULE 1
#define CONFIG_AR5523_MODULE 1
#define CONFIG_MLX4_CORE_GEN2 1
#define CONFIG_CROS_EC_DEBUGFS_MODULE 1
#define CONFIG_CHROMEOS_TBMC_MODULE 1
#define CONFIG_HW_RANDOM 1
#define CONFIG_IP_NF_TARGET_MASQUERADE_MODULE 1
#define CONFIG_RCU_NEED_SEGCBLIST 1
#define CONFIG_SENSORS_LM80_MODULE 1
#define CONFIG_USB_NET_CX82310_ETH_MODULE 1
#define CONFIG_AD5755_MODULE 1
#define CONFIG_BT_QCA_MODULE 1
#define CONFIG_SLIP_SMART 1
#define CONFIG_NEED_DMA_MAP_STATE 1
#define CONFIG_TCM_FC_MODULE 1
#define CONFIG_UBIFS_FS_MODULE 1
#define CONFIG_RCU_EXP_CPU_STALL_TIMEOUT 20
#define CONFIG_ROCKCHIP_DW_MIPI_DSI 1
#define CONFIG_OF 1
#define CONFIG_ARCH_MMAP_RND_BITS_MIN 18
#define CONFIG_USB_SERIAL_SYMBOL_MODULE 1
#define CONFIG_OPENVSWITCH_VXLAN_MODULE 1
#define CONFIG_CRYPTO_BLAKE2B_MODULE 1
#define CONFIG_MMC_SDHCI_OF_DWCMSHC 1
#define CONFIG_GFS2_FS_MODULE 1
#define CONFIG_ATA_FORCE 1
#define CONFIG_ARM64_POE 1
#define CONFIG_IP_SET_HASH_IPMAC_MODULE 1
#define CONFIG_SND_HDA_CODEC_CA0132_DSP 1
#define CONFIG_SENSORS_DS1621_MODULE 1
#define CONFIG_RT2800USB_UNKNOWN 1
#define CONFIG_SYSVIPC 1
#define CONFIG_LOCKDEP_SUPPORT 1
#define CONFIG_VIDEO_OV5640_MODULE 1
#define CONFIG_ARM64_ERRATUM_1319367 1
#define CONFIG_ARCH_MESON 1
#define CONFIG_REGMAP_AC97_MODULE 1
#define CONFIG_NFT_FIB_IPV6_MODULE 1
#define CONFIG_MEDIA_TUNER_TDA18250_MODULE 1
#define CONFIG_MEDIA_TUNER_QM1D1B0004_MODULE 1
#define CONFIG_NET_VENDOR_NETERION 1
#define CONFIG_IIO_ST_GYRO_3AXIS_MODULE 1
#define CONFIG_BT_HCIUART_LL 1
#define CONFIG_AD5933_MODULE 1
#define CONFIG_TCP_SIGPOOL 1
#define CONFIG_RT2X00_LIB_PCI_MODULE 1
#define CONFIG_FIREWIRE_NOSY_MODULE 1
#define CONFIG_LTC2632_MODULE 1
#define CONFIG_FSCACHE 1
#define CONFIG_MFD_SM501_MODULE 1
#define CONFIG_DM_INTEGRITY_MODULE 1
#define CONFIG_USB_CONFIGFS 1
#define CONFIG_USB_F_ECM 1
#define CONFIG_CRYPTO 1
#define CONFIG_NF_TABLES_BRIDGE_MODULE 1
#define CONFIG_USB_XHCI_TEGRA_MODULE 1
#define CONFIG_PCIEAER_CXL 1
#define CONFIG_PARAVIRT 1
#define CONFIG_HID_ELECOM_MODULE 1
#define CONFIG_HAVE_ARCH_SECCOMP 1
#define CONFIG_DVB_TUNER_CX24113_MODULE 1
#define CONFIG_IOMMU_IOPF 1
#define CONFIG_CAVIUM_ERRATUM_30115 1
#define CONFIG_AIC_LOADFW_SUPPORT_MODULE 1
#define CONFIG_AMPERE_ERRATUM_AC03_CPU_38 1
#define CONFIG_TI_ADC081C_MODULE 1
#define CONFIG_USB_ISP1760_DUAL_ROLE 1
#define CONFIG_DRM_ETNAVIV 1
#define CONFIG_SENSORS_LM70_MODULE 1
#define CONFIG_PINCTRL_MESON_GXL 1
#define CONFIG_PROC_FS 1
#define CONFIG_USB_ALI_M5632 1
#define CONFIG_RSI_SDIO_MODULE 1
#define CONFIG_SND_CTL_FAST_LOOKUP 1
#define CONFIG_DVB_USB_DIBUSB_MC_MODULE 1
#define CONFIG_ARCH_USE_MEMREMAP_PROT 1
#define CONFIG_BMA220_MODULE 1
#define CONFIG_SENSORS_MAX16064_MODULE 1
#define CONFIG_CRYPTO_SM4_ARM64_CE 1
#define CONFIG_CRYPTO_MICHAEL_MIC_MODULE 1
#define CONFIG_PM_DEVFREQ 1
#define CONFIG_DRM_CLIENT_SELECTION 1
#define CONFIG_SENSORS_F71805F_MODULE 1
#define CONFIG_NET_VENDOR_TEHUTI 1
#define CONFIG_NF_CT_PROTO_SCTP 1
#define CONFIG_USB_GSPCA_SPCA508_MODULE 1
#define CONFIG_MEDIA_PLATFORM_SUPPORT 1
#define CONFIG_HEADERS_INSTALL 1
#define CONFIG_PAGE_BLOCK_ORDER 10
#define CONFIG_POSIX_TIMERS 1
#define CONFIG_HID_BELKIN_MODULE 1
#define CONFIG_DVB_ISL6421_MODULE 1
#define CONFIG_UIO_CIF_MODULE 1
#define CONFIG_SND_OXYGEN_MODULE 1
#define CONFIG_ARM_RK3399_DMC_DEVFREQ 1
#define CONFIG_USB_SERIAL_KLSI_MODULE 1
#define CONFIG_BME680_MODULE 1
#define CONFIG_PCS_LYNX_MODULE 1
#define CONFIG_NOP_USB_XCEIV 1
#define CONFIG_NET_DSA_MT7530_MDIO_MODULE 1
#define CONFIG_ATA_GENERIC_MODULE 1
#define CONFIG_SQUASHFS_XZ 1
#define CONFIG_REGULATOR_SUN20I 1
#define CONFIG_OCTEONTX2_VF_MODULE 1
#define CONFIG_PPP_ASYNC_MODULE 1
#define CONFIG_SND_SOC_TEGRA210_DMIC_MODULE 1
#define CONFIG_DVB_MAX_ADAPTERS 8
#define CONFIG_BCMA_BLOCKIO 1
#define CONFIG_V4L2_MEM2MEM_DEV_MODULE 1
#define CONFIG_CC_HAVE_SHADOW_CALL_STACK 1
#define CONFIG_RTC_DRV_RS5C372_MODULE 1
#define CONFIG_HAVE_PREEMPT_DYNAMIC_KEY 1
#define CONFIG_XFRM_AH_MODULE 1
#define CONFIG_DVB_DIB8000_MODULE 1
#define CONFIG_EFI_BOOTLOADER_CONTROL 1
#define CONFIG_INET_DIAG 1
#define CONFIG_FS_IOMAP 1
#define CONFIG_FB_IOMEM_HELPERS 1
#define CONFIG_NATIONAL_PHY_MODULE 1
#define CONFIG_INPUT_VIVALDIFMAP 1
#define CONFIG_CHARGER_CROS_USBPD_MODULE 1
#define CONFIG_THERMAL_STATISTICS 1
#define CONFIG_USB_GSPCA_DTCS033_MODULE 1
#define CONFIG_IP_VS_PROTO_TCP 1
#define CONFIG_USB_GL860_MODULE 1
#define CONFIG_ALLOW_DEV_COREDUMP 1
#define CONFIG_ASN1 1
#define CONFIG_NF_NAT_PPTP_MODULE 1
#define CONFIG_STAGING_MEDIA 1
#define CONFIG_RAID_ATTRS_MODULE 1
#define CONFIG_MMC_CQHCI 1
#define CONFIG_DRM_DISPLAY_HDMI_AUDIO_HELPER 1
#define CONFIG_AD5624R_SPI_MODULE 1
#define CONFIG_DEVTMPFS_MOUNT 1
#define CONFIG_NFC_ST21NFCA_I2C_MODULE 1
#define CONFIG_HAVE_FTRACE_MCOUNT_RECORD 1
#define CONFIG_FIREWIRE_NET_MODULE 1
#define CONFIG_HID_LOGITECH_DJ_MODULE 1
#define CONFIG_SND_SOC_TEGRA20_I2S_MODULE 1
#define CONFIG_USB_SERIAL_EDGEPORT_TI_MODULE 1
#define CONFIG_CONSOLE_LOGLEVEL_QUIET 4
#define CONFIG_AD5592R_BASE_MODULE 1
#define CONFIG_ACPI_PCC 1
#define CONFIG_NETFILTER_XT_TARGET_NFLOG_MODULE 1
#define CONFIG_USB_ETH_MODULE 1
#define CONFIG_INET_TUNNEL_MODULE 1
#define CONFIG_VIDEO_WM8739_MODULE 1
#define CONFIG_DMA_SUN6I 1
#define CONFIG_PCI_SYSCALL 1
#define CONFIG_LZO_DECOMPRESS 1
#define CONFIG_DM_UNSTRIPED_MODULE 1
#define CONFIG_VIRTIO_VSOCKETS_MODULE 1
#define CONFIG_JUMP_LABEL 1
#define CONFIG_GENERIC_SMP_IDLE_THREAD 1
#define CONFIG_USB_ARCH_HAS_HCD 1
#define CONFIG_REED_SOLOMON_ENC8 1
#define CONFIG_NET_EMATCH_STACK 32
#define CONFIG_NF_NAT_FTP_MODULE 1
#define CONFIG_CRYPTO_AEGIS128_MODULE 1
#define CONFIG_DPOT_DAC_MODULE 1
#define CONFIG_LDM_PARTITION 1
#define CONFIG_IAVF_MODULE 1
#define CONFIG_VIDEO_UDA1342_MODULE 1
#define CONFIG_ARCH_MMAP_RND_COMPAT_BITS_MIN 11
#define CONFIG_NET_EMATCH_CANID_MODULE 1
#define CONFIG_RTC_HCTOSYS 1
#define CONFIG_EXTRA_FIRMWARE ""
#define CONFIG_IP_MULTICAST 1
#define CONFIG_PNPACPI 1
#define CONFIG_ARM_SMCCC_SOC_ID 1
#define CONFIG_ASYMMETRIC_PUBLIC_KEY_SUBTYPE 1
#define CONFIG_DVB_STV0288_MODULE 1
#define CONFIG_SENSORS_MAX6650_MODULE 1
#define CONFIG_LOCKUP_DETECTOR 1
#define CONFIG_FUSE_FS_MODULE 1
#define CONFIG_USB_STV06XX_MODULE 1
#define CONFIG_SENSORS_LIS3LV02D_MODULE 1
#define CONFIG_CRYPTO_NHPOLY1305_NEON 1
#define CONFIG_SUN55I_A523_CCU 1
#define CONFIG_SERIAL_8250_DWLIB 1
#define CONFIG_ZRAM_BACKEND_ZSTD 1
#define CONFIG_OF_KOBJ 1
#define CONFIG_USB_F_ACM 1
#define CONFIG_ARM_GIC 1
#define CONFIG_RTW88_CORE_MODULE 1
#define CONFIG_PSTORE_RAM_MODULE 1
#define CONFIG_EFI_CUSTOM_SSDT_OVERLAYS 1
#define CONFIG_MEDIA_TUNER_TUA9001_MODULE 1
#define CONFIG_HAVE_KVM_DIRTY_RING_ACQ_REL 1
#define CONFIG_MICREL_PHY 1
#define CONFIG_LZ4_COMPRESS 1
#define CONFIG_TOUCHSCREEN_USB_ETURBO 1
#define CONFIG_NETFILTER_XT_MATCH_SOCKET_MODULE 1
#define CONFIG_ARCH_HAS_FORCE_DMA_UNENCRYPTED 1
#define CONFIG_IP_VS_LBLCR_MODULE 1
#define CONFIG_PHY_NS2_USB_DRD 1
#define CONFIG_BTREE 1
#define CONFIG_DRM_SII9234_MODULE 1
#define CONFIG_PNFS_FLEXFILE_LAYOUT 1
#define CONFIG_GENERIC_PINMUX_FUNCTIONS 1
#define CONFIG_PINCTRL_SUN50I_A100_R 1
#define CONFIG_VETH_MODULE 1
#define CONFIG_USB_GSPCA_CONEX_MODULE 1
#define CONFIG_USB_ULPI_VIEWPORT 1
#define CONFIG_IP_VS_NFCT 1
#define CONFIG_USB_FEW_INIT_RETRIES 1
#define CONFIG_AD7303_MODULE 1
#define CONFIG_DVB_CXD2099_MODULE 1
#define CONFIG_SDIO_UART_MODULE 1
#define CONFIG_SSB_B43_PCI_BRIDGE 1
#define CONFIG_NET_SCH_PIE_MODULE 1
#define CONFIG_USB_SERIAL_MOS7720_MODULE 1
#define CONFIG_VIDEO_V4L2_SUBDEV_API 1
#define CONFIG_SCSI_SAS_ATA 1
#define CONFIG_INV_MPU6050_I2C_MODULE 1
#define CONFIG_SND_SOC_TEGRA_WM8753_MODULE 1
#define CONFIG_SENSORS_MAX31722_MODULE 1
#define CONFIG_SENSORS_MAX16065_MODULE 1
#define CONFIG_VIRTIO_BALLOON_MODULE 1
#define CONFIG_BCMA_HOST_PCI_POSSIBLE 1
#define CONFIG_CPU_ISOLATION 1
#define CONFIG_SERIAL_8250_PCI 1
#define CONFIG_USB_GSPCA_SONIXJ_MODULE 1
#define CONFIG_ARM_SCMI_PERF_DOMAIN 1
#define CONFIG_ZSWAP_ZPOOL_DEFAULT "zsmalloc"
#define CONFIG_LWTUNNEL_BPF 1
#define CONFIG_DLM_MODULE 1
#define CONFIG_SERIO 1
#define CONFIG_CHR_DEV_SCH_MODULE 1
#define CONFIG_USB_GSPCA_OV519_MODULE 1
#define CONFIG_NET_VRF_MODULE 1
#define CONFIG_DRM_GEM_SHMEM_HELPER_MODULE 1
#define CONFIG_ARCH_SUPPORTS_ATOMIC_RMW 1
#define CONFIG_FB_TFT_ST7789V_MODULE 1
#define CONFIG_DVB_TDA826X_MODULE 1
#define CONFIG_CRYPTO_LIB_AES 1
#define CONFIG_MMC_SDHCI_BRCMSTB 1
#define CONFIG_ARM64_E0PD 1
#define CONFIG_BMI160_SPI_MODULE 1
#define CONFIG_SND_GINA24_MODULE 1
#define CONFIG_CROS_USBPD_LOGGER_MODULE 1
#define CONFIG_LEGACY_TIOCSTI 1
#define CONFIG_DM_VERITY_FEC 1
#define CONFIG_SND_MESON_AXG_FIFO_MODULE 1
#define CONFIG_NF_LOG_SYSLOG_MODULE 1
#define CONFIG_USB_OHCI_HCD 1
#define CONFIG_VFIO_VIRQFD 1
#define CONFIG_ARCH_TEGRA_186_SOC 1
#define CONFIG_FB_IOMEM_FOPS 1
#define CONFIG_FW_CACHE 1
#define CONFIG_ARCH_HAS_SYNC_DMA_FOR_DEVICE 1
#define CONFIG_REGMAP_I2C 1
#define CONFIG_STMMAC_PLATFORM 1
#define CONFIG_IP_NF_TARGET_REDIRECT_MODULE 1
#define CONFIG_MMC_SDHCI 1
#define CONFIG_IOMMUFD_DRIVER 1
#define CONFIG_PRINTK 1
#define CONFIG_LEDS_TRIGGER_DISK 1
#define CONFIG_FIREWIRE_MODULE 1
#define CONFIG_DM_SWITCH_MODULE 1
#define CONFIG_VIDEO_EM28XX_DVB_MODULE 1
#define CONFIG_SND_MESON_AXG_SPDIFOUT_MODULE 1
#define CONFIG_NETLABEL 1
#define CONFIG_ARCH_SUPPORTS_KEXEC_HANDOVER 1
#define CONFIG_MEMCG_V1 1
#define CONFIG_PCI_STUB 1
#define CONFIG_CRYPTO_RMD160_MODULE 1
#define CONFIG_UEVENT_HELPER_PATH ""
#define CONFIG_USB_NET_CDC_NCM_MODULE 1
#define CONFIG_CC_VERSION_TEXT "gcc (GCC) 14.2.1 20250207"
#define CONFIG_EQUALIZER_MODULE 1
#define CONFIG_VIDEO_IVTV_MODULE 1
#define CONFIG_IP6_NF_MATCH_SRH_MODULE 1
#define CONFIG_BINFMT_ELF 1
#define CONFIG_I2C_ARB_GPIO_CHALLENGE_MODULE 1
#define CONFIG_HIGH_RES_TIMERS 1
#define CONFIG_I2C_SMBUS_MODULE 1
#define CONFIG_DEBUG_FS_ALLOW_ALL 1
#define CONFIG_NULL_TTY_MODULE 1
#define CONFIG_HAVE_SAMPLE_FTRACE_DIRECT_MULTI 1
#define CONFIG_WIREGUARD_MODULE 1
#define CONFIG_MEMFD_CREATE 1
#define CONFIG_FB_TFT_SSD1351_MODULE 1
#define CONFIG_USB_HSIC_USB3503_MODULE 1
#define CONFIG_SENSORS_HMC5843_I2C_MODULE 1
#define CONFIG_MTD_OF_PARTS 1
